/*===========================================================================*/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*===========================================================================*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "rssma.h"
#include "rssma_utility.h"

#define IEEE_ERROR_BITS         0x3C    /* AND mask for ESR checking */
#define BUFFER_SIZE             2048UL    /* File I/O buffer size */

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rssma_invalidViBooleanRange (ViSession instrSession, ViBoolean val)
{
    rssma_instrRange    instrPtr = NULL;

    viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr);

    /* Error Checking Enabled? */
    if (instrPtr != NULL)
    {
        if (instrPtr -> rangeChecking != VI_FALSE)
        {
            return(ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
        }
    }
    return VI_FALSE;
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rssma_invalidViInt32Range (ViSession instrSession, ViInt32 val, ViInt32 min, ViInt32 max)
{
    rssma_instrRange    instrPtr = NULL;

    viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr);

    /* Error Checking Enabled? */
    if (instrPtr != NULL)
    {
        if (instrPtr -> rangeChecking != VI_FALSE)
        {
            return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
        }
    }
    return VI_FALSE;
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rssma_invalidViReal64Range (ViSession instrSession, ViReal64 val, ViReal64 min, ViReal64 max)
{
    rssma_instrRange    instrPtr = NULL;

    viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr);

    /* Error Checking Enabled? */
    if (instrPtr != NULL)
    {
        if (instrPtr -> rangeChecking != VI_FALSE)
        {
            return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
        }
    }
    return VI_FALSE;
}
/*=========================================================================*/
/* Function: Check Status                                                  */
/* Purpose:  This function reads Event Status Register to check for        */
/*           possible errors.                                              */
/*=========================================================================*/
ViStatus rssma_checkStatus (ViSession instrSession)
{
    ViStatus            rssma_status = VI_SUCCESS;
    rssma_instrRange    instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Error Checking Enabled? */
    if (instrPtr != NULL)
    {
        if (instrPtr -> errorChecking != VI_FALSE)
        {
            ViInt32 esrByte = 0;

            /*--------------- Read the Standard Event Register ------------------*/
            CHECKERR (viQueryf (instrSession, "*ESR?\n", "%ld", &esrByte));

            /*---------------- Check if any error or message bit was asserted ---*/
            rssma_status = ((esrByte & IEEE_ERROR_BITS) != 0) ? RSSMA_ERROR_INSTRUMENT_ERROR : VI_SUCCESS;
        }
    }

    return rssma_status;
}

/*===========================================================================*/
/* Function: Wait for OPC                                                    */
/* Purpose:  This function sends a command to the instrument and waits until */
/*           the instrument response is available (queries MAV in STB).      */
/*===========================================================================*/
ViStatus rssma_waitOPC (ViSession instrSession, ViPString command,
                ViPString instrResp, ViInt32 timeout)
{
    ViStatus    rssma_status    = VI_SUCCESS;
    ViUInt16    response        = 0;
    ViInt32     counter         = 0;
    ViInt32     oldTimeout      = 0;
    ViUInt16    intf       = VI_INTF_GPIB;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf));

    /* Interface specific operations */
    switch (intf)
        {
        case VI_INTF_ASRL: /* Serial intf */
            CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, (ViAttrState) timeout));
            if ((rssma_status = viQueryf (instrSession, "%s\n", "%[^\r\n]", command, instrResp)) < 0)
            {
                (void) viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, (ViAttrState) oldTimeout);
                return rssma_status;
            }
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, (ViAttrState) oldTimeout));
        break;
        default: /* Default case */
            CHECKERR (viPrintf (instrSession, "%s\n", command));

            while ((response & 0x10) == 0) /* (queries MAV in STB) */
                {

                CHECKERR (viReadSTB (instrSession, &response));

                (void) rssma_delay (instrSession, 0.005);
                counter+= 5;
                if (counter >= timeout)
                    return VI_ERROR_TMO;
                }

            CHECKERR (viScanf (instrSession, "%[^\r\n]", instrResp));
        break;
        }

    return rssma_status;
}

/*===========================================================================*/
/* Function: Send Command                                                    */
/* Purpose:  This function sends command to the instrument with error check. */
/*===========================================================================*/
ViStatus rssma_sendCmd (ViSession instrSession, ViString cmd)
{
    ViStatus    rssma_status    = VI_SUCCESS;
    size_t      length          = 0;
    ViChar      *pbuffer;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc(length + 1);
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;

    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    length++;
    GOCHECKERR (viWrite (instrSession, (ViPBuf) cmd, (ViUInt32) length, VI_NULL));
    GOCHECKERR (rssma_checkStatus (instrSession));

Error:

    free(pbuffer);

    return rssma_status;
}

/*===========================================================================*/
/* Function: Set ViInt32                                                     */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssma_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViString parameterName)
{
    ViChar      *wrt_buf;
    ViStatus    rssma_status = VI_SUCCESS;

    if ((max - min) > 0)
        if (rssma_invalidViInt32Range (instrSession, value, min, max) == VI_TRUE)
		{
		    rssma_instrRange   instrPtr;
		    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
			strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
	        return VI_ERROR_INSTR_PARAMETER;
		}

    
    wrt_buf = (ViChar*) malloc (strlen (cmd) + 20);
    
    sprintf (wrt_buf, "%s %ld", cmd, value);

    rssma_status = rssma_sendCmd (instrSession, wrt_buf);
    
    free (wrt_buf);
    return rssma_status;
}

/*===========================================================================*/
/* Function: Get ViInt32                                                     */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssma_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;

    if (value == NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    CHECKERR( viPrintf(instrSession, "%s\n", cmd));
    CHECKERR( viScanf(instrSession, "%ld", value));
    CHECKERR( rssma_checkStatus (instrSession));

    return rssma_status;
}

/*===========================================================================*/
/* Function: Get ViUInt32                                                    */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssma_getViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 *value,
                           ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;

    if (value == NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    CHECKERR( viPrintf(instrSession, "%s\n", cmd));
    CHECKERR( viScanf(instrSession, "%u", value));
    CHECKERR( rssma_checkStatus (instrSession));

    return rssma_status;
}

/*===========================================================================*/
/* Function: Set ViBoolean                                                   */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssma_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViString parameterName)
{
    return rssma_setViString (instrSession,
                              cmd,
                              value ? "ON" : "OFF",
                              parameterName);
}

/*===========================================================================*/
/* Function: Get ViBoolean                                                   */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssma_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViString parameterName)
{
    ViStatus    rssma_status;
    ViChar      buffer[4];

	if (value == NULL)
	{
		rssma_instrRange   instrPtr;
		CHECKERR(viGetAttribute(instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy(instrPtr->parameterErrorMessage, parameterName, 255);
		return VI_ERROR_INSTR_PARAMETER;
	}

    CHECKERR (rssma_getViString (instrSession,
                                cmd,
                                buffer, 4,
                                0));
    *value = (ViBoolean) ((strncmp (buffer, "1", 1) == 0) || (strncmp (buffer,"ON",2) == 0));

    return rssma_status;
}

/*===========================================================================*/
/* Function: Set ViReal64                                                    */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rssma_setViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViString parameterName)
{
    ViChar      buffer[BUFFER_SIZE];
    ViStatus    rssma_status = VI_SUCCESS;

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rssma_invalidViReal64Range (instrSession, value, min, max) == VI_TRUE)
		{
		    rssma_instrRange   instrPtr;
		    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
			strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
	        return VI_ERROR_INSTR_PARAMETER;
		}

    sprintf(buffer,"%s %.12lG", cmd, value);

    return rssma_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get ViReal64                                                    */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rssma_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;

    if (value == NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    CHECKERR(viPrintf(instrSession, "%s\n",cmd));
    CHECKERR(viScanf (instrSession, "%le",value));
    CHECKERR(rssma_checkStatus (instrSession));

    return rssma_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 with Units                                         */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rssma_setViReal64Units (ViSession instrSession,
                                 ViString cmd,
                                 ViReal64 value,
                                 ViString units,
                                 ViReal64 min,
                                 ViReal64 max,
                                 ViString parameterName)
{
    ViChar      buffer[BUFFER_SIZE];
    ViStatus    rssma_status = VI_SUCCESS;

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rssma_invalidViReal64Range (instrSession, value, min, max) == VI_TRUE)
		{
		    rssma_instrRange   instrPtr;
		    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
			strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
	        return VI_ERROR_INSTR_PARAMETER;
		}

    sprintf(buffer,"%s %.12f %s",cmd, value, units);

    return rssma_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Set String                                                      */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rssma_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 4 );
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, "%s %s\n", cmd, value);

    GOCHECKERR(viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32)(p2buf - wrt_buf), VI_NULL));
    GOCHECKERR(rssma_checkStatus(instrSession));

Error:

    free(wrt_buf);

    return rssma_status;
}

/*===========================================================================*/
/* Function: Get String                                                      */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rssma_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViUInt32 buffer_size,
                            ViString parameterName)
{
    ViStatus    rssma_status    = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    ViUInt32    count           = 0;
    size_t      length          = 0;

    if (value == NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    
    memcpy (pbuffer, cmd, length);
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    pbuffer[length] = '\n';

    GOCHECKERR (viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) (length + 1), VI_NULL));

    free (pbuffer);
    
    pbuffer = NULL;

    GOCHECKERR (rssma_readDataUnknownLength (instrSession, &pbuffer, &count));
    
    if (pbuffer == NULL)
        return rssma_status;
    
    GOCHECKERR (rssma_checkStatus (instrSession));

    if (count > buffer_size)
    {
        rssma_status = RSSMA_WARNING_SMALL_BUFFER;
        count = buffer_size;
        memcpy (value, pbuffer, (size_t) count);
        value[count] = '\0';
    }
    else
        if ((p2buffer = strpbrk (pbuffer, "\r\n")) != VI_NULL)
        {
            rssma_status = VI_SUCCESS;
            memcpy (value, pbuffer, (size_t) (p2buffer - pbuffer));
            value[p2buffer - pbuffer] = '\0';
        }

Error:

    if (pbuffer != NULL)
        free (pbuffer);

    return rssma_status;
}

/*===========================================================================*/
/* Function: Set Quoted String                                               */
/* Purpose:  This function sets command with quoted string parameter.        */
/*===========================================================================*/
ViStatus rssma_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    wrt_buf = (ViChar*)malloc(strlen(cmd) + strlen(value) + 2*strlen(quotes) + 4);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf,"%s %s%s%s\n", cmd, quotes, value, quotes);

    GOCHECKERR (viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL));
    GOCHECKERR (rssma_checkStatus(instrSession));

Error:

    free (wrt_buf);

    return rssma_status;
}

/*===========================================================================*/
/* Function: Set Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rssma_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViString parameterName)
{
    ViStatus rssma_status = VI_SUCCESS;

    CHECKERR (rssma_checkIndexInList (instrSession, value, list, parameterName));

    return rssma_setViString (instrSession, cmd, list[value], parameterName);
}

/*===========================================================================*/
/* Function: Get Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rssma_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE] = "";

    CHECKERR( rssma_getViString(instrSession, cmd, buffer, BUFFER_SIZE, parameterName));
    CHECKERR( rssma_getIndexFromString (buffer, list, value));

    return rssma_status;
}

/*===========================================================================*/
/* Function: Check Index List                                                */
/* Purpose:  This function checks if the passed index is valid for given list*/
/*===========================================================================*/
ViStatus rssma_checkIndexInList (ViSession instrSession,
								 ViInt32 index,
                                 ViString list[],
                                 ViString parameterName)

{
    ViInt32     i=0;
	ViStatus    rssma_status = VI_SUCCESS;

    if (index < 0)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    while (list[i++] != VI_NULL);
    if (index >= i)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Get Index From String                                           */
/* Purpose:  This function returns index for string.                         */
/*===========================================================================*/
ViStatus rssma_getIndexFromString (ViString string,
                                   ViString *list,
                                   ViInt32 *index)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViUInt32    i;
    size_t      len;

    for (i = 0; list[i]; i++)
        if ((len = strlen (string)) == strlen (list[i])) {
            if (strncmp(string, list[i], strlen (string)) == 0) break;
        }

    *index = (ViInt32) i;

    return (list[i]) ? rssma_status : VI_ERROR_INSTR_INTERPRETING_RESPONSE;
}

/*===========================================================================*/
/* Function: Get Catalog                                                     */
/*===========================================================================*/
ViStatus rssma_getCatalog (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 arraySize,
                           ViChar* array,
                           ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    size_t      count  = 0,
                length = 0;
    ViUInt32    unknown_count = 0;

    if (array == NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    GOCHECKERR (viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) (length + 1), VI_NULL));

    free(pbuffer);
    
    pbuffer = NULL;

    GOCHECKERR (rssma_readDataUnknownLength (instrSession, &pbuffer, &unknown_count));
    GOCHECKERR (rssma_checkStatus (instrSession));

    if (count > (size_t) arraySize)
        {
        rssma_status = RSSMA_WARNING_SMALL_BUFFER;
        count = (size_t)arraySize;
        memcpy (array, pbuffer, count);
        array[count-1] = '\0';
        }
    else
        {
        if ((p2buffer = strchr (pbuffer, ';')) != VI_NULL)
            {
            rssma_status = VI_SUCCESS;
            memcpy (array, pbuffer, (size_t)(p2buffer - pbuffer));
            array[p2buffer - pbuffer] = '\0';
            }
        else
            {
            memcpy (array, pbuffer, count);
            array[count-1] = '\0';
            }
        }
Error:

    if (pbuffer != NULL)
        free (pbuffer);

    return rssma_status;
}

/*===========================================================================*/
/* Function: Get Catalog With Separator                                      */
/*===========================================================================*/
ViStatus rssma_getCatalogWithSeparator (ViSession instrSession,
                           ViString cmd,
						   ViChar separator,
                           ViUInt32 arraySize,
                           ViChar* array,
                           ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    size_t      count  = 0,
                length = 0;
    ViUInt32    unknown_count = 0;

    if (array == NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    GOCHECKERR (viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) (length + 1), VI_NULL));

    free(pbuffer);
    
    pbuffer = NULL;

    GOCHECKERR (rssma_readDataUnknownLength (instrSession, &pbuffer, &unknown_count));
    GOCHECKERR (rssma_checkStatus (instrSession));

    if (count > (size_t) arraySize)
        {
        rssma_status = RSSMA_WARNING_SMALL_BUFFER;
        count = (size_t)arraySize;
        memcpy (array, pbuffer, count);
        array[count-1] = '\0';
        }
    else
        {
        if ((p2buffer = strchr (pbuffer, separator)) != VI_NULL)
            {
            rssma_status = VI_SUCCESS;
            memcpy (array, pbuffer, (size_t)(p2buffer - pbuffer));
            array[p2buffer - pbuffer] = '\0';
            }
        else
            {
            memcpy (array, pbuffer, count);
            array[count-1] = '\0';
            }
        }
Error:

    if (pbuffer != NULL)
        free (pbuffer);

    return rssma_status;
}
/*===========================================================================*/
/* Function: Set ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rssma_setViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value[],
                            ViInt32  count,
                            ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViInt32     i = 0;

    if (value == NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    pbuffer = (ViChar*) malloc (count * 15 + strlen (cmd) + 200);
	
	p2buf = pbuffer + sprintf (pbuffer, "%s ", cmd);
    
	while ((p2buf != NULL) && (i < count))
        p2buf += sprintf (p2buf, "%.12lG,", value[i++]);
	
	*--p2buf = '\n';
	
	GOCHECKERR (viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) strlen (pbuffer), NULL));

	GOCHECKERR (rssma_checkStatus (instrSession));

Error:

    free(pbuffer);

    return rssma_status;
}

/*===========================================================================*/
/* Function: Get ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rssma_getViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value[],
                            ViInt32  count,
                            ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count = 0;
    ViInt32     i = 0;

    if (value == NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    GOCHECKERR (viPrintf(instrSession, "%s\n",cmd));
    GOCHECKERR (rssma_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while ((p2buf != NULL) && (i < count))
    {
        value[i++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
    };

    GOCHECKERR (rssma_checkStatus (instrSession));

Error:

    free(pbuffer);

    return rssma_status;
}

/*===========================================================================*/
/* Function: Get ViInt32 Array                                               */
/*===========================================================================*/
ViStatus rssma_getViInt32Array (ViSession instrSession,
                            ViString cmd,
                            ViInt32 *value,
                            ViInt32  count,
                            ViString parameterName)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count = 0;
    ViInt32     i = 0;

    if (value == NULL)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
        return VI_ERROR_INSTR_PARAMETER;
	}

    GOCHECKERR (viPrintf(instrSession, "%s\n",cmd));
    GOCHECKERR (rssma_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while ((p2buf != NULL) && (i < count))
    {
        value[i++] = atol(p2buf);
        p2buf = strtok(NULL, ",");
    };

    GOCHECKERR (rssma_checkStatus (instrSession));

Error:

    free(pbuffer);

    return rssma_status;
}

/*===========================================================================*/
/* Function: Read Data With Unknown length                                   */
/* Purpose:  This function allocates buffer and reads data from the          */
/*           instrument                                                      */
/* WARNING:  You must deallocate buffer, if any pointer has been returned.   */
/*           This function does not work correctly, when you read bin data   */
/*           via serial intf                                                 */
/*===========================================================================*/
ViStatus rssma_readDataUnknownLength(ViSession instrSession,
                                     /*@null@*/ViChar **outBuffer,
                                     ViUInt32 *count)
{
    ViStatus rssma_status = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = NULL, *p2buffer = NULL;

    *outBuffer = VI_NULL;
    if (count) *count = 0;

    /* Allocate Buffer */
    actSize = BUFFER_SIZE;
    buffer = (ViChar *)calloc(actSize, (size_t) sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
    {
        if ((rssma_status = viRead (instrSession, (ViPBuf)p2buffer, BUFFER_SIZE, &retCnt)) < 0)
        {
            free(buffer);
            *outBuffer = VI_NULL;
            return rssma_status;
        }
        if (retCnt == BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, (size_t) actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string at the end of data */
        }
    } while (rssma_status == VI_SUCCESS_MAX_CNT); /* if buffer was small, continue with next read */

    *outBuffer = buffer;
    if (count != NULL) *count = (actSize - BUFFER_SIZE) + retCnt;

    return rssma_status;
}

/*===========================================================================*/
/* Function: Read Data To File                                               */
/* Purpose:  This function read data from the instrument and                 */
/*           write it to a user specified file.                              */
/*===========================================================================*/
ViStatus rssma_readDataToFile (ViSession instrSession, ViString cmd,
                           ViString destination)
{
    ViStatus    rssma_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViUInt32    count           = 0,
                lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
                format[32],
                *pbuffer = NULL, *p2buffer = NULL;
    FILE        *targetFile;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    sprintf (buffer, "%s\n", cmd);

    CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL));
    CHECKERR (rssma_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1lu", &lengthOfLength) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    sprintf (format, "%%%lud", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    p2buffer += lengthOfLength;

    if (intfType == VI_INTF_GPIB)
        if ((rssma_status = viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL)) < VI_SUCCESS)
            {
            free (pbuffer);
            return rssma_status;
            }

    if ((rssma_status = rssma_checkStatus (instrSession)) < VI_SUCCESS)
        {
        free (pbuffer);
        return rssma_status;
        }

    /* Store data from instrument to file on host computer */
    if ((targetFile = fopen (destination, "wb")) == NULL)
        {
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    if (count != (ViUInt32) fwrite (p2buffer, sizeof (ViByte), (size_t)(count), targetFile))
        {
        (void) fclose (targetFile);
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_WRITE;
        }

    (void) fclose (targetFile);

    if (pbuffer) /* If not NULL pointer, then deallocate memory. */
        free (pbuffer);

    if ((rssma_status = rssma_checkStatus (instrSession)) < 0)
        return rssma_status;

    return rssma_status;
}

ViStatus rssma_ParameterError (ViSession io, ViString parameterName)
{
	ViStatus    rssma_status = VI_SUCCESS;
	rssma_instrRange   instrPtr;
	CHECKERR (viGetAttribute (io, VI_ATTR_USER_DATA, &instrPtr));
	strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
	return VI_ERROR_INSTR_PARAMETER;
}
