/*= R&S SMA Signal Generator ================================================
 * LabWindows/CVI 8.0 VXIpnp Instrument Driver
 * Original Release: May 2006
 * By: Jiri Kominek
 *
 * Should you have any technical questions please contact the hotline of
 * Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 * e-mail: CustomerSupport@rsd.rohde-schwarz.com
 *
 * Modification History:
 *
 *   see file rssma_vxi.chm
 *
*===========================================================================*/
#ifdef _WIN32
    #ifdef _CVI_
        #include <userint.h>
    #else
        #include <windows.h>
    #endif
#endif
#include <math.h>
#include <visa.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include "rssma.h"
#include "rssma_utility.h"
#include <limits.h>
#include <locale.h>  

/*= Macros ==================================================================*/

#define CHECKERR_CU(fCal) if (rssma_status = (fCal), rssma_status < VI_SUCCESS) \
    return rssma_initCleanUp (rmSession, instrSession, rssma_status); else

/* Checking ViInt32 argument */
#define CHECKVIINT32(io, arg, min, max) if ( rssma_invalidViInt32Range (io, arg,min,max)) return rssma_ParameterError(io,#arg)
/* Checking ViReal64 argument */
#define CHECKVIREAL64(io, arg, min, max) if ( rssma_invalidViReal64Range (io, arg,min,max)) return rssma_ParameterError(io,#arg)

#define SETPARAMBOOL(io,cmd,value) rssma_setViBoolean (io,cmd,value,#value)
#define GETPARAMBOOL(io,cmd,value) rssma_getViBoolean (io,cmd,value,#value)

#define SETPARAMINT32(io,cmd,value,min,max) rssma_setViInt32 (io,cmd,value,min,max,#value)
#define GETPARAMINT32(io,cmd,value) rssma_getViInt32 (io,cmd,value,#value)
#define GETPARAMUINT32(io,cmd,value) rssma_getViUInt32 (io,cmd,value,#value)
#define GETPARAMINT32ARRAY(io,cmd,value,count) rssma_getViInt32Array (io,cmd,value,count,#value)

#define SETPARAMENUM(io,cmd,value,list) rssma_setEnum (io,cmd,value,list,#value)
#define GETPARAMENUM(io,cmd,value,list) rssma_getEnum (io,cmd,value,list,#value)

#define SETPARAMREAL64(io,cmd,value,min,max) rssma_setViReal64 (io,cmd,value,min,max,#value)
#define SETPARAMREAL64UNITS(io,cmd,value,units,min,max) rssma_setViReal64Units (io,cmd,value,units,min,max,#value)
#define GETPARAMREAL64(io,cmd,value) rssma_getViReal64 (io,cmd,value,#value)
#define GETPARAMREAL64ARRAY(io,cmd,value,count) rssma_getViReal64Array (io,cmd,value,count,#value)
#define SETPARAMREAL64ARRAY(io,cmd,value,count) rssma_setViReal64Array (io,cmd,value,count,#value)

#define SETPARAMSTRINGQUOTED(io,cmd,quotes,value) rssma_setViStringQuoted (io,cmd,quotes,value,#value)
#define GETPARAMSTRINGQUOTED(io,cmd,quotes,buf_size,value) rssma_getViStringQuoted (io,cmd,quotes,buf_size,value,#value)

#define SETPARAMSTRING(io,cmd,value) rssma_setViString (io,cmd,value,#value)
#define GETPARAMSTRING(io,cmd,value,buf_size) rssma_getViString (io,cmd,value,buf_size,#value)

/*= DEFINES =================================================================*/
#define rssma_REVISION          "Rev 2.21.0, 07/2016, CVI 2015" /* Instrument driver revision */
#define BUFFER_SIZE             2048L    /* File I/O buffer size */

#define rssma_DEFAULT_TIMEOUT   10000   /* Default Timeout value in ms (10 sec) */

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString triggerRFSourceArr[] = {"AUTO","IMM","SING","EXT","BUS","EAUT", VI_NULL};
static ViString onRecallArr[] = {"INCL","EXCL", VI_NULL};
static ViString sourceArr[] = {"INT","EXT", VI_NULL};
static ViString externReferenceFreqArr[] = {"5MHZ","10MHZ","13MHZ", VI_NULL};
static ViString bandwidthArr[] = {"NARR","WIDE", VI_NULL};
static ViString attenuatorModeArr[] = {"AUTO","FIX","NORM","HPOW", VI_NULL};
static ViString attenuatorRFOffMode[] = {"UNCH","FATT",VI_NULL};
static ViString powerOnStateArr[] = {"OFF","UNCH", VI_NULL};
static ViString alcStateArr[] = {"OFF","ON","AUTO", VI_NULL};
static ViString lfoSourceArr[] = {"LF1","LF2","LF12","NOIS","LF1N","LF2N", VI_NULL};
static ViString shapeArr[] = {"SINE","SQU","TRI","TRAP", VI_NULL};
static ViString amExtCouplingArr[] = {"AC","DC", VI_NULL};
static ViString polarityArr[] = {"NORM","INV", VI_NULL};
static ViString pulmSourceArr[] = {"INT","EXT","COD", VI_NULL};
static ViString pulmTriggerModeArr[] = {"AUTO","EXT","EGAT", VI_NULL};
static ViString fmSourceArr[] = {"INT","EXT","INT,EXT", "EDIG", VI_NULL};
static ViString fmModeArr[] = {"NORM","LNO", VI_NULL};
static ViString pmModeArr[] = {"HBAN","HDEV","LNO", VI_NULL};
static ViString fmExtCouplingArr[] = {"AC","DC", VI_NULL};
static ViString fmExtBinaryFormatArr[] = {"BOFF","DCOD", VI_NULL};
static ViString distributionArr[] = {"GAUS","EQU", VI_NULL};
static ViString slopeArr[] = {"POS","NEG", VI_NULL};
static ViString impedanceArr[] = {"G50","HIGH",VI_NULL};
static ViString testCommandsArr[] = {"SSYN","SSYNEX","IQOP3","IQOP6","SATT3",
                                     "SATT6","BBINS","DIRECTLAST", VI_NULL};
static ViString spacingArr[] = {"LIN","LOG", VI_NULL};
static ViString regGroupArr[] = {"OPER","QUES", VI_NULL};
static ViString regTypeArr[] = {"COND","EVEN","ENAB","NTR","PTR", VI_NULL};
static ViString lfSweepModeArr[] = {"AUTO","MAN","STEP", VI_NULL};
static ViString frequencyModeArr[] = {"CW","SWE","LIST", VI_NULL};
static ViString lfOutputModeArr[] = {"CW","SWE", VI_NULL};
static ViString variationArr[] = {"DEC","USER",VI_NULL};
static ViString listModeArr[] = {"AUTO","STEP", VI_NULL};
static ViString listTriggerSourceArr[] = {"AUTO","SING","EXT","HOP","DHOP", VI_NULL};
static ViString dexcListModeArr[] = {"IMP","EXP", VI_NULL};
static ViString dexcExtensionArr[] = {"TXT","CSV", VI_NULL};
static ViString dexcSeparatorArr[] = {"TAB","SEM","COMM","SPAC", VI_NULL};
static ViString dexcDecimalSeparatorArr[] = {"DOT","COMM", VI_NULL};
static ViString identificationArr[] = {"HP8662","HP8663","ORIG", VI_NULL};
static ViString identification2Arr[] = {"AUTO","USER", VI_NULL};   
static ViString ipAddressModeArr[] = {"STAT","STAT","AUTO", VI_NULL};
static ViString VORModeArr[] = {"NORM","VAR","SUBC","FMS", VI_NULL};
static ViString VORDirectionArr[] = {"FROM","TO", VI_NULL};
static ViString EXTAMArr[] = {"INT","INT,EXT", VI_NULL};
static ViString VORFreqStepArr[] = {"DEC","ICAO", VI_NULL};
static ViInt32  VORICAOArr[] = {17,19,21,23,25,27,29,31,33,35,37,39,41,43,45,47,49,51,53,55,57,58,59,
								70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,
								93,94,95,96,97,98,99,100,102,102,103,104,105,106,107,108,109,110,111,
								112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,VI_NULL};
static ViString ILSGSModeArr[] = {"NORM","ULOB","LLOB", VI_NULL};
static ViString ILSGSDDMDirectionArr[] = {"UP","DOWN", VI_NULL};
static ViString ILSLOCModeArr[] = {"NORM","LLOB","RLOB", VI_NULL};
static ViString ILSLOCDDMDirectionArr[] = {"LEFT","RIGH", VI_NULL};
static ViString MBMMarkerFrequencyArr[] = {"400","1300","3000", VI_NULL};
static ViString ILSCoupArr[] = {"FIX","SDM",NULL};
static ViString carrierFrequencyStepArr[] = {"DEC","ICAO", VI_NULL};
static ViString alcOffModeArr[] = {"SHOL", VI_NULL};
static ViString frequencySweepShapeArr[] = {"SAWT","TRI", VI_NULL};
static ViString PULMModeArr[] = {"DOUB","SING","PTR", VI_NULL};
static ViString phaseContinuousFrequencyRangeArr[] = {"NARR","WIDE", VI_NULL};
static ViString icaoChannelArr[] = {"CH18X","CH18Y","CH20X","CH20Y","CH22X","CH22Y",
                                    "CH24X","CH24Y","CH26X","CH26Y","CH28X","CH28Y",
                                    "CH30X","CH30Y","CH32X","CH32Y","CH34X","CH34Y",
                                    "CH36X","CH36Y","CH38X","CH38Y","CH40X","CH40Y",
                                    "CH42X","CH42Y","CH44X","CH44Y","CH46X","CH46Y",
                                    "CH48X","CH48Y","CH50X","CH50Y","CH52X","CH52Y",
                                    "CH54X","CH54Y","CH56X","CH56Y", VI_NULL};
static ViString chirpTriggerModeArr[] = {"AUTO","EXT","EGAT","PSEN", VI_NULL};
static ViString chirpTriggerSlopeArr[] = {"POS","NEG", VI_NULL};
static ViString chirpGatePolarityArr[] = {"INV","NORM", VI_NULL};
static ViString chirpImpedanceArr[] = {"G10K","G50", VI_NULL};
static ViString dmemExternalInputImpedance[] = {"G50","G10K", VI_NULL};
static ViString chirpDirectionArr[] = {"UP","DOWN", VI_NULL};
static ViString dmemModeArr[] = {"INT","REPL", VI_NULL};
static ViString dmemPulseShapeArr[] = {"COS","COS2","LIN", VI_NULL};
static ViString dmemChannelSuffixArr[] = {"X","Y", "ICAO", VI_NULL};
static ViString angleUnitsArr[] = {"DEG","RAD", VI_NULL};
static ViString powerUnitsArr[] = {"V","DBM", VI_NULL};
static ViString velocityUnitsArr[] = {"MPS","KMH","MPH", VI_NULL};
static ViString imageFormatArr[] = {"BMP","JPG","XPM","PNG", VI_NULL};
static ViString powerSourceArr[] = {"A","USER","B", VI_NULL};
static ViString sensePowerUnitArr[] = {"DBM","DBUV","WATT", VI_NULL};
static ViString powerPriorityArr[] = {"AVER","PEAK", VI_NULL};
static ViString powerGateTraceArr[] = {"TRAC1","TRAC2","TRAC3", VI_NULL};
static ViString powerTimeTriggerSourceArr[] = {"AUTO","FREE","EXT", VI_NULL};
static ViString powerFilterModeArr[] = {"AUTO","USER","NSR", VI_NULL};
static ViString powerFrequencyTimingArr[] = {"NORM","FAST", VI_NULL};
static ViString powerFrequencySpacingArr[] = {"LIN","LOG", VI_NULL};
static ViString powerFrequencyYScaleAutoArr[] = {"OFF","CEXP","FEXP","CFL","FFL", VI_NULL};
static ViString powerTimingArr[] = {"NORM","FAST", VI_NULL};
static ViString senseModeArr[] = {"SING","CONT", VI_NULL};
static ViString runModeArr[] = {"SING","CONT", VI_NULL};
static ViString sweepModeArr[] = {"FREQ","POW","TIME", VI_NULL};
static ViString sweepHardcopyFormatArr[] = {"BMP","JPG","XPM","PNG","CSV", VI_NULL};
static ViString powerSensorDisplayColorArr[] = {"BLAC","WHIT", VI_NULL};
static ViString calLevelAdjDataArr[] = {"FACT","CUST", VI_NULL};
static ViString hcopImageSizeArr[] = {"320,240","640,480","800,600","1024,768", VI_NULL};
static ViString hcopDeviceArr[] = {"FILE", VI_NULL};
static ViString TimeSchemaArr[] = {"STD","USER", VI_NULL};
static ViString ILSPolarityArr[] = {"P90_150","P150_90", VI_NULL};
static ViString ListAttenuatorModeArr[] = {"NORM","HPOW", VI_NULL};
static ViString pulseExternalInputImpedanceArr[] = {"G50","1G0", VI_NULL};
static ViString hcopHeaderArr[] = {"OFF","STAN", VI_NULL};
static ViString hcopOrientationArr[] = {"HOR","VERT", VI_NULL};
static ViString hcopColumnSeparatorArr[] = {"TAB","SEM","COMM","BLAN", VI_NULL};
static ViString pulseThresholdBaseArr[] = {"VOLT","POW", VI_NULL};
static ViString pulseThresholdModeArr[] = {"HREF","REF","LREF", VI_NULL};
static ViString traceColorArr[] = {"INV","GRAY","YELL","BLUE","GRE","RED","MAG", VI_NULL};
static ViString traceCopyArr[] = {"REF", VI_NULL};
static ViString traceSourceArr[] = {"SENS1","SENS2","SENS3","REF","NONE", VI_NULL};
static ViString eventsArr[] = {"AND","OR", VI_NULL};
static ViString subtractArr[] = {"T1T1","T1T2","T1T3","T1T4","T1REF","T2T1","T2T2",
                                 "T2T3","T2T4","T2REF","T3T1","T3T2","T3T3","T3T4",
                                 "T3REF","T4T1","T4T2","T4T3","T4T4","T4REF", VI_NULL};
static ViString externalFrequencyArr[] = {"5MHZ","10MHZ", VI_NULL};
static ViString icaoChanArr[] = {"X", "Y", VI_NULL};
static ViString powerControlSensorArr[] = {"SENS1","SENS2","SENS3","SENS4",VI_NULL};
static ViString keybLangArr[] = {"US","DE",VI_NULL};
static ViString keybLayoutArr[] = {"CHIN","DAN","DUTC","ENGUK","ENGUS","FINN","FREN","FREB","FREC","GERM","ITAL","JAP","KOR","NORW","PORT","RUSS","SPAN","SWED", VI_NULL};
/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViStatus rssma_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
                             ViStatus currentStatus);
ViStatus rssma_defaultInstrSetup (ViSession openInstrSession);
ViStatus rssma_SetSweepMode (ViSession instrSession, ViInt32 mode);
ViStatus rssma_GetSweepMode (ViSession instrSession, ViPInt32 sweepMode);
ViStatus rssma_SetFrequencySweepTriggerSource (ViSession instrSession, ViInt32 triggerSourceRFFreqSweep);
ViStatus rssma_GetFrequencySweepTriggerSource (ViSession instrSession, ViPInt32 triggerSourceRFFreqSweep);
ViStatus rssma_SetFrequencyMode (ViSession instrSession, ViInt32 frequencyMode);
ViStatus rssma_GetFrequencyMode (ViSession instrSession, ViPInt32 frequencyMode);
ViStatus rssma_SetSweepLevelMode (ViSession instrSession, ViInt32 sweepLevelMode);
ViStatus rssma_GetSweepLevelMode (ViSession instrSession, ViPInt32 sweepLevelMode);
ViStatus rssma_SetLevelSweepTriggerSource (ViSession instrSession, ViInt32 triggerSourceRFLevelSweep);
ViStatus rssma_GetLevelSweepTriggerSource (ViSession instrSession, ViPInt32 triggerSourceRFLevelSweep);
ViStatus rssma_SetLevelMode (ViSession instrSession, ViInt32 RFLevelMode);
ViStatus rssma_GetLevelMode (ViSession instrSession, ViPInt32 RFLevelMode);
ViStatus rssma_SetSweepLFMode (ViSession instrSession, ViInt32 LFSweepMode);
ViStatus rssma_GetSweepLFMode (ViSession instrSession, ViPInt32 LFSweepMode);
ViStatus rssma_SetLFSweepTriggerSource (ViSession instrSession, ViInt32 triggerSourceLFSweep);
ViStatus rssma_GetLFSweepTriggerSource (ViSession instrSession, ViPInt32 triggerSourceLFSweep);
ViStatus rssma_SetLFMode (ViSession instrSession, ViInt32 LFOutputMode);
ViStatus rssma_GetLFMode (ViSession instrSession, ViPInt32 LFOutputMode);
ViStatus rssma_SetModeLIST (ViSession instrSession, ViInt32 listMode);
ViStatus rssma_SetLISTTriggerSource (ViSession instrSession,ViInt32 triggerSource);

/*****************************************************************************
 * Function: Initialize
 * Purpose:  This function performs the following initialization actions:
 *           
 *           - Opens a session to the Default Resource Manager resource and a 
 *           session to the specified device using the interface and address 
 *           specified in the Resource_Name control.
 *           
 *           - Performs an identification query on the Instrument.
 *           
 *           - Resets the instrument to a known state.
 *           
 *           - Sends initialization commands to the instrument that set any 
 *           necessary programmatic variables such as Headers Off, Short 
 *           Command form, and Data Transfer Binary to the state necessary for 
 *           the operation of the instrument driver.
 *           
 *           - Returns an Instrument Handle which is used to differentiate 
 *           between different sessions of this instrument driver.
 *           
 *           - Each time this function is invoked a Unique Session is opened.  
 *           It is possible to have more than one session open for the same 
 *           resource.
 *           
 *****************************************************************************/

ViStatus _VI_FUNC rssma_init(
    ViRsrc  resourceName,
    ViBoolean   idQuery,
    ViBoolean   resetDevice,
    ViSession*  instrSession
)
{
{
    ViStatus            rssma_status = VI_SUCCESS;
    ViSession           rmSession = 0;
    ViChar              buffer[BUFFER_SIZE] = "";
    rssma_instrRange    instrPtr = NULL;

    /* --- Set standard "C" locale --- */
    {
       char *locale;

       locale = setlocale(LC_ALL, "C");
    }
    
    /*- Check input parameter ranges ----------------------------------------*/
    /*
    CHECKPARAM (rssma_invalidViBooleanRange (idQuery), 2);
    CHECKPARAM (rssma_invalidViBooleanRange (resetDevice), 3);
    */
    /*- Open instrument session ---------------------------------------------*/
    CHECKERR( viOpenDefaultRM (&rmSession));

    if ((rssma_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG,
                                 VI_NULL, instrSession)) < 0)
    {
        (void)viClose (rmSession);
        return rssma_status;
    }

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */

    if (instrPtr == NULL)
        instrPtr = calloc (1, sizeof (struct rssma_statusDataRanges));
        

    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_USER_DATA, (ViAttrState)instrPtr));

    //With remote control via Ethernet, remote control is not automatically
    //set by means of a command. The instrument must be explicitly set to
    //the REMOTE state, e.g. by sending the intf command &GTR
    CHECKERR (viPrintf (*instrSession, "&GTR\n"));

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE,
                                 rssma_DEFAULT_TIMEOUT));
    CHECKERR( viSetBuf (*instrSession, (ViUInt16) (VI_READ_BUF|VI_WRITE_BUF), 4000));
    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));
    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));

    /* -------------------------------------------------------------------- */
    /* Drawback of Agilent VISA is patched by adding extra termination      */
    /* character "\n" to each I/O operation                                 */
    /*                                                                      */
    /* VI_ATTR_SEND_END_EN will stay with this attribute set to VI_TRUE     */
    /* -------------------------------------------------------------------- */

    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_SEND_END_EN, VI_TRUE));

    // read/reset ESR which may still indicate device dependent error from 
    // previous handling!
    (void) rssma_checkStatus(*instrSession);
    
    /*- Identification Query ------------------------------------------------*/
    if (idQuery == VI_TRUE)
    {
        CHECKERR( viQueryf (*instrSession, "*IDN?\n", "%*[^,],%[^,],%*[\r\n]",
                               buffer));

        /* Check if instrument is supported or not */
        if (strncmp (buffer, "SMA", 3) != 0)
            return rssma_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    }

    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
        CHECKERR( rssma_reset (*instrSession));
    else  /*- Send Default Instrument Setup ---------------------------------*/
        CHECKERR( rssma_defaultInstrSetup (*instrSession));

    return rssma_status;
}
}

/*============================================================================
 * Function: rssma_ConfigureVORModulation
 * Purpose:  This function configures the VOR modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ConfigureVORModulation (ViSession instrSession,
                                                ViBoolean state, ViInt32 mode,
                                                ViReal64 bearingAngle,
                                                ViInt32 direction,
                                                ViReal64 VARREFFrequency,
                                                ViReal64 VARDepth,
                                                ViReal64 subcarrierFrequency,
                                                ViReal64 subcarrierDepth,
                                                ViInt32 REFDeviation)
{
    INIT;
    CHECKOPT (K25);
    
    CHECKERR (rssma_SetVORMState (instrSession, state));
    CHECKERR (rssma_SetVORMMode (instrSession, mode));
    CHECKERR (rssma_SetVORMBearingAngle (instrSession, bearingAngle));
    CHECKERR (rssma_SetVORMDirection (instrSession, direction));
    CHECKERR (rssma_SetVORMVARREFFrequency (instrSession, VARREFFrequency));
    CHECKERR (rssma_SetVORMVARDepth (instrSession, VARDepth));
    CHECKERR (rssma_SetVORMSubcarrierFrequency (instrSession, subcarrierFrequency));
    CHECKERR (rssma_SetVORMSubcarrierDepth (instrSession, subcarrierDepth));
    CHECKERR (rssma_SetVORMREFDeviation (instrSession, REFDeviation));
    
    return rssma_status;
    
}

/*============================================================================
 * Function: rssma_ConfigureVORModulation
 * Purpose:  This function configures the additional communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ConfigureVORMCOMID (ViSession instrSession,
                                            ViBoolean COMIDState,
                                            ViString COMIDCode,
                                            ViReal64 COMIDFrequency,
                                            ViReal64 COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    CHECKERR (rssma_SetVORMCOMIDState (instrSession, COMIDState));
    CHECKERR (rssma_SetVORMCOMIDCode (instrSession, COMIDCode));
    CHECKERR (rssma_SetVORMCOMIDFrequency (instrSession, COMIDFrequency));
    CHECKERR (rssma_SetVORMCOMIDDepth (instrSession, COMIDDepth));
    
    return rssma_status;    
}

/*============================================================================
 * Function: rssma_SetVORMState
 * Purpose:  This function activates/deactivates the VOR modulation. Activation 
 *           of VOR modulation deactivates AM, ILS, Marker Beacon and ADF
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMState (ViSession instrSession,
                                      ViBoolean state)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:VOR:STAT",
                               state);
}

/*============================================================================
 * Function: rssma_GetVORMState
 * Purpose:  This function returns the state of the VOR modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMState (ViSession instrSession,
                                      ViBoolean *state)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:VOR:STAT?",
                               state);
}

/*============================================================================
 * Function: rssma_SetVORMMode
 * Purpose:  This function selects the operating mode for the VOR modulation 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMMode (ViSession instrSession, ViInt32 mode)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:VOR:MODE",
                          mode,
                          VORModeArr);
}

/*============================================================================
 * Function: rssma_GetVORMMode
 * Purpose:  This function returns the operating mode for the VOR modulation 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMMode (ViSession instrSession, ViInt32 *mode)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:VOR:MODE?",
                          mode,
                          VORModeArr);    
}

/*============================================================================
 * Function: rssma_SetVORMBearingAngle
 * Purpose:  This function sets the Bearing ANGle between the VAR signal and 
 *           the reference signal. The orientation of the angle depends on the 
 *           setting under SOURce:VOR:BANGle:DIRection.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMBearingAngle (ViSession instrSession,
                                             ViReal64 bearingAngle)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:BANG",
                              bearingAngle,
                              0.0, 360.0);    
}

/*============================================================================
 * Function: rssma_GetVORMBearingAngle
 * Purpose:  This function returns the Bearing ANGle between the VAR signal and 
 *           the reference signal. The orientation of the angle depends on the 
 *           setting under SOURce:VOR:BANGle:DIRection.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMBearingAngle (ViSession instrSession,
                                             ViReal64 *bearingAngle)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:BANG?",
                              bearingAngle);    
}

/*============================================================================
 * Function: rssma_SetVORMDirection
 * Purpose:  This function selects the reference position of the phase information.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMDirection (ViSession instrSession,
                                          ViInt32 direction)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:VOR:BANG:DIR",
                          direction,
                          VORDirectionArr);    
}

/*============================================================================
 * Function: rssma_GetVORMDirection
 * Purpose:  This function returns the reference position of the phase information.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMDirection (ViSession instrSession,
                                          ViInt32 *direction)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:VOR:BANG:DIR?",
                          direction,
                          VORDirectionArr);    
}

/*============================================================================
 * Function: rssma_SetVORMVARREFFrequency
 * Purpose:  This function sets the frequency of the variable and the reference 
 *           signal. As the two signals must have the same frequency, the 
 *           setting is valid for both signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMVARREFFrequency (ViSession instrSession,
                                                ViReal64 VARREFFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:VAR:FREQ",
                              VARREFFrequency,
                              20.0, 60.0);    
}

/*============================================================================
 * Function: rssma_GetVORMVARREFFrequency
 * Purpose:  This function returns the frequency of the variable and the reference 
 *           signal. As the two signals must have the same frequency, the 
 *           setting is valid for both signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMVARREFFrequency (ViSession instrSession,
                                                ViReal64 *VARREFFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:VAR:FREQ?",
                              VARREFFrequency);    
}

/*============================================================================
 * Function: rssma_SetVORMVARDepth
 * Purpose:  This function sets the AM modulation depth of the 30Hz variable 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMVARDepth (ViSession instrSession,
                                         ViReal64 VARDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:VAR:DEPT",
                              VARDepth,
                              0.0, 100.0);    
}

/*============================================================================
 * Function: rssma_GetVORMVARDepth
 * Purpose:  This function returns the AM modulation depth of the 30Hz variable 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMVARDepth (ViSession instrSession,
                                         ViReal64 *VARDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:VAR:DEPT?",
                              VARDepth);    
}

/*============================================================================
 * Function: rssma_SetVORMSubcarrierFrequency
 * Purpose:  This function sets the frequency of the FM carrier.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMSubcarrierFrequency (ViSession instrSession,
                                                    ViReal64 subcarrierFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:SUBC:FREQ",
                              subcarrierFrequency,
                              5000.0, 15000.0);    
}

/*============================================================================
 * Function: rssma_GetVORMSubcarrierFrequency
 * Purpose:  This function returns the frequency of the FM carrier.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMSubcarrierFrequency (ViSession instrSession,
                                                    ViReal64 *subcarrierFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:SUBC:FREQ?",
                              subcarrierFrequency);    
}

/*============================================================================
 * Function: rssma_SetVORMSubcarrierDepth
 * Purpose:  This function sets the AM modulation depth of the FM carrier.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMSubcarrierDepth (ViSession instrSession,
                                                ViReal64 subcarrierDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:SUBC:DEPT",
                              subcarrierDepth,
                              0.0, 100.0);    
}

/*============================================================================
 * Function: rssma_GetVORMSubcarrierDepth
 * Purpose:  This function returns the AM modulation depth of the FM carrier.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMSubcarrierDepth (ViSession instrSession,
                                                ViReal64 *subcarrierDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:SUBC:DEPT?",
                              subcarrierDepth);    
}

/*============================================================================
 * Function: rssma_SetVORMREFDeviation
 * Purpose:  This function sets the frequency deviation of the reference signal 
 *           on the FM carrier.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMREFDeviation (ViSession instrSession,
                                             ViInt32 REFDeviation)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMINT32 (instrSession,
                             "SOUR:VOR:REF:DEV",
                             REFDeviation,
                             0, 960);    
}

/*============================================================================
 * Function: rssma_GetVORMREFDeviation
 * Purpose:  This function returns the frequency deviation of the reference signal 
 *           on the FM carrier.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMREFDeviation (ViSession instrSession,
                                             ViInt32 *REFDeviation)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:VOR:REF:DEV?",
                             REFDeviation);     
}

/*============================================================================
 * Function: rssma_SetVORMToDefault
 * Purpose:  This function activates the VOR default setting.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMToDefault (ViSession instrSession)
{
    INIT;
    CHECKOPT (K25);
    
    return rssma_sendCmd (instrSession, "SOUR:VOR:PRES");
}

/*============================================================================
 * Function: rssma_SetVORMCOMIDState
 * Purpose:  This function switches on or off the additional communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMCOMIDState (ViSession instrSession,
                                           ViBoolean COMIDState)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:VOR:COM:STAT",
                               COMIDState);    
}

/*============================================================================
 * Function: rssma_GetVORMCOMIDState
 * Purpose:  This function returns the state of the additional communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMCOMIDState (ViSession instrSession,
                                           ViBoolean *COMIDState)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:VOR:COM:STAT?",
                               COMIDState);    
}

/*============================================================================
 * Function: rssma_SetVORMCOMIDCode
 * Purpose:  This function sets the coding of the COM/ID signal by the 
 *           international short name of the airport (e.g. MUC for the Munic 
 *           airport).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMCOMIDCode (ViSession instrSession,
                                          ViString COMIDCode)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:VOR:COM:CODE",
                                    "'",
                                    COMIDCode);    
}

/*============================================================================
 * Function: rssma_GetVORMCOMIDCode
 * Purpose:  This function returns the coding of the COM/ID signal by the 
 *           international short name of the airport (e.g. MUC for the Munic 
 *           airport).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMCOMIDCode (ViSession instrSession,
                                          ViChar COMIDCode[])
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMSTRING (instrSession,
                              "SOUR:VOR:COM:CODE?",
                              COMIDCode,
                              64);     
}

/*============================================================================
 * Function: rssma_SetVORMCOMIDFrequency
 * Purpose:  This function sets the frequency of the communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMCOMIDFrequency (ViSession instrSession,
                                               ViReal64 COMIDFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:FREQ",
                              COMIDFrequency,
                              0.1, 20000.0);    
}

/*============================================================================
 * Function: rssma_GetVORMCOMIDFrequency
 * Purpose:  This function returns the frequency of the communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMCOMIDFrequency (ViSession instrSession,
                                               ViReal64 *COMIDFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:FREQ?",
                              COMIDFrequency);    
}

/*============================================================================
 * Function: rssma_SetVORMCOMIDDepth
 * Purpose:  This function sets the depth of the communication/identification 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMCOMIDDepth (ViSession instrSession,
                                           ViReal64 COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:DEPT",
                              COMIDDepth,
                              0.0, 100.0);    
}

/*============================================================================
 * Function: rssma_GetVORMCOMIDDepth
 * Purpose:  This function returns the depth of the communication/identification 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMCOMIDDepth (ViSession instrSession,
                                           ViReal64 *COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:DEPT?",
                              COMIDDepth);    
}

/*============================================================================
 * Function: Set VORM COM/ID Time Schema
 * Purpose:  This function selects if the set dot length determines the dash
 *           length (= three times the dot length) (setting STD) or if all
 *           length parameters can be set independently (setting USER.
 *           (COM/ID signal).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMCOMIDTimeSchema (ViSession instrSession,
                                                ViInt32 COMIDTimeSchema)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:VOR:COM:TSCH",
                          COMIDTimeSchema,
                          TimeSchemaArr);    
}

/*============================================================================
 * Function: Get VORM COM/ID Time Schema
 * Purpose:  This function returns whether the set dot length is determined by the dash
 *           length (= three times the dot length) (setting STD) or all
 *           length parameters is set independently (setting USER. (COM/ID
 *           signal).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMCOMIDTimeSchema (ViSession instrSession,
                                                ViInt32 *COMIDTimeSchema)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:VOR:COM:TSCH?",
                          COMIDTimeSchema,
                          TimeSchemaArr);    
}

/*============================================================================
 * Function: Set VORM COM/ID Dash Length
 * Purpose:  This function sets the length of a morse dash by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetVORMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMCOMIDDashLength (ViSession instrSession,
                                                ViReal64 COMIDDashLength)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:DASH",
                              COMIDDashLength,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get VORM COM/ID Dash Length
 * Purpose:  This function returns the length of a morse dash by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetVORMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMCOMIDDashLength (ViSession instrSession,
                                                ViReal64 *COMIDDashLength)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:DASH?",
                              COMIDDashLength);    
}

/*============================================================================
 * Function: Set VORM COM/ID Dot Length
 * Purpose:  This function sets the length of a morse dot by means of
 *           seconds. The length of the dash (3xdot), symbol space (=dot) and
 *           letter space (=3xdot) is also determined by this entry .
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMCOMIDDotLength (ViSession instrSession,
                                               ViReal64 COMIDDotLength)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:DOT",
                              COMIDDotLength,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get VORM COM/ID Dot Length
 * Purpose:  This function returns the length of a morse dot by means of
 *           seconds. The length of the dash (3xdot), symbol space (=dot) and
 *           letter space (=3xdot) is also determined by this entry .
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMCOMIDDotLength (ViSession instrSession,
                                               ViReal64 *COMIDDotLength)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:DOT?",
                              COMIDDotLength);    
}

/*============================================================================
 * Function: Set VORM COM/ID Letter Space
 * Purpose:  This function sets the length of a letter space by means of
 *           seconds. This function is available user time schema only
 *           (rssma_SetVORMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMCOMIDLetterSpace (ViSession instrSession,
                                                 ViReal64 COMIDLetterSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:LETT",
                              COMIDLetterSpace,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get VORM COM/ID Letter Space
 * Purpose:  This function returns the length of a letter space by means of
 *           seconds. This function is available user time schema only
 *           (rssma_SetVORMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMCOMIDLetterSpace (ViSession instrSession,
                                                 ViReal64 *COMIDLetterSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:LETT?",
                              COMIDLetterSpace);    
}

/*============================================================================
 * Function: Set VORM COM/ID Symbol Space
 * Purpose:  This function sets the length of the symbol space by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetVORMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMCOMIDSymbolSpace (ViSession instrSession,
                                                 ViReal64 COMIDSymbolSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:SYMB",
                              COMIDSymbolSpace,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get VORM COM/ID Symbol Space
 * Purpose:  This function returns the length of the symbol space by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetVORMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMCOMIDSymbolSpace (ViSession instrSession,
                                                 ViReal64 *COMIDSymbolSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:SYMB?",
                              COMIDSymbolSpace);    
}

/*****************************************************************************
 * Function: Set VORM COM ID Period
 * Purpose:  This function sets the period of the COM/ID signal in seconds.
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *           
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetVORMCOMIDPeriod(
    ViSession   instrSession,
    ViReal64    idPeriod
)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:PER",
                              idPeriod,
                              0.0, 120.0);    
}

/*****************************************************************************
 * Function: Get VORM COM ID Period
 * Purpose:  This function queries the period of the COM/ID signal in seconds.
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *           
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetVORMCOMIDPeriod(
    ViSession   instrSession,
    ViReal64*   idPeriod
)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:VOR:COM:PER?",
                              idPeriod);    
}

/*============================================================================
 * Function: rssma_SetVORMEXTAM
 * Purpose:  This function selects the modulation source for VOR modulation. 
 *           INT is the internal modulation source. The external signal is 
 *           input at the AM EXT connector. The external signal is added to 
 *           the internal signal. Switching off the internal source is not 
 *           possible.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMEXTAM (ViSession instrSession, ViInt32 EXTAM)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:VOR:SOUR",
                          EXTAM,
                          EXTAMArr);    
}

/*============================================================================
 * Function: rssma_GetVORMEXTAM
 * Purpose:  This function returns the modulation source for VOR modulation. 
 *           INT is the internal modulation source.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMEXTAM (ViSession instrSession, ViInt32 *EXTAM)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:VOR:SOUR?",
                          EXTAM,
                          EXTAMArr);    
}

/*============================================================================
 * Function: Set VORM Carrier Frequency Step
 * Purpose:  This function sets the variation of the carrier frequency via 
/*			 the rotary knob.
/*
/* Required Option(s):
/* 			R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMCarrierFrequencyStep (ViSession instrSession,
                                                     ViInt32 carrierFrequencyStep)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          ":SOUR:VOR:FREQ:STEP",
                          carrierFrequencyStep,
                          VORFreqStepArr);    
}

/*============================================================================
 * Function: Set VORM Carrier Frequency Step
 * Purpose:  This function returns the variation of the carrier frequency 
/*			 via the rotary knob.
/*
/* Required Option(s):
/* 			R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMCarrierFrequencyStep (ViSession instrSession,
                                                     ViInt32 *carrierFrequencyStep)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          ":SOUR:VOR:FREQ:STEP?",
                          carrierFrequencyStep,
                          VORFreqStepArr);        
}

/*============================================================================
 * Function: Set VORM ICAO Channel
 * Purpose:  This function selects the  ICAO channel i.e., the VOR 
/*			 transmitting frequency.The RF-frequency is set to the value 
/*			 selected here.
/*
/* Required Option(s):
/* 			R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetVORMICAOChannel (ViSession instrSession,
                                            ViInt32 ICAOChannel,
                                            ViInt32 ICAONumber)
{
	ViChar              buffer[BUFFER_SIZE] = "";
	INIT;
    CHECKOPT (K25); 
	
	CHECKVIINT32 (instrSession, ICAOChannel, RSSMA_SOURCE_ICAO_CHANNEL_X, RSSMA_SOURCE_ICAO_CHANNEL_Y);
	CHECKVIINT32 (instrSession, ICAONumber, 17, 126);
	sprintf  (buffer, ":SOUR:VOR:ICAO:CHAN %s%ld", icaoChanArr[ICAOChannel], ICAONumber);
	
	return rssma_sendCmd(instrSession, buffer);    
}


/*============================================================================
 * Function: Get VORM ICAO Channel
 * Purpose:  This function returnsthe selected ICAO channel.
/*
/* Required Option(s):
/* 			R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetVORMICAOChannel (ViSession instrSession,
                                            ViInt32 *ICAOChannel,
                                            ViInt32 *ICAONumber)
{
 	ViChar buffer[BUFFER_SIZE]="";
	ViChar channel[5]="";
	
	
	INIT;
    CHECKOPT (K25); 
	
	CHECKERR (GETPARAMSTRING(instrSession, ":SOUR:VOR:ICAO:CHAN?",buffer, BUFFER_SIZE));
	if (sscanf (buffer, "%1s%ld", channel, ICAONumber) != 2)
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE; 
		
	return rssma_getIndexFromString (channel, icaoChanArr, ICAOChannel);    
}

/*============================================================================
 * Function: rssma_ConfigureILSGSModulation
 * Purpose:  This function configures the ILS-GS modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ConfigureILSGSModulation (ViSession instrSession,
                                                  ViBoolean state, ViInt32 mode,
                                                  ViReal64 DDMDepth,
                                                  ViReal64 DDMCurrent,
                                                  ViReal64 DDMLogarithmic,
                                                  ViInt32 DDMDirection,
                                                  ViReal64 sumOfDepth,
                                                  ViReal64 upFrequency,
                                                  ViReal64 downFrequency,
                                                  ViReal64 upDownPhase)
{
    INIT;
    CHECKOPT (K25);
    
    CHECKERR (rssma_SetILSGSMState (instrSession, state));
    CHECKERR (rssma_SetILSGSMMode (instrSession, mode));
    CHECKERR (rssma_SetILSGSMDDMDepth (instrSession, DDMDepth));
    CHECKERR (rssma_SetILSGSMDDMCurrent (instrSession, DDMCurrent));
    CHECKERR (rssma_SetILSGSMDDMLogarithmic (instrSession, DDMLogarithmic));
    CHECKERR (rssma_SetILSGSMFly (instrSession, DDMDirection));
    CHECKERR (rssma_SetILSGSMSumOfDepth (instrSession, sumOfDepth));
    CHECKERR (rssma_SetILSGSMUpFrequency (instrSession, upFrequency));
    CHECKERR (rssma_SetILSGSMDownFrequency (instrSession, downFrequency));
    CHECKERR (rssma_SetILSGSMUpDownPhase (instrSession, upDownPhase));
    
    return rssma_status;
}

/*============================================================================
 * Function: rssma_SetILSGSMState
 * Purpose:  This function activates/deactivates the ILS-GS modulation. 
 *           Activation of ILS-GS modulation deactivates AM, VOR, ILS-LOC,
 *           Marker Beacon and ADF modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMState (ViSession instrSession,
                                        ViBoolean state)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:ILS:GS:STAT",
                               state);    
}

/*============================================================================
 * Function: rssma_GetILSGSMState
 * Purpose:  This function returns the state of the ILS-GS modulation. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMState (ViSession instrSession,
                                        ViBoolean *state)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:ILS:GS:STAT?",
                               state);    
}

/*============================================================================
 * Function: rssma_SetILSGSMMode
 * Purpose:  This function selects the operating mode for the ILS-GS modulation 
 *           signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMMode (ViSession instrSession, ViInt32 mode)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:MODE",
                          mode,
                          ILSGSModeArr);     
}

/*============================================================================
 * Function: rssma_GetILSGSMMode
 * Purpose:  This function returns the operating mode for the ILS-GS modulation 
 *           signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMMode (ViSession instrSession, ViInt32 *mode)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:MODE?",
                          mode,
                          ILSGSModeArr);    
}

/*============================================================================
 * Function: rssma_SetILSGSMDDMDepth
 * Purpose:  This function sets the difference in depth of modulation between 
 *           the signal of the upper lobe (90 Hz) and the lower lobe (150 Hz). 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMDDMDepth (ViSession instrSession,
                                           ViReal64 DDMDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:DDM:DEPT",
                              DDMDepth,
                              -0.8, 0.8);    
}

/*============================================================================
 * Function: rssma_GetILSGSMDDMDepth
 * Purpose:  This function returns the difference in depth of modulation between 
 *           the signal of the upper lobe (90 Hz) and the lower lobe (150 Hz). 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMDDMDepth (ViSession instrSession,
                                           ViReal64 *DDMDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:DDM:DEPT?",
                              DDMDepth);    
}

/*============================================================================
 * Function: rssma_SetILSGSMDDMCurrent
 * Purpose:  This function sets the DDM value alternatively as a current by 
 *           means of the ILS indicating instrument.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMDDMCurrent (ViSession instrSession,
                                             ViReal64 DDMCurrent)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:DDM:CURR",
                              DDMCurrent,
                              -857.125, 857.12);     
}

/*============================================================================
 * Function: rssma_GetILSGSMDDMCurrent
 * Purpose:  This function returns the DDM value alternatively as a current by 
 *           means of the ILS indicating instrument.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMDDMCurrent (ViSession instrSession,
                                             ViReal64 *DDMCurrent)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:DDM:CURR?",
                              DDMCurrent);    
}

/*============================================================================
 * Function: rssma_SetILSGSMDDMLogarithmic
 * Purpose:  This function sets the DDM value in dB.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMDDMLogarithmic (ViSession instrSession,
                                                 ViReal64 DDMLogarithmic)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:DDM:LOG",
                              DDMLogarithmic,
                              -999.9, 999.9);    
}

/*============================================================================
 * Function: rssma_GetILSGSMDDMLogarithmic
 * Purpose:  This function returns the DDM value in dB.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMDDMLogarithmic (ViSession instrSession,
                                                 ViReal64 *DDMLogarithmic)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:DDM:LOG?",
                              DDMLogarithmic);    
}

/*============================================================================
 * Function: Set ILS-GSM DDM Coupling
 * Purpose:  This function selects if the DDM value is fixed or is changed
 *           with a change of sum of modulation depths.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMDDMCoupling (ViSession instrSession,
                                              ViInt32 DDMCoupling)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:DDM:COUP",
                          DDMCoupling,
                          ILSCoupArr);    
}

/*============================================================================
 * Function: Get ILS-GSM DDM Coupling
 * Purpose:  This function queries whether the DDM value is fixed or is changed
 *           with a change of sum of modulation depths.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMDDMCoupling (ViSession instrSession,
                                              ViInt32 *DDMCoupling)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:DDM:COUP?",
                          DDMCoupling,
                          ILSCoupArr);    
}

/*============================================================================
 * Function: Set ILS-GSM DDM Percent
 * Purpose:  This function enters the difference in depth of modulation
 *           between the signal of the upper/left lobe (90 Hz) and the
 *           lower/right lobe (150 Hz).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMDDMPercent (ViSession instrSession,
                                             ViInt32 DDMPercent)
{
    return SETPARAMINT32 (instrSession,
                              "SOUR:ILS:GS:DDM:PCT",
                              DDMPercent,
                              0, 0);    
}

/*============================================================================
 * Function: Get ILS-GSM DDM Percent
 * Purpose:  This function returns the difference in depth of modulation
 *           between the signal of the upper/left lobe (90 Hz) and the
 *           lower/right lobe (150 Hz).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMDDMPercent (ViSession instrSession,
                                             ViInt32 *DDMPercent)
{
    return GETPARAMINT32 (instrSession,
                              "SOUR:ILS:GS:DDM:PCT?",
                              DDMPercent);    
}

/*============================================================================
 * Function: Set ILS-GSM DDM Polarity
 * Purpose:  This function enters the polarity for DDM calculation (see DDM
 *           Depth).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMDDMPolarity (ViSession instrSession,
                                              ViInt32 DDMPolarity)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:DDM:POL",
                          DDMPolarity,
                          ILSPolarityArr);    
}

/*============================================================================
 * Function: Get ILS-GSM DDM Polarity
 * Purpose:  This function returns the polarity for DDM calculation (see DDM
 *           Depth).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMDDMPolarity (ViSession instrSession,
                                              ViInt32 *DDMPolarity)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:DDM:POL?",
                          DDMPolarity,
                          ILSPolarityArr);    
}

/*============================================================================
 * Function: rssma_SetILSGSMFly
 * Purpose:  This function selects the simulation mode for the ILS-GS 
 *           modulation signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMFly (ViSession instrSession,
                                      ViInt32 DDMDirection)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:DDM:DIR",
                          DDMDirection,
                          ILSGSDDMDirectionArr);    
}

/*============================================================================
 * Function: rssma_GetILSGSMFly
 * Purpose:  This function returns the simulation mode for the ILS-GS 
 *           modulation signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMFly (ViSession instrSession,
                                      ViInt32 *DDMDirection)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:DDM:DIR?",
                          DDMDirection,
                          ILSGSDDMDirectionArr);    
}

/*============================================================================
 * Function: rssma_SetILSGSMSumOfDepth
 * Purpose:  This function sets the arithmetic sum of the modulation depths 
 *           of the upper lobe (90Hz) and lower lobe (150Hz) ILS-GS signal 
 *           contents. The RMS modulation depth of the sum signal depends on 
 *           the phase setting of both modulation tones.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMSumOfDepth (ViSession instrSession,
                                             ViReal64 sumOfDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:SDM",
                              sumOfDepth,
                              0.0, 100.0);    
}

/*============================================================================
 * Function: rssmaGetILSGSMSumOfDepth
 * Purpose:  This function returns the arithmetic sum of the modulation depths 
 *           of the upper lobe (90Hz) and lower lobe (150Hz) ILS-GS signal 
 *           contents. The RMS modulation depth of the sum signal depends on 
 *           the phase setting of both modulation tones.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMSumOfDepth (ViSession instrSession,
                                             ViReal64 *sumOfDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:SDM?",
                              sumOfDepth);    
}

/*============================================================================
 * Function: rssma_SetILSGSMUpFrequency
 * Purpose:  This function sets the modulation frequency of the antenna lobe 
 *           arranged at the top viewed from the air plane.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMUpFrequency (ViSession instrSession,
                                              ViReal64 upFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:ULOB:FREQ",
                              upFrequency,
                              60.0, 120.0);    
}

/*============================================================================
 * Function: rssma_GetILSGSMUpFrequency
 * Purpose:  This function returns the modulation frequency of the antenna lobe 
 *           arranged at the top viewed from the air plane.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMUpFrequency (ViSession instrSession,
                                              ViReal64 *upFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:ULOB:FREQ?",
                              upFrequency);    
}

/*============================================================================
 * Function: rssma_SetILSGSMDownFrequency
 * Purpose:  This function sets the modulation frequency of the antenna lobe 
 *           arranged at the bottom viewed from the air plane.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMDownFrequency (ViSession instrSession,
                                                ViReal64 downFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:LLOB:FREQ",
                              downFrequency,
                              100.0, 200.0);    
}

/*============================================================================
 * Function: rssma_GetILSGSMDownFrequency
 * Purpose:  This function returns the modulation frequency of the antenna lobe 
 *           arranged at the bottom viewed from the air plane.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMDownFrequency (ViSession instrSession,
                                                ViReal64 *downFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:LLOB:FREQ?",
                              downFrequency);    
}

/*============================================================================
 * Function: rssma_SetILSGSMUpDownPhase
 * Purpose:  This function sets the phase between the modulation signals of 
 *           the upper and lower antenna lobe. The zero crossing of the lower 
 *           lobe (150Hz) signal serves as a reference. The angle refers to 
 *           the period of the signal of the lower antenna lobe.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMUpDownPhase (ViSession instrSession,
                                              ViReal64 upDownPhase)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:PHAS",
                              upDownPhase,
                              0.0, 120.0);    
}

/*============================================================================
 * Function: rssma_GetILSGSMUpDownPhase
 * Purpose:  This function returns the phase between the modulation signals of 
 *           the upper and lower antenna lobe. The zero crossing of the lower 
 *           lobe (150Hz) signal serves as a reference. The angle refers to 
 *           the period of the signal of the lower antenna lobe.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMUpDownPhase (ViSession instrSession,
                                              ViReal64 *upDownPhase)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:GS:PHAS?",
                              upDownPhase);    
}

/*============================================================================
 * Function: rssma_SetILSGSMToDefault
 * Purpose:  This function activate the ILS-GS default setting.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMToDefault (ViSession instrSession)
{
    INIT;
    CHECKOPT (K25);
    
    return rssma_sendCmd (instrSession, "SOUR:ILS:GS:PRES");    
}

/*===========================================================================*/
/* Function: Set ILS-GSM Carrier Frequency Step
/* Purpose:  This function selects the variation of the carrier frequency.
/*           
/*           Required Option(s):
/*           R&S SMA-K25
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMCarrierFrequencyStep(
    ViSession   instrSession,
    ViInt32 carrierFrequencyStep
)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:FREQ:STEP",
                          carrierFrequencyStep,
                          carrierFrequencyStepArr);    
}

/*===========================================================================*/
/* Function: Get ILS-GSM Carrier Frequency Step
/* Purpose:  This function returns the variation of the carrier frequency.
/*           
/*           Required Option(s):
/*           R&S SMA-K25
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMCarrierFrequencyStep(
    ViSession   instrSession,
    ViInt32*    carrierFrequencyStep
)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:FREQ:STEP?",
                          carrierFrequencyStep,
                          carrierFrequencyStepArr);    
}

/*===========================================================================*/
/* Function: Set ILS-GSM ICAO Channel
/* Purpose:  This function selects the ICAO channel  i.e, the ILS 
/*           transmitting frequency. The RF-frequency is set to the value 
/*           selected here. A variation of the carrier frequency by the 
/*           rotary knob is performed in predefined steps  according to the 
/*           standardized ILS transmitting frequencies. The ICAO channel 
/*           settings for ILS-GS and ILS-LOC are coupled.
/*           
/*           
/*           Required Option(s):
/*           R&S SMA-K25
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMICAOChannel(
    ViSession   instrSession,
    ViInt32 icaoChannel
)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:ICAO:CHAN",
                          icaoChannel,
                          icaoChannelArr);    
}

/*===========================================================================*/
/* Function: Get ILS-GSM ICAO Channel
/* Purpose:  This function returns the ICAO channel  i.e, the ILS 
/*           transmitting frequency. 
/*           
/*           Required Option(s):
/*           R&S SMA-K25
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMICAOChannel(
    ViSession   instrSession,
    ViInt32*    icaoChannel
)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:ICAO:CHAN?",
                          icaoChannel,
                          icaoChannelArr);    
}

/*============================================================================
 * Function: rssma_SetILSGSMEXTAM
 * Purpose:  This function selects the modulation source for ILS-GS modulation. 
 *           INT is the internal modulation source. The external signal is 
 *           input at the AM EXT connector. The external signal is added to the 
 *           internal signal. Switching off the internal source is not possible.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSGSMEXTAM (ViSession instrSession,
                                        ViInt32 EXTAM)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:SOUR",
                          EXTAM,
                          EXTAMArr);
}

/*============================================================================
 * Function: rssma_GetILSGSMEXTAM
 * Purpose:  This function returns the modulation source for ILS-GS modulation. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSGSMEXTAM (ViSession instrSession,
                                        ViInt32 *EXTAM)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:GS:SOUR?",
                          EXTAM,
                          EXTAMArr);    
}

/*============================================================================
 * Function: rssma_ConfigureILSLOCModulation
 * Purpose:  This function configures the ILS-LOC modulation. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ConfigureILSLOCModulation (ViSession instrSession,
                                                   ViBoolean state, ViInt32 mode,
                                                   ViReal64 DDMDepth,
                                                   ViReal64 DDMCurrent,
                                                   ViReal64 DDMLogarithmic,
                                                   ViInt32 DDMDirection,
                                                   ViReal64 sumOfDepth,
                                                   ViReal64 leftFrequency,
                                                   ViReal64 rightFrequency,
                                                   ViReal64 leftRightPhase)
{
    INIT;
    CHECKOPT (K25);
    
    CHECKERR (rssma_SetILSLOCMState (instrSession, state));
    CHECKERR (rssma_SetILSLOCMMode (instrSession, mode));
    CHECKERR (rssma_SetILSLOCMDDMDepth (instrSession, DDMDepth));
    CHECKERR (rssma_SetILSLOCMDDMCurrent (instrSession, DDMCurrent));
    CHECKERR (rssma_SetILSLOCMDDMLogarithmic (instrSession, DDMLogarithmic));
    CHECKERR (rssma_SetILSLOCMFly (instrSession, DDMDirection));
    CHECKERR (rssma_SetILSLOCMSumOfDepth (instrSession, sumOfDepth));
    CHECKERR (rssma_SetILSLOCMLeftFrequency (instrSession, leftFrequency));
    CHECKERR (rssma_SetILSLOCMRightFrequency (instrSession, rightFrequency));
    CHECKERR (rssma_SetILSLOCMLeftRightPhase (instrSession, leftRightPhase));
    
    return rssma_status;    
}

/*============================================================================
 * Function: rssma_ConfigureILSLOCMCOMID
 * Purpose:  This function configures the additional communication/
 *           identification signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ConfigureILSLOCMCOMID (ViSession instrSession,
                                               ViBoolean COMIDState,
                                               ViString COMIDCode,
                                               ViReal64 COMIDFrequency,
                                               ViReal64 COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    CHECKERR (rssma_SetILSLOCMCOMIDState (instrSession, COMIDState));
    CHECKERR (rssma_SetILSLOCMCOMIDCode (instrSession, COMIDCode));
    CHECKERR (rssma_SetILSLOCMCOMIDFrequency (instrSession, COMIDFrequency));
    CHECKERR (rssma_SetILSLOCMCOMIDDepth (instrSession, COMIDDepth));

    return rssma_status;    
}

/*============================================================================
 * Function: rssma_SetILSLOCMState
 * Purpose:  This function activates/deactivates the ILS-LOC modulation. 
 *           Activation of ILS-LOC modulation deactivates AM, VOR, ILS-GS,
 *           Marker Beacon and ADF modulation. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMState (ViSession instrSession,
                                         ViBoolean state)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:ILS:LOC:STAT",
                               state);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMState
 * Purpose:  This function returns the state of the ILS-LOC modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMState (ViSession instrSession,
                                         ViBoolean *state)
{   INIT;
    CHECKOPT (K25);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:ILS:LOC:STAT?",
                               state);
}

/*============================================================================
 * Function: rssma_SetILSLOCMMode
 * Purpose:  This function selects the operating mode for the ILS-LOC 
 *           modulation signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMMode (ViSession instrSession, ViInt32 mode)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:MODE",
                          mode,
                          ILSLOCModeArr);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMMode
 * Purpose:  This function returns the operating mode for the ILS-LOC 
 *           modulation signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMMode (ViSession instrSession,
                                        ViInt32 *mode)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:MODE?",
                          mode,
                          ILSLOCModeArr);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMDDMDepth
 * Purpose:  This function sets the difference in depth of modulation between 
 *           the signal of the left lobe (90 Hz) and the right lobe (150 Hz).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMDDMDepth (ViSession instrSession,
                                            ViReal64 DDMDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:DDM:DEPT",
                              DDMDepth,
                              -0.4, 0.4);   
}

/*============================================================================
 * Function: rssma_GetILSLOCMDDMDepth
 * Purpose:  This function returns the difference in depth of modulation between 
 *           the signal of the left lobe (90 Hz) and the right lobe (150 Hz).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMDDMDepth (ViSession instrSession,
                                            ViReal64 *DDMDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:DDM:DEPT?",
                              DDMDepth);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMDDMCurrent
 * Purpose:  This function sets the DDM value alternatively as a current by 
 *           means of the ILS indicating instrument.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMDDMCurrent (ViSession instrSession,
                                              ViReal64 DDMCurrent)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:DDM:CURR",
                              DDMCurrent,
                              -967.75, 967.75);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMDDMCurrent
 * Purpose:  This function returns the DDM value alternatively as a current by 
 *           means of the ILS indicating instrument.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMDDMCurrent (ViSession instrSession,
                                              ViReal64 *DDMCurrent)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:DDM:CURR?",
                              DDMCurrent);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMDDMLogarithmic
 * Purpose:  This function sets the DDM value in dB.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMDDMLogarithmic (ViSession instrSession,
                                                  ViReal64 DDMLogarithmic)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:DDM:LOG",
                              DDMLogarithmic,
                              -999.9, 999.9);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMDDMLogarithmic
 * Purpose:  This function returns the DDM value in dB.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMDDMLogarithmic (ViSession instrSession,
                                                  ViReal64 *DDMLogarithmic)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:DDM:LOG?",
                              DDMLogarithmic);    
}

/*============================================================================
 * Function: Set ILS-LOCM DDM Coupling
 * Purpose:  This function selects if the DDM value is fixed or is changed
 *           with a change of sum of modulation depths.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMDDMCoupling (ViSession instrSession,
                                               ViInt32 DDMCoupling)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:DDM:COUP",
                          DDMCoupling,
                          ILSCoupArr);    
}

/*============================================================================
 * Function: Get ILS-LOCM DDM Coupling
 * Purpose:  This function queries whether the DDM value is fixed or is changed
 *           with a change of sum of modulation depths.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMDDMCoupling (ViSession instrSession,
                                               ViInt32 *DDMCoupling)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:DDM:COUP?",
                          DDMCoupling,
                          ILSCoupArr);    
}

/*============================================================================
 * Function: Set ILS-LOCM DDM Percent
 * Purpose:  This function enters the difference in depth of modulation
 *           between the signal of the upper/left lobe (90 Hz) and the
 *           lower/right lobe (150 Hz).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMDDMPercent (ViSession instrSession,
                                              ViInt32 DDMPercent)
{
    return SETPARAMINT32 (instrSession,
                              "SOUR:ILS:LOC:DDM:PCT",
                              DDMPercent,
                              0, 0);    
}

/*============================================================================
 * Function: Get ILS-LOCM DDM Percent
 * Purpose:  This function returns the difference in depth of modulation
 *           between the signal of the upper/left lobe (90 Hz) and the
 *           lower/right lobe (150 Hz).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMDDMPercent (ViSession instrSession,
                                              ViInt32 *DDMPercent)
{
    return GETPARAMINT32 (instrSession,
                              "SOUR:ILS:LOC:DDM:PCT?",
                              DDMPercent);    
}

/*============================================================================
 * Function: Set ILS-LOCM DDM Polarity
 * Purpose:  This function enters the polarity for DDM calculation (see DDM
 *           Depth).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMDDMPolarity (ViSession instrSession,
                                               ViInt32 DDMPolarity)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:DDM:POL",
                          DDMPolarity,
                          ILSPolarityArr);    
}

/*============================================================================
 * Function: Get ILS-LOCM DDM Polarity
 * Purpose:  This function returns the polarity for DDM calculation (see DDM
 *           Depth).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMDDMPolarity (ViSession instrSession,
                                               ViInt32 *DDMPolarity)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:DDM:POL?",
                          DDMPolarity,
                          ILSPolarityArr);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMFly
 * Purpose:  This function selects the simulation mode for the ILS-LOC 
 *           modulation signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMFly (ViSession instrSession,
                                       ViInt32 DDMDirection)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:DDM:DIR",
                          DDMDirection,
                          ILSLOCDDMDirectionArr);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMFly
 * Purpose:  This function returns the simulation mode for the ILS-LOC 
 *           modulation signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMFly (ViSession instrSession,
                                       ViInt32 *DDMDirection)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:DDM:DIR?",
                          DDMDirection,
                          ILSLOCDDMDirectionArr);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMSumOfDepth
 * Purpose:  This function sets the arithmetic sum of the modulation depths 
 *           of the left lobe (90Hz) and right lobe (150Hz) ILS-LOC signal 
 *           contents. The RMS modulation depth of the sum signal depends on 
 *           the phase setting of both modulation tones.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMSumOfDepth (ViSession instrSession,
                                              ViReal64 sumOfDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:SDM",
                              sumOfDepth,
                              0.0, 100.0);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMSumOfDepth
 * Purpose:  This function returns the arithmetic sum of the modulation depths 
 *           of the left lobe (90Hz) and right lobe (150Hz) ILS-LOC signal 
 *           contents. The RMS modulation depth of the sum signal depends on 
 *           the phase setting of both modulation tones.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMSumOfDepth (ViSession instrSession,
                                              ViReal64 *sumOfDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:SDM?",
                              sumOfDepth);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMLeftFrequency
 * Purpose:  This function sets the modulation frequency of the antenna lobe 
 *           arranged at the left viewed from the air plane.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMLeftFrequency (ViSession instrSession,
                                                 ViReal64 leftFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:LLOB:FREQ",
                              leftFrequency,
                              60.0, 120.0);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMLeftFrequency
 * Purpose:  This function returns the modulation frequency of the antenna lobe 
 *           arranged at the left viewed from the air plane.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMLeftFrequency (ViSession instrSession,
                                                 ViReal64 *leftFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:LLOB:FREQ?",
                              leftFrequency);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMRightFrequency
 * Purpose:  This function sets the modulation frequency of the antenna lobe 
 *           arranged at the right viewed from the air plane.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMRightFrequency (ViSession instrSession,
                                                  ViReal64 rightFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:RLOB:FREQ",
                              rightFrequency,
                              100.0, 200.0);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMRightFrequency
 * Purpose:  This function returns the modulation frequency of the antenna lobe 
 *           arranged at the right viewed from the air plane.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMRightFrequency (ViSession instrSession,
                                                  ViReal64 *rightFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:RLOB:FREQ?",
                              rightFrequency);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMLeftRightPhase
 * Purpose:  This function sets the phase between the modulation signals of 
 *           the left and right antenna lobe. The zero crossing of the right 
 *           lobe (150Hz) signal serves as a reference. The angle refers to 
 *           the period of the signal of the right antenna lobe.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMLeftRightPhase (ViSession instrSession,
                                                  ViReal64 leftRightPhase)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:PHAS",
                              leftRightPhase,
                              0.0, 120.0);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMLeftRightPhase
 * Purpose:  This function returns the phase between the modulation signals of 
 *           the left and right antenna lobe. The zero crossing of the right 
 *           lobe (150Hz) signal serves as a reference. The angle refers to 
 *           the period of the signal of the right antenna lobe.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMLeftRightPhase (ViSession instrSession,
                                                  ViReal64 *leftRightPhase)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:PHAS?",
                              leftRightPhase);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMToDefault
 * Purpose:  This function activate the ILS-LOC default setting.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMToDefault (ViSession instrSession)
{
    INIT;
    CHECKOPT (K25);
    
    return rssma_sendCmd (instrSession, "SOUR:ILS:LOC:PRES");    
}

/*============================================================================
 * Function: rssma_SetILSLOCMCOMIDState
 * Purpose:  This function switches on or off the additional communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDState (ViSession instrSession,
                                              ViBoolean COMIDState)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:ILS:LOC:COM:STAT",
                               COMIDState);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMCOMIDState
 * Purpose:  This function returns the state of the additional communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDState (ViSession instrSession,
                                              ViBoolean *COMIDState)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:ILS:LOC:COM:STAT?",
                               COMIDState);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMCOMIDCode
 * Purpose:  This function sets the coding of the COM/ID signal by the 
 *           international short name of the airport (e.g. MUC for the Munic 
 *           airport).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDCode (ViSession instrSession,
                                             ViString COMIDCode)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:ILS:LOC:COM:CODE",
                                    "'",
                                    COMIDCode);     
}

/*============================================================================
 * Function: rssma_GetILSLOCMCOMIDCode
 * Purpose:  This function returns the coding of the COM/ID signal by the 
 *           international short name of the airport (e.g. MUC for the Munic 
 *           airport).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDCode (ViSession instrSession,
                                             ViChar COMIDCode[])
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMSTRING (instrSession,
                              "SOUR:ILS:LOC:COM:CODE?",
                              COMIDCode,
                              64);
}

/*============================================================================
 * Function: rssma_SetILSLOCMCOMIDFrequency
 * Purpose:  This function sets the frequency of the communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDFrequency (ViSession instrSession,
                                                  ViReal64 COMIDFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:FREQ",
                              COMIDFrequency,
                              0.1, 20000.0);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMCOMIDFrequency
 * Purpose:  This function returns the frequency of the communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDFrequency (ViSession instrSession,
                                                  ViReal64 *COMIDFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:FREQ?",
                              COMIDFrequency);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMCOMIDDepth
 * Purpose:  This function sets the depth of the communication/identification 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDDepth (ViSession instrSession,
                                              ViReal64 COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:DEPT",
                              COMIDDepth,
                              0.0, 100.0);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMCOMIDDepth
 * Purpose:  This function returns the depth of the communication/identification 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDDepth (ViSession instrSession,
                                              ViReal64 *COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:DEPT?",
                              COMIDDepth);    
}

/*============================================================================
 * Function: rssma_SetILSLOCMEXTAM
 * Purpose:  This function selects the modulation source for ILS-LOC 
 *           modulation. INT is the internal modulation source. The external 
 *           signal is input at the AM EXT connector. The external signal is 
 *           added to the internal signal. Switching off the internal source 
 *           is not possible.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMEXTAM (ViSession instrSession,
                                         ViInt32 EXTAM)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:SOUR",
                          EXTAM,
                          EXTAMArr);    
}

/*============================================================================
 * Function: Set ILS-LOCM COM/ID Time Schema
 * Purpose:  This function selects if the set dot length determines the dash
 *           length (= three times the dot length) (setting STD) or if all
 *           length parameters can be set independently (setting USER.
 *           (COM/ID signal).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDTimeSchema (ViSession instrSession,
                                                   ViInt32 COMIDTimeSchema)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:COM:TSCH",
                          COMIDTimeSchema,
                          TimeSchemaArr);    
}

/*============================================================================
 * Function: Get ILS-LOCM COM/ID Time Schema
 * Purpose:  This function returns whether the set dot length is determined
 *           by the dash length (= three times the dot length) (setting STD)
 *           or all length parameters is set independently (setting USER.
 *           (COM/ID signal).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDTimeSchema (ViSession instrSession,
                                                   ViInt32 *COMIDTimeSchema)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:COM:TSCH?",
                          COMIDTimeSchema,
                          TimeSchemaArr);    
}

/*============================================================================
 * Function: Set ILS-LOCM COM/ID Dash Length
 * Purpose:  This function sets the length of a morse dash by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetILSLOCMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDDashLength (ViSession instrSession,
                                                   ViReal64 COMIDDashLength)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:DASH",
                              COMIDDashLength,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get ILS-LOCM COM/ID Dash Length
 * Purpose:  This function returns the length of a morse dash by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetILSLOCMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDDashLength (ViSession instrSession,
                                                   ViReal64 *COMIDDashLength)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:DASH?",
                              COMIDDashLength);    
}

/*============================================================================
 * Function: Set ILS-LOCM COM/ID Dot Length
 * Purpose:  This function sets the length of a morse dot by means of
 *           seconds. The length of the dash (3xdot), symbol space (=dot) and
 *           letter space (=3xdot) is also determined by this entry .
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDDotLength (ViSession instrSession,
                                                  ViReal64 COMIDDotLength)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:DOT",
                              COMIDDotLength,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get ILS-LOCM COM/ID Dot Length
 * Purpose:  This function returns the length of a morse dot by means of
 *           seconds. The length of the dash (3xdot), symbol space (=dot) and
 *           letter space (=3xdot) is also determined by this entry .
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDDotLength (ViSession instrSession,
                                                  ViReal64 *COMIDDotLength)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:DOT?",
                              COMIDDotLength);    
}

/*============================================================================
 * Function: Set ILS-LOCM COM/ID Letter Space
 * Purpose:  This function sets the length of a letter space by means of
 *           seconds. This function is available user time schema only
 *           (rssma_SetILSLOCMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDLetterSpace (ViSession instrSession,
                                                    ViReal64 COMIDLetterSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:LETT",
                              COMIDLetterSpace,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get ILS-LOCM COM/ID Letter Space
 * Purpose:  This function returns the length of a letter space by means of
 *           seconds. This function is available user time schema only
 *           (rssma_SetILSLOCMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDLetterSpace (ViSession instrSession,
                                                    ViReal64 *COMIDLetterSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:LETT?",
                              COMIDLetterSpace);    
}

/*============================================================================
 * Function: Set ILS-LOCM COM/ID Symbol Space
 * Purpose:  This function sets the length of the symbol space by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetILSLOCMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMCOMIDSymbolSpace (ViSession instrSession,
                                                    ViReal64 COMIDSymbolSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:SYMB",
                              COMIDSymbolSpace,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get ILS-LOCM COM/ID Symbol Space
 * Purpose:  This function returns the length of the symbol space by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetILSLOCMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMCOMIDSymbolSpace (ViSession instrSession,
                                                    ViReal64 *COMIDSymbolSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ILS:LOC:COM:SYMB?",
                              COMIDSymbolSpace);    
}

/*============================================================================
 * Function: Set ILS-LOCM COM/ID Period
 * Purpose:  This function sets the period of the ID sequence by means of
 *           seconds.
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCCOMIDPeriod (ViSession instrSession,
                                              ViInt32 idPeriod)
{
    INIT;
    CHECKOPT(K25);
    
    return SETPARAMINT32 (instrSession,
                             "SOUR:ILS:LOC:COM:PER",
                             idPeriod,
                             0,120);    
}

/*============================================================================
 * Function: Get ILS-LOCM COM/ID Period
 * Purpose:  This function queries the period of the ID sequence by means of
 *           seconds.
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCCOMIDPeriod (ViSession instrSession,
                                              ViInt32* idPeriod)
{
    INIT;
    CHECKOPT(K25);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:ILS:LOC:COM:PER?",
                             idPeriod);    
}

/*===========================================================================*/
/* Function: Set ILS-LOCM Carrier Frequency Step
/* Purpose:  This function selects the variation of the carrier frequency.
/*           
/*           Required Option(s):
/*           R&S SMA-K25
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMCarrierFrequencyStep(
    ViSession   instrSession,
    ViInt32 carrierFrequencyStep
)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:FREQ:STEP",
                          carrierFrequencyStep,
                          carrierFrequencyStepArr);    
}

/*===========================================================================*/
/* Function: Get ILS-LOCM Carrier Frequency Step
/* Purpose:  This function returns the variation of the carrier frequency.
/*           
/*           Required Option(s):
/*           R&S SMA-K25
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMCarrierFrequencyStep(
    ViSession   instrSession,
    ViInt32*    carrierFrequencyStep
)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:FREQ:STEP?",
                          carrierFrequencyStep,
                          carrierFrequencyStepArr);    
}

/*===========================================================================*/
/* Function: Set ILS-LOCM ICAO Channel
/* Purpose:  This function selects the ICAO channel  i.e, the ILS 
/*           transmitting frequency. The RF-frequency is set to the value 
/*           selected here. A variation of the carrier frequency by the 
/*           rotary knob is performed in predefined steps  according to the 
/*           standardized ILS transmitting frequencies. The ICAO channel 
/*           settings for ILS-GS and ILS-LOC are coupled.
/*           
/*           
/*           Required Option(s):
/*           R&S SMA-K25
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetILSLOCMICAOChannel(
    ViSession   instrSession,
    ViInt32 icaoChannel
)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:ICAO:CHAN",
                          icaoChannel,
                          icaoChannelArr);    
}

/*===========================================================================*/
/* Function: Get ILS-LOCM ICAO Channel
/* Purpose:  This function returns the ICAO channel  i.e, the ILS 
/*           transmitting frequency. 
/*           
/*           Required Option(s):
/*           R&S SMA-K25
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMICAOChannel(
    ViSession   instrSession,
    ViInt32*    icaoChannel
)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:ICAO:CHAN?",
                          icaoChannel,
                          icaoChannelArr);    
}

/*============================================================================
 * Function: rssma_GetILSLOCMEXTAM
 * Purpose:  This function returns the modulation source for ILS-LOC 
 *           modulation. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetILSLOCMEXTAM (ViSession instrSession,
                                         ViInt32 *EXTAM)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ILS:LOC:SOUR?",
                          EXTAM,
                          EXTAMArr);    
}

/*============================================================================
 * Function: rssma_ConfigureMBModulation
 * Purpose:  This function configures the Marker Beacon modulation. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ConfigureMBModulation (ViSession instrSession,
                                               ViBoolean state,
                                               ViInt32 markerFrequency,
                                               ViReal64 markerDepth,
                                               ViBoolean pulsedMarker)
{
    INIT;
    CHECKOPT (K25);
    
    CHECKERR (rssma_SetMBMState (instrSession, state));
    CHECKERR (rssma_SetMBMMarkerFrequency (instrSession, markerFrequency));
    CHECKERR (rssma_SetMBMMarkerDepth (instrSession, markerDepth));
    CHECKERR (rssma_SetMBMPulsedMarker (instrSession, pulsedMarker));
        
    return rssma_status;   
}

/*============================================================================
 * Function: rssma_ConfigureMBMCOMID
 * Purpose:  This function configures the additional communication/
 *           identification signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ConfigureMBMCOMID (ViSession instrSession,
                                           ViBoolean COMIDState,
                                           ViString COMIDCode,
                                           ViReal64 COMIDFrequency,
                                           ViReal64 COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    CHECKERR (rssma_SetMBMCOMIDState (instrSession, COMIDState));
    CHECKERR (rssma_SetMBMCOMIDCode (instrSession, COMIDCode));
    CHECKERR (rssma_SetMBMCOMIDFrequency (instrSession, COMIDFrequency));
    CHECKERR (rssma_SetMBMCOMIDDepth (instrSession, COMIDDepth));
        
    return rssma_status;    
}

/*============================================================================
 * Function: rssma_SetMBMState
 * Purpose:  This function activates/deactivates the Marker Beacon modulation. 
 *           Activation of Marker Beacon modulation deactivates AM, VOR, ILS 
 *           and ADF modulation. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMState (ViSession instrSession, ViBoolean state)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:MBE:STAT",
                               state);    
}

/*============================================================================
 * Function: rssma_GetMBMState
 * Purpose:  This function returns the state of the Marker Beacon modulation. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMState (ViSession instrSession,
                                     ViBoolean *state)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:MBE:STAT?",
                               state);    
}

/*============================================================================
 * Function: rssma_SetMBMMarkerFrequency
 * Purpose:  This function selects the modulation frequency of the marker 
 *           signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMMarkerFrequency (ViSession instrSession,
                                               ViInt32 markerFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:MBE:MARK:FREQ",
                          markerFrequency,
                          MBMMarkerFrequencyArr);    
}

/*============================================================================
 * Function: rssma_GetMBMMarkerFrequency
 * Purpose:  This function returns the modulation frequency of the marker 
 *           signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMMarkerFrequency (ViSession instrSession,
                                               ViInt32 *markerFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:MBE:MARK:FREQ?",
                          markerFrequency,
                          MBMMarkerFrequencyArr);    
}

/*============================================================================
 * Function: rssma_SetMBMMarkerDepth
 * Purpose:  This function selects the modulation depth of the marker signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMMarkerDepth (ViSession instrSession,
                                           ViReal64 markerDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:MBE:MARK:DEPT",
                              markerDepth,
                              0.0, 100.0);    
}

/*============================================================================
 * Function: rssma_GetMBMMarkerDepth
 * Purpose:  This function returns the modulation depth of the marker signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMMarkerDepth (ViSession instrSession,
                                           ViReal64 *markerDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:MBE:MARK:DEPT?",
                              markerDepth);    
}

/*============================================================================
 * Function: rssma_SetMBMPulsedMarker
 * Purpose:  This function activates/deactivates the pulsed marker. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMPulsedMarker (ViSession instrSession,
                                            ViBoolean pulsedMarker)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:MBE:MARK:PULS",
                               pulsedMarker);    
}

/*============================================================================
 * Function: rssma_GetMBMPulsedMarker
 * Purpose:  This function returns the state of the pulsed marker. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMPulsedMarker (ViSession instrSession,
                                            ViBoolean *pulsedMarker)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:MBE:MARK:PULS?",
                               pulsedMarker);    
}

/*============================================================================
 * Function: rssma_SetMBMCOMIDState
 * Purpose:  This function switches on or off the additional communication/
 *           identification signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMCOMIDState (ViSession instrSession,
                                          ViBoolean COMIDState)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:MBE:COM:STAT",
                               COMIDState);    
}

/*============================================================================
 * Function: rssma_GetMBMCOMIDState
 * Purpose:  This function returns the state of the additional communication/
 *           identification signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMCOMIDState (ViSession instrSession,
                                          ViBoolean *COMIDState)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:MBE:COM:STAT?",
                               COMIDState);    
}

/*============================================================================
 * Function: rssma_SetMBMCOMIDCode
 * Purpose:  This function sets the coding of the COM/ID signal by the 
 *           international short name of the airport (e.g. MUC for the Munic 
 *           airport). 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMCOMIDCode (ViSession instrSession,
                                         ViString COMIDCode)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:MBE:COM:CODE",
                                    "'",
                                    COMIDCode);    
}

/*============================================================================
 * Function: rssma_GetMBMCOMIDCode
 * Purpose:  This function returns the coding of the COM/ID signal by the 
 *           international short name of the airport (e.g. MUC for the Munic 
 *           airport). 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMCOMIDCode (ViSession instrSession,
                                         ViChar COMIDCode[])
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMSTRING (instrSession,
                              "SOUR:MBE:COM:CODE?",
                              COMIDCode,
                              64);    
}

/*============================================================================
 * Function: rssma_SetMBMCOMIDFrequency
 * Purpose:  This function sets the frequency of the communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMCOMIDFrequency (ViSession instrSession,
                                              ViReal64 COMIDFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:FREQ",
                              COMIDFrequency,
                              0.1, 20000.0);
}

/*============================================================================
 * Function: rssma_GetMBMCOMIDFrequency
 * Purpose:  This function returns the frequency of the communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMCOMIDFrequency (ViSession instrSession,
                                              ViReal64 *COMIDFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:FREQ?",
                              COMIDFrequency);    
}

/*============================================================================
 * Function: rssma_SetMBMCOMIDDepth
 * Purpose:  This function sets the depth of the communication/identification 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMCOMIDDepth (ViSession instrSession,
                                          ViReal64 COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:DEPT",
                              COMIDDepth,
                              0.0, 100.0);    
}

/*============================================================================
 * Function: rssma_GetMBMCOMIDDepth
 * Purpose:  This function returns the depth of the communication/identification 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMCOMIDDepth (ViSession instrSession,
                                          ViReal64 *COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:DEPT?",
                              COMIDDepth);    
}

/*============================================================================
 * Function: Set MBM COM/ID Time Schema
 * Purpose:  This function selects if the set dot length determines the dash
 *           length (= three times the dot length) (setting STD) or if all
 *           length parameters can be set independently (setting USER.
 *           (COM/ID signal).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMCOMIDTimeSchema (ViSession instrSession,
                                               ViInt32 COMIDTimeSchema)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:MBE:COM:TSCH",
                          COMIDTimeSchema,
                          TimeSchemaArr);    
}

/*============================================================================
 * Function: Get MBM COM/ID Time Schema
 * Purpose:  This function returns whether the set dot length is determined by 
 *           the dash length (= three times the dot length) (setting STD) or 
 *           if all length parameters can be set independently (setting USER.
 *           (COM/ID signal).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMCOMIDTimeSchema (ViSession instrSession,
                                               ViInt32 *COMIDTimeSchema)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:MBE:COM:TSCH?",
                          COMIDTimeSchema,
                          TimeSchemaArr);    
}

/*============================================================================
 * Function: Set MBM COM/ID Dash Length
 * Purpose:  This function sets the length of a morse dash by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetMBMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMCOMIDDashLength (ViSession instrSession,
                                               ViReal64 COMIDDashLength)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:DASH",
                              COMIDDashLength,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get MBM COM/ID Dash Length
 * Purpose:  This function returns the length of a morse dash by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetMBMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMCOMIDDashLength (ViSession instrSession,
                                               ViReal64 *COMIDDashLength)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:DASH?",
                              COMIDDashLength);    
}

/*============================================================================
 * Function: Set MBM COM/ID Dot Length
 * Purpose:  This function sets the length of a morse dot by means of
 *           seconds. The length of the dash (3xdot), symbol space (=dot) and
 *           letter space (=3xdot) is also determined by this entry .
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMCOMIDDotLength (ViSession instrSession,
                                              ViReal64 COMIDDotLength)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:DOT",
                              COMIDDotLength,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get MBM COM/ID Dot Length
 * Purpose:  This function returns the length of a morse dot by means of
 *           seconds. The length of the dash (3xdot), symbol space (=dot) and
 *           letter space (=3xdot) is also determined by this entry .
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMCOMIDDotLength (ViSession instrSession,
                                              ViReal64 *COMIDDotLength)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:DOT?",
                              COMIDDotLength);    
}

/*============================================================================
 * Function: Set MBM COM/ID Letter Space
 * Purpose:  This function sets the length of a letter space by means of
 *           seconds. This function is available user time schema only
 *           (rssma_SetMBMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMCOMIDLetterSpace (ViSession instrSession,
                                                ViReal64 COMIDLetterSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:LETT",
                              COMIDLetterSpace,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get MBM COM/ID Letter Space
 * Purpose:  This function returns the length of a letter space by means of
 *           seconds. This function is available user time schema only
 *           (rssma_SetMBMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMCOMIDLetterSpace (ViSession instrSession,
                                                ViReal64 *COMIDLetterSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:LETT?",
                              COMIDLetterSpace);    
}

/*============================================================================
 * Function: Set MBM COM/ID Symbol Space
 * Purpose:  This function sets the length of the symbol space by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetMBMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMCOMIDSymbolSpace (ViSession instrSession,
                                                ViReal64 COMIDSymbolSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:SYMB",
                              COMIDSymbolSpace,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get MBM COM/ID Symbol Space
 * Purpose:  This function returns the length of the symbol space by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetMBMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetMBMCOMIDSymbolSpace (ViSession instrSession,
                                                ViReal64 *COMIDSymbolSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:SYMB?",
                              COMIDSymbolSpace);    
}

/*****************************************************************************
 * Function: Set MBM COM ID Period
 * Purpose:  This function sets the period of the COM/ID signal in seconds.
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *           
 *           Remote-control command(s):
 *           [:SOURce]:MBEacon:COMid:PERiod
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetMBMCOMIDPeriod(
    ViSession   instrSession,
    ViReal64    idPeriod
)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:PER",
                              idPeriod,
                              0.0, 120.0);    
}

/*****************************************************************************
 * Function: Get MBM COM ID Period
 * Purpose:  This function queries the period of the COM/ID signal in seconds.
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *           
 *           Remote-control command(s):
 *           [:SOURce]:MBEacon:COMid:PERiod?
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetMBMCOMIDPeriod(
    ViSession   instrSession,
    ViReal64*   idPeriod
)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:MBE:COM:PER?",
                              idPeriod);    
}

/*============================================================================
 * Function: rssma_SetMBMToDefault
 * Purpose:  This function activate the Marker Beacon default setting.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetMBMToDefault (ViSession instrSession)
{
    INIT;
    CHECKOPT (K25);
    
    return rssma_sendCmd (instrSession, "SOUR:MBE:PRES");    
}

/*============================================================================
 * Function: rssma_ConfigureADFMCOMID
 * Purpose:  This function configures the additional communication/
 *           identification signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ConfigureADFMCOMID (ViSession instrSession,
                                            ViBoolean COMIDState,
                                            ViString COMIDCode,
                                            ViReal64 COMIDFrequency,
                                            ViReal64 COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    CHECKERR (rssma_SetADFMCOMIDState (instrSession, COMIDState));
    CHECKERR (rssma_SetADFMCOMIDCode (instrSession, COMIDCode));
    CHECKERR (rssma_SetADFMCOMIDFrequency (instrSession, COMIDFrequency));
    CHECKERR (rssma_SetADFMCOMIDDepth (instrSession, COMIDDepth));
        
    return rssma_status;    
    
}

/*============================================================================
 * Function: rssma_SetADFMState
 * Purpose:  This function activates/deactivates the ADF modulation. A 
 *           unmodulated carrier with a frequency of 190 kHz (default) is 
 *           generated. After switching on the COM/ID tone the carrier is 
 *           modulated with 95 percent AM depth at a 1,02 kHz modulation rate. 
 *           Activation of ADF modulation deactivates AM, VOR, ILS and Marker 
 *           Beacon modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMState (ViSession instrSession,
                                      ViBoolean state)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:ADF:STAT",
                               state);    
}

/*============================================================================
 * Function: rssma_GetADFMState
 * Purpose:  This function returns the state of the ADF modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMState (ViSession instrSession,
                                      ViBoolean *state)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:ADF:STAT?",
                               state);    
}

/*============================================================================
 * Function: rssma_SetADFMCOMIDState
 * Purpose:  This function switches on or off the additional communication/
 *           identification signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMCOMIDState (ViSession instrSession,
                                           ViBoolean COMIDState)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:ADF:COM:STAT",
                               COMIDState);    
}

/*============================================================================
 * Function: rssma_GetADFMCOMIDState
 * Purpose:  This function returns the state of the additional communication/
 *           identification signal. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMCOMIDState (ViSession instrSession,
                                           ViBoolean *COMIDState)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:ADF:COM:STAT?",
                               COMIDState);    
}

/*============================================================================
 * Function: rssma_SetADFMCOMIDCode
 * Purpose:  This function sets the coding of the COM/ID signal by the 
 *           international short name of the airport (e.g. MUC for the Munic 
 *           airport). 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMCOMIDCode (ViSession instrSession,
                                          ViString COMIDCode)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:ADF:COM:CODE",
                                    "'",
                                    COMIDCode);    
    
}

/*============================================================================
 * Function: rssma_GetADFMCOMIDCode
 * Purpose:  This function returns the coding of the COM/ID signal by the 
 *           international short name of the airport (e.g. MUC for the Munic 
 *           airport). 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMCOMIDCode (ViSession instrSession,
                                          ViChar COMIDCode[])
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMSTRING (instrSession,
                              "SOUR:ADF:COM:CODE?",
                              COMIDCode,
                              64);    
    
}

/*============================================================================
 * Function: rssma_SetADFMCOMIDFrequency
 * Purpose:  This function sets the frequency of the communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMCOMIDFrequency (ViSession instrSession,
                                               ViReal64 COMIDFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:FREQ",
                              COMIDFrequency,
                              0.1, 20000.0);
    
}

/*============================================================================
 * Function: rssma_GetADFMCOMIDFrequency
 * Purpose:  This function returns the frequency of the communication/
 *           identification signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMCOMIDFrequency (ViSession instrSession,
                                               ViReal64 *COMIDFrequency)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:FREQ?",
                              COMIDFrequency);    
    
}

/*============================================================================
 * Function: rssma_SetADFMCOMIDDepth
 * Purpose:  This function sets the depth of the communication/identification 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMCOMIDDepth (ViSession instrSession,
                                           ViReal64 COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:DEPT",
                              COMIDDepth,
                              0.0, 100.0);    
    
}

/*============================================================================
 * Function: rssma_GetADFMCOMIDDepth
 * Purpose:  This function returns the depth of the communication/identification 
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMCOMIDDepth (ViSession instrSession,
                                           ViReal64 *COMIDDepth)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:DEPT?",
                              COMIDDepth);    
    
}

/*============================================================================
 * Function: Set ADFM COM/ID Time Schema
 * Purpose:  This function selects if the set dot length determines the dash
 *           length (= three times the dot length) (setting STD) or if all
 *           length parameters can be set independently (setting USER.
 *           (COM/ID signal).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMCOMIDTimeSchema (ViSession instrSession,
                                                ViInt32 COMIDTimeSchema)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:ADF:COM:TSCH",
                          COMIDTimeSchema,
                          TimeSchemaArr);    
}

/*============================================================================
 * Function: Get ADFM COM/ID Time Schema
 * Purpose:  This function returns whether the set dot length is determined
 *           by the dash length (= three times the dot length) (setting STD)
 *           or all length parameters is set independently (setting USER.
 *           (COM/ID signal).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMCOMIDTimeSchema (ViSession instrSession,
                                                ViInt32 *COMIDTimeSchema)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:ADF:COM:TSCH?",
                          COMIDTimeSchema,
                          TimeSchemaArr);    
}

/*============================================================================
 * Function: Set ADFM COM/ID Dash Length
 * Purpose:  This function sets the length of a morse dash by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetADFMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMCOMIDDashLength (ViSession instrSession,
                                                ViReal64 COMIDDashLength)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:DASH",
                              COMIDDashLength,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get ADFM COM/ID Dash Length
 * Purpose:  This function returns the length of a morse dash by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetADFMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMCOMIDDashLength (ViSession instrSession,
                                                ViReal64 *COMIDDashLength)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:DASH?",
                              COMIDDashLength);    
}

/*============================================================================
 * Function: Set ADFM COM/ID Dot Length
 * Purpose:  This function sets the length of a morse dot by means of
 *           seconds. The length of the dash (3xdot), symbol space (=dot) and
 *           letter space (=3xdot) is also determined by this entry .
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMCOMIDDotLength (ViSession instrSession,
                                               ViReal64 COMIDDotLength)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:DOT",
                              COMIDDotLength,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get ADFM COM/ID Dot Length
 * Purpose:  This function returns the length of a morse dot by means of
 *           seconds. The length of the dash (3xdot), symbol space (=dot) and
 *           letter space (=3xdot) is also determined by this entry .
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMCOMIDDotLength (ViSession instrSession,
                                               ViReal64 *COMIDDotLength)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:DOT?",
                              COMIDDotLength);    
}

/*============================================================================
 * Function: Set ADFM COM/ID Letter Space
 * Purpose:  This function sets the length of a letter space by means of
 *           seconds. This function is available user time schema only
 *           (rssma_SetADFMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMCOMIDLetterSpace (ViSession instrSession,
                                                 ViReal64 COMIDLetterSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:LETT",
                              COMIDLetterSpace,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get ADFM COM/ID Letter Space
 * Purpose:  This function returns the length of a letter space by means of
 *           seconds. This function is available user time schema only
 *           (rssma_SetADFMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMCOMIDLetterSpace (ViSession instrSession,
                                                 ViReal64 *COMIDLetterSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:LETT?",
                              COMIDLetterSpace);    
}

/*============================================================================
 * Function: Set ADFM COM/ID Symbol Space
 * Purpose:  This function sets the length of the symbol space by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetADFMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMCOMIDSymbolSpace (ViSession instrSession,
                                                 ViReal64 COMIDSymbolSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:SYMB",
                              COMIDSymbolSpace,
                              0.0, 0.0);    
}

/*============================================================================
 * Function: Get ADFM COM/ID Symbol Space
 * Purpose:  This function returns the length of the symbol space by means of
 *           seconds. This function is available for user time schema only
 *           (rssma_SetADFMCOMIDTimeSchema).
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMCOMIDSymbolSpace (ViSession instrSession,
                                                 ViReal64 *COMIDSymbolSpace)
{
    INIT;
    CHECKOPT (K25);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:ADF:COM:SYMB?",
                              COMIDSymbolSpace);    
}

/*============================================================================
 * Function: Set ADFM COM/ID Period
 * Purpose:  This function sets the period of the ID sequence by means of 
 *           seconds. This function is available for reply mode only.
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetADFMCOMIDPeriod (ViSession instrSession,
                                            ViInt32 idPeriod)
{
    INIT;
    CHECKOPT(K25);
    
    return SETPARAMINT32 (instrSession,
                             "SOUR:ADF:COM:PER",
                             idPeriod,
                             0,120);    
}

/*============================================================================
 * Function: Get ADFM COM/ID Period
 * Purpose:  This function queries the period of the ID sequence by means of
 *           seconds. This function is available for reply mode only.
 *           
 *           Required Option(s):
 *           R&S SMA-K25
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetADFMCOMIDPeriod (ViSession instrSession,
                                            ViInt32 *idPeriod)
{
    INIT;
    CHECKOPT(K25);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:ADF:COM:PER?",
                             idPeriod);    
}

/*===========================================================================*/
/* Function: Configure CHIRP
/* Purpose:  This function configures the generation of modulation chirps.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23, 
/*           K20/22 and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confCHIRP(
    ViSession   instrSession,
    ViBoolean   chripState,
    ViInt32 direction,
    ViReal64    bandwidth,
    ViReal64    pulsePeriod,
    ViReal64    pulseWidth,
    ViInt32 triggerMode,
    ViInt32 triggerSlope,
    ViInt32 impedance
)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetCHIRPState (instrSession, chripState));
    CHECKERR (rssma_SetCHIRPDirection (instrSession, direction));
    CHECKERR (rssma_SetCHIRPBandwidth (instrSession, bandwidth));
    CHECKERR (rssma_SetCHIRPPulsePeriod (instrSession, pulsePeriod));
    CHECKERR (rssma_SetCHIRPPulseWidth (instrSession, pulseWidth));
    CHECKERR (rssma_SetCHIRPTriggerMode (instrSession, triggerMode));
    CHECKERR (rssma_SetCHIRPTriggerSlope (instrSession, triggerSlope));
    CHECKERR (rssma_SetCHIRPExternalImpedance (instrSession, impedance));
    
    return rssma_status;    
}

/*===========================================================================*/
/* Function: Configure CHIRP Trigger
/* Purpose:  This function configures the trigger for modulation chirp. 
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/22, 
/*           R&S SMA-K20/23 and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confCHIRPTrigger(
    ViSession   instrSession,
    ViInt32 triggerSlope,
    ViInt32 gatePolarity,
    ViInt32 impedance,
    ViInt32 triggerMode,
    ViInt32 triggerLevel
)
{
    ViStatus    rssma_status;
	triggerLevel;

    CHECKERR (rssma_SetCHIRPTriggerSlope (instrSession, triggerSlope));
    CHECKERR (rssma_SetCHIRPGatePolarity (instrSession, gatePolarity));
    CHECKERR (rssma_SetCHIRPExternalImpedance (instrSession, impedance));
    CHECKERR (rssma_SetCHIRPTriggerMode (instrSession, triggerMode));
    
    return rssma_status;    
}

/*===========================================================================*/
/* Function: Set CHIRP Trigger Mode
/* Purpose:  This function selects the trigger mode for modulation chirp. 
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/22 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetCHIRPTriggerMode(
    ViSession   instrSession,
    ViInt32 triggerMode
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K22);
    }
    
    return SETPARAMENUM (instrSession,
                          "SOUR:CHIR:TRIG:MODE",
                          triggerMode,
                          chirpTriggerModeArr);

}

/*===========================================================================*/
/* Function: Get CHIRP Trigger Mode
/* Purpose:  This function returns the trigger mode for modulation chirp. 
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/22 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetCHIRPTriggerMode(
    ViSession   instrSession,
    ViInt32*    triggerMode
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K22);
    }
    
    return GETPARAMENUM (instrSession,
                          "SOUR:CHIR:TRIG:MODE?",
                          triggerMode,
                          chirpTriggerModeArr);

}

/*===========================================================================*/
/* Function: Set CHIRP Trigger Slope
/* Purpose:  This function sets the active slope of an externally applied 
/*           trigger signal at the PULSE EXT input (BNC connector at the rear 
/*           of the instrument).
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/22 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetCHIRPTriggerSlope(
    ViSession   instrSession,
    ViInt32 triggerSlope
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K22);
    }
    
    return SETPARAMENUM (instrSession,
                          "SOUR:CHIR:TRIG:EXT:SLOP",
                          triggerSlope,
                          chirpTriggerSlopeArr);

}

/*===========================================================================*/
/* Function: Get CHIRP Trigger Slope
/* Purpose:  This function return the active slope of an externally applied 
/*           trigger signal at the PULSE EXT input (BNC connector at the rear 
/*           of the instrument).
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/22 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetCHIRPTriggerSlope(
    ViSession   instrSession,
    ViInt32*    triggerSlope
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K22);
    }
    
    return GETPARAMENUM (instrSession,
                          "SOUR:CHIR:TRIG:EXT:SLOP?",
                          triggerSlope,
                          chirpTriggerSlopeArr);

}

/*===========================================================================*/
/* Function: Set CHIRP Gate Polarity
/* Purpose:  This function selects the active level of the gate signal.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetCHIRPGatePolarity(
    ViSession   instrSession,
    ViInt32 gatePolarity
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return SETPARAMENUM (instrSession,
                          "SOUR:CHIR:TRIG:EXT:GATE:POL",
                          gatePolarity,
                          chirpGatePolarityArr);

}

/*===========================================================================*/
/* Function: Get CHIRP Gate Polarity
/* Purpose:  This function returns the active level of the gate signal.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetCHIRPGatePolarity(
    ViSession   instrSession,
    ViInt32*    gatePolarity
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return GETPARAMENUM (instrSession,
                          "SOUR:CHIR:TRIG:EXT:GATE:POL?",
                          gatePolarity,
                          chirpGatePolarityArr);

}

/*===========================================================================*/
/* Function: Set CHIRP External Impedance
/* Purpose:  This function set the impedance for external feed via the PULSE 
/*           EXT input 
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/22 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetCHIRPExternalImpedance(
    ViSession   instrSession,
    ViInt32 impedance
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K22);
    }
    
    return SETPARAMENUM (instrSession,
                          "SOUR:CHIR:TRIG:EXT:IMP",
                          impedance,
                          chirpImpedanceArr);

}

/*===========================================================================*/
/* Function: Get CHIRP External Impedance
/* Purpose:  This function returns the impedance for external feed via the 
/*           PULSE EXT input 
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/22 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetCHIRPExternalImpedance(
    ViSession   instrSession,
    ViInt32*    impedance
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K22);
    }
    
    return GETPARAMENUM (instrSession,
                          "SOUR:CHIR:TRIG:EXT:IMP?",
                          impedance,
                          chirpImpedanceArr);

}

/*===========================================================================*/
/* Function: Set CHIRP Direction
/* Purpose:  This function selects the direction of the chirp modulation. 
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/22 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetCHIRPDirection(
    ViSession   instrSession,
    ViInt32 direction
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K22);
    }
    
    return SETPARAMENUM (instrSession,
                          "SOUR:CHIR:DIR",
                          direction,
                          chirpDirectionArr);

}

/*===========================================================================*/
/* Function: Get CHIRP Direction
/* Purpose:  This function returns the direction of the chirp modulation. 
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/22 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetCHIRPDirection(
    ViSession   instrSession,
    ViInt32*    direction
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K22);
    }
    
    return GETPARAMENUM (instrSession,
                          "SOUR:CHIR:DIR?",
                          direction,
                          chirpDirectionArr);

}

/*===========================================================================*/
/* Function: Set CHIRP State
/* Purpose:  This function activates/deactivates the generation of 
/*           modulation chirps.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetCHIRPState(
    ViSession   instrSession,
    ViBoolean   chripState
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:CHIR:STAT",
                               chripState);

}

/*===========================================================================*/
/* Function: Get CHIRP State
/* Purpose:  This function returns the generation of modulation chirps 
/*           state.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetCHIRPState(
    ViSession   instrSession,
    ViBoolean*  chripState
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:CHIR:STAT?",
                               chripState);

}

/*===========================================================================*/
/* Function: Set CHIRP Bandwidth
/* Purpose:  This function sets the modulation bandwidth of the modulation 
/*           chirp in Hz. The maximal  bandwidth depends on the RF frequency 
/*           set and the selected modulation mode (see data sheet).
/*            
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetCHIRPBandwidth(
    ViSession   instrSession,
    ViReal64    bandwidth
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:CHIR:BAND",
                              bandwidth,
                              0.0,80.0e6);

}

/*===========================================================================*/
/* Function: Get CHIRP Bandwidth
/* Purpose:  This function returns the modulation bandwidth of the 
/*           modulation chirp in Hz. The maximal  bandwidth depends on the RF 
/*           frequency set and the selected modulation mode (see data sheet).
/*            
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetCHIRPBandwidth(
    ViSession   instrSession,
    ViReal64*   bandwidth
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:CHIR:BAND?",
                              bandwidth);

}

/*===========================================================================*/
/* Function: Set CHIRP Pulse Period
/* Purpose:  This function sets the period of the generated modulation 
/*           chirp. The period determines the repetition frequency of the 
/*           internal signal. 
/*            
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetCHIRPPulsePeriod(
    ViSession   instrSession,
    ViReal64    pulsePeriod
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:CHIR:PULS:PER",
                              pulsePeriod,
                              200.0e-9,100.0);

}

/*===========================================================================*/
/* Function: Get CHIRP Pulse Period
/* Purpose:  This function sets the period of the generated modulation 
/*           chirp. The period determines the repetition frequency of the 
/*           internal signal. 
/*            
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetCHIRPPulsePeriod(
    ViSession   instrSession,
    ViReal64*   pulsePeriod
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:CHIR:PULS:PER?",
                              pulsePeriod);

}

/*===========================================================================*/
/* Function: Set CHIRP Pulse Width
/* Purpose:  This function sets the width of the generated pulse. The pulse 
/*           width must be at least 1us less than the set pulse period. 
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetCHIRPPulseWidth(
    ViSession   instrSession,
    ViReal64    pulseWidth
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:CHIR:PULS:WIDT",
                              pulseWidth,
                              100.0e-9,100.0);

}

/*===========================================================================*/
/* Function: Get CHIRP Pulse Width
/* Purpose:  This function returns the width of the generated pulse. The 
/*           pulse width must be at least 1us less than the set pulse period. 
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetCHIRPPulseWidth(
    ViSession   instrSession,
    ViReal64*   pulseWidth
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:CHIR:PULS:WIDT?",
                              pulseWidth);

}

/*===========================================================================*/
/* Function: Get CHIRP Compression Ratio
/* Purpose:  This function queries the pulse compression ratio, i.e. the 
/*           product of pulse width (s) and bandwidth (Hz). Pulse compression 
/*           increases the range resolution as well as the signal to noise 
/*           ratio of pulsed signals.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-K20/23 
/*           and interface board 1400.0530.02 or 1400.0552.02
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetCHIRPCompressionRatio(
    ViSession   instrSession,
    ViReal64*   compressionRatio
)
{
    INIT;
    CHECKOPTCONT (K20);

    if (rssma_status != VI_SUCCESS)
    {
        rssma_status = VI_SUCCESS;
        CHECKOPTCONT (K23);
    }
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:CHIR:COMP:RAT?",
                              compressionRatio);

}

/*============================================================================
 * Function: rssma_SetAllRFOutputsState
 * Purpose:  This function switches on or off all RF output signals of the
 *           instrument.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAllRFOutputsState(
    ViSession instrSession,
    ViBoolean state
)
{
    return SETPARAMBOOL (instrSession,
                               "OUTP:ALL:STAT",
                               state);
}

/*============================================================================
 * Function: rssma_TriggerSourceAllSweeps
 * Purpose:  This function sets the trigger source for all sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_TriggerSourceAllSweeps (ViSession instrSession,
                                                ViInt32 triggerAllSweeps)
{
    return SETPARAMENUM (instrSession,
                          "TRIG:SWE:SOUR",
                          triggerAllSweeps,
                          triggerRFSourceArr);
}

/*============================================================================
 * Function: rssma_SetOutputState
 * Purpose:  This function activates and deactivates the RF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetOutputState(
    ViSession instrSession,
    ViBoolean outputState
)
{
    return SETPARAMBOOL (instrSession,
                               "OUTP:STAT",
                               outputState);
}

/*============================================================================
 * Function: rssma_GetOutputState
 * Purpose:  This function returns state of the RF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetOutputState (ViSession  instrSession,
                                        ViBoolean* outputState)
{
    return GETPARAMBOOL (instrSession,
                               "OUTP:STAT?",
                               outputState);
}

/*============================================================================
 * Function: rssma_SetRFFrequencyonRecall
 * Purpose:  This function determines whether the current frequency setting
 *           is retained or whether the stored frequency setting is adopted
 *           when an instrument configuration is loaded.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetRFFrequencyonRecall (ViSession instrSession,
                                                ViInt32   frequencyOnRecall)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:FREQ:RCL",
                          frequencyOnRecall,
                          onRecallArr);
}

/*============================================================================
 * Function: rssma_GetRFFrequencyonRecall
 * Purpose:  This function returns whether the current frequency setting is
 *           retained or whether the stored frequency setting is adopted when 
 *           an instrument configuration is loaded.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFFrequencyonRecall (ViSession instrSession,
                                                ViInt32*  frequencyOnRecall)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:FREQ:RCL?",
                          frequencyOnRecall,
                          onRecallArr);
}

/*============================================================================
 * Function: rssma_SetRFLevelonRecall
 * Purpose:  This function determines whether the current level is retained
 *           or whether the stored level setting is adopted when an
 *           instrument configuration is loaded.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetRFLevelonRecall (ViSession instrSession,
                                            ViInt32   levelOnRecall)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:POW:RCL",
                          levelOnRecall,
                          onRecallArr);
}

/*============================================================================
 * Function: rssma_GetRFLevelonRecall
 * Purpose:  This function returns whether the current level is retained or
 *           whether the stored level setting is adopted when an instrument
 *           configuration is loaded.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFLevelonRecall (ViSession instrSession,
                                            ViInt32*  levelOnRecall)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:POW:RCL?",
                          levelOnRecall,
                          onRecallArr);
}

/*============================================================================
 * Function: rssma_OvervoltageProtectionClear
 * Purpose:  This function resets the protective circuit after it has been
 *           tripped. The state of the output is again determined by
 *           OUTPut:STATe.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_OvervoltageProtectionClear (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, ":OUTP:PROT:CLE");
}

/*============================================================================
 * Function: rssma_OvervoltageProtectionTripped
 * Purpose:  This function queries the state of the protective circuit.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_OvervoltageProtectionTripped (ViSession instrSession,
                                                      ViInt32*  stateProtective)
{
    return GETPARAMINT32 (instrSession,
                             "OUTP:PROT:TRIP?",
                             stateProtective);
}
 
/*============================================================================
 * Function: Get Output Impedance
 * Purpose:  This function queries the impedance of the RF output. This 
 *           permits converting the output level between units V and W. 
 *           The impedances cannot be changed.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetOutputImpedance (ViSession instrSession,
                                            ViInt32 *impedance)
{
    return GETPARAMENUM (instrSession,
                          "OUTP:IMP?",
                          impedance,
                          dmemExternalInputImpedance);
}

/*============================================================================
 * Function: rssma_confRFFreq
 * Purpose:  This function selects Fixed frequency mode and sets the
 *           frequency and frequency offset of the RF output signal for CW
 *           mode (:SOURce:FREQuency:MODE CW).
 *
 *           As with the FREQ value entered in the display, the OFFSet value
 *           is also taken into consideration with this function. The
 *           specified value range is therefore only effective if OFFSet is
 *           set to 0. The value range for other OFFset values can be
 *           calculated using the following formula:
 *
 *           300 kHz + OFFSet ... RFmax + OFFSet
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confRFFreq (ViSession instrSession, ViReal64 frequency,
                                    ViReal64 offset, ViBoolean variationActive,
                                    ViReal64 variationStep)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetRFFrequency (instrSession, frequency));
    CHECKERR (rssma_SetRFFrequencyOffset (instrSession, offset));
    CHECKERR (rssma_SetRFVariationActive (instrSession, variationActive));
    if (variationActive == VI_TRUE)
    {
        CHECKERR (rssma_SetRFVariationStep (instrSession, variationStep));
    }

    RETURN;
}

/*===========================================================================*/
/* Function: Configure RF Phase Continuous Frequency Settings
/* Purpose:  This function configures the RF phase continuous frequency 
/*           settings.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-B22.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_ConfigureRFPhaseContinuousFrequencySettings(
    ViSession   instrSession,
    ViBoolean   state,
    ViInt32 frequencyRange
)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetRFPhaseContinuousActive (instrSession, state));
    CHECKERR (rssma_SetRFPhaseContinuousFrequencyRange (instrSession, frequencyRange));

    RETURN;
}


/*============================================================================
 * Function: rssma_SetRFFrequency
 * Purpose:  This function sets the frequency of the RF output signal for CW
 *           mode (:SOURce:FREQuency:MODE CW). In Sweep mode
 *           (:SOURce:FREQuency:MODE SWEep), this value is linked to the
 *           current sweep frequency.
 *
 *           In addition to a numerical value, it is also possible to specify
 *           UP and DOWN. The frequency is then increased or decreased by the
 *           value which is set under [SOURce<1|2>:]FREQuency:STEP.
 *
 *           As with the FREQ value entered in the display, the OFFSet value
 *           is also taken into consideration with this function. The
 *           specified value range is therefore only effective if OFFSet is
 *           set to 0. The value range for other OFFset values can be
 *           calculated using the following formula:
 *
 *           300 kHz + OFFSet ... RFmax + OFFSet
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetRFFrequency (ViSession instrSession,
                                        ViReal64  frequency)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ",
                              frequency,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetRFFrequency
 * Purpose:  This function returns the frequency of the RF output signal for
             CW mode (:SOURce:FREQuency:MODE CW). In Sweep mode
 *           (:SOURce:FREQuency:MODE SWEep), this value is linked to the
 *           current sweep frequency.
 *
 *           In addition to a numerical value, it is also possible to specify
 *           UP and DOWN. The frequency is then increased or decreased by the
 *           value which is set under [SOURce<1|2>:]FREQuency:STEP.
 *
 *           As with the FREQ value entered in the display, the OFFSet value
 *           is also taken into consideration with this function. The
 *           specified value range is therefore only effective if OFFSet is
 *           set to 0. The value range for other OFFset values can be
 *           calculated using the following formula:
 *
 *           300 kHz + OFFSet ... RFmax + OFFSet
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFFrequency (ViSession instrSession,
                                        ViReal64* frequency)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ?",
                              frequency);
}

/*============================================================================
 * Function: rssma_SetRFFrequencyOffset
 * Purpose:  This function sets the frequency offset of a downstream
 *           instrument, e.g. a mixer (see section RF Frequency and Phase -
 *           Frequency - Phase). If a frequency offset is entered, the
 *           frequency entered with SOURce:FREQuency:... no longer
 *           corresponds to the RF output frequency. The following
 *           correlation applies:
 *
 *           SOURce:FREQuency:... = RF output frequency  +
 *           SOURce:FREQuency:OFFSet.
 *
 *           Entering an offset does not change the RF output frequency, but
 *           rather the query value of SOURce:FREQuency:...
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetRFFrequencyOffset (ViSession instrSession,
                                              ViReal64  offset)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:OFFS",
                              offset,
                              -67e9,67e9);
}

/*============================================================================
 * Function: rssma_GetRFFrequencyOffset
 * Purpose:  This function returns the frequency offset of a downstream
 *           instrument, e.g. a mixer (see section RF Frequency and Phase -
 *           Frequency - Phase). If a frequency offset is entered, the
 *           frequency entered with SOURce:FREQuency:... no longer
 *           corresponds to the RF output frequency. The following
 *           correlation applies:
 *
 *           SOURce:FREQuency:... = RF output frequency  +
 *           SOURce:FREQuency:OFFSet.
 *
 *           Entering an offset does not change the RF output frequency, but
 *           rather the query value of SOURce:FREQuency:...
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFFrequencyOffset (ViSession instrSession,
                                              ViReal64* offset)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:OFFS?",
                              offset);
}

/*============================================================================
 * Function: Set RF Multiplier
 * Purpose:  This function sets the multiplication factor for the RF frequency. 
 *			 The entry does not change the RF frequency at the RF output of 
 *			 the R&S SMA.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetRFMultiplier (ViSession instrSession,
                                         ViReal64 multiplier)
{
	return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:MULT",
                              multiplier,
                              0,0);   
}

/*============================================================================
 * Function: Get RF Multiplier
 * Purpose:  This function sets the multiplication factor for the RF frequency. 
 *
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFMultiplier (ViSession instrSession,
                                         ViReal64 *multiplier)
{
 	return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:MULT?",
                              multiplier);      
}

/*===========================================================================*/
/* Function: Set RF Phase Continuous Active
/* Purpose:  This function activates/deactivates phase continuous frequency 
/*           settings. For a given RF frequency setting, phase continuous 
/*           frequency changes are possible in a limited frequency range. The 
/*           output sinewave is phase continuous, i.e there will be no phase 
/*           discontinuity or glitch when changing the frequency.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetRFPhaseContinuousActive(
    ViSession   instrSession,
    ViBoolean   state
)
{
    INIT;
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:FREQ:PHAS:CONT:STAT",
                               state);    
}

/*===========================================================================*/
/* Function: Get RF Phase Continuous Active
/* Purpose:  This function returns the state of the phase continuous 
/*           frequency settings. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFPhaseContinuousActive(
    ViSession   instrSession,
    ViBoolean*  state
)
{
    INIT;
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:FREQ:PHAS:CONT:STAT?",
                               state);    
}

/*===========================================================================*/
/* Function: Set RF Phase Continuous Frequency Range
/* Purpose:  This function selects the mode for determining the frequency 
/*           range for the phase continuous signal.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-B22.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetRFPhaseContinuousFrequencyRange(
    ViSession   instrSession,
    ViInt32 frequencyRange
)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "SOUR:FREQ:PHAS:CONT:MODE",
                          frequencyRange,
                          phaseContinuousFrequencyRangeArr);    
}

/*===========================================================================*/
/* Function: Get RF Phase Continuous Frequency Range
/* Purpose:  This function returns the mode for determining the frequency 
/*           range for the phase continuous signal.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-B22.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFPhaseContinuousFrequencyRange(
    ViSession   instrSession,
    ViInt32*    frequencyRange
)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "SOUR:FREQ:PHAS:CONT:MODE?",
                          frequencyRange,
                          phaseContinuousFrequencyRangeArr);    
}

/*===========================================================================*/
/* Function: Get RF Phase Continuous Minimum Frequency
/* Purpose:  This function queries the minimum frequency of the frequency 
/*           range for for phase continuous settings. The minimum frequency 
/*           of the frequency range depends on the mode selected with the 
/*           command SOURce:FREQuency:PHASe:CONTinuous:MODE.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-B22.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFPhaseContinuousMinimumFrequency(
    ViSession   instrSession,
    ViReal64*   minimumFrequency
)
{
    INIT;
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:PHAS:CONT:LOW?",
                              minimumFrequency);    
}

/*===========================================================================*/
/* Function: Get RF Phase Continuous Maximum Frequency
/* Purpose:  This function queries the maximum frequency of the frequency 
/*           range for for phase continuous settings. The maximum frequency 
/*           of the frequency range depends on the mode selected with the 
/*           command SOURce:FREQuency:PHASe:CONTinuous:MODE.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMA-B22.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFPhaseContinuousMaximumFrequency(
    ViSession   instrSession,
    ViReal64*   maximumFrequency
)
{
    INIT;
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:PHAS:CONT:HIGH?",
                              maximumFrequency);    
}

/*============================================================================
 * Function: rssma_SetRFVariationActive
 * Purpose:  This function activates (USER) or deactivates (DECimal) the
 *           user-defined step width used when varying the frequency value
 *           with the frequency values UP/DOWN. This function is linked to
 *           the function Variation Active for manual control, i.e. the
 *           function also activates/deactives the user-defined step width
 *           used when varying the frequency value with the rotary knob.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetRFVariationActive (ViSession instrSession,
                                              ViBoolean variationActive)
{
    return SETPARAMENUM (instrSession,
                           "SOUR:FREQ:STEP:MODE",
                           (ViInt32) variationActive,
                           variationArr);
}

/*============================================================================
 * Function: rssma_GetRFVariationActive
 * Purpose:  This function queries the frequency step mode. This function is
 *           linked to the function Variation Active for manual control, i.e.
 *           the function also activates/deactives the user-defined step
 *           width used when varying the frequency value with the rotary
 *           knob.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFVariationActive (ViSession  instrSession,
                                              ViBoolean* variationActive)
{
    ViStatus    rssma_status;
    ViInt32     variation = 0;
    
    CHECKERR (GETPARAMENUM (instrSession,
                             "SOUR:FREQ:STEP:MODE?",
                             &variation,
                             variationArr));
    if (variationActive != NULL)
        *variationActive = (ViBoolean)((variation == 1)? 1 : 0);

    return rssma_status;
}

/*============================================================================
 * Function: rssma_SetRFVariationStep
 * Purpose:  This function sets the step width for the frequency setting if
 *           the frequency values UP/DOWN are used and variation mode
 *           :SOUR:FREQ:STEP:MODE USER is selected. This function is linked
 *           to  Variation Step for manual control, i.e. the command also
 *           sets the step width of the rotary knob for Variation Active on.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetRFVariationStep (ViSession instrSession,
                                            ViReal64  variationStep)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STEP",
                              variationStep,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetRFVariationStep
 * Purpose:  This function returns the step width of the frequency setting if
 *           the frequency values UP/DOWN are used and variation mode
 *           :SOUR:FREQ:STEP:MODE USER is selected. This function is linked
 *           to  Variation Step for manual control, i.e. the command also
 *           sets the step width of the rotary knob for Variation Active on.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFVariationStep (ViSession instrSession,
                                            ViReal64* variationStep)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STEP?",
                              variationStep);
}

/*============================================================================
 * Function: rssma_SetRFDeltaPhase
 * Purpose:  This function specifies the phase variation relative to the
 *           current phase. The variation can be specified in RADians.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetRFDeltaPhase (ViSession instrSession,
                                         ViReal64  deltaPhase)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:PHAS",
                              deltaPhase,
                              -359.9,359.9);
}

/*============================================================================
 * Function: rssma_GetRFDeltaPhase
 * Purpose:  This function returns the phase variation relative to the
 *           current phase. The variation can be specified in RADians.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFDeltaPhase (ViSession instrSession,
                                         ViReal64* deltaPhase)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:PHAS?",
                              deltaPhase);
}

/*============================================================================
 * Function: rssma_ResetRFDeltaPhaseDisplay
 * Purpose:  This function adopts the phase set with SOURce:PHASe:ADJust as
 *           the current phase.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_ResetRFDeltaPhaseDisplay (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SOUR:PHAS:REF");
}

/*============================================================================
 * Function: rssma_confRosc
 * Purpose:  This function configures reference oscillator for external or
 *           internal reference.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_confRosc (ViSession instrSession, ViInt32 ROSCSource,
                                  ViInt32 externalReferenceFreq,
                                  ViInt32 ROSCSynchronizationBandwidth,
                                  ViBoolean adjustmentActive,
                                  ViInt32 adjustmentFrequency)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetROSCSource (instrSession, ROSCSource));
    CHECKERR (rssma_SetROSCExternalReferenceFrequency (instrSession, externalReferenceFreq));
    CHECKERR (rssma_SetROSCSynchronizationBandwidth (instrSession, ROSCSynchronizationBandwidth));
    CHECKERR (rssma_SetROSCAdjustmentActive (instrSession, adjustmentActive));
    CHECKERR (rssma_SetROSCAdjustmentFrequency (instrSession, adjustmentFrequency));

    RETURN;
}

/*============================================================================
 * Function: rssma_SetROSCSource
 * Purpose:  This function selects between internal and external reference
 *           frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetROSCSource (ViSession instrSession,
                                       ViInt32   roscSource)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:ROSC:SOUR",
                          roscSource,
                          sourceArr);
}

/*============================================================================
 * Function: rssma_GetROSCSource
 * Purpose:  This function returns reference frequency source.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetROSCSource (ViSession instrSession,
                                       ViInt32*  roscSource)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:ROSC:SOUR?",
                          roscSource,
                          sourceArr);
}

/*============================================================================
 * Function: Set ROSC RF Output
 * Purpose:  This function determines if the RF output is switched off in
 *           case of a missing external reference signal for selection
 *           external source. If enabled, this setting ensures that no
 *           improper RF signal due to the missing external reference signal
 *           is output and used for measurements. In addition to the error
 *           message "Ext Ref missing", message "RF output deactivated' is
 *           generated
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetROSCRFOutput (ViSession instrSession,
                                         ViBoolean RFOutput)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:ROSC:EXT:RFOF",
                               RFOutput);
}

/*============================================================================
 * Function: Get ROSC RF Output
 * Purpose:  This function returns whether the RF output is switched off in
 *           case of a missing external reference signal for selection
 *           external source.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetROSCRFOutput (ViSession instrSession,
                                         ViBoolean *RFOutput)
{
    return GETPARAMBOOL (instrSession,
                               "SOUR:ROSC:EXT:RFOF?",
                               RFOutput);
}

/*============================================================================
 * Function: rssma_SetROSCExternalReferenceFrequency
 * Purpose:  This function informs the instrument of the frequency of the
 *           external reference.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetROSCExternalReferenceFrequency (ViSession instrSession,
                                                           ViInt32   externReferenceFreq)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:ROSC:EXT:FREQ",
                          externReferenceFreq,
                          externReferenceFreqArr);
}

/*============================================================================
 * Function: rssma_GetROSCExternalReferenceFrequency
 * Purpose:  This function returns frequency of the external reference.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetROSCExternalReferenceFrequency (ViSession instrSession,
                                                           ViInt32*  externReferenceFreq)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:ROSC:EXT:FREQ?",
                          externReferenceFreq,
                          externReferenceFreqArr);
}

/*============================================================================
 * Function: rssma_SetROSCSynchronizationBandwidth
 * Purpose:  This function selects the synchronization bandwidth for an
 *           external reference signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetROSCSynchronizationBandwidth (ViSession instrSession,
                                                         ViInt32   bandwidth)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:ROSC:EXT:SBAN",
                          bandwidth,
                          bandwidthArr);
}

/*============================================================================
 * Function: rssma_GetROSCSynchronizationBandwidth
 * Purpose:  This function returns the synchronization bandwidth for an
 *           external reference signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetROSCSynchronizationBandwidth (ViSession instrSession,
                                                         ViInt32*  bandwidth)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:ROSC:EXT:SBAN?",
                          bandwidth,
                          bandwidthArr);
}

/*============================================================================
 * Function: rssma_SetROSCExternalFrequency
 * Purpose:  This function informs the instrument of the frequency of the
 *           external reference.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetROSCExternalFrequency (ViSession instrSession,
                                                  ViInt32 externalFrequency)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:ROSC:EXT:FREQ",
                          externalFrequency,
                          externalFrequencyArr);
}

/*============================================================================
 * Function: rssma_GetROSCExternalFrequency
 * Purpose:  This function returns the instrument's frequency of the external
 *           reference.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetROSCExternalFrequency (ViSession instrSession,
                                                  ViInt32 *externalFrequency)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:ROSC:EXT:FREQ?",
                          externalFrequency,
                          externalFrequencyArr);
}

/*============================================================================
 * Function: rssma_SetROSCAdjustmentActive
 * Purpose:  This function determines whether the calibrated (OFF) or a
 *           user-defined (ON) adjustment value is used for fine adjustment
 *           of the frequency. With STATe ON, the instrument is no longer in
 *           the calibrated state.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetROSCAdjustmentActive (ViSession instrSession,
                                                 ViBoolean adjustmentActive)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:ROSC:ADJ",
                               adjustmentActive);
}

/*============================================================================
 * Function: rssma_GetROSCAdjustmentActive
 * Purpose:  This function determines whether the calibrated (OFF) or a
 *           user-defined (ON) adjustment value is used for fine adjustment
 *           of the frequency. With STATe ON, the instrument is no longer in
 *           the calibrated state.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetROSCAdjustmentActive (ViSession  instrSession,
                                                 ViBoolean* adjustmentActive)
{
    return GETPARAMBOOL (instrSession,
                               "SOUR:ROSC:ADJ?",
                               adjustmentActive);
}

/*============================================================================
 * Function: rssma_SetROSCAdjustmentFrequency
 * Purpose:  This function specifies the frequency correction value
 *           (adjustment value). A detailed definition is given in  section
 *           RF Reference Frequency - Reference Oscillator.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetROSCAdjustmentFrequency (ViSession instrSession,
                                                    ViInt32   adjustmentFreq)
{
    return SETPARAMINT32 (instrSession,
                             "SOUR:ROSC:ADJ:VAL",
                             adjustmentFreq,
                             0,4095);
}

/*============================================================================
 * Function: rssma_GetROSCAdjustmentFrequency
 * Purpose:  This function returns the frequency correction value (adjustment
 *           value). A detailed definition is given in  section  RF Reference
 *           Frequency - Reference Oscillator.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetROSCAdjustmentFrequency (ViSession  instrSession,
                                                    ViInt32*   adjustmentFreq)
{
    return GETPARAMINT32 (instrSession,
                             "SOUR:ROSC:ADJ:VAL?",
                             adjustmentFreq);
}

/*============================================================================
 * Function: rssma_confRFLevel
 * Purpose:  This function sets all parameters of fixed RF level and changes
 *           the mode to fixed RF level.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confRFLevel (ViSession instrSession, ViReal64 amplitude,
                                     ViReal64 offset, ViReal64 limit,
                                     ViBoolean variationActive,
                                     ViReal64 variationStep, ViInt32 attenuatorMode,
                                     ViInt32 RFLevel_onRecall)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetRFAmplitude (instrSession, amplitude));
    CHECKERR (rssma_SetRFLevelLimit (instrSession, limit));
    CHECKERR (rssma_SetRFLevelOffset (instrSession, offset));
    CHECKERR (rssma_SetRFAttenuatorMode (instrSession, attenuatorMode));
    CHECKERR (rssma_SetRFVariationActive (instrSession, variationActive));
    if (variationActive == VI_TRUE)
    {
        CHECKERR (rssma_SetRFVariationStep (instrSession, variationStep));
    }
    CHECKERR (rssma_SetRFLevelonRecall (instrSession, RFLevel_onRecall));

    RETURN;
}

/*============================================================================
 * Function: rssma_SetRFAmplitude
 * Purpose:  This function sets the RF output level in CW mode.
 *
 *           As with the Level value entered in the header, the OFFSet value
 *           is also taken into consideration with this command.
 *
 *           The specified value range is therefore only effective if
 *           :SOURce:POWer:OFFSet is set to 0. The value range for other
 *           OFFSet values can be calculated using the following formula:
 *
 *           Minimum level + OFFSet ... Maximum level + OFFSet
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetRFAmplitude (ViSession instrSession,
                                        ViReal64  amplitude)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW",
                              amplitude,
                              0.0,0.0);
}

/*============================================================================
 * Function: rssma_GetRFAmplitude
 * Purpose:  This function returns the RF output level in CW mode.
 *
 *           As with the Level value entered in the header, the OFFSet value
 *           is also taken into consideration with this function.
 *
 *           The specified value range is therefore only effective if
 *           :SOURce:POWer:OFFSet is set to 0. The value range for other
 *           OFFSet values can be calculated using the following formula:
 *
 *           Minimum level + OFFSet ... Maximum level + OFFSet
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFAmplitude (ViSession instrSession,
                                        ViReal64* amplitude)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW?",
                              amplitude);
}

/*============================================================================
 * Function: Set RF Output Level
 * Purpose:  This function sets the RF level of the RF output connector.
 *			 
 *			 The level entered with this command corresponds to the level at
 * 			 the RF output, i.e. any offset entry is not taken into consideration.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetRFOutputLevel (ViSession instrSession,
                                          ViReal64 RFOutputLevel)
{
   return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:POW",
                              RFOutputLevel,
                              0.0,0.0);    
}

/*============================================================================
 * Function: Get RF Output Level
 * Purpose:  This function returns the RF level of the RF output connector.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFOutputLevel (ViSession instrSession,
                                          ViReal64 *RFOutputLevel)
{
   return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:POW?",
                              RFOutputLevel);        
}

/*============================================================================
 * Function: rssma_SetRFLevelLimit
 * Purpose:  This function limits the maximum RF output level in CW and SWEEP
 *           mode. It does not influence the LEVEL display or the response to
 *           the POW? query command.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetRFLevelLimit (ViSession instrSession,
                                         ViReal64  limit)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:LIM",
                              limit,
                              0.0,0.0);
}

/*============================================================================
 * Function: rssma_GetRFLevelLimit
 * Purpose:  This function returns the RF output level limits in CW and SWEEP
 *           mode.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFLevelLimit (ViSession instrSession,
                                         ViReal64* limit)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:LIM?",
                              limit);
}

/*============================================================================
 * Function: rssma_SetRFLevelOffset
 * Purpose:  This function specifies the constant level offset of a
 *           downstream attenuator/amplifier. If a level offset is entered,
 *           the level entered with :POWer no longer corresponds to the RF
 *           output level. The following correlation applies:
 *
 *           POWer = RF output level + POWer:OFFSet.
 *
 *           Entering a level offset does not change the RF output level, but
 *           rather the query value of :POWer.
 *
 *           Only dB is permitted as the unit here. The linear units (V, W,
 *           etc.) are not permitted.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetRFLevelOffset (ViSession instrSession,
                                          ViReal64  offset)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:OFFS",
                              offset,
                              0.0,0.0);
}

/*============================================================================
 * Function: rssma_GetRFLevelOffset
 * Purpose:  This function returns the constant level offset of a downstream
 *           attenuator/amplifier. If a level offset is entered, the level
 *           entered with :POWer no longer corresponds to the RF output
 *           level. The following correlation applies:
 *
 *           POWer = RF output level + POWer:OFFSet.
 *
 *           Entering a level offset does not change the RF output level, but
 *           rather the query value of :POWer.
 *
 *           Only dB is permitted as the unit here. The linear units (V, W,
 *           etc.) are not permitted.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFLevelOffset (ViSession instrSession,
                                          ViReal64* offset)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:OFFS?",
                              offset);
}

/*============================================================================
 * Function: rssma_SetRFAttenuatorMode
 * Purpose:  This function switches the mode of the attenuator at the RF
 *           output (Attenuator MODe).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetRFAttenuatorMode (ViSession instrSession,
                                             ViInt32   attenuatorMode)
{
    return SETPARAMENUM (instrSession,
                          "OUTP:AMOD",
                          attenuatorMode,
                          attenuatorModeArr);
}

/*============================================================================
 * Function: rssma_GetRFAttenuatorMode
 * Purpose:  This function returns the mode of the attenuator at the RF
 *           output (Attenuator MODe).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFAttenuatorMode (ViSession instrSession,
                                             ViInt32*  attenuatorMode)
{
    return GETPARAMENUM (instrSession,
                          "OUTP:AMOD?",
                          attenuatorMode,
                          attenuatorModeArr);
}

/*============================================================================
 * Function: rssma_GetRFAttenuatorLowerRange
 * Purpose:  This function queries the minimum level which can be set without
 *           the attenuator being adjusted (Attenuator FIXed).
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFAttenuatorLowerRange (ViSession instrSession,
                                                   ViReal64* attenuatorLowRange)
{
    return GETPARAMREAL64 (instrSession,
                              "OUTP:AFIX:RANG:LOW?",
                              attenuatorLowRange);
}

/*============================================================================
 * Function: rssma_GetRFAttenuatorUpperRange
 * Purpose:  This function queries the maximum level which can be set without
 *           the attenuator being adjusted (Attenuator FIXed).
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFAttenuatorUpperRange (ViSession instrSession,
                                                   ViReal64* attenuatorUppRange)
{
    return GETPARAMREAL64 (instrSession,
                              "OUTP:AFIX:RANG:UPP?",
                              attenuatorUppRange);
}

/*============================================================================
 * Function: Set RF Attenuator RF Off Mode
 * Purpose:  This function selects the attenuator mode, when the RF signal 
 *			 is switched off.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetRFAttenuatorRFOffMode (ViSession instrSession,
                                                  ViInt32 RFOFFMode)
{
   return SETPARAMENUM (instrSession,
                          "SOUR:POW:ATT:RFOF:MODE",
                          RFOFFMode,
                          attenuatorRFOffMode);    
}

/*============================================================================
 * Function: Get RF Attenuator RF Off Mode
 * Purpose:  This function returns the RF Off attenuator mode,
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFAttenuatorRFOffMode (ViSession instrSession,
                                                  ViInt32 *RFOFFMode)
{
return GETPARAMENUM (instrSession,
                          "SOUR:POW:ATT:RFOF:MODE?",
                          RFOFFMode,
                          attenuatorRFOffMode);      
}

/*============================================================================
 * Function: rssma_SetRFLevelVariationActive
 * Purpose:  This function activates (USER) or deactivates (DECimal) the
 *           user-defined step width used when varying the level value with
 *           the level values UP/DOWN. This function is linked to setting
 *           Variation Active for manual control, i.e. the command also
 *           activates/deactives the user-defined step width used when
 *           varying the level value with the rotary knob.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetRFLevelVariationActive (ViSession instrSession,
                                                   ViBoolean variationActive)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:POW:STEP:MODE",
                          (ViInt32) variationActive,
                          variationArr);
}

/*============================================================================
 * Function: rssma_GetRFLevelVariationActive
 * Purpose:  This function returns level step mode. This function is linked
 *           to setting Variation Active for manual control, i.e. the
 *           function also activates/deactives the user-defined step width
 *           used when varying the level value with the rotary knob.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetRFLevelVariationActive (ViSession  instrSession,
                                                   ViBoolean* variationActive)
{
    ViStatus    rssma_status;
    ViInt32     variation = 0;
    
    CHECKERR (GETPARAMENUM (instrSession,
                             "SOUR:POW:STEP:MODE?",
                             &variation,
                             variationArr));
    if (variationActive != NULL)
        *variationActive = (ViBoolean)((variation == 1)? 1 : 0);

    return rssma_status;
}

/*============================================================================
 * Function: rssma_SetRFLevelVariationStep
 * Purpose:  This function sets the step width for the level setting if UP
 *           and DOWN are used as the level values and variation mode
 *           :SOUR:POW:STEP:MODE USER is selected. This function is linked to
 *           setting Variation Step for manual control, i.e. the function
 *           also sets the step width of the rotary knob for Variation Active
 *           on.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetRFLevelVariationStep (ViSession instrSession,
                                                 ViReal64  variationStep)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:STEP",
                              variationStep,
                              0.0,100.0);
}

/*============================================================================
 * Function: rssma_GetRFLevelVariationStep
 * Purpose:  This function returns the step width for the level setting if UP
 *           and DOWN are used as the level values and variation mode
 *           :SOUR:POW:STEP:MODE USER is selected. This function is linked to
 *           setting Variation Step for manual control, i.e. the function
 *           also sets the step width of the rotary knob for Variation Active
 *           on.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFLevelVariationStep (ViSession instrSession,
                                                 ViReal64* variationStep)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:STEP?",
                              variationStep);
}

/*============================================================================
 * Function: Configure NRP-Z Power Control
 * Purpose:  This function ets the parameters of Power Control using 
 *			 NRP-Z sensor. With this function you can achieve a very stable 
 *			 and accurate RF power supplied to your DUT.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confNRPZPowerControl (ViSession instrSession,
                                              ViInt32 sensor, 
											  ViBoolean state,
                                              ViReal64 targetLevel,
                                              ViReal64 RFLevelLimit,
                                              ViReal64 catchRange,
                                              ViInt32 delayTime,
                                              ViBoolean usePeakPower)
{
	INIT;
	CHECKERR (rssma_SetPowerControlSensor (instrSession, sensor));

    CHECKERR (rssma_SetPowerControlSensorState (instrSession, state));
    CHECKERR (rssma_SetPowerControlSensorTargetLevel (instrSession, targetLevel));
    CHECKERR (rssma_SetRFLevelLimit (instrSession, RFLevelLimit));
    CHECKERR (rssma_SetPowerControlSensorCatchRange (instrSession, catchRange));	
	CHECKERR (rssma_SetPowerControlSensorDelayTime (instrSession, delayTime));
    CHECKERR (rssma_SetPowerControlSensorPeakPowerState (instrSession, usePeakPower));
    RETURN;   
}

/*============================================================================
 * Function: Set Power Control Sensor
 * Purpose:  This function sets the sensor to be used for power control.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerControlSensor (ViSession instrSession,
                                               ViInt32 sensor)
{
	return SETPARAMENUM (instrSession,
						"SOUR:POW:SPC:SEL",	
						 sensor,
						 powerControlSensorArr);
						 
}

/*============================================================================
 * Function: Get Power Control Sensor
 * Purpose:  This function the currently selected sensor to be used for power 
 *			 control.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerControlSensor (ViSession instrSession,
                                               ViInt32 *sensor)
{
 return GETPARAMENUM (instrSession,
						"SOUR:POW:SPC:SEL?",	
						 sensor,
						 powerControlSensorArr);   
}

/*============================================================================
 * Function: Set Power Control Sensor State
 * Purpose:  This function activates/deactivates power control using 
 *			 the selected sensor
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerControlSensorState (ViSession instrSession,
                                                    ViBoolean state)
{
	return SETPARAMBOOL (instrSession,
                               "SOUR:POW:SPC:STAT",
                               state);      
}

/*============================================================================
 * Function: Get Power Control Sensor State
 * Purpose:  This function returns the power control state of selected sensor
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerControlSensorState (ViSession instrSession,
                                                    ViBoolean *state)
{
 	return GETPARAMBOOL (instrSession,
                               "SOUR:POW:SPC:STAT?",
                               state);      
}

/*============================================================================
 * Function: Set Power Control Sensor Target Level
 * Purpose:  This function sets nominal level expected at the input of the sensor.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerControlSensorTargetLevel (ViSession instrSession,
                                                          ViReal64 level)
{
	return SETPARAMREAL64 (instrSession,
                               "SOUR:POW:SPC:TARG",
                               level,
							   -50.0,
							   30.0);       
}

/*============================================================================
 * Function: Get Power Control Sensor Target Level
 * Purpose:  This function returns the nominal level expected at the input of 
 *			 the sensor.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerControlSensorTargetLevel (ViSession instrSession,
                                                          ViReal64 *level)
{
	return GETPARAMREAL64 (instrSession,
                               "SOUR:POW:SPC:TARG?",
                               level);       
}

/*============================================================================
 * Function: Set Power Control Sensor Catch Range
 * Purpose:  This function sets the capture range of the power control system.
 *===========================================================================*/     
ViStatus _VI_FUNC rssma_SetPowerControlSensorCatchRange (ViSession instrSession,
                                                         ViReal64 catchRange)
{
	return SETPARAMREAL64 (instrSession,
                               "SOUR:POW:SPC:CRAN",
                               catchRange,
							   0.0,
							   50.0);        
}

/*============================================================================
 * Function: Get Power Control Sensor Catch Range
 * Purpose:  This function returns the capture range of the power control system.
 *===========================================================================*/ 
ViStatus _VI_FUNC rssma_GetPowerControlSensorCatchRange (ViSession instrSession,
                                                         ViReal64 *catchRange)
{
	return GETPARAMREAL64 (instrSession,
                               "SOUR:POW:SPC:CRAN?",
                               catchRange);           
}

/*============================================================================
 * Function: Set Power Control Sensor Delay Time
 * Purpose:  This function sets  a waiting period between the level 
 *			 adjustment of the generator and the next measurement of 
 *			 the power sensor.
 *===========================================================================*/  
ViStatus _VI_FUNC rssma_SetPowerControlSensorDelayTime (ViSession instrSession,
                                                        ViInt32 delayTime)
{
	return SETPARAMINT32 (instrSession,
                               "SOUR:POW:SPC:DEL",
                               delayTime,
							   0,
							   1000);        
}

/*============================================================================
 * Function: Get Power Control Sensor Delay Time
 * Purpose:  This function returns  a waiting period between the level 
 *			 adjustment of the generator and the next measurement of 
 *			 the power sensor.
 *===========================================================================*/  
ViStatus _VI_FUNC rssma_GetPowerControlSensorDelayTime (ViSession instrSession,
                                                        ViInt32 *delayTime)
{
	return GETPARAMINT32 (instrSession,
                               "SOUR:POW:SPC:DEL?",
                               delayTime);      
}

/*============================================================================
 * Function: Set Power Control Sensor Peak Power State
 * Purpose:  This function activates/deactivates control by means of the peak 
 *			 power values, provided the power sensor supports this function.
 *===========================================================================*/  
ViStatus _VI_FUNC rssma_SetPowerControlSensorPeakPowerState (ViSession instrSession,
                                                            ViBoolean state)
{
	return SETPARAMBOOL (instrSession,
                               "SOUR:POW:SPC:PEAK",
                               state);        
}

/*============================================================================
 * Function: Get Power Control Sensor Peak Power State
 * Purpose:  This function returns the state of peak power values control.
 *===========================================================================*/  
ViStatus _VI_FUNC rssma_GetPowerControlSensorPeakPowerState (ViSession instrSession,
                                                            ViBoolean *state)
{
	return GETPARAMBOOL (instrSession,
                               "SOUR:POW:SPC:PEAK?",
                               state);     
}

/*============================================================================
 * Function: rssma_SetRFPowerOnState
 * Purpose:  This function selects the state which the RF output assumes when
 *           the instrument is switched on.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetRFPowerOnState (ViSession instrSession,
                                           ViInt32   powerOnState)
{
    return SETPARAMENUM (instrSession,
                          "OUTP:PON",
                          powerOnState,
                          powerOnStateArr);
}

/*============================================================================
 * Function: rssma_GetRFPowerOnState
 * Purpose:  This function returns the state which the RF output assumes when
 *           the instrument is switched on.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_GetRFPowerOnState (ViSession instrSession,
                                           ViInt32*  powerOnState)
{
    return GETPARAMENUM (instrSession,
                          "OUTP:PON?",
                          powerOnState,
                          powerOnStateArr);
}

/*============================================================================
 * Function: rssma_SetALCState
 * Purpose:  This function activates/deactivates automatic level control.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_SetALCState (ViSession instrSession,
                                     ViInt32   alcState)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:POW:ALC",
                          alcState,
                          alcStateArr);
}

/*============================================================================
 * Function: rssma_GetALCState
 * Purpose:  This function state of the automatic level control.
 *===========================================================================*/
 
ViStatus _VI_FUNC rssma_GetALCState (ViSession instrSession,
                                     ViInt32*  alcState)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:POW:ALC?",
                          alcState,
                          alcStateArr);
}

/*============================================================================
 * Function: rssma_ALCSearchOnce
 * Purpose:  This function briefly activates level control for correction
 *           purposes.
 *===========================================================================*/

ViStatus _VI_FUNC rssma_ALCSearchOnce (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SOUR:POW:ALC:SONC");
    
}

/*============================================================================
 * Function: rssma_SetUserCorrectionDataState
 * Purpose:  This function activates/deactivates level correction. Level
 *           correction is performed using the table which has been selected
 *           with the command CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetUserCorrectionDataState (ViSession instrSession,
                                                    ViBoolean correctionState)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:CORR",
                               correctionState);
}

/*============================================================================
 * Function: rssma_GetUserCorrectionDataState
 * Purpose:  This function queries state of the level correction. Level
 *           correction is performed using the table which has been selected
 *           with the command CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionDataState (ViSession instrSession,
                                                    ViBoolean *correctionState)
{
    return GETPARAMBOOL (instrSession,
                               "SOUR:CORR?",
                               correctionState);
}

/*============================================================================
 * Function: rssma_GetUserCorrectionDataCatalog
 * Purpose:  This function requests a list of user correction tables. The
 *           individual lists are separated by commas.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionDataCatalog (ViSession instrSession,
                                                      ViString directoryPath, 
                                                      ViInt32 arraySize,
                                                      ViChar _VI_FAR catalogList[])
{
    ViChar buffer[BUFFER_SIZE];

    if (!directoryPath)
        strncpy (buffer, "CORR:CSET:CAT?;*OPC?", (size_t)BUFFER_SIZE);
    else
        sprintf (buffer, "CORR:CSET:CAT? '%s';*OPC?", directoryPath);

    return rssma_getCatalog (instrSession,
                             buffer,
                             (ViUInt32) arraySize,
                             catalogList,"catalogList");
}

/*============================================================================
 * Function: rssma_SetUserCorrectionDataTable
 * Purpose:  This function selects the table for user correction. Level
 *           correction must also be activated with the command
 *           SOURce<1|2>:CORRection:CSET:STATe ON.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetUserCorrectionDataTable (ViSession instrSession,
                                                    ViString correctionTable)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:CORR:CSET",
                                    "'",
                                    correctionTable);
}

/*============================================================================
 * Function: rssma_GetUserCorrectionDataTable
 * Purpose:  This function returns the table name for user correction. Level
 *           correction must also be activated with the command
 *           SOURce<1|2>:CORRection:CSET:STATe ON.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionDataTable (ViSession instrSession,
                                                    ViChar _VI_FAR correctionTable[])
{
    return GETPARAMSTRING (instrSession,
                              "SOUR:CORR:CSET?",
                              correctionTable,
                              128);
}

/*============================================================================
 * Function: Get User Correction Value
 * Purpose:  This function requests the current value for user correction.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionValue (ViSession instrSession,
                                                ViReal64 *userCorrectionValue)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:CORR:VAL?",
                              userCorrectionValue);
}

/*============================================================================
 * Function: rssma_SetUserCorrectionDataFrequencyValues
 * Purpose:  This function transfers the frequency data to the table selected
 *           with :CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetUserCorrectionDataFrequencyValues (ViSession instrSession,
                                                              ViInt32 arraySize,
                                                              ViReal64 _VI_FAR frequencyValues[])
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    ViStatus rssma_status;

    CHECKNULL (frequencyValues);
    p2buf = pbuffer = (ViChar*) malloc ((size_t) arraySize * 20 + 20);

    p2buf += sprintf (pbuffer, "CORR:CSET:DATA:FREQ ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%f,", frequencyValues[i]);

    *--p2buf = '\0';

    rssma_status = rssma_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssma_status;
}

/*============================================================================
 * Function: rssma_GetUserCorrectionDataFrequencyValues
 * Purpose:  This function queries the frequency data from the table selected
 *           with :CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionDataFrequencyValues (ViSession instrSession,
                                                              ViInt32 arraySize,
                                                              ViReal64 _VI_FAR frequencyValues[])
{
    return GETPARAMREAL64ARRAY (instrSession,
                                   "CORR:CSET:DATA:FREQ?",
                                   frequencyValues,
                                   arraySize);
}

/*============================================================================
 * Function: rssma_GetUserCorrectionDataNumberofFrequencyValues
 * Purpose:  This function queries the number of frequency values in the table
 *           selected with :CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionDataNumberofFrequencyValues (ViSession instrSession,
                                                                      ViInt32 *number_ofFrequencyValues)
{
    return GETPARAMINT32 (instrSession,
                             "CORR:CSET:DATA:FREQ:POIN?",
                             number_ofFrequencyValues);
}

/*============================================================================
 * Function: rssma_SetUserCorrectionDataLevelValues
 * Purpose:  This function transfers the level correction data to the table
 *           selected with :CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetUserCorrectionDataLevelValues (ViSession instrSession,
                                                          ViInt32 arraySize,
                                                          ViReal64 _VI_FAR levelValues[])
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    ViStatus rssma_status;

    CHECKNULL (levelValues);
    p2buf = pbuffer = malloc ((size_t) arraySize * 20 + 20);

    p2buf += sprintf (pbuffer, "CORR:CSET:DATA:POW ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%f,", levelValues[i]);

    *--p2buf = '\0';

    rssma_status = rssma_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssma_status;
}

/*============================================================================
 * Function: rssma_GetUserCorrectionDataLevelValues
 * Purpose:  This function queries the level correction data of the table
 *           selected with :CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionDataLevelValues (ViSession instrSession,
                                                          ViInt32 arraySize,
                                                          ViReal64 _VI_FAR levelValues[])
{
    return GETPARAMREAL64ARRAY (instrSession,
                                   "CORR:CSET:DATA:POW?",
                                   levelValues,
                                   arraySize);
}

/*============================================================================
 * Function: rssma_GetUserCorrectionDataNumberofLevelValues
 * Purpose:  This function queries the number of level values in the selected
 *           table.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionDataNumberofLevelValues (ViSession instrSession,
                                                                  ViInt32 *number_ofLevelValues)
{
    return GETPARAMINT32 (instrSession,
                             "CORR:CSET:DATA:POW:POIN?",
                             number_ofLevelValues);
}

/*============================================================================
 * Function: User Correction  Fill List With Sensor
 * Purpose:  This function fills the selected user correction list with the
 *           level values measured by the power sensor for the given
 *           frequencies
 *===========================================================================*/
ViStatus _VI_FUNC rssma_UserCorrectionFillListWithSensor (ViSession instrSession,
                                                          ViInt32 sensor,
                                                          ViInt32 timeout)
{
    ViChar  buffer[BUFFER_SIZE] = "";
    
    INIT;

    CHECKVIINT32 (instrSession, sensor, 1, 3);
    
    sprintf (buffer, "SOUR:CORR:CSET:DATA:SENS%ld:POW:SONC;*OPC?", sensor);
    
    CHECKERR (rssma_waitOPC (instrSession, buffer, buffer, timeout));
    
    return rssma_status;
}

/*============================================================================
 * Function: rssma_UserCorrectionDataDeleteTable
 * Purpose:  This function deletes the specified table.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_UserCorrectionDataDeleteTable (ViSession instrSession,
                                                       ViString tableName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "CORR:CSET:DEL",
                                    "'",
                                    tableName);
}

/*===========================================================================*/
/* Function: Set ALC Off Mode
/* Purpose:  This function sets the level control mode which becomes active 
/*           when automatic level control is deactivated (ALC Off). 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetALCOffMode(
    ViSession   instrSession,
    ViInt32 listMode
)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "SOUR:POW:ALC:OMOD",
                          listMode,
                          alcOffModeArr);    
}

/*===========================================================================*/
/* Function: Get ALC Off Mode
/* Purpose:  This function returns the level control mode which becomes 
/*           active when automatic level control is deactivated (ALC Off). 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetALCOffMode(
    ViSession   instrSession,
    ViInt32*    listMode
)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "SOUR:POW:ALC:OMOD?",
                          listMode,
                          alcOffModeArr);    
}

/*===========================================================================
 * Function: Set User Correction Import Export List Mode
 * Purpose:  This function selects if user correction lists should be
 *           imported or exported.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportListMode (ViSession instrSession,
                                                               ViInt32 listMode)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:MODE",
                          listMode,
                          dexcListModeArr);
}

/*===========================================================================
 * Function: Get User Correction Import Export List Mode
 * Purpose:  This function returns whether user correction lists is imported 
 *           or exported.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportListMode (ViSession instrSession,
                                                               ViInt32 *listMode)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:MODE?",
                          listMode,
                          dexcListModeArr);
}

/*===========================================================================
 * Function: Set User Correction Import Export List File
 * Purpose:  This function calls the File Manager for selecting the user
 *           correction list to be exported (source) into an ASCII file or
 *           the destination for the ASCII file to be imported (destination)
 *           in.
 *           
 *           The user correction files are stored with the fixed file
 *           extensions *.lsw in a directory of the user's choice. The
 *           directory applicable to the commands is defined with the command
 *           MMEMory:CDIR. A path can also be specified in command
 *           SOUR:LIST:DEXC:SEL, in which case the files are stored or loaded
 *           in the specified directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportListFile (ViSession instrSession,
                                                               ViString sourceFile)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:CORR:DEXC:SEL",
                                    "'",
                                    sourceFile);
}

/*===========================================================================
 * Function: Get User Correction Import Export List File
 * Purpose:  This function calls the File Manager for querying the user
 *           correction list to be exported (source) into an ASCII file or
 *           the destination for the ASCII file to be imported (destination)
 *           in.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportListFile (ViSession instrSession,
                                                               ViChar sourceFile[])
{
    return GETPARAMSTRING (instrSession,
                              "SOUR:CORR:DEXC:SEL?",
                              sourceFile,
                              128);
}

/*===========================================================================
 * Function: User Correction Import Export Execute
 * Purpose:  This function starts the export or import of the selected file. 
 *           
 *           When import is selected, the ASCII file is imported as user 
 *           correction list.
 *           
 *           When export is selected, the user correction list is exported into 
 *           the selected ASCII file.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_UserCorrectionImportExportExecute (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SOUR:CORR:DEXC:EXEC");
}

/*===========================================================================
 * Function: Set User Correction Import Export ASCII File
 * Purpose:  This function calls the File Manager for selecting the ASCII
 *           file to be imported into a user correction list (source) or the
 *           ASCII file the user correction list is exported (destination)
 *           in.
 *           
 *           The ASCII files are stored with the fixed file extensions *.txt
 *           or *.csv in a directory of the user's choice. The directory
 *           applicable to the commands is defined with the command
 *           MMEMory:CDIR.  A path can also be specified in command
 *           SOUR:LIST:DEXC:AFIL:SEL, in which case the files are stored or
 *           loaded in the specified directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportASCIIFile (ViSession instrSession,
                                                                ViString ASCIISourceFile)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:CORR:DEXC:AFIL:SEL",
                                    "'",
                                    ASCIISourceFile);
}

/*===========================================================================
 * Function: Get User Correction Import Export ASCII File
 * Purpose:  This function calls the File Manager for querying the ASCII file
 *           to be imported into a user correction list (source) or the ASCII
 *           file the user correction list is exported (destination) in.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportASCIIFile (ViSession instrSession,
                                                                ViChar ASCIISourceFile[])
{
    return GETPARAMSTRING (instrSession,
                              "SOUR:CORR:DEXC:AFIL:SEL?",
                              ASCIISourceFile,
                              128);
}

/*===========================================================================
 * Function: Set User Correction Import Export ASCII Extension
 * Purpose:  This function selects the file extension of the ASCII file to 
 *           be imported or exported. Selection TXT (text file) or CSV (Excel 
 *           file) is available. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportASCIIExtension (ViSession instrSession,
                                                                     ViInt32 extension)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:EXT",
                          extension,
                          dexcExtensionArr);
}

/*===========================================================================
 * Function: Get User Correction Import Export ASCII Extension
 * Purpose:  This function returns the file extension of the ASCII file to 
 *           be imported or exported. Selection TXT (text file) or CSV (Excel 
 *           file) is available. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportASCIIExtension (ViSession instrSession,
                                                                     ViInt32 *extension)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:EXT?",
                          extension,
                          dexcExtensionArr);
}

/*===========================================================================
 * Function: Set User Correction Import Export ASCII Column Separator
 * Purpose:  This function selects the separator between the frequency and 
 *           level column of the ASCII table. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportASCIIColumnSeparator (ViSession instrSession,
                                                                           ViInt32 separator)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:SEP:COL",
                          separator,
                          dexcSeparatorArr);
}

/*===========================================================================
 * Function: Get User Correction Import Export ASCII Column Separator
 * Purpose:  This function returns the separator between the frequency and 
 *           level column of the ASCII table. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportASCIIColumnSeparator (ViSession instrSession,
                                                                           ViInt32 *separator)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:SEP:COL?",
                          separator,
                          dexcSeparatorArr);
}

/*===========================================================================
 * Function: Set User Correction Import Export ASCII Decimal Separator
 * Purpose:  This function selects the decimal separator used in the ASCII 
 *           data between '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetUserCorrectionImportExportASCIIDecimalSeparator (ViSession instrSession,
                                                                            ViInt32 decimalSeparator)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:SEP:DEC",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*===========================================================================
 * Function: Get User Correction Import Export ASCII Decimal Separator
 * Purpose:  This function returns the decimal separator used in the ASCII 
 *           data between '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetUserCorrectionImportExportASCIIDecimalSeparator (ViSession instrSession,
                                                                            ViInt32 *decimalSeparator)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:SEP:DEC?",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*===========================================================================
 * Function: User Correction Import Export Catalog
 * Purpose:  This function requests a list of available ASCII files for 
 *           export/import of user correction data. The individual files are 
 *           separated by commas.
 *            
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified in command 
 *           SOUR:LIST:DEXC:AFIL:CAT?, in which case the files in the 
 *           specified directory are read.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_UserCorrectionImportExportCatalog (ViSession instrSession,
                                                           ViInt32 arraySize,
                                                           ViChar catalogList[])
{
    return rssma_getCatalog (instrSession,
                             ":CORR:DEXC:AFIL:CAT?;*OPC?",
                             arraySize,
                             catalogList, "catalogList");

}

/*===========================================================================*/
/* Function: Configure Power Sense
/* Purpose:  This function configure the power sense (NRP-Z Power View)
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confPowerSense (ViSession instrSession,
                                        ViInt32 sense, ViBoolean initPowerMeas,
                                        ViInt32 unit, ViBoolean permanentDisplay,
                                        ViInt32 source, ViReal64 frequency,
                                        ViInt32 filterMode)
{
    ViStatus rssma_status;
    
    CHECKERR (rssma_SetInitiatePowerMeasurement (instrSession, sense, initPowerMeas));
    CHECKERR (rssma_SetPowerUnit (instrSession, sense, unit));
    CHECKERR (rssma_SetPowerDisplayPermanentState (instrSession, sense, permanentDisplay));
    CHECKERR (rssma_SetPowerSource (instrSession, sense, source));
    if (source == RSSMA_SENSE_POWER_SOURCE_USER)
    {
        CHECKERR (rssma_SetPowerFrequency (instrSession, sense, frequency));
    }
    CHECKERR (rssma_SetPowerFilterMode (instrSession, sense, filterMode));
    
    return rssma_status;
}



/*===========================================================================*/
/* Function: Configure Power Response Measurement
/* Purpose:  This function configure the power sense response measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confPowerResponseMeasurement (ViSession instrSession,
                                                      ViInt32 sense,
                                                      ViInt32 sweepMode,
                                                      ViReal64 start,
                                                      ViReal64 stop,
                                                      ViInt32 timing,
                                                      ViInt32 steps,
                                                      ViInt32 spacing)
{
    ViStatus rssma_status;
    
    CHECKERR (rssma_SetPowerSweepMode (instrSession, sense, sweepMode));
    
    switch (sweepMode)
    {
    case RSSMA_SENSE_POWER_SWEEP_MODE_FREQ: 
        CHECKERR (rssma_SetPowerSweepFrequencyStart (instrSession, sense, start));
        CHECKERR (rssma_SetPowerSweepFrequencyStop (instrSession, sense, stop));
        CHECKERR (rssma_SetPowerSweepFrequencyTiming (instrSession, sense, timing));
        CHECKERR (rssma_SetPowerSweepFrequencySteps (instrSession, sense, steps));
        CHECKERR (rssma_SetPowerSweepFrequencySpacing (instrSession, sense, spacing));
        break;
        
    case RSSMA_SENSE_POWER_SWEEP_MODE_POW:
        CHECKERR (rssma_SetPowerSweepPowerStart (instrSession, sense, start));
        CHECKERR (rssma_SetPowerSweepPowerStop (instrSession, sense, stop));
        CHECKERR (rssma_SetPowerSweepPowerTiming (instrSession, sense, timing));
        CHECKERR (rssma_SetPowerSweepPowerSteps (instrSession, sense, steps));
        CHECKVIINT32 (instrSession, spacing, 0, 0);
        CHECKERR (rssma_SetPowerSweepPowerSpacing (instrSession, sense, spacing));
        break;
    case RSSMA_SENSE_POWER_SWEEP_MODE_TIME:
        CHECKERR (rssma_SetPowerSweepTimeStart (instrSession, sense, start));
        CHECKERR (rssma_SetPowerSweepTimeStop (instrSession, sense, stop));
        CHECKERR (rssma_SetPowerSweepTimeSteps (instrSession, sense, steps));
        break;
    default:
        rssma_status = rssma_ParameterError (instrSession, "sweepMode");
        
    }
    
    return rssma_status;
}

/*===========================================================================
 * Function: Configure Power Response Measurement Level Offset
 * Purpose:  This function  configures the power sense level offset.
*===========================================================================*/
ViStatus _VI_FUNC rssma_confPowerResponseMeasurementLevelOffset (ViSession instrSession,
                                                   ViInt32 sense,
                                                   ViInt32 sweepMode,
                                                   ViBoolean offsetState,
                                                   ViReal64 offset)
{
    INIT;
	
    CHECKERR (rssma_SetPowerSweepMode (instrSession, sense, sweepMode));    
    
	switch (sweepMode)
    {
    case RSSMA_SENSE_POWER_SWEEP_MODE_FREQ: 
        CHECKERR (rssma_SetPowerSweepFrequencyLevelOffsetState (instrSession, sense, offsetState));
        CHECKERR (rssma_SetPowerSweepFrequencyLevelOffset (instrSession, sense, offset));
    break;
        
    case RSSMA_SENSE_POWER_SWEEP_MODE_POW:
        CHECKERR (rssma_SetPowerSweepPowerLevelOffsetState (instrSession, sense, offsetState));
        CHECKERR (rssma_SetPowerSweepPowerLevelOffset (instrSession, sense, offset));
    break;
    case RSSMA_SENSE_POWER_SWEEP_MODE_TIME:
        CHECKERR (rssma_SetPowerSweepTimeLevelOffsetState (instrSession, sense, offsetState));
        CHECKERR (rssma_SetPowerSweepTimeLevelOffset (instrSession, sense, offset));
    break;
    default:
        rssma_status = rssma_ParameterError (instrSession, "sweepMode");
    }
    
    return rssma_status;    
}

/*===========================================================================*/
/* Function: Configure Power Response Measurement Diagram
/* Purpose:  This function configure the power sense response measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confPowerResponseMeasurementDiagram (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViReal64 frequencyYScaleMinimum,
                                                             ViReal64 frequencyYScaleMaximum,
                                                             ViReal64 powerYScaleMinimum,
                                                             ViReal64 powerYScaleMaximum,
                                                             ViInt32 senseMode)
{
    ViStatus rssma_status;
    
    CHECKERR (rssma_SetPowerSweepFrequencyYScaleMinimum (instrSession, sense, frequencyYScaleMinimum));
    CHECKERR (rssma_SetPowerSweepFrequencyYScaleMaximum (instrSession, sense, frequencyYScaleMaximum));
    CHECKERR (rssma_SetPowerSweepPowerYScaleMinimum (instrSession, sense, powerYScaleMinimum));
    CHECKERR (rssma_SetPowerSweepPowerYScaleMaximum (instrSession, sense, powerYScaleMaximum));
    CHECKERR (rssma_SetPowerSweepSenseMode (instrSession, sense, senseMode));
    
    return rssma_status;
}

/*===========================================================================*/
/* Function: Power Zero
/* Purpose:  This function activates the autozero function. Zeroing is 
/*           required in regular interval (at least once a day)and if the 
/*           temperature has varied more than about 5 °C, if the sensor has 
/*           been replaced or if measurements of signals with very low power 
/*           are to be performed. The sensor must be disconnected from all 
/*           power sources before starting the autozero function.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PowerZero (ViSession instrSession, ViInt32 sense)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:ZERO", sense);

    return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Set Initiate Power Measurement
/* Purpose:  This function activates/deactivates the continuous power 
/*           measurement by the R&S NRP-Z power sensors. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetInitiatePowerMeasurement (ViSession instrSession,
                                                     ViInt32 sense,
                                                     ViBoolean initPowerMeas)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "INIT%ld:POW:CONT", sense);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               initPowerMeas);
}

/*===========================================================================*/
/* Function: Get Initiate Power Measurement
/* Purpose:  This function returns activation the continuous power 
/*           measurement by the R&S NRP-Z power sensors. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetInitiatePowerMeasurement (ViSession instrSession,
                                                     ViInt32 sense,
                                                     ViBoolean *initPowerMeas)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "INIT%ld:POW:CONT?", sense);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               initPowerMeas);
}

/*===========================================================================*/
/* Function: Set Power Use SParameters
/* Purpose:  This function activates/deactivates the use of the s-parameters 
/*           correction data of the connected power sensor. 
/*           
/*           Note(s):
/*           (1) For power sensor with attenuator this command is 
/*           automatically set to ON.
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerUseSParameters (ViSession instrSession,
                                                ViInt32 sense,
                                                ViBoolean useSParameters)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:CORR:SPD:STAT", sense);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               useSParameters);
}

/*===========================================================================*/
/* Function: Get Power Use SParameters
/* Purpose:  This function returns the state of usage of the s-parameters 
/*           correction data of the connected power sensor. 
/*           
/*           Note(s):
/*           (1) For power sensor with attenuator this command is 
/*           automatically set to ON.
/*           
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerUseSParameters (ViSession instrSession,
                                                ViInt32 sense,
                                                ViBoolean *useSParameters)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:CORR:SPD:STAT?", sense);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               useSParameters);
}

/*===========================================================================*/
/* Function: Set Power Frequency
/* Purpose:  This function sets the RF frequency of the source if the user 
/*           source is selected (SENSe[:POWer]:SOURce USER.)
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerFrequency (ViSession instrSession,
                                           ViInt32 sense, ViReal64 frequency)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FREQ", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              frequency,
                              0.0,0.0);
}

/*===========================================================================*/
/* Function: Get Power Frequency
/* Purpose:  This function returns the RF frequency of the source if the 
/*           user source is selected (SENSe[:POWer]:SOURce USER.)
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerFrequency (ViSession instrSession,
                                           ViInt32 sense, ViReal64 *frequency)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FREQ?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              frequency);
}

/*===========================================================================*/
/* Function: Set Power Offset
/* Purpose:  This function sets a level offset which is added to the 
/*           measured level value after activation with command 
/*           SENSe[:POWer]:OFFSet:STATe ON. This allows e.g. an attenuator in 
/*           the signal path to be taken into account.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerOffset (ViSession instrSession,
                                        ViInt32 sense, ViReal64 offset)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    CHECKVIREAL64(instrSession, offset, -100.0, 100.0);
    
    sprintf (buffer, "SENS%ld:POW:OFFS %f", sense, offset);

    return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get Power Offset
/* Purpose:  This function returns a level offset which is added to the 
/*           measured level value after activation with command 
/*           SENSe[:POWer]:OFFSet:STATe ON. This allows e.g. an attenuator in 
/*           the signal path to be taken into account.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerOffset (ViSession instrSession,
                                        ViInt32 sense, ViReal64 *offset)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:OFFS?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              offset);
}

/*===========================================================================*/
/* Function: Set Power Offset State
/* Purpose:  This function activates/deactivates the addition of  the level 
/*           offset to the measured value. The level offset value is set with 
/*           command SENSe[:POWer]:OFFSet.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerOffsetState (ViSession instrSession,
                                             ViInt32 sense,
                                             ViBoolean offsetState)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:OFFS:STAT", sense);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               offsetState);
}

/*===========================================================================*/
/* Function: Get Power Offset State
/* Purpose:  This function returns the state of activation the addition of  
/*           the level offset to the measured value. The level offset value 
/*           is set with command SENSe[:POWer]:OFFSet.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerOffsetState (ViSession instrSession,
                                             ViInt32 sense,
                                             ViBoolean *offsetState)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:OFFS:STAT?", sense);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               offsetState);
}

/*===========================================================================*/
/* Function: Set Power Source
/* Purpose:  The command selects the signal source for the measurement. The 
/*           RF signal or a user defined source can be selected.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSource (ViSession instrSession,
                                        ViInt32 sense, ViInt32 source)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SOUR", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          source,
                          powerSourceArr);
}

/*===========================================================================*/
/* Function: Get Power Source
/* Purpose:  The command queries the signal source for the measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSource (ViSession instrSession,
                                        ViInt32 sense, ViInt32 *source)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SOUR?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          source,
                          powerSourceArr);
}

/*===========================================================================*/
/* Function: Set Power Unit
/* Purpose:  The command selects the unit used for result query with command 
/*           READ. The power sensor provides the measured value in Watt. In 
/*           which unit the measured value is returned is selected here
/*           and might be either Watt, dBm or dBuV.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerUnit (ViSession instrSession, ViInt32 sense,
                                      ViInt32 unit)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:UNIT:POW", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          unit,
                          sensePowerUnitArr);
}

/*===========================================================================*/
/* Function: Get Power Unit
/* Purpose:  This function queries the unit used for result query with 
/*           command READ.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerUnit (ViSession instrSession, ViInt32 sense,
                                      ViInt32 *unit)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:UNIT:POW?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          unit,
                          sensePowerUnitArr);
}

/*===========================================================================*/
/* Function: Set Power Display Permanent State
/* Purpose:  This function switches on and off  the permanent indication of 
/*           the power measurement result in the upper right corner of the 
/*           block diagram. For each sensor, the type of sensor, the 
/*           connector, the measurement source and - if set - the offset is 
/*           indicated.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerDisplayPermanentState (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViBoolean permanentDisplay)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:DISP:PERM:STAT", sense);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               permanentDisplay);
}

/*===========================================================================*/
/* Function: Get Power Display Permanent State
/* Purpose:  This function returns on and off state of the permanent 
/*           indication of the power measurement result in the upper right 
/*           corner of the block diagram. For each sensor, the type of 
/*           sensor, the connector, the measurement source and - if set - the 
/*           offset is indicated.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerDisplayPermanentState (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViBoolean *permanentDisplay)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:DISP:PERM:STAT?", sense);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               permanentDisplay);
}

/*===========================================================================
 * Function: Set Power Display Permanent Priority
 * Purpose:  The function selects which power measurement result (average or
 *           peak power) is indicated when permanent display is active. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerDisplayPermanentPriority (ViSession instrSession,
                                                          ViInt32 trace,
                                                          ViInt32 priority)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:DISP:PERM:PRI", trace);
    
    return SETPARAMENUM (instrSession,
                          buffer,
                          priority,
                          powerPriorityArr);
}

/*===========================================================================
 * Function: Get Power Display Permanent Priority
 * Purpose:  The function returns which power measurement result (average or
 *           peak power) is indicated when permanent display is active. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerDisplayPermanentPriority (ViSession instrSession,
                                                          ViInt32 trace,
                                                          ViInt32 *priority)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:DISP:PERM:PRI?", trace);
    
    return GETPARAMENUM (instrSession,
                          buffer,
                          priority,
                          powerPriorityArr);
}

/*===========================================================================*/
/* Function: Configure Power Filter
/* Purpose:  This function selects the filter mode. For mode NSR selects 
/*           noise content and noise timeout.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confPowerFilter (ViSession instrSession,
                                         ViInt32 sense, ViInt32 filterMode,
                                         ViReal64 noiseContent, ViReal64 timeout)
{
    ViStatus rssma_status;
    
    CHECKERR (rssma_SetPowerFilterMode (instrSession, sense, filterMode));
    CHECKERR (rssma_SetPowerFilterFixedNoiseContent (instrSession, sense, noiseContent));
    CHECKERR (rssma_SetPowerFilterFixedNoiseTimeout (instrSession, sense, timeout));

    return rssma_status;
}

/*===========================================================================*/
/* Function: Set Power Filter Length
/* Purpose:  This function selects the filter length for user filter mode
/*           (:SENSe<[1]...3>:POWer:FILTer:TYPE USER). As the filter length 
/*           works as a multiplier for the time window, a constant filter 
/*           length results in a constant measurement time. Values 1 and 2^n 
/*           are setable. In remote control, when using command READ to 
/*           retrieve the measurement result, the time window is fixed to 20 
/*           ms. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerFilterLength (ViSession instrSession,
                                              ViInt32 sense,
                                              ViInt32 filterLength)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:LENG:USER", sense);

    return SETPARAMINT32 (instrSession, 
                             buffer,
                             filterLength,
                             1, 65536);
}

/*===========================================================================*/
/* Function: Get Power Filter Length
/* Purpose:  This function selects the filter length for user filter mode. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerFilterLength (ViSession instrSession,
                                              ViInt32 sense,
                                              ViInt32 *filterLength)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:LENG:USER?", sense);

    return GETPARAMINT32 (instrSession, 
                             buffer,
                             filterLength);
}

/*===========================================================================*/
/* Function: Set Power Filter Mode
/* Purpose:  This function selects the filter mode. The filter length is the 
/*           multiplier for the time window and thus directly influences the 
/*           measurement time. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerFilterMode (ViSession instrSession,
                                            ViInt32 sense, ViInt32 filterMode)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:TYPE", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          filterMode,
                          powerFilterModeArr);
}

/*===========================================================================*/
/* Function: Get Power Filter Mode
/* Purpose:  This function queriess the filter mode for user filter mode. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerFilterMode (ViSession instrSession,
                                            ViInt32 sense, ViInt32 *filterMode)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:TYPE?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          filterMode,
                          powerFilterModeArr);
}

/*===========================================================================*/
/* Function: Set Power Filter Fixed Noise Content
/* Purpose:  This function sets the noise content for fixed noise filter 
/*           mode (:SENSe:POWer:FILTer:TYPE NSRation). This value determines 
/*           the proportion of intrinsic noise in the measured result.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerFilterFixedNoiseContent (ViSession instrSession,
                                                         ViInt32 sense,
                                                         ViReal64 noiseContent)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:NSR", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              noiseContent,
                              0.0001, 1.0);
}

/*===========================================================================*/
/* Function: Get Power Filter Fixed Noise Content
/* Purpose:  This function returns the noise content for fixed noise filter 
/*           mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerFilterFixedNoiseContent (ViSession instrSession,
                                                         ViInt32 sense,
                                                         ViReal64 *noiseContent)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:NSR?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              noiseContent);
}

/*===========================================================================*/
/* Function: Set Power Filter Fixed Noise Timeout
/* Purpose:  This function sets the timeout for fixed noise filter mode 
/*           (:SENSe:POWer:FILTer:TYPE NSRation). This value ensures limited 
/*           settling times.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerFilterFixedNoiseTimeout (ViSession instrSession,
                                                         ViInt32 sense,
                                                         ViReal64 timeout)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:NSR:MTIM", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              timeout,
                              1.0, 999.99);
}

/*===========================================================================*/
/* Function: Get Power Filter Fixed Noise Timeout
/* Purpose:  This function returns the timeout for fixed noise filter mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerFilterFixedNoiseTimeout (ViSession instrSession,
                                                         ViInt32 sense,
                                                         ViReal64 *timeout)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:NSR:MTIM?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              timeout);
}

/*===========================================================================*/
/* Function: Get Power Filter Length Auto
/* Purpose:  This function returns the filter length automatically set 
/*           value.
/*           
/*           Example:
/*            SENS1:FILT:TYPE AUTO
/*            selects auto filter mode for the power sensor connected to the 
/*            
/*            SENSOR connector.
/*           
/*            SENS1:FILT:LENG:AUTO?
/*            queries the automatically set filter length.
/*           
/*            Response: 1024
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerFilterLengthAuto (ViSession instrSession,
                                                  ViInt32 sense,
                                                  ViInt32 *filterLengthAuto)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:LENG:AUTO?", sense);

    return GETPARAMINT32 (instrSession, 
                             buffer,
                             filterLengthAuto);
}

/*===========================================================================*/
/* Function: Power Filter Search Once
/* Purpose:  This function the search for the optimum filter length for the 
/*           current measurement conditions. The found filter length can be 
/*           retrieved with command :SENSe:POWer:FILTer:LENGth:USER?. This 
/*           command is only available for user filter mode 
/*           (:SENSe:POWer:FILTer:TYPE USER).
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PowerFilterSearchOnce (ViSession instrSession,
                                               ViInt32 sense)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:SONC", sense);

    return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================
 * Function: Set Power Sensor Gate State
 * Purpose:  The function activates gate settings for the selected trace. The
 *           measurement is started with command SENS:POW:SWE:INIT (function
 *           rssma_StartPowerSensorMeasurement). Both gates are active at one
 *           time.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSensorGateState (ViSession instrSession,
                                                 ViBoolean state)
{
    INIT;
    
    CHECKOPT (K28);
    
    return SETPARAMBOOL (instrSession,
                               "CALC:POW:SWE:TIME:GATE:STAT",
                               state);
}

/*===========================================================================
 * Function: Get Power Sensor Gate State
 * Purpose:  The function returns the state of the gate settings for the
 *           selected trace.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSensorGateState (ViSession instrSession,
                                                 ViBoolean *state)
{
    INIT;
    
    CHECKOPT (K28);
    
    return GETPARAMBOOL (instrSession,
                               "CALC:POW:SWE:TIME:GATE:STAT?",
                               state);
}

/*===========================================================================
 * Function: Set Power Sensor Gate Start
 * Purpose:  The function sets the start time of the selected gate.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSensorGateStart (ViSession instrSession,
                                                 ViInt32 gate, ViReal64 start)
{
    ViChar buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, gate, 1, 2);
    
    sprintf (buffer, "CALC:POW:SWE:TIME:GATE%ld:STAR", gate);
    
    return SETPARAMREAL64 (instrSession,
                              buffer,
                              start,
                              0.0, 0.0);
}

/*===========================================================================
 * Function: Get Power Sensor Gate Start
 * Purpose:  The function returns the start time of the selected gate.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSensorGateStart (ViSession instrSession,
                                                 ViInt32 gate, ViReal64 *start)
{
    ViChar buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, gate, 1, 2);
    
    sprintf (buffer, "CALC:POW:SWE:TIME:GATE%ld:STAR?", gate);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              start);
}

/*===========================================================================
 * Function: Set Power Sensor Gate Stop
 * Purpose:  The function sets the stop time of the selected gate.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSensorGateStop (ViSession instrSession,
                                                ViInt32 gate, ViReal64 stop)
{
    ViChar buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, gate, 1, 2);
    
    sprintf (buffer, "CALC:POW:SWE:TIME:GATE%ld:STOP", gate);
    
    return SETPARAMREAL64 (instrSession,
                              buffer,
                              stop,
                              0.0, 0.0);
}

/*===========================================================================
 * Function: Get Power Sensor Gate Stop
 * Purpose:  The function retruns the stop time of the selected gate.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSensorGateStop (ViSession instrSession,
                                                ViInt32 gate, ViReal64 *stop)
{
    ViChar buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, gate, 1, 2);
    
    sprintf (buffer, "CALC:POW:SWE:TIME:GATE%ld:STOP?", gate);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              stop);
}

/*===========================================================================
 * Function: Set Power Sensor Gate Trace
 * Purpose:  The function selects the trace for time gated measurement. Both
 *           gates are assigned to the same trace.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSensorGateTrace (ViSession instrSession,
                                                 ViInt32 trace)
{
    INIT;
    
    CHECKOPT (K28);
    
    return SETPARAMENUM (instrSession,
                          "CALC:POW:SWE:TIME:GATE:FEED",
                          trace,
                          powerGateTraceArr);
}

/*===========================================================================
 * Function: Get Power Sensor Gate Trace
 * Purpose:  The function returns the trace for time gated measurement. 
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSensorGateTrace (ViSession instrSession,
                                                 ViInt32 *trace)
{
    INIT;
    
    CHECKOPT (K28);
    
    return GETPARAMENUM (instrSession,
                          "CALC:POW:SWE:TIME:GATE:FEED?",
                          trace,
                          powerGateTraceArr);
}

/*===========================================================================
 * Function: Get Power Sensor Gate Average Power
 * Purpose:  The function queries the average power value of the time gated
 *           measurement.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSensorGateAveragePower (ViSession instrSession,
                                                        ViInt32 gate,
                                                        ViReal64 *averagePower)
{
    ViChar buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, gate, 1, 2);
    
    sprintf (buffer, "CALC:POW:SWE:TIME:GATE%ld:AVER?", gate);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              averagePower);
}

/*===========================================================================
 * Function: Get Power Sensor Gate Peak Power
 * Purpose:  The function queries the peak power value of the time gated
 *           measurement.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSensorGatePeakPower (ViSession instrSession,
                                                     ViInt32 gate,
                                                     ViReal64 *peakPower)
{
    ViChar buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, gate, 1, 2);
    
    sprintf (buffer, "CALC:POW:SWE:TIME:GATE%ld:MAX?", gate);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              peakPower);
}

/*===========================================================================*/
/* Function: Power Sweep Initiate
/* Purpose:  This function starts the response measurement with NRP power 
/*           sensor 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PowerSweepInitiate (ViSession instrSession,
                                            ViInt32 sense, ViInt32 timeout)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:INIT;*OPC?", sense);
    
    CHECKERR (rssma_waitOPC (instrSession, buffer, buffer, timeout));
    
    return rssma_status; //rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Power Sweep Abort
/* Purpose:  This function aborts the response measurement with NRP power 
/*           sensors.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PowerSweepAbort (ViSession instrSession,
                                         ViInt32 sense)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:ABOR", sense);

    return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Configure Power Sweep Frequency Separate Frequency Range
/* Purpose:  The function configures using of Separate Frequency.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confPowerSweepFrequencySeparateFrequencyRange (ViSession instrSession,
                                                              ViInt32 sense,
                                                              ViBoolean state,
                                                              ViReal64 separateFrequencyStart,
                                                              ViReal64 separateFrequencyStop)
{
    ViStatus rssma_status;
    
    CHECKERR (rssma_SetPowerSweepFrequencySeparateFrequencyRange (instrSession, sense, state));
    CHECKERR (rssma_SetPowerSweepFrequencySeparateFrequencyStart (instrSession, sense, separateFrequencyStart));
    CHECKERR (rssma_SetPowerSweepFrequencySeparateFrequencyStop (instrSession, sense, separateFrequencyStop));

	return rssma_status;    
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Execution
/* Purpose:  This function selects single or continuous mode in power
/*           analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyExecution (ViSession instrSession,
                                                         ViInt32 execution)
{
    return SETPARAMENUM (instrSession,
                          "SENS:SWE:FREQ:RMOD",
                          execution,
                          runModeArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Execution
/* Purpose:  This function returns single or continuous mode in power
/*           analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyExecution (ViSession instrSession,
                                                         ViInt32 *execution)
{
    return GETPARAMENUM (instrSession,
                          "SENS:SWE:FREQ:RMOD?",
                          execution,
                          runModeArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Y Auto Scale
/* Purpose:  This function activates/deactivates autoscaling of the Y axis of
/*           the diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyYAutoScale (ViSession instrSession,
                                                          ViInt32 autoScale)
{
    return SETPARAMENUM (instrSession,
                          "SENS:POW:SWE:FREQ:YSC:AUTO",
                          autoScale,
                          powerFrequencyYScaleAutoArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Y Auto Scale
/* Purpose:  This function returns the state of the autoscaling of the Y axis of
/*           the diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyYAutoScale (ViSession instrSession,
                                                          ViInt32 *autoScale)
{
    return GETPARAMENUM (instrSession,
                          "SENS:POW:SWE:FREQ:YSC:AUTO?",
                          autoScale,
                          powerFrequencyYScaleAutoArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Power Y Auto Scale
/* Purpose:  This function activates/deactivates autoscaling of the Y axis of
/*           the diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerYAutoScale (ViSession instrSession,
                                                      ViInt32 autoScale)
{
    return SETPARAMENUM (instrSession,
                          "SENS:POW:SWE:POW:YSC:AUTO",
                          autoScale,
                          powerFrequencyYScaleAutoArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Power Y Auto Scale
/* Purpose:  This function returns autoscaling of the Y axis of
/*           the diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerYAutoScale (ViSession instrSession,
                                                      ViInt32 *autoScale)
{
    return GETPARAMENUM (instrSession,
                          "SENS:POW:SWE:POW:YSC:AUTO?",
                          autoScale,
                          powerFrequencyYScaleAutoArr);
}

/*===========================================================================*/
/* Function: Reset Power Sweep Power Y Scale
/* Purpose:  This function resets the Y scale to suitable values after the
/*           use of auto scaling in the expanding mode. For this mode, the
/*           scale might get expanded because of temporarily high power
/*           values. The reset function resets the diagram in such a way that
/*           it matches smaller power values again.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_ResetPowerSweepPowerYScale (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SENS:POW:SWE:POW:YSC:AUTO:RES");   
}

/*===========================================================================*/
/* Function: Reset Power Sweep Frequency Y Scale
/* Purpose:  This function resets the Y scale to suitable values after the
/*           use of auto scaling in the expanding mode. For this mode, the
/*           scale might get expanded because of temporarily high power
/*           values. The reset function resets the diagram in such a way that
/*           it matches smaller power values again.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_ResetPowerSweepFrequencyYScale (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SENS:POW:SWE:FREQ:YSC:AUTO:RES");   
}

/*===========================================================================
 * Function: Set Power Sweep Frequency Level Offset State
 * Purpose:  This function  activates/deactivates a level offset at 
 *			 the sensor input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyLevelOffsetState (ViSession instrSession,
                                                                ViInt32 sense,
                                                                ViBoolean offsetState)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28); 
		
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:FREQ:OFFS:STAT", sense);

    return SETPARAMBOOL (instrSession, 
                              buffer,
                              offsetState);    
}

/*===========================================================================*
 * Function: Get Power Sweep Frequency Level Offset State
 * Purpose:  This function returns the state of activation a level offset
 *			 at the sensor input..
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyLevelOffsetState (ViSession instrSession,
                                                                ViInt32 sense,
                                                                ViBoolean *offsetState)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28);  
	
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:FREQ:OFFS:STAT?", sense);

    return GETPARAMBOOL (instrSession, 
                              buffer,
                              offsetState);        
}

/*===========================================================================
 * Function: Set Power Sweep Frequency Level Offset
 * Purpose:  This function  activates/deactivates a level offset at 
 *			 the sensor input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyLevelOffset (ViSession instrSession,
                                                           ViInt32 sense,
                                                           ViReal64 offset)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28); 
		
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:FREQ:OFFS", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              offset,
							  -100.0,100.0);     
}

/*===========================================================================
 * Function: Get Power Sweep Frequency Level Offset
 * Purpose:  This function  returns a level ofset at the sensor input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyLevelOffset (ViSession instrSession,
                                                           ViInt32 sense,
                                                           ViReal64 *offset)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28); 
		
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:FREQ:OFFS?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              offset);     
}

/*===========================================================================*/
/* Function: Configure Power vs Power Sweep Separate Frequency Range
/* Purpose:  The function configures using of Separate Frequency for power
/*           vs. power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confPowerSweepPowerSeparateFrequencyRange (ViSession instrSession,
                                                                     ViInt32 sense,
                                                                     ViBoolean state,
                                                                     ViReal64 separateFrequency)
{
    ViStatus rssma_status;
    
    CHECKERR (rssma_SetPowerSweepPowerSeparateFrequencyState (instrSession, sense, state));
    CHECKERR (rssma_SetPowerSweepPowerSeparateFrequency (instrSession, sense, separateFrequency));
   
    return rssma_status;
}

/*===========================================================================*/
/* Function: Set Power Sweep Power Execution
/* Purpose:  This function selects single or continuous mode in power
/*           analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerExecution (ViSession instrSession,
                                                     ViInt32 execution)
{
    return SETPARAMENUM (instrSession,
                          "SENS:SWE:POW:RMOD",
                          execution,
                          runModeArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Power Execution
/* Purpose:  This function returns single or continuous mode in power
/*           analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerExecution (ViSession instrSession,
                                                     ViInt32 *execution)
{
    return GETPARAMENUM (instrSession,
                          "SENS:SWE:POW:RMOD?",
                          execution,
                          runModeArr);
}

/*===========================================================================
 * Function: Set Power Sweep Power Level Offset State
 * Purpose:  This function  activates/deactivates  a level offset at 
 *           the sensor input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerLevelOffsetState (ViSession instrSession,
                                                            ViInt32 sense,
                                                            ViBoolean offsetState)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28); 
		
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:POW:OFFS:STAT", sense);

    return SETPARAMBOOL (instrSession, 
                              buffer,
                              offsetState);        
}

/*===========================================================================
 * Function: Get Power Sweep Power Level Offset State
 * Purpose:  This function returns the state of activation a level offset 
 *			 at the sensor input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerLevelOffsetState (ViSession instrSession,
                                                            ViInt32 sense,
                                                            ViBoolean *offsetState)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28);  
	
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:POW:OFFS:STAT?", sense);

    return GETPARAMBOOL (instrSession, 
                              buffer,
                              offsetState);    
}

/*===========================================================================
 * Function: Set Power Sweep Power Level Offset
 * Purpose:  This function sets level offset at the sensor input in dB.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerLevelOffset (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViReal64 offset)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28); 
		
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:POW:OFFS", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              offset,
							  -100.0,100.0);       
}

/*===========================================================================
 * Function: Get Power Sweep Power Level Offset
 * Purpose:  This function returns a level ofset at the sensor input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerLevelOffset (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViReal64 *offset)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28); 
		
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:POW:OFFS?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              offset);       
}

/*===========================================================================*/
/* Function: Configure Power vs Time Sweep Separate Frequency Range
/* Purpose:  The function configures using of Separate Frequency for power
/*           vs. time measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confPowerSweepTimeSeparateFrequencyRange (ViSession instrSession,
                                                                    ViInt32 sense,
                                                                    ViBoolean state,
                                                                    ViReal64 separateFrequency)
{
    ViStatus rssma_status;
    
    CHECKERR (rssma_SetPowerSweepTimeSeparateFrequencyState (instrSession, sense, state));
    CHECKERR (rssma_SetPowerSweepTimeSeparateFrequency (instrSession, sense, separateFrequency));
   
    return rssma_status;
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Separate Frequency Range
/* Purpose:  The function activates the use of a frequency range for the
/*           power measurement that is different to the set signal generator
/*           frequency range. The separate frequency range is entered  with
/*           commands SENS:SWE:FREQ:SENS:STAR  and SENS:SWE:FREQ:SENS:STop.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencySeparateFrequencyRange (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViBoolean state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:SENS:SRAN:STAT", sense);
    
    return SETPARAMBOOL (instrSession, 
                               buffer,
                               state);    
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Separate Frequency Range
/* Purpose:  The function displays the use of a frequency range for the
/*           power measurement that is different to the set signal generator
/*           frequency range. The separate frequency range is entered  with
/*           commands SENS:SWE:FREQ:SENS:STAR  and SENS:SWE:FREQ:SENS:STop.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencySeparateFrequencyRange (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViBoolean *state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:SENS:SRAN:STAT?", sense);
    
    return GETPARAMBOOL (instrSession, 
                               buffer,
                               state);    
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Separate Frequency Start
/* Purpose:  This function enters the start frequency for the frequency power
/*           analysis with separate frequencies
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencySeparateFrequencyStart (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViReal64 separateFrequencyStart)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:SENS:SRAN:STAR", sense);
    
    return SETPARAMREAL64 (instrSession, 
                               buffer,
                               separateFrequencyStart,
                               0,
                               22.0e12);
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Separate Frequency Start
/* Purpose:  This function displays the start frequency for the frequency power
/*           analysis with separate frequencies
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencySeparateFrequencyStart (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViReal64 *separateFrequencyStart)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:SENS:SRAN:STAR?", sense);
    
    return GETPARAMREAL64 (instrSession, 
                               buffer,
                               separateFrequencyStart);
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Separate Frequency Stop
/* Purpose:  This function enters the stop frequency for the frequency power
/*           analysis with separate frequencies.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencySeparateFrequencyStop (ViSession instrSession,
                                                            ViInt32 sense,
                                                            ViReal64 separateFrequencyStop)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:SENS:SRAN:STOP", sense);
    
    return SETPARAMREAL64 (instrSession, 
                               buffer,
                               separateFrequencyStop,
                               0,
                               22.0e12);    
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Separate Frequency Stop
/* Purpose:  This function displays the stop frequency for the frequency power
/*           analysis with separate frequencies.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencySeparateFrequencyStop (ViSession instrSession,
                                                            ViInt32 sense,
                                                            ViReal64 *separateFrequencyStop)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:SENS:SRAN:STOP?", sense);
    
    return GETPARAMREAL64 (instrSession, 
                               buffer,
                               separateFrequencyStop);    
}

/*****************************************************************************
 * Function: Power Sweep Frequency Reference Curve
 * Purpose:  This function generates a reference curve for "Frequency" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_PowerSweepFrequencyReferenceCurve(
    ViSession   instrSession,
    ViInt32     sense
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:REF:DATA:COPY", sense);
    
    return rssma_sendCmd (instrSession, buffer);    
}

/*****************************************************************************
 * Function: Get Power Sweep Frequency Reference Curve Points
 * Purpose:  This function reads the number of points from the reference curve 
 *           in "Frequency" measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyReferenceCurvePoints(
    ViSession   instrSession,
    ViInt32     sense,
    ViInt32*    points
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:REF:DATA:POIN?", sense);
    
    return GETPARAMINT32 (instrSession, 
                             buffer,
                             points);    
}

/*****************************************************************************
 * Function: Set Power Sweep Frequency Reference Curve Frequency X
 * Purpose:  This function sets the x values of the two reference points, i.e. 
 *           "Frequency X (Point A)" and "Frequency X (Point B)" in "Frequency" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyReferenceCurveFrequencyX (ViSession instrSession,
                                                                        ViInt32 sense,
                                                                        ViInt32 arraySize,
                                                                        ViReal64 frequencyX[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:REF:DATA:XVAL", sense);
    
    return SETPARAMREAL64ARRAY (instrSession, 
                              buffer,
                              frequencyX,
							  arraySize);    
}

/*****************************************************************************
 * Function: Get Power Sweep Frequency Reference Curve Frequency X
 * Purpose:  This function queries the x values of the two reference points, 
 *           i.e. "Frequency X (Point A)" and "Frequency X (Point B)" in 
 *           "Frequency" measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyReferenceCurveFrequencyX (ViSession instrSession,
                                                                        ViInt32 sense,
                                                                        ViInt32 arraySize,
                                                                        ViReal64 frequencyX[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:REF:DATA:XVAL?", sense);
    
    return GETPARAMREAL64ARRAY (instrSession, 
                                   buffer,
                                   frequencyX,
                                   arraySize);    
}

/*****************************************************************************
 * Function: Set Power Sweep Frequency Reference Curve Power Y
 * Purpose:  This function sets the y values of the two reference points, i.e. 
 *           "Pow Y (Point A)" and "Power Y (Point B)" in "Frequency" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyReferenceCurvePowerY (ViSession instrSession,
                                                                    ViInt32 sense,
                                                                    ViInt32 arraySize,
                                                                    ViReal64 powerY[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:REF:DATA:YVAL", sense);
    
    return SETPARAMREAL64ARRAY (instrSession, 
	                              buffer,
	                              powerY,
								  arraySize);    
}

/*****************************************************************************
 * Function: Get Power Sweep Frequency Reference Curve Power Y
 * Purpose:  This function queries the y values of the two reference points, 
 *           i.e. "Pow Y (Point A)" and "Power Y (Point B)" in "Frequency" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyReferenceCurvePowerY (ViSession instrSession,
                                                                    ViInt32 sense,
                                                                    ViInt32 arraySize,
                                                                    ViReal64 powerY[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:REF:DATA:YVAL?", sense);
    
    return GETPARAMREAL64ARRAY (instrSession, 
	                              buffer,
	                              powerY,
	                              arraySize);    
}

/*===========================================================================*/
/* Function: Set Power vs Power Sweep Separate Frequency
/* Purpose:  This function enters the separate frequency used for power vs.
/*           power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerSeparateFrequency (ViSession instrSession,
                                                               ViInt32 sense,
                                                               ViReal64 separateFrequency)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:SENS:SFR", sense);

    return SETPARAMREAL64 (instrSession, 
                               buffer,
                               separateFrequency,
                               0,
                               22.0e12);
}

/*===========================================================================*/
/* Function: Get Power vs Power Sweep Separate Frequency
/* Purpose:  This function returns the separate frequency used for power vs.
/*           power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerSeparateFrequency (ViSession instrSession,
                                                               ViInt32 sense,
                                                               ViReal64 *separateFrequency)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:SENS:SFR?", sense);

    return GETPARAMREAL64 (instrSession, 
                               buffer,
                               separateFrequency);
}

/*****************************************************************************
 * Function: Power Sweep Power Reference Curve
 * Purpose:  This function generates a reference curve for "Power" 
 *           measurement.
 *           
 *           Remote-control command(s):
 *           SENSe<[1]...3>[:POWer]:SWEep:POWer:REFerence:DATA:COPY
 *****************************************************************************/
ViStatus _VI_FUNC rssma_PowerSweepPowerReferenceCurve(
    ViSession   instrSession,
    ViInt32 sense
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:REF:DATA:COPY", sense);

    return rssma_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Power Sweep Power Reference Curve Points
 * Purpose:  This function reads the number of points from the reference curve 
 *           in "Power" measurement.
 *           
 *           Remote-control command(s):
 *           SENSe<[1]...3>[:POWer]:SWEep:POWer:REFerence:DATA:POINts?
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerReferenceCurvePoints(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32*    points
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:REF:DATA:POIN?", sense);

    return GETPARAMINT32 (instrSession, 
                             buffer,
                             points);
}

/*****************************************************************************
 * Function: Set Power Sweep Power Reference Curve Power X
 * Purpose:  This function sets the x values of the two reference points, i.e. 
 *           "Power X (Point A)" and "Power X (Point B)" in "Power" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerReferenceCurvePowerX (ViSession instrSession,
                                                                ViInt32 sense,
                                                                ViInt32 arraySize,
                                                                ViReal64 powerX[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:REF:DATA:XVAL", sense);

    return SETPARAMREAL64ARRAY (instrSession, 
                                   buffer,
                                   powerX,
								   arraySize);
}

/*****************************************************************************
 * Function: Get Power Sweep Power Reference Curve Power X
 * Purpose:  This function queries the x values of the two reference points, 
 *           i.e. "Power X (Point A)" and "Power X (Point B)" in "Power" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerReferenceCurvePowerX (ViSession instrSession,
                                                                ViInt32 sense,
                                                                ViInt32 arraySize,
                                                                ViReal64 powerX[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:REF:DATA:XVAL?", sense);

    return GETPARAMREAL64ARRAY (instrSession, 
                              	   buffer,
                                   powerX,
                                   arraySize);
}

/*****************************************************************************
 * Function: Set Power Sweep Power Reference Curve Power Y
 * Purpose:  This function sets the y values of the two reference points, i.e. 
 *           "Power Y (Point A)" and "Power Y (Point B)" in "Power" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerReferenceCurvePowerY (ViSession instrSession,
                                                                ViInt32 sense,
                                                                ViInt32 arraySize,
                                                                ViReal64 powerY[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:REF:DATA:YVAL", sense);

    return SETPARAMREAL64ARRAY (instrSession, 
	                              buffer,
	                              powerY,
								  arraySize);
}

/*****************************************************************************
 * Function: Get Power Sweep Power Reference Curve Power Y
 * Purpose:  This function queries the y values of the two reference points, 
 *           i.e. "Power Y (Point A)" and "Power Y (Point B)" in "Power" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerReferenceCurvePowerY (ViSession instrSession,
                                                                ViInt32 sense,
                                                                ViInt32 arraySize,
                                                                ViReal64 powerY[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:REF:DATA:YVAL?", sense);

    return GETPARAMREAL64ARRAY (instrSession, 
	                              buffer,
	                              powerY,
	                              arraySize);
}

/*===========================================================================*/
/* Function: Set Power vs Power Sweep Separate Frequency State
/* Purpose:  The function activates/deactivates the use of a different
/*           frequency for the power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerSeparateFrequencyState (ViSession instrSession,
                                                                    ViInt32 sense,
                                                                    ViBoolean state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:SENS:SFR:STAT", sense);
    
    return SETPARAMBOOL (instrSession, 
                               buffer,
                               state); 
}

/*===========================================================================*/
/* Function: Get Power vs Power Sweep Separate Frequency State
/* Purpose:  The function returns the use of a different
/*           frequency for the power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerSeparateFrequencyState (ViSession instrSession,
                                                                    ViInt32 sense,
                                                                    ViBoolean *state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:SENS:SFR:STAT?", sense);
    
    return GETPARAMBOOL (instrSession, 
                               buffer,
                               state); 
}

/*===========================================================================*/
/* Function: Set Power vs Time Sweep Separate Frequency
/* Purpose:  This function enters the separate frequency used for power vs.
/*           time measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeSeparateFrequency (ViSession instrSession,
                                                              ViInt32 sense,
                                                              ViReal64 separateFrequency)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:SFR", sense);

    return SETPARAMREAL64 (instrSession, 
                               buffer,
                               separateFrequency,
                               0,
                               22.0e12);
}

/*===========================================================================*/
/* Function: Get Power vs Time Sweep Separate Frequency
/* Purpose:  This function returns the separate frequency used for power vs.
/*           time measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeSeparateFrequency (ViSession instrSession,
                                                              ViInt32 sense,
                                                              ViReal64 *separateFrequency)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:SFR?", sense);

    return GETPARAMREAL64 (instrSession, 
                               buffer,
                               separateFrequency);
}

/*****************************************************************************
 * Function: Power Sweep Time Reference Curve
 * Purpose:  This function generates a reference curve for "Time" measurement.
 *           
 *           Remote-control command(s):
 *           SENSe<[1]...3>[:POWer]:SWEep:TIME:REFerence:DATA:COPY
 *****************************************************************************/
ViStatus _VI_FUNC rssma_PowerSweepTimeReferenceCurve(
    ViSession   instrSession,
    ViInt32 sense
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:REF:DATA:COPY", sense);

    return rssma_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Power Sweep Time Reference Curve Points
 * Purpose:  This function reads the number of points from the reference curve 
 *           in "Time" measurement.
 *           
 *           Remote-control command(s):
 *           SENSe<[1]...3>[:POWer]:SWEep:TIME:REFerence:DATA:POINts?
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeReferenceCurvePoints(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32*    points
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:REF:DATA:POIN?", sense);

    return GETPARAMINT32 (instrSession, 
                             buffer,
                             points);
}

/*****************************************************************************
 * Function: Set Power Sweep Time Reference Curve Time X
 * Purpose:  This function sets the x values of the two reference points, i.e. 
 *           "Time X (Point A)" and "Time X (Point B)" in "Time" measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeReferenceCurveTimeX (ViSession instrSession,
                                                              ViInt32 sense,
                                                              ViInt32 arraySize,
                                                              ViReal64 timeX[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:REF:DATA:XVAL", sense);

    return SETPARAMREAL64ARRAY (instrSession, 
	                              buffer,
	                              timeX,
								  arraySize);
}

/*****************************************************************************
 * Function: Get Power Sweep Time Reference Curve Time X
 * Purpose:  This function queries the x values of the two reference points, 
 *           i.e. "Time X (Point A)" and "Time X (Point B)" in "Time" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeReferenceCurveTimeX (ViSession instrSession,
                                                              ViInt32 sense,
                                                              ViInt32 arraySize,
                                                              ViReal64 timeX[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:REF:DATA:XVAL?", sense);

    return GETPARAMREAL64ARRAY (instrSession, 
	                              buffer,
	                              timeX,
	                              arraySize);
}

/*****************************************************************************
 * Function: Set Power Sweep Time Reference Curve Power Y
 * Purpose:  This function sets the y values of the two reference points, i.e. 
 *           "Power Y (Point A)" and "Power Y (Point B)" in "Time" measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeReferenceCurvePowerY (ViSession instrSession,
                                                               ViInt32 sense,
                                                               ViInt32 arraySize,
                                                               ViReal64 powerY[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:REF:DATA:YVAL", sense);

    return SETPARAMREAL64ARRAY (instrSession, 
	                              buffer,
	                              powerY,
								  arraySize);
}

/*****************************************************************************
 * Function: Get Power Sweep Time Reference Curve Power Y
 * Purpose:  This function queries the y values of the two reference points, 
 *           i.e. "Power Y (Point A)" and "Power Y (Point B)" in "Time" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeReferenceCurvePowerY (ViSession instrSession,
                                                               ViInt32 sense,
                                                               ViInt32 arraySize,
                                                               ViReal64 powerY[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:REF:DATA:YVAL?", sense);

    return GETPARAMREAL64ARRAY (instrSession, 
	                              buffer,
	                              powerY,
	                              arraySize);
}

/*===========================================================================*/
/* Function: Set Power vs Time Sweep Separate Frequency State
/* Purpose:  The function activates/deactivates the use of a different
/*           frequency for the power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeSeparateFrequencyState (ViSession instrSession,
                                                                   ViInt32 sense,
                                                                   ViBoolean state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:SFR:STAT", sense);
    
    return SETPARAMBOOL (instrSession, 
                               buffer,
                               state); 
}

/*===========================================================================*/
/* Function: Get Power vs Time Sweep Separate Frequency State
/* Purpose:  The function returns the usage of a different
/*           frequency for the power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeSeparateFrequencyState (ViSession instrSession,
                                                                   ViInt32 sense,
                                                                   ViBoolean *state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:SFR:STAT?", sense);
    
    return GETPARAMBOOL (instrSession, 
                               buffer,
                               state); 
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Start
/* Purpose:  This function enters the start frequency for the frequency 
/*           response measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyStart (ViSession instrSession,
                                                     ViInt32 sense,
                                                     ViReal64 frequencyStart)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;

    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:STAR", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              frequencyStart,
                              1.0e9,22.0e9);
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Start
/* Purpose:  This function returns the start frequency for the frequency 
/*           response measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyStart (ViSession instrSession,
                                                     ViInt32 sense,
                                                     ViReal64 *frequencyStart)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:STAR?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              frequencyStart);
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Stop
/* Purpose:  This function enters the stop frequency for the frequency 
/*           response measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyStop (ViSession instrSession,
                                                    ViInt32 sense,
                                                    ViReal64 frequencyStop)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:STOP", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              frequencyStop,
                              1.0e9,22.0e9);
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Stop
/* Purpose:  This function returns the stop frequency for the frequency 
/*           response measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyStop (ViSession instrSession,
                                                    ViInt32 sense,
                                                    ViReal64 *frequencyStop)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:STOP?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              frequencyStop);
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Timing
/* Purpose:  This function selects the timing mode of the measurement. 
/*           Selection FAST leads to a fast measurement with a short 
/*           integration times for each measurement step, NORMal leads to a 
/*           longer but more precise measurement due to a higher integration 
/*           time for each step.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyTiming (ViSession instrSession,
                                                      ViInt32 sense,
                                                      ViInt32 frequencyTiming)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:TIM:MODE", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          frequencyTiming,
                          powerFrequencyTimingArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Timing
/* Purpose:  This function returns the timing mode of the measurement. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyTiming (ViSession instrSession,
                                                      ViInt32 sense,
                                                      ViInt32 *frequencyTiming)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:TIM:MODE?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          frequencyTiming,
                          powerFrequencyTimingArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Spacing
/* Purpose:  This function selects the spacing for the frequency response 
/*           measurement. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencySpacing (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViInt32 frequencySpacing)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:SPAC:MODE", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          frequencySpacing,
                          powerFrequencySpacingArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Spacing
/* Purpose:  This function returns the spacing for the frequency response 
/*           measurement. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencySpacing (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViInt32 *frequencySpacing)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:SPAC:MODE?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          frequencySpacing,
                          powerFrequencySpacingArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Steps
/* Purpose:  This function enters the number of measurement steps for the 
/*           frequency response measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencySteps (ViSession instrSession,
                                                     ViInt32 sense,
                                                     ViInt32 frequencySteps)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:STEP", sense);

    return SETPARAMINT32 (instrSession, 
                             buffer,
                             frequencySteps,
                             10,1000);
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Steps
/* Purpose:  This function returns the number of measurement steps for the 
/*           frequency response measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencySteps (ViSession instrSession,
                                                     ViInt32 sense,
                                                     ViInt32 *frequencySteps)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:STEP?", sense);

    return GETPARAMINT32 (instrSession, 
                             buffer,
                             frequencySteps);
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Y Scale Minimum
/* Purpose:  This function sets the minimum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyYScaleMinimum (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViReal64 yScaleMinimum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    CHECKVIREAL64 (instrSession, yScaleMinimum, -200.0,100.0);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:YSC:MIN %f", sense, yScaleMinimum);
    
return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Y Scale Minimum
/* Purpose:  This function returns the minimum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyYScaleMinimum (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViReal64 *yScaleMinimum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:YSC:MIN?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              yScaleMinimum);
}

/*===========================================================================*/
/* Function: Set Power Sweep Frequency Y Scale Maximum
/* Purpose:  This function sets the maximum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepFrequencyYScaleMaximum (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViReal64 yScaleMaximum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    CHECKVIREAL64 (instrSession, yScaleMaximum, -200.0, 100.0);
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:YSC:MAX %f", sense, yScaleMaximum);

    return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get Power Sweep Frequency Y Scale Maximum
/* Purpose:  This function returns the maximum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepFrequencyYScaleMaximum (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViReal64 *yScaleMaximum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    sprintf (buffer, "SENS%ld:POW:SWE:FREQ:YSC:MAX?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              yScaleMaximum);
}

/*===========================================================================*/
/* Function: Set Power Sweep Power Start
/* Purpose:  This function sets the start level for the power versus power 
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerStart (ViSession instrSession,
                                                 ViInt32 sense,
                                                 ViReal64 powerStart)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    CHECKVIREAL64 (instrSession, powerStart, -140.0, 30.0);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:STAR %f", sense, powerStart);

    return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get Power Sweep Power Start
/* Purpose:  This function returns the start level for the power versus 
/*           power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerStart (ViSession instrSession,
                                                 ViInt32 sense,
                                                 ViReal64 *powerStart)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:STAR?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              powerStart);
}

/*===========================================================================*/
/* Function: Set Power Sweep Power Stop
/* Purpose:  This function sets the stop level for the power versus power 
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerStop (ViSession instrSession,
                                                ViInt32 sense,
                                                ViReal64 powerStop)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    CHECKVIREAL64 (instrSession, powerStop, -140.0, 30.0);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:STOP %f", sense, powerStop);

    return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get Power Sweep Power Stop
/* Purpose:  This function returns the stop level for the power versus power 
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerStop (ViSession instrSession,
                                                ViInt32 sense,
                                                ViReal64 *powerStop)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:STOP?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              powerStop);
}

/*===========================================================================*/
/* Function: Set Power Sweep Power Spacing
/* Purpose:  This function selects the sweep spacing for the power versus 
/*           power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerSpacing (ViSession instrSession,
                                                   ViInt32 sense,
                                                   ViInt32 powerSpacing)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:SPAC:MODE", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          powerSpacing,
                          powerFrequencySpacingArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Power Spacing
/* Purpose:  This function returns the sweep spacing for the power versus 
/*           power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerSpacing (ViSession instrSession,
                                                   ViInt32 sense,
                                                   ViInt32 *powerSpacing)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:SPAC:MODE?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          powerSpacing,
                          powerFrequencySpacingArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Power Timing
/* Purpose:  This function selects the timing mode of the measurement. 
/*           Selection FAST leads to a fast measurement with a short 
/*           integration times for each measurement step, NORMal leads to a 
/*           longer but more precise measurement due to a higher integration 
/*           time for each step.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerTiming (ViSession instrSession,
                                                  ViInt32 sense,
                                                  ViInt32 powerTiming)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:TIM:MODE", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          powerTiming,
                          powerTimingArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Power Timing
/* Purpose:  This function returns the timing mode of the measurement. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerTiming (ViSession instrSession,
                                                  ViInt32 sense,
                                                  ViInt32 *powerTiming)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:TIM:MODE?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          powerTiming,
                          powerTimingArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Power Steps
/* Purpose:  This function enters the number of measurement steps for the 
/*           power versus power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerSteps (ViSession instrSession,
                                                 ViInt32 sense,
                                                 ViInt32 powerSteps)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:STEP", sense);

    return SETPARAMINT32 (instrSession, 
                             buffer,
                             powerSteps,
                             10,1000);
}

/*===========================================================================*/
/* Function: Get Power Sweep Power Steps
/* Purpose:  This function returns the number of measurement steps for the 
/*           power versus power measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerSteps (ViSession instrSession,
                                                 ViInt32 sense,
                                                 ViInt32 *powerSteps)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:STEP?", sense);

    return GETPARAMINT32 (instrSession, 
                             buffer,
                             powerSteps);
}

/*===========================================================================*/
/* Function: Set Power Sweep Power Y Scale Minimum
/* Purpose:  This function sets the minimum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerYScaleMinimum (ViSession instrSession,
                                                         ViInt32 sense,
                                                         ViReal64 yScaleMinimum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    CHECKVIREAL64 (instrSession, yScaleMinimum, -200.0, 100.0);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:YSC:MIN %f", sense, yScaleMinimum);

    return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get Power Sweep Power Y Scale Minimum
/* Purpose:  This function returns the minimum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerYScaleMinimum (ViSession instrSession,
                                                         ViInt32 sense,
                                                         ViReal64 *yScaleMinimum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:YSC:MIN?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              yScaleMinimum);
}

/*===========================================================================*/
/* Function: Set Power Sweep Power Y Scale Maximum
/* Purpose:  This function sets the maximum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepPowerYScaleMaximum (ViSession instrSession,
                                                         ViInt32 sense,
                                                         ViReal64 yScaleMaximum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    CHECKVIREAL64 (instrSession, yScaleMaximum, -200.0, 100.0);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:YSC:MAX %f", sense, yScaleMaximum);

    return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get Power Sweep Power Y Scale Maximum
/* Purpose:  This function returns the maximum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepPowerYScaleMaximum (ViSession instrSession,
                                                         ViInt32 sense,
                                                         ViReal64 *yScaleMaximum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:POW:YSC:MAX?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              yScaleMaximum);
}

/*===========================================================================*/
/* Function: Get Power Sweep Trace Pulse Data Analysis State 
/* Purpose:  The function queries the state of pulse data analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTracePulseDataAnalysisState (ViSession instrSession,
                                                                  ViInt32 trace,
                                                                  ViBoolean *state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);

    CHECKVIINT32 (instrSession, trace, 1, 3);

    sprintf (buffer, "TRAC%ld:POW:SWE:PULS:STAT?", trace);
                                                       
    return GETPARAMBOOL (instrSession, 
                              buffer,
                              state);
}

/*===========================================================================*/
/* Function: Get Power Sweep Trace Pulse Threshold Base
/* Purpose:  The function queries how the threshold parameters are calculated.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTracePulseThresholdBase (ViSession instrSession,
                                                              ViInt32 trace,
                                                              ViInt32 *base)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 3);

    sprintf (buffer, "TRAC%ld:POW:SWE:PULS:THR:BASE?", trace);
    
    return GETPARAMENUM (instrSession, 
                              buffer,
                              base,
                              pulseThresholdBaseArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Trace Pulse Threshold Reference Level
/* Purpose:  The function returns threshold level.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTracePulseThresholdReferenceLevel (ViSession instrSession,
                                                                        ViInt32 trace,
                                                                        ViInt32 mode,
                                                                        ViReal64 *referenceLevel)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    CHECKVIINT32 (instrSession, mode, 0, 2);

    sprintf (buffer, "TRAC%ld:POW:SWE:PULS:THR:POW:%s?", trace,pulseThresholdModeArr[mode]);
    
    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              referenceLevel);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Y Scale Minimum
/* Purpose:  This function sets the minimum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeYScaleMinimum (ViSession instrSession,
                                                        ViInt32 sense,
                                                        ViReal64 yScaleMinimum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:YSC:MIN", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              yScaleMinimum,
                              -200.0, 100.0);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Y Scale Minimum
/* Purpose:  This function returns the minimum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeYScaleMinimum (ViSession instrSession,
                                                        ViInt32 sense,
                                                        ViReal64 *yScaleMinimum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:YSC:MIN?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              yScaleMinimum);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Y Scale Maximum
/* Purpose:  This function sets the maximum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeYScaleMaximum (ViSession instrSession,
                                                        ViInt32 sense,
                                                        ViReal64 yScaleMaximum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:YSC:MAX", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              yScaleMaximum,
                              -200.0, 100.0);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Y Scale Maximum
/* Purpose:  This function returns the maximum value for the y axis of the 
/*           measurement diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeYScaleMaximum (ViSession instrSession,
                                                        ViInt32 sense,
                                                        ViReal64 *yScaleMaximum)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:YSC:MAX?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              yScaleMaximum);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Execution
/* Purpose:  This function selects single or continuous mode in power
/*           analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeExecution (ViSession instrSession,
                                                    ViInt32 execution)
{
    return SETPARAMENUM (instrSession,
                          "SENS:SWE:TIME:RMOD",
                          execution,
                          runModeArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Execution
/* Purpose:  This function returns single or continuous mode in power
/*           analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeExecution (ViSession instrSession,
                                                    ViInt32 *execution)
{
    return GETPARAMENUM (instrSession,
                          "SENS:SWE:TIME:RMOD?",
                          execution,
                          runModeArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Y Auto Scale
/* Purpose:  This function activates/deactivates autoscaling of the Y axis of
/*           the diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeYAutoScale (ViSession instrSession,
                                                     ViInt32 autoScale)
{
    return SETPARAMENUM (instrSession,
                          "SENS:POW:SWE:TIME:YSC:AUTO",
                          autoScale,
                          powerFrequencyYScaleAutoArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Y Auto Scale
/* Purpose:  This function returns autoscaling of the Y axis of
/*           the diagram.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeYAutoScale (ViSession instrSession,
                                                     ViInt32 *autoScale)
{
    return GETPARAMENUM (instrSession,
                          "SENS:POW:SWE:TIME:YSC:AUTO?",
                          autoScale,
                          powerFrequencyYScaleAutoArr);
}

/*===========================================================================*/
/* Function: Reset Power Sweep Time Y Scale
/* Purpose:  This function resets the Y scale to suitable values after the
/*           use of auto scaling in the expanding mode. For this mode, the
/*           scale might get expanded because of temporarily high power
/*           values. The reset function resets the diagram in such a way that
/*           it matches smaller power values again.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_ResetPowerSweepTimeYScale (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SENS:POW:SWE:TIME:YSC:AUTO:RES");   
}

/*===========================================================================
 * Function: Set Power Sweep Time Level Offset State
 * Purpose:  This function  activates/deactivates  a level offset at 
 *           the sensor input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeLevelOffsetState (ViSession instrSession,
                                                            ViInt32 sense,
                                                            ViBoolean offsetState)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28); 
		
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:TIME:OFFS:STAT", sense);

    return SETPARAMBOOL (instrSession, 
                              buffer,
                              offsetState);        
}

/*===========================================================================
 * Function: Get Power Sweep Time Level Offset State
 * Purpose:  This function returns the state of activation a level offset 
 *			 at the sensor input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeLevelOffsetState (ViSession instrSession,
                                                            ViInt32 sense,
                                                            ViBoolean *offsetState)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28);  
	
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:TIME:OFFS:STAT?", sense);

    return GETPARAMBOOL (instrSession, 
                              buffer,
                              offsetState);    
}

/*===========================================================================
 * Function: Set Power Sweep Time Level Offset
 * Purpose:  This function sets level offset at the sensor input in dB.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeLevelOffset (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViReal64 offset)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28); 
		
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:TIME:OFFS", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              offset,
							  -100.0,100.0);       
}

/*===========================================================================
 * Function: Get Power Sweep Time Level Offset
 * Purpose:  This function returns a level ofset at the sensor input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeLevelOffset (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViReal64 *offset)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT(K28); 
		
    CHECKVIINT32 (instrSession, sense, 1, 4);
    
    sprintf (buffer, ":SENS%ld:SWE:TIME:OFFS?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              offset);       
}

/*===========================================================================*/
/* Function: Set Power Sweep Trace State
/* Purpose:  The function activates/deactivates the selected trace.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTraceState (ViSession instrSession,
                                                 ViInt32 trace, ViBoolean state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "TRAC%ld:POW:SWE:STAT", trace);

    return SETPARAMBOOL (instrSession, 
                              buffer,
                              state);
}

/*===========================================================================*/
/* Function: Get Power Sweep Trace State
/* Purpose:  The function returns the selected trace state.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTraceState (ViSession instrSession,
                                                 ViInt32 trace, ViBoolean *state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "TRAC%ld:POW:SWE:STAT?", trace);

    return GETPARAMBOOL (instrSession, 
                              buffer,
                              state);    
}

/*===========================================================================*/
/* Function: Set Power Sweep Trace Color
/* Purpose:  The function defines the color of each trace.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTraceColor (ViSession instrSession,
                                                 ViInt32 trace, ViInt32 color)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "TRAC%ld:POW:SWE:COL", trace);

    return SETPARAMENUM (instrSession, 
                              buffer,
                              color,
                              traceColorArr);    
}

/*===========================================================================*/
/* Function: Get Power Sweep Trace Color
/* Purpose:  The function returns the color of each trace.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTraceColor (ViSession instrSession,
                                                 ViInt32 trace, ViInt32 *color)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "TRAC%ld:POW:SWE:COL?", trace);

    return GETPARAMENUM (instrSession, 
                              buffer,
                              color,
                              traceColorArr);    
}

/*===========================================================================*/
/* Function: Copy Power Sweep Trace
/* Purpose:  The function stores the selected trace data as reference trace.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_copyPowerSweepTrace (ViSession instrSession,
                                             ViInt32 trace, ViInt32 copyMode)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "TRAC%ld:POW:SWE:COPY", trace);

    return SETPARAMENUM (instrSession, 
                              buffer,
                              copyMode,
                              traceCopyArr);    
}

/*===========================================================================*/
/* Function: Get Power Sweep Trace Data Points
/* Purpose:  This function queries the number of measurement points of the
/*           selected trace of the current power analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTraceDataPoints (ViSession instrSession,
                                                      ViInt32 trace,
                                                      ViInt32 *dataPoints)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "TRAC%ld:POW:SWE:DATA:POIN?", trace);

    return GETPARAMINT32 (instrSession, 
                             buffer,
                             dataPoints);
}

/*===========================================================================*/
/* Function: Get Power Sweep Trace X Values
/* Purpose:  This function queries the x-axis values - frequency, power or
/*           time values -  of the selected trace of the current power
/*           analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTraceXValues (ViSession instrSession,
                                                   ViInt32 trace,
                                                   ViInt32 arraySize,
                                                   ViReal64 xValues[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "TRAC%ld:POW:SWE:DATA:XVAL?", trace);

    return GETPARAMREAL64ARRAY (instrSession,
                                    buffer,
                                   xValues,
                                   arraySize);
}

/*===========================================================================*/
/* Function: Get Power Sweep Trace Y Values
/* Purpose:  This function queries the measurement (y-axis) values of the
/*           selected trace of the current power analysis.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTraceYValues (ViSession instrSession,
                                                   ViInt32 trace,
                                                   ViInt32 arraySize,
                                                   ViReal64 yValues[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "TRAC%ld:POW:SWE:DATA:YVAL?", trace);

    return GETPARAMREAL64ARRAY (instrSession,
                                   buffer,
                                   yValues,
                                   arraySize);
}

/*===========================================================================*/
/* Function: Set Power Sweep Trace Source
/* Purpose:  The function defines the source for the data trace.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTraceSource (ViSession instrSession,
                                                  ViInt32 trace, ViInt32 source)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "TRAC%ld:POW:SWE:FEED", trace);

    return SETPARAMENUM (instrSession, 
                              buffer,
                              source,
                              traceSourceArr); 
}

/*===========================================================================*/
/* Function: Get Power Sweep Trace Source
/* Purpose:  The function returns the source for the data trace.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTraceSource (ViSession instrSession,
                                                  ViInt32 trace, ViInt32 *source)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, trace, 1, 3);
    
    sprintf (buffer, "TRAC%ld:POW:SWE:FEED?", trace);

    return GETPARAMENUM (instrSession, 
                              buffer,
                              source,
                              traceSourceArr); 
}

/*===========================================================================*/
/* Function: Set Power Sweep Sense Mode
/* Purpose:  This function selects single or contimuous mode for response 
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepSenseMode (ViSession instrSession,
                                                ViInt32 sense, ViInt32 senseMode)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:RMOD", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          senseMode,
                          senseModeArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Sense Mode
/* Purpose:  This function returns single or contimuous mode for response 
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepSenseMode (ViSession instrSession,
                                                ViInt32 sense,
                                                ViInt32 *senseMode)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:RMOD?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          senseMode,
                          senseModeArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Mode
/* Purpose:  This function selects power versus frequency measurement 
/*           (frequency response) or power vs power measurment (power sweep, 
/*           AM/AM) 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepMode (ViSession instrSession,
                                           ViInt32 sense, ViInt32 sweepMode)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:MODE", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          sweepMode,
                          sweepModeArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Mode
/* Purpose:  This function returns power versus frequency measurement 
/*           (frequency response) or power vs power measurment (power sweep, 
/*           AM/AM) 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepMode (ViSession instrSession,
                                           ViInt32 sense, ViInt32 *sweepMode)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:MODE?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          sweepMode,
                          sweepModeArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Hardcopy LNG Format
/* Purpose:  This function selects the bitmap graphic format for the 
/*           screenshot of the power analysis trace.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyLNGFormat (ViSession instrSession,
                                                        ViInt32 sense,
                                                        ViInt32 sweepHardcopyFormat)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:HCOP:DEV:LANG", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          sweepHardcopyFormat,
                          sweepHardcopyFormatArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Hardcopy LNG Format
/* Purpose:  This function returns the bitmap graphic format for the 
/*           screenshot of the power analysis trace.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyLNGFormat (ViSession instrSession,
                                                        ViInt32 sense,
                                                        ViInt32 *sweepHardcopyFormat)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:HCOP:DEV:LANG?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          sweepHardcopyFormat,
                          sweepHardcopyFormatArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Hardcopy File
/* Purpose:  This function stores the Hardcopy in the selected output format 
/*           into a file.
/*           
/*           Example:
/*            SENS:SWE:HCOP:DEV:LANG BMP
/*            selects output format *,bmp.
/*           
/*            MMEM:DIR 'var/nrp_sweep'
/*            sets the directory 
/*           
/*            SENS:SWE:HCOP:FILE 'trace1'
/*            triggers a hardcopy of the current trace and stores it in the  
/*            
/*            file trace1.bmp in the set directory. The measurement diagram 
/*            has to be visible in the foreground. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyFile (ViSession instrSession,
                                                   ViString fileName)
{
    INIT;
    
    return SETPARAMSTRINGQUOTED (instrSession, 
                              "SENS:POW:SWE:HCOP:FILE",
                              "'",
                              fileName);
}

/*===========================================================================*/
/* Function: Get Power Sweep Hardcopy File
/* Purpose:  This function returns name of the Hardcopy file.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyFile (ViSession instrSession,
                                                   ViChar fileName[])
{
    INIT;
    
    return GETPARAMSTRING (instrSession, 
                              "SENS:POW:SWE:HCOP:FILE?",
                              fileName,
                              1024);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Pulse Data Analysis State
/* Purpose:  The function activates/deactivates pulse data analysis. The
/*           measurement is started with command INITiate.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimePulseDataAnalysisState (ViSession instrSession,
                                                                 ViInt32 sense,
                                                                 ViBoolean state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);

    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:PULS:STAT", sense);
    
    return SETPARAMBOOL (instrSession, 
                              buffer,
                              state);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Pulse Data Analysis State
/* Purpose:  The function returns pulse data analysis state.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimePulseDataAnalysisState (ViSession instrSession,
                                                                 ViInt32 sense,
                                                                 ViBoolean *state)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:PULS:STAT?", sense);
    
    return GETPARAMBOOL (instrSession, 
                              buffer,
                              state);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Pulse Threshold Base
/* Purpose:  The function selects how the threshold parameters for pulse
/*           analysis are calculated.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimePulseThresholdBase (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViInt32 base)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:PULS:THR:BASE", sense);
    
    return SETPARAMENUM (instrSession, 
                              buffer,
                              base,
                              pulseThresholdBaseArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Pulse Threshold Base
/* Purpose:  The function returns how the threshold parameters for pulse
/*           analysis are calculated.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimePulseThresholdBase (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViInt32 *base)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:PULS:THR:BASE?", sense);
    
    return GETPARAMENUM (instrSession, 
                              buffer,
                              base,
                              pulseThresholdBaseArr);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Pulse Threshold Reference Level
/* Purpose:  The function selects reference level in terms of percentage of
/*           the overall pulse level
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimePulseThresholdReferenceLevel (ViSession instrSession,
                                                                       ViInt32 sense,
                                                                       ViInt32 mode,
                                                                       ViReal64 referenceLevel)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    CHECKVIINT32 (instrSession, mode, 0, 2);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:PULS:THR:POW:%s", sense,pulseThresholdModeArr[mode]);
    
    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              referenceLevel,
                              0,100);    
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Pulse Threshold Reference Level
/* Purpose:  The function returns reference level in terms of percentage of
/*           the overall pulse level
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimePulseThresholdReferenceLevel (ViSession instrSession,
                                                                       ViInt32 sense,
                                                                       ViInt32 mode,
                                                                       ViReal64 *referenceLevel)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    CHECKVIINT32 (instrSession, mode, 0, 2);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:PULS:THR:POW:%s?", sense,pulseThresholdModeArr[mode]);
    
    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              referenceLevel);    
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Trigger Auto Set Time Sweep
/* Purpose:  The function determines and sets the trigger level. The
/*           hysteresis and drop out time are not influenced.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerAutoSet (ViSession instrSession,
                                                         ViInt32 sense,
                                                         ViInt32 timeout)
{
    ViChar      buffer[BUFFER_SIZE];
    INIT;   
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:TRIG:AUTO ONCE;*OPC?", sense);
   
    CHECKERR (rssma_waitOPC (instrSession, buffer, buffer, timeout));
    
    return rssma_status; //return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Trigger Drop Out Time
/* Purpose:  This function determines the minimum time for which the signal
/*           must be below (above) the power level defined by level and
/*           hysteresis before triggering can occur again.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerDropOutTime (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViReal64 dropOutTime)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;   
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:TRIG:DTIM", sense);
   
    return SETPARAMREAL64 (instrSession, 
                          buffer,
                          dropOutTime,
                          0,10);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Trigger Drop Out Time
/* Purpose:  This function returns the minimum time for which the signal must
/*           be below (above) the power level defined by level and hysteresis
/*           before triggering can occur again.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerDropOutTime (ViSession instrSession,
                                                             ViInt32 sense,
                                                             ViReal64 *dropOutTime)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;   
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:TRIG:DTIM?", sense);
   
    return GETPARAMREAL64 (instrSession, 
                          buffer,
                          dropOutTime);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Trigger Hysteresis
/* Purpose:  This function sets the hysteresis of the internal trigger
/*           threshold. Hysteresis is the magnitude (in dB) the trigger
/*           signal level must drop below the trigger threshold (positive
/*           trigger slope) before triggering can occur again.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerHysteresis (ViSession instrSession,
                                                            ViInt32 sense,
                                                            ViReal64 hysteresis)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;   
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:TRIG:HYST", sense);
   
    return SETPARAMREAL64 (instrSession, 
                          buffer,
                          hysteresis,
                          -10,10);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Trigger Hysteresis
/* Purpose:  This function returns the hysteresis of the internal trigger
/*           threshold. Hysteresis is the magnitude (in dB) the trigger
/*           signal level must drop below the trigger threshold (positive
/*           trigger slope) before triggering can occur again.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerHysteresis (ViSession instrSession,
                                                            ViInt32 sense,
                                                            ViReal64 *hysteresis)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;   
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:TRIG:HYST?", sense);
   
    return GETPARAMREAL64 (instrSession, 
                          buffer,
                          hysteresis);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Trigger Threshold
/* Purpose:  This function sets the trigger threshold.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerThreshold (ViSession instrSession,
                                                           ViInt32 sense,
                                                           ViReal64 threshold)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;   
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:TRIG:LEV", sense);
   
    return SETPARAMREAL64 (instrSession, 
                          buffer,
                          threshold,
                          -200,100);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Trigger Threshold
/* Purpose:  This function returns the trigger threshold.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerThreshold (ViSession instrSession,
                                                           ViInt32 sense,
                                                           ViReal64 *threshold)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;   
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:TRIG:LEV?", sense);
   
    return GETPARAMREAL64 (instrSession, 
                          buffer,
                          threshold);
}

/*****************************************************************************
 * Function: Set Power Sweep Time Trigger Events
 * Purpose:  This function sets the measurement data processing starts with a 
 *           trigger event in one of the sensors (Logical OR), or whether all 
 *           channels have to be triggered (logical AND). Each sensor evaluates 
 *           a trigger event according to its setting independently. This 
 *           function supports the internal or external trigger modes with 
 *           multi-channel time measurements.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *           
 *           Remote-control command(s):
 *           SENSe<[1]...3>[:POWer]:SWEep:TIME:TEVents
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerEvents(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32 events
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:TEV", sense);
    
    return SETPARAMENUM (instrSession, 
                          buffer,
                          events,
                          eventsArr);
}

/*****************************************************************************
 * Function: Get Power Sweep Time Trigger Events
 * Purpose:  This function returns the measurement data processing starts with 
 *           a trigger event in one of the sensors (Logical OR), or whether all 
 *           channels have to be triggered (logical AND).
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *           
 *           Remote-control command(s):
 *           SENSe<[1]...3>[:POWer]:SWEep:TIME:TEVents?
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerEvents(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32*    events
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, sense, 1, 3);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:TEV?", sense);
    
    return GETPARAMENUM (instrSession, 
                          buffer,
                          events,
                          eventsArr);
}

/*****************************************************************************
 * Function: Set Power Sweep Time Trigger Slope
 * Purpose:  This function sets the polarity of the active slope for the 
 *			 trigger signals.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *           
 *           Remote-control command(s):
 *           :SENSe<ch>[:POWer]:SWEep:TIME[:SENSor]:TRIGger:SLOPe
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerSlope (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViInt32 slope)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, sense, 1, 4);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:TRIG:SLOP", sense);
    
    return SETPARAMENUM (instrSession, 
                          buffer,
                          slope,
                          slopeArr);    
}

/*****************************************************************************
 * Function: Get Power Sweep Time Trigger Slope
 * Purpose:  This function returns the polarity of the active slope for the 
 *			 trigger signals.
 *           
 *           Required Option(s):
 *           R&S SMA-K28
 *           
 *           Remote-control command(s):
 *           :SENSe<ch>[:POWer]:SWEep:TIME[:SENSor]:TRIGger:SLOPe?
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerSlope (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViInt32 *slope)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, sense, 1, 4);

    sprintf (buffer, "SENS%ld:POW:SWE:TIME:TRIG:SLOP", sense);
    
    return GETPARAMENUM (instrSession, 
                          buffer,
                          slope,
                          slopeArr);        
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Start
/* Purpose:  This function enters the start time for the power versus time 
/*           measurement. Value 0 defines the trigger point. By choosing a 
/*           negative time value, the trace can be shifted in the diagram. 
/*           It is possible, that the measurement cannot be performed over 
/*           the complete time range because of limitations due to sensor 
/*           settings. In this case, an error message is output.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeStart (ViSession instrSession,
                                                ViInt32 sense,
                                                ViReal64 timeStart)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:STAR", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              timeStart,
                              -1.0,1.0);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Start
/* Purpose:  This function returns the start time for the power versus time 
/*           measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeStart (ViSession instrSession,
                                                ViInt32 sense,
                                                ViReal64 *timeStart)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:STAR?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              timeStart);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Stop
/* Purpose:  This function enters the stop time for the power versus time
/*           measurement
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeStop (ViSession instrSession,
                                               ViInt32 sense, ViReal64 timeStop)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:STOP", sense);

    return SETPARAMREAL64 (instrSession, 
                              buffer,
                              timeStop,
                              0.0,2.0);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Stop
/* Purpose:  This function returns the stop time for the power versus time
/*           measurement
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeStop (ViSession instrSession,
                                               ViInt32 sense, ViReal64 *timeStop)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:STOP?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              timeStop);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Trigger Event
/* Purpose:  This function determines, whether the measurement data
/*           processing starts with a trigger event in one of the sensors
/*           (Logical OR), or whether all channels have to be triggered
/*           (logical AND). Each sensor evaluates a trigger event according
/*           to its setting independently. This function supports the
/*           internal or external trigger modes with multi-channel time
/*           measurements.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerEvent (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViInt32 triggerEvents)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:TEV", sense);

    return SETPARAMENUM (instrSession, 
                             buffer,
                             triggerEvents,
                             eventsArr);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Trigger Event *N
/* Purpose:  This function returns the measurement data processing starts
/*           with a trigger event in one of the sensors (Logical OR), or
/*           whether all channels have to be triggered (logical AND).
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerEvent (ViSession instrSession,
                                                       ViInt32 sense,
                                                       ViInt32 *triggerEvents)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:TEV?", sense);

    return GETPARAMENUM (instrSession, 
                             buffer,
                             triggerEvents,
							 eventsArr);
    
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Step
/* Purpose:  This function enters the number of measurement steps for the 
/*           power versus time measurement. Value 0 defines the trigger point. 
/*           It is possible, that the measurement cannot be performed over the 
/*           complete time range because of limitations due to sensor settings. 
/*           In this case, an error message is output.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeSteps (ViSession instrSession,
                                                ViInt32 sense, ViInt32 timeSteps)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:STEP", sense);

    return SETPARAMINT32 (instrSession, 
                             buffer,
                             timeSteps,
                             10,1000);
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Step
/* Purpose:  This function returns the number of measurement steps for the 
/*           power versus time measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeSteps (ViSession instrSession,
                                                ViInt32 sense,
                                                ViInt32 *timeSteps)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:STEP?", sense);

    return GETPARAMINT32 (instrSession, 
                             buffer,
                             timeSteps);
}

/*===========================================================================*/
/* Function: Set Power Sweep Time Spacing
/* Purpose:  This function selects the sweep spacing for the power versus
/*           time measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeSpacing (ViSession instrSession,
                                                  ViInt32 sense,
                                                  ViInt32 powerSpacing)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SPAC:MODE", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          powerSpacing,
                          powerFrequencySpacingArr);
    
}

/*===========================================================================*/
/* Function: Get Power Sweep Time Spacing
/* Purpose:  This function returns the sweep spacing for the power versus
/*           time measurement.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeSpacing (ViSession instrSession,
                                                  ViInt32 sense,
                                                  ViInt32 *powerSpacing)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SPAC:MODE?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          powerSpacing,
                          powerFrequencySpacingArr);
    
}

/*===========================================================================
 * Function: Set Power Sweep Time Averaging Factor
 * Purpose:  Selects the averaging factor in time mode. The count
 *           number determines how many measurement cycles are used to form a
 *           measurement result. Higher averaging counts reduce noise but
 *           increase the measurement time. Averaging requires a stable
 *           trigger event so that the measurement cycles have the same
 *           timing.
 *           
 *           Required Option(s):
 *           R&S SMA-K28 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeAveragingFactor (ViSession instrSession,
                                                     ViInt32 averagingFactor)
{
    INIT;
    
    CHECKOPT (K28);
    
    return SETPARAMINT32 (instrSession,
                             "SENS:POW:SWE:TIME:AVER:COUN",
                             averagingFactor,
                             1, 1024);
}

/*===========================================================================
 * Function: Get Power Sweep Time Averaging Factor
 * Purpose:  Queriess the averaging factor in time mode.
 *           
 *           Required Option(s):
 *           R&S SMA-K28 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeAveragingFactor (ViSession instrSession,
                                                     ViInt32 *averagingFactor)
{
    INIT;
    
    CHECKOPT (K28);
    
    return GETPARAMINT32 (instrSession,
                             "SENS:POW:SWE:TIME:AVER:COUN?",
                             averagingFactor);
}

/*===========================================================================
 * Function: Set Power Sweep Time Trigger Source
 * Purpose:  The function selects if the measurement is free running or
 *           starts only after a trigger event. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepTimeTriggerSource (ViSession instrSession,
                                                   ViInt32 sense, ViInt32 source)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:TRIG:SOUR", sense);
    
    return SETPARAMENUM (instrSession,
                          buffer,
                          source,
                          powerTimeTriggerSourceArr);
}

/*===========================================================================
 * Function: Get Power Sweep Time Trigger Source
 * Purpose:  The function returns if the measurement is free running or starts 
 *           only after a trigger event. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepTimeTriggerSource (ViSession instrSession,
                                                   ViInt32 sense,
                                                   ViInt32 *source)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SWE:TIME:SENS:TRIG:SOUR?", sense);
    
    return GETPARAMENUM (instrSession,
                          buffer,
                          source,
                          powerTimeTriggerSourceArr);
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy CSV Data
 * Purpose:  This function queries the measurement data directly. The data is
 *           transferred to the remote client as data stream, e.g. for
 *           further processing.
 *           Readable ASCII data is available for hardcopy language CSV. The
 *           representation of the values depends on the selected orientation
 *           for the CSV format.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyCSVData (ViSession instrSession,
                                                   ViString destination)
{
    return rssma_readDataToFile (instrSession, "SENS:POW:SWE:HCOP:DATA?", destination); 
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy Device
 * Purpose:  This function selects the output device. The setting is fixed to
 *           FILE, i.e. the  hardcopy is stored in a file.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyDevice (ViSession instrSession,
                                                     ViInt32 hardcopyDevice)
{
    INIT;
    return SETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV",
                          hardcopyDevice,
                          hcopDeviceArr);    
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Device
 * Purpose:  This function returns the output device. The setting is fixed to
 *           FILE, i.e. the  hardcopy is stored in a file.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyDevice (ViSession instrSession,
                                                     ViInt32 *hardcopyDevice)
{
    INIT;
    return GETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV?",
                          hardcopyDevice,
                          hcopDeviceArr);
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy CSV Decimal Point
 * Purpose:  This function defines which character should be used as the
 *           decimal point of the values, either dot or comma.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyCSVDecimalPoint (ViSession instrSession,
                                                              ViInt32 CSVDecimalPoint)
{
    INIT;
    return SETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV:LANG:CSV:DPO",
                          CSVDecimalPoint,
                          dexcDecimalSeparatorArr);    
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy CSV Decimal Point 
 * Purpose:  This function returns which character should be used as the
 *           decimal point of the values, either dot or comma.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyCSVDecimalPoint (ViSession instrSession,
                                                              ViInt32 *CSVDecimalPoint)
{
    INIT;
    return GETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV:LANG:CSV:DPO?",
                          CSVDecimalPoint,
                          dexcDecimalSeparatorArr);    
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy CSV Decimal Header
 * Purpose:  This function defines whether each row (or column depending on
 *           the orientation) should be preceded by a header containing
 *           information about the trace (see also
 *           'SENSe[:POWer]:SWEep:HCOPy:DATA?').
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyCSVHeader (ViSession instrSession,
                                                               ViInt32 CSVHeader)
{
    INIT;
    return SETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV:LANG:CSV:HEAD",
                          CSVHeader,
                          hcopHeaderArr);    
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy CSV Decimal Header
 * Purpose:  This function returns whether each row (or column depending on
 *           the orientation) should be preceded by a header containing
 *           information about the trace (see also
 *           'SENSe[:POWer]:SWEep:HCOPy:DATA?').
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyCSVHeader (ViSession instrSession,
                                                               ViInt32 *CSVHeader)
{
    INIT;
    return GETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV:LANG:CSV:HEAD?",
                          CSVHeader,
                          hcopHeaderArr);    
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy CSV Orientation
 * Purpose:  This function defines the orientation of the X/Y value pairs.
 *           For examples on how the data are arranged see , section CSV
 *           Options Hardkey.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyCSVOrientation (ViSession instrSession,
                                                             ViInt32 CSVOrientation)
{
   INIT;
   return SETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV:LANG:CSV:ORI",
                          CSVOrientation,
                          hcopOrientationArr);    
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy CSV Orientation
 * Purpose:  This function returns the orientation of the X/Y value pairs.
 *           For examples on how the data are arranged see , section CSV
 *           Options Hardkey.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyCSVOrientation (ViSession instrSession,
                                                             ViInt32 *CSVOrientation)
{
    INIT;
    return GETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV:LANG:CSV:ORI?",
                          CSVOrientation,
                          hcopOrientationArr);    
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy CSV Column Separator
 * Purpose:  This function defines which character is to separate the values,
 *           either tabulator, semicolon, comma or blank.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyCSVColumnSeparator (ViSession instrSession,
                                                                 ViInt32 CSVColumnSeparator)
{
    INIT;
    return SETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV:LANG:CSV:COL:SEP",
                          CSVColumnSeparator,
                          hcopColumnSeparatorArr);    
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy CSV Column Separator
 * Purpose:  This function returns which character is to separate the values,
 *           either tabulator, semicolon, comma or blank.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyCSVColumnSeparator (ViSession instrSession,
                                                                 ViInt32 *CSVColumnSeparator)
{
    INIT;
    return GETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV:LANG:CSV:COL:SEP?",
                          CSVColumnSeparator,
                          hcopColumnSeparatorArr);    
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy Image Size
 * Purpose:  This function sets the size of the hardcopy in number of pixels.
 *           The first value of the size setting defines the width, the
 *           second value the height of the image.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyImageSize (ViSession instrSession,
                                                        ViInt32 imageSize)
{
    INIT;
    return SETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV:SIZE",
                          imageSize,
                          hcopImageSizeArr);    
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Image Size
 * Purpose:  This function returns the size of the hardcopy in number of pixels.
 *           The first value of the size setting defines the width, the
 *           second value the height of the image.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyImageSize (ViSession instrSession,
                                                        ViInt32 *imageSize)
{
    INIT;
    return GETPARAMENUM (instrSession,
                          "SENS:POW:SWE:HCOP:DEV:SIZE?",
                          imageSize,
                          hcopImageSizeArr);    
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy Automatic Directory
 * Purpose:  This function defines the directory into which the hardcopy
 *           files will be stored if auto naming is activated
 *           (SENS:SWE:HCOP:FILE:AUTO:STAT ON).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyAutomaticDirectory (ViSession instrSession,
                                                                 ViString directoryName)
{
    INIT;
    
    return SETPARAMSTRINGQUOTED (instrSession,
                              "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:DIR",
                              "'",
                              directoryName);
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Automatic Directory 
 * Purpose:  This function returns the directory into which the hardcopy
 *           files will be stored if auto naming is activated
 *           (SENS:SWE:HCOP:FILE:AUTO:STAT ON).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticDirectory (ViSession instrSession,
                                                                 ViChar _VI_FAR directoryName[])
{
    INIT;
    
    return GETPARAMSTRING (instrSession,
                              "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:DIR?",
                              directoryName,
                              1204);
}

/*===========================================================================
 * Function: Delete Power Sweep Hardcopy Image Files 
 * Purpose:  This function deletes all files with extensions "bmp", "img",
 *           "png", "xpm" and "csv" in the directory set for automatic
 *           naming.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_DeletePowerSweepHardcopyImageFiles (ViSession instrSession)
{
    INIT;

    return rssma_sendCmd (instrSession,
                    "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:DIR:CLE");
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy Automatic Naming State 
 * Purpose:  This function activates/deactivates automatic naming of the
 *           hardcopy files.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyAutomaticNamingState (ViSession instrSession,
                                                                   ViBoolean state)
{
    INIT;
    return SETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:STAT",
                               state);
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Automatic Naming State 
 * Purpose:  This function returns automatic naming of the hardcopy files.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticNamingState (ViSession instrSession,
                                                                   ViBoolean *state)
{
    INIT;
    return GETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:STAT?",
                               state);
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Automatic File Name 
 * Purpose:  This function returns the file name that what generated using
 *           the automatic naming settings.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFileName (ViSession instrSession,
                                                                ViChar fileName[])
{
    INIT;
    return GETPARAMSTRING (instrSession,
                              "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE?",
                              fileName,
                              1024);
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy Automatic File Date State 
 * Purpose:  This function activates/deactivates the usage of the
 *           day/month/year in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyAutomaticFileDateState (ViSession instrSession,
                                                                     ViInt32 fileNamePart,
                                                                     ViBoolean state)
{
    INIT;
    
    CHECKVIINT32 (instrSession, fileNamePart, 0, 2);
    
    switch (fileNamePart){
        case RSSMA_HCOPY_AUTO_FILE_DAY:
            rssma_status = SETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:DAY:STAT",
                               state);        
        break;
        case RSSMA_HCOPY_AUTO_FILE_MONTH:
            rssma_status = SETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:MONTH:STAT",
                               state);
        break;                         
        case RSSMA_HCOPY_AUTO_FILE_YEAR:
            rssma_status = SETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:YEAR:STAT",
                               state);
        break;
    }
    
    return rssma_status;
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Automatic File Date State 
 * Purpose:  This function returns the usage of the
 *           day/month/year in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFileDateState (ViSession instrSession,
                                                                     ViInt32 fileNamePart,
                                                                     ViBoolean *state)
{
    INIT;

    CHECKVIINT32 (instrSession, fileNamePart, 0, 2);
    
    switch (fileNamePart){
        case RSSMA_HCOPY_AUTO_FILE_DAY:
            rssma_status = GETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:DAY:STAT?",
                               state);        
        break;
        case RSSMA_HCOPY_AUTO_FILE_MONTH:
            rssma_status = GETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:MONTH:STAT?",
                               state);
        break;                         
        case RSSMA_HCOPY_AUTO_FILE_YEAR:
            rssma_status = GETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:YEAR:STAT?",
                               state);
        break;
    }
    return rssma_status;
   
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Automatic File Date 
 * Purpose:  This function queries the day/month/year part in the automatic
 *           file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFileDate (ViSession instrSession,
                                                                ViInt32 fileNamePart,
                                                                ViChar part[])
{
    INIT;
    
    CHECKVIINT32 (instrSession, fileNamePart, 0, 2);
    
    switch (fileNamePart){
        case RSSMA_HCOPY_AUTO_FILE_DAY:
            rssma_status = GETPARAMSTRING (instrSession,
                              "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:DAY?",
                              part,
                              1024);         
        break;
        case RSSMA_HCOPY_AUTO_FILE_MONTH:
            rssma_status = GETPARAMSTRING (instrSession,
                              "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:MONTH?",
                              part,
                              1024);  
        break;                         
        case RSSMA_HCOPY_AUTO_FILE_YEAR:
            rssma_status = GETPARAMSTRING (instrSession,
                              "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:YEAR?",
                              part,
                              1024);  
        break;
    } 
    return rssma_status; 
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Automatic File Number 
 * Purpose:  This function  queries the generated number in the automatic
 *           file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFileNumber (ViSession instrSession,
                                                                  ViInt32 *number)
{
    INIT;
    return GETPARAMINT32 (instrSession,
                              "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:NUMB?",
                              number);        
}

/*****************************************************************************
 * Function: Set Power Math Frequency State
 * Purpose:  The function sets the trace mathematics mode for "Frequency" 
 *           measurement. This feature enables to calculate the difference 
 *           between the measurement values of two traces. Additionally, for 
 *           further calculation a math result can also be assigned to a trace.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerMathFrequencyState(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean   state
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 4);
    
    sprintf (buffer, "CALC:POW:SWE:FREQ:MATH%ld:STAT", trace);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               state);
}

/*****************************************************************************
 * Function: Get Power Math Frequency State
 * Purpose:  The function returns the trace mathematics mode for "Power" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerMathFrequencyState(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean*  state
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 4);
    
    sprintf (buffer, "CALC:POW:SWE:FREQ:MATH%ld:STAT?", trace);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               state);
}

/*****************************************************************************
 * Function: Power Math Frequency Subtract
 * Purpose:  The function executes the subtraction of the operands 1 and 2 and 
 *           assigns the result to the selected trace in "Frequency" 
 *           measurement mode.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_PowerMathFrequencySubtract(
    ViSession   instrSession,
    ViInt32 trace,
    ViInt32 subtract
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 4);
    CHECKVIINT32 (instrSession, subtract, 0, 19);
    
    sprintf (buffer, "CALC:POW:SWE:FREQ:MATH%ld:SUBT %s", trace, subtractArr[subtract]);

    return rssma_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Set Power Math Power State
 * Purpose:  The function sets the trace mathematics mode for "Power" 
 *           measurement. This feature enables to calculate the difference 
 *           between the measurement values of two traces. Additionally, for 
 *           further calculation a math result can also be assigned to a trace.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerMathPowerState(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean   state
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 4);
    
    sprintf (buffer, "CALC:POW:SWE:POW:MATH%ld:STAT", trace);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               state);
}

/*****************************************************************************
 * Function: Get Power Math Power State
 * Purpose:  The function returns the trace mathematics mode for "Frequency" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerMathPowerState(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean*  state
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 4);
    
    sprintf (buffer, "CALC:POW:SWE:POW:MATH%ld:STAT?", trace);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               state);
}

/*****************************************************************************
 * Function: Power Math Power Subtract
 * Purpose:  The function executes the subtraction of the operands 1 and 2 and 
 *           assigns the result to the selected trace in "Power" measurement 
 *           mode.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_PowerMathPowerSubtract(
    ViSession   instrSession,
    ViInt32 trace,
    ViInt32 subtract
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 4);
    CHECKVIINT32 (instrSession, subtract, 0, 19);
    
    sprintf (buffer, "CALC:POW:SWE:POW:MATH%ld:SUBT %s", trace, subtractArr[subtract]);

    return rssma_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Set Power Math Time State
 * Purpose:  The function sets the trace mathematics mode for "Time" 
 *           measurement. This feature enables to calculate the difference 
 *           between the measurement values of two traces. Additionally, for 
 *           further calculation a math result can also be assigned to a
 *           trace.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetPowerMathTimeState(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean   state
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 4);
    
    sprintf (buffer, "CALC:POW:SWE:TIME:MATH%ld:STAT", trace);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               state);
}

/*****************************************************************************
 * Function: Get Power Math Time State
 * Purpose:  The function returns the trace mathematics mode for "Time" 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetPowerMathTimeState(
    ViSession   instrSession,
    ViInt32 trace,
    ViBoolean*  state
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 4);
    
    sprintf (buffer, "CALC:POW:SWE:TIME:MATH%ld:STAT?", trace);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               state);
}

/*****************************************************************************
 * Function: Power Math Time Subtract
 * Purpose:  The function executes the subtraction of the operands 1 and 2 and 
 *           assigns the result to the selected trace in "Time" measurement 
 *           mode.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_PowerMathTimeSubtract(
    ViSession   instrSession,
    ViInt32 trace,
    ViInt32 subtract
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    CHECKOPT (K28);
    
    CHECKVIINT32 (instrSession, trace, 1, 4);
    CHECKVIINT32 (instrSession, subtract, 0, 19);
    
    sprintf (buffer, "CALC:POW:SWE:TIME:MATH%ld:SUBT %s", trace, subtractArr[subtract]);

    return rssma_sendCmd (instrSession, buffer);
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy Automatic File Prefix 
 * Purpose:  This function defines the prefix part in the automatic file
 *           name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyAutomaticFilePrefix (ViSession instrSession,
                                                                  ViString prefix)
{
    INIT;
    return SETPARAMSTRINGQUOTED (instrSession,
                              "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:PREF",
                              "'",
                              prefix); 
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Automatic File Prefix 
 * Purpose:  This function returns the prefix part in the automatic file
 *           name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFilePrefix (ViSession instrSession,
                                                                  ViChar prefix[])
{
    INIT;
    return GETPARAMSTRING (instrSession,
                              "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:FILE:PREF?",
                              prefix,
                              1024); 
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy Automatic File Prefix State 
 * Purpose:  This function activates/deactivates the usage of the prefix in
 *           the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyAutomaticFilePrefixState (ViSession instrSession,
                                                                       ViBoolean state)
{
    INIT;
    return SETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:PREF:STAT",
                               state);
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Automatic File Prefix State 
 * Purpose:  This function returns the usage of the prefix in
 *           the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyAutomaticFilePrefixState (ViSession instrSession,
                                                                       ViBoolean *state)
{
    INIT;
    return GETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:FILE:NAME:AUTO:PREF:STAT?",
                               state);
}

/*===========================================================================
 * Function: Set Power Sweep Hardcopy Marker 
 * Purpose:  This function selects if the markers in the measurement diagram
 *           and the marker list below the diagram are also indicated in the
 *           hardcopy file.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerSweepHardcopyMarkerState  (ViSession instrSession,
                                                     ViBoolean state)
{
    INIT;
    return SETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:ITEM:WIND:STAT:MARK:STAT",
                               state);
}

/*===========================================================================
 * Function: Get Power Sweep Hardcopy Marker 
 * Purpose:  This function returns if the markers in the measurement diagram
 *           and the marker list below the diagram are also indicated in the
 *           hardcopy file.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSweepHardcopyMarkerState  (ViSession instrSession,
                                                     ViBoolean *state)
{
    INIT;
    return GETPARAMBOOL (instrSession,
                               "SENS:POW:SWE:HCOP:ITEM:WIND:STAT:MARK:STAT?",
                               state);
}

/*===========================================================================
 * Function: Save Power Sweep Hardcopy 
 * Purpose:  This function triggers the generation of a hardcopy of the
 *           current measurement diagram.. The data is written into the file
 *           selected/created with the SENS:SWE:HCOP:FILE command.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SavePowerSweepHardcopy (ViSession instrSession)
{
    INIT;
    return rssma_sendCmd (instrSession,"SENS:POW:SWE:HCOP:EXEC");
}

/*===========================================================================
 * Function: Set Power Display Indication
 * Purpose:  The function switches the indication of the selected type on or
 *           off.
 *           
 *           Required Option(s):
 *           R&S SMA-K28 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerDisplayIndication (ViSession instrSession,
                                                   ViInt32 trace, ViInt32 type,
                                                   ViBoolean indication)
{
    ViChar buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPT (K28);
    
    switch (type)
    {
        case RSSMA_SENSE_POWER_DISP_PULS_ALL:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:ALL:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_STAN:
            sprintf (buffer, "TRAC:POW:SWE:MEAS:STAN:DISP:ANN:STAT");
        break;
        case RSSMA_SENSE_POWER_DISP_MARK:
            sprintf (buffer, "TRAC:POW:SWE:MEAS:MARK:DISP:ANN:STAT");
        break;
        case RSSMA_SENSE_POWER_DISP_GATE:
            sprintf (buffer, "TRAC:POW:SWE:MEAS:GATE:DISP:ANN:STAT");
        break;
        case RSSMA_SENSE_POWER_DISP_PULS:
            sprintf (buffer, "TRAC:POW:SWE:MEAS:PULS:DISP:ANN:STAT");
        break;
        case RSSMA_SENSE_POWER_DISP_FULL_SCR:
            sprintf (buffer, "TRAC:POW:SWE:MEAS:FULL:DISP:ANN:STAT");
        break;

        case RSSMA_SENSE_POWER_DISP_POWER_AVER:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:AVER:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_HREF:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:HREF:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_REF:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:REF:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_LREF:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:LREF:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_MAX:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:MAX:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_MIN:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:MIN:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_PULSE_BASE:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:PULS:BASE:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_PULSE_TOP:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:PULS:TOP:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_DCYCLE :
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:DCYC:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_DUR:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:DUR:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_PERIOD:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:PER:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_SEP:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:SEP:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_NEG_DUR:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:NEG:DUR:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_NEG_OCC:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:NEG:OCC:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_NEG_OVER :
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:NEG:OVER:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_POS_DUR:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:POS:DUR:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_POS_OCC:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:POS:OCC:DISP:ANN:STAT", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_POS_OVER:      
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:POS:OVER:DISP:ANN:STAT", trace);
        break;
        
        default:
        	return rssma_ParameterError (instrSession, "type");
    }
    
    return SETPARAMBOOL (instrSession,
                               buffer,
                               indication);
}

/*===========================================================================
 * Function: Get Power Display Indication
 * Purpose:  The function returns the state of the indication of the selected
 *           type.
 *           
 *           Required Option(s):
 *           R&S SMA-K28 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerDisplayIndication (ViSession instrSession,
                                                   ViInt32 trace, ViInt32 type,
                                                   ViBoolean *indication)
{
    ViChar buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPT (K28);
    
    switch (type)
    {
        case RSSMA_SENSE_POWER_DISP_PULS_ALL:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:ALL:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_STAN:
            sprintf (buffer, "TRAC:POW:SWE:MEAS:STAN:DISP:ANN:STAT?");
        break;
        case RSSMA_SENSE_POWER_DISP_MARK:
            sprintf (buffer, "TRAC:POW:SWE:MEAS:MARK:DISP:ANN:STAT?");
        break;
        case RSSMA_SENSE_POWER_DISP_GATE:
            sprintf (buffer, "TRAC:POW:SWE:MEAS:GATE:DISP:ANN:STAT?");
        break;
        case RSSMA_SENSE_POWER_DISP_PULS:
            sprintf (buffer, "TRAC:POW:SWE:MEAS:PULS:DISP:ANN:STAT?");
        break;
        case RSSMA_SENSE_POWER_DISP_FULL_SCR:
            sprintf (buffer, "TRAC:POW:SWE:MEAS:FULL:DISP:ANN:STAT?");
        break;

        case RSSMA_SENSE_POWER_DISP_POWER_AVER:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:AVER:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_HREF:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:HREF:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_REF:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:REF:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_LREF:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:LREF:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_MAX:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:MAX:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_MIN:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:MIN:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_PULSE_BASE:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:PULS:BASE:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_PULSE_TOP:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:PULS:TOP:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_DCYCLE :
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:DCYC:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_DUR:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:DUR:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_PERIOD:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:PER:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_SEP:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:SEP:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_NEG_DUR:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:NEG:DUR:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_NEG_OCC:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:NEG:OCC:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_NEG_OVER :
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:NEG:OVER:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_POS_DUR:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:POS:DUR:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_POS_OCC:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:POS:OCC:DISP:ANN:STAT?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_POS_OVER:      
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:POS:OVER:DISP:ANN:STAT?", trace);
        break;
        default:
	        return rssma_ParameterError (instrSession, "type");
    }
    
    return GETPARAMBOOL (instrSession,
                               buffer,
                               indication);
}

/*===========================================================================*/
/* Function: Get Power Display Pulse Data Analysis
/* Purpose:  The function returns the measured pulse parameters values.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerDisplayPulseDataAnalysis (ViSession instrSession,
                                                          ViInt32 trace,
                                                          ViInt32 type,
                                                          ViReal64 *pulseDataAnalysis)
{
        ViChar buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPT (K28);
    
    switch (type)
    {
        
        case RSSMA_SENSE_POWER_DISP_POWER_AVER:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:AVER?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_HREF:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:HREF?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_REF:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:REF?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_LREF:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:LREF?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_MAX:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:MAX?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_MIN:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:MIN?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_PULSE_BASE:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:PULS:BASE?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_POWER_PULSE_TOP:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:POW:PULS:TOP?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_DCYCLE :
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:DCYC?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_DUR:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:DUR?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_PERIOD:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:PER?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_PULSE_SEP:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:PULS:SEP?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_NEG_DUR:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:NEG:DUR?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_NEG_OCC:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:NEG:OCC?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_NEG_OVER :
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:NEG:OVER?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_POS_DUR:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:POS:DUR?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_POS_OCC:
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:POS:OCC?", trace);
        break;
        case RSSMA_SENSE_POWER_DISP_TRAN_POS_OVER:      
            sprintf (buffer, "TRAC%ld:POW:SWE:MEAS:TRAN:POS:OVER?", trace);
        break;
        default:
        	return rssma_ParameterError (instrSession, "type");
    }
    
    return GETPARAMREAL64 (instrSession,
                               buffer,
                               pulseDataAnalysis);
}

/*===========================================================================*/
/* Function: Power Sensor Device
/* Purpose:  This function queries if a sensor is connected at the selected 
/*           port to the signal generator. The sensor is selected by suffix 
/*           1, 2 or 3 in key word SENSe or READ of the command header. 
/*           Suffix 1 denotes the sensor connected to the SENSOR connector, 
/*           suffix 2 the sensor connected first to one of the USB interfaces 
/*           and suffix 3 the sensor connected second to one of the USB 
/*           interfaces.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PowerSensorDevice (ViSession instrSession,
                                           ViInt32 sense, ViInt32 *device)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:STAT:DEV?", sense);

    return GETPARAMINT32 (instrSession, 
                             buffer,
                             device);
}

/*===========================================================================*/
/* Function: Power Sensor Type
/* Purpose:  This function queries the type of sensor connected at the 
/*           selected port. The type is automatically detected.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PowerSensorType (ViSession instrSession,
                                         ViInt32 sense, ViChar type[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:TYPE?", sense);

    return GETPARAMSTRING (instrSession, 
                              buffer,
                              type,
                              1024);
}

/*===========================================================================*/
/* Function: Power Read Power
/* Purpose:  This function retrieves the power measurement result of the 
/*           selected power sensor. The value is provided with the unit set 
/*           with command SENSe<[1]...3>:UNIT[:POWer]. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PowerReadPower (ViSession instrSession, ViInt32 read,
                                        ViReal64 *power)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, read, 1, 3);
    
    sprintf (buffer, "READ%ld:POW?", read);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              power);
}

/*===========================================================================*/
/* Function: Power Sensor Version
/* Purpose:  This function queries the software version of the connected NRP 
/*           power sensor.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerSensorVersion (ViSession instrSession,
                                               ViInt32 sense, ViChar version[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SVER?", sense);

    return GETPARAMSTRING (instrSession, 
                              buffer,
                              version,
                              1024);
}

/*===========================================================================*/
/* Function: Power Sensor Serial Number
/* Purpose:  This function queries the serial number of the sensor. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PowerSensorSerialNumber (ViSession instrSession,
                                                 ViInt32 sense,
                                                 ViChar serialNumber[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32 (instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SNUM?", sense);

    return GETPARAMSTRING (instrSession, 
                              buffer,
                              serialNumber,
                              1024);
}

/*===========================================================================*/
/* Function: Configure Diagram Display
/* Purpose:  This function configures the diagram display. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confDiagramDisplay (ViSession instrSession,
                                           ViBoolean grid,
                                           ViInt32 backgroundColor)
{
    ViStatus rssma_status;
    
    CHECKERR (rssma_SetDiagramGrid (instrSession, grid));
    CHECKERR (rssma_SetDiagramBackgroundColor (instrSession, backgroundColor));
   
    return rssma_status;
}

/*===========================================================================*/
/* Function: Set Diagram Grid
/* Purpose:  This function activates/deactivates the indication of a grid in
/*           the diagram area.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDiagramGrid (ViSession instrSession,
                                        ViBoolean grid)
{
    INIT;
    return SETPARAMBOOL (instrSession, 
                               "DISP:WIND:POW:SWE:GRID:STAT",
                               grid);
}

/*===========================================================================*/
/* Function: Get Diagram Grid
/* Purpose:  This function returns the indication of a grid in the diagram
/*           area.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDiagramGrid (ViSession instrSession,
                                        ViBoolean *grid)
{
    INIT;
    return GETPARAMBOOL (instrSession, 
                               "DISP:WIND:POW:SWE:GRID:STAT?",
                               grid);
}

/*===========================================================================*/
/* Function: Set Diagram Background Color
/* Purpose:  This function defines the background color of the measurement
/*           diagram. The background color is also effective for the hardcopy
/*           of the diagram
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDiagramBackgroundColor (ViSession instrSession,
                                                   ViInt32 backgroundColor)
{
    INIT;
    return SETPARAMENUM (instrSession, 
                          "DISP:WIND:POW:SWE:BACK:COL",
                          backgroundColor,
                          powerSensorDisplayColorArr);
}

/*===========================================================================*/
/* Function: Get Diagram Background Color
/* Purpose:  This function returns the background color of the measurement
/*           diagram. The background color is also effective for the hardcopy
/*           of the diagram
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDiagramBackgroundColor (ViSession instrSession,
                                                   ViInt32 *backgroundColor)
{
    INIT;
    return GETPARAMENUM (instrSession, 
                          "DISP:WIND:POW:SWE:BACK:COL?",
                          backgroundColor,
                          powerSensorDisplayColorArr);
}

/*============================================================================
 * Function: rssma_confFrequencySweep
 * Purpose:  This function sets the frequency parameters of sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confFrequencySweep (ViSession instrSession,
                                            ViInt32 mode,
                                            ViInt32 frequencySetting,
                                            ViReal64 centerFrequency,
                                            ViReal64 span,
                                            ViReal64 startFrequency,
                                            ViReal64 stopFrequency,
                                            ViInt32 spacing, ViReal64 step,
                                            ViReal64 dwellTime)
{
    ViStatus    rssma_status;

    CHECKVIINT32 (instrSession, frequencySetting, 0, 1);

    CHECKERR (rssma_SetFrequencySweepMode (instrSession, mode));

    if (frequencySetting == RSSMA_FREQ_CENTER_SPAN)
    {
        CHECKERR (rssma_SetFrequencySweepCenterFreq (instrSession, centerFrequency));
        CHECKERR (rssma_SetFrequencySweepSpan (instrSession, span));
	}
    else
    {
        CHECKERR (rssma_SetFrequencySweepStartFreq (instrSession, startFrequency));
		CHECKERR (rssma_SetFrequencySweepStopFreq (instrSession, stopFrequency));
	}

    CHECKERR (rssma_SetFrequencySweepSpacing (instrSession, spacing));
    if (spacing == RSSMA_FREQ_SWEEP_SPACING_LIN)
    {
        CHECKERR (rssma_SetFrequencySweepStepLin (instrSession, step));
	}
    else
    {
        CHECKERR (rssma_SetFrequencySweepStepLog (instrSession, step));
    }

    CHECKERR (rssma_SetFrequencySweepDwellTime (instrSession, dwellTime));

    RETURN;
}

/*============================================================================
 * Function: rssma_FrequencySweepExecute
 * Purpose:  This function triggers a sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_FrequencySweepExecute (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SOUR:SWE:FREQ:EXEC");
}

/*============================================================================
 * Function: rssma_SetFrequencySweepMode
 * Purpose:  This function sets the sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepMode (ViSession instrSession,
                                               ViInt32 mode)
{
    INIT; 
    
    switch (mode)
    {
    case RSSMA_SWEEP_MODE_AUTO:
        CHECKERR (rssma_SetSweepMode (instrSession, RSSMA_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssma_SetFrequencySweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_AUTO));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_SINGLE:
        CHECKERR (rssma_SetSweepMode (instrSession, RSSMA_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssma_SetFrequencySweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_SINGLE));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_SWEEP));
        CHECKERR (rssma_FrequencySweepExecute (instrSession));
        break;
    case RSSMA_SWEEP_MODE_STEP:
        CHECKERR (rssma_SetSweepMode (instrSession, RSSMA_SWEEP_LOW_MODE_MAN));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_EXT_SINGLE:
        CHECKERR (rssma_SetSweepMode (instrSession, RSSMA_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssma_SetFrequencySweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_EXT_STEP:
        CHECKERR (rssma_SetSweepMode (instrSession, RSSMA_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssma_SetFrequencySweepSpacing (instrSession, RSSMA_FREQ_SWEEP_SPACING_LIN));
        CHECKERR (rssma_SetFrequencySweepStepLin (instrSession, 1.0e6));
        CHECKERR (rssma_SetFrequencySweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_EXT_START:
        CHECKERR (rssma_SetSweepMode (instrSession, RSSMA_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssma_SetFrequencySweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_EAUTO));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_SWEEP));
        break;
    default:
        rssma_status = rssma_ParameterError (instrSession, "mode");
    }

    RETURN;
}

/*============================================================================
 * Function: rssma_SetFrequencySweepStartFreq
 * Purpose:  This function sets the start frequency for the sweep mode. STARt
 *           can be greater than STOP.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepStartFreq (ViSession instrSession,
                                                    ViReal64 startFrequency)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STAR",
                              startFrequency,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetFrequencySweepStartFreq
 * Purpose:  This function returns the start frequency of the sweep mode.
 *           STARt can be greater than STOP.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepStartFreq (ViSession instrSession,
                                                    ViReal64 *startFrequency)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STAR?",
                              startFrequency);
}

/*============================================================================
 * Function: rssma_SetFrequencySweepStopFreq
 * Purpose:  This function sets the stop frequency for the sweep mode. STOP
 *           can be less than STARt.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepStopFreq (ViSession instrSession,
                                                   ViReal64 stopFrequency)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STOP",
                              stopFrequency,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetFrequencySweepStopFreq
 * Purpose:  This function returns the stop frequency of the sweep mode. STOP
 *           can be less than STARt.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepStopFreq (ViSession instrSession,
                                                   ViReal64 *stopFrequency)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STOP?",
                              stopFrequency);
}

/*============================================================================
 * Function: rssma_SetFrequencySweepCenterFreq
 * Purpose:  This function sets the center frequency of the sweep. This
 *           setting in combination with the span setting
 *           ([:SOURce]:FREQuency:SPAN) defines the sweep range.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepCenterFreq (ViSession instrSession,
                                                     ViReal64 centerFrequency)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:CENT",
                              centerFrequency,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetFrequencySweepCenterFreq
 * Purpose:  This function returns the center frequency of the sweep. This
 *           setting in combination with the span setting
 *           ([:SOURce]:FREQuency:SPAN) defines the sweep range.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepCenterFreq (ViSession instrSession,
                                                     ViReal64 *centerFrequency)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:CENT?",
                              centerFrequency);
}

/*============================================================================
 * Function: rssma_SetFrequencySweepSpan
 * Purpose:  This function specifies the span for the sweep. This setting in
 *           combination with the center frequency setting ([:SOUR]:FREQ:CENT)
 *           defines the sweep range.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepSpan (ViSession instrSession, 
                                               ViReal64 span)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:SPAN",
                              span,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetFrequencySweepSpan
 * Purpose:  This function returns the span of the sweep. This setting in
 *           combination with the center frequency setting ([:SOUR]:FREQ:CENT)
 *           defines the sweep range.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepSpan (ViSession instrSession, 
                                               ViReal64 *span)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:SPAN?",
                              span);
}

/*============================================================================
 * Function: rssma_SetFrequencySweepSpacing
 * Purpose:  This function selects linear or logarithmic sweep spacing.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepSpacing (ViSession instrSession,
                                                  ViInt32 spacing)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:SWE:SPAC",
                          spacing,
                          spacingArr);
}

/*============================================================================
 * Function: rssma_GetFrequencySweepSpacing
 * Purpose:  This function returns linear or logarithmic sweep spacing.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepSpacing (ViSession instrSession,
                                                  ViInt32 *spacing)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:SWE:SPAC?",
                          spacing,
                          spacingArr);
}

/*============================================================================
 * Function: rssma_SetFrequencySweepStepLin
 * Purpose:  This function sets the step width for linear sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepStepLin (ViSession instrSession,
                                                  ViReal64 stepLin)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:STEP",
                              stepLin,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetFrequencySweepStepLin
 * Purpose:  This function returns the step width for linear sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepStepLin (ViSession instrSession,
                                                  ViReal64 *stepLin)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:STEP?",
                              stepLin);
}

/*============================================================================
 * Function: rssma_SetFrequencySweepStepLog
 * Purpose:  This function specifies the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepStepLog (ViSession instrSession,
                                                  ViReal64 stepLog)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:STEP:LOG",
                              stepLog,
                              0.01, 100.0);
}

/*============================================================================
 * Function: rssma_GetFrequencySweepStepLog
 * Purpose:  This function returns the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepStepLog (ViSession instrSession,
                                                  ViReal64 *stepLog)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:STEP:LOG?",
                              stepLog);
}

/*============================================================================
 * Function: rssma_SetFrequencySweepDwellTime
 * Purpose:  This function sets the time taken for each frequency step of the
 *           sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepDwellTime (ViSession instrSession,
                                                    ViReal64 dwellTime)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:DWEL",
                              dwellTime,
                              3.0e-3, 10.0);
}

/*============================================================================
 * Function: rssma_GetFrequencySweepDwellTime
 * Purpose:  This function returns the time taken for each frequency step of
 *           the sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepDwellTime (ViSession instrSession,
                                                    ViReal64 *dwellTime)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:DWEL?",
                              dwellTime);
}


/*============================================================================
 * Function: rssma_SetFrequencySweepManualStep
 * Purpose:  In Sweep mode (:SOUR:FREQ:MODE SWE) this function sets the
 *           frequency for the next sweep step in the Step sweep mode
 *           (:SOUR:SWE:MODE MAN). Here only frequency values between the
 *           settings [:SOUR]:FREQ:STAR and ...:STOP are permitted. Each sweep
 *           step is triggered by a separate :SOUR:FREQ:MAN command.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepManualStep (ViSession instrSession,
                                                     ViReal64 manualStep)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:MAN",
                              manualStep,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetFrequencySweepManualStep
 * Purpose:  In Sweep mode (:SOUR:FREQ:MODE SWE) this function returns the
 *           frequency for the next sweep step in the Step sweep mode
 *           (:SOUR:SWE:MODE MAN). Here only frequency values between the
 *           settings [:SOUR]:FREQ:STAR and ...:STOP are permitted. Each sweep
 *           step is triggered by a separate :SOUR:FREQ:MAN command.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepManualStep (ViSession instrSession,
                                                     ViReal64 *manualStep)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:MAN?",
                              manualStep);
}

/*============================================================================
 * Function: rssma_SetFrequencySweepPoints
 * Purpose:  This function sets the number of steps in an RF sweep. This
 *           function is linked to the command :SWEep[:FREQuency]:STEP as
 *           follows:
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepPoints (ViSession instrSession,
                                                 ViInt32 frequencySweepPoints)
{
    return SETPARAMINT32 (instrSession,
                             "SOUR:SWE:POIN",
                             frequencySweepPoints,
                             0, 0);
}

/*============================================================================
 * Function: rssma_GetFrequencySweepPoints
 * Purpose:  This function returns the number of steps in an RF sweep. This
 *           function is linked to the command :SWEep[:FREQuency]:STEP as
 *           follows:
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepPoints (ViSession instrSession,
                                                 ViInt32 *frequencySweepPoints)
{
    return GETPARAMINT32 (instrSession,
                             "SOUR:SWE:POIN?",
                             frequencySweepPoints);
}

/*============================================================================
 * Function: rssma_confRFLevelSweep
 * Purpose:  This function sets the RF sweep parameters and changes the mode
 *           of RF level to Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confRFLevelSweep (ViSession instrSession,
                                          ViInt32 levelSweepMode,
                                          ViReal64 startLevel,
                                          ViReal64 stopLevel, ViReal64 stepLin,
                                          ViReal64 dwellTime)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetLevelSweepMode (instrSession, levelSweepMode));

    CHECKERR (rssma_SetLevelSweepStartLevel (instrSession, startLevel));
    CHECKERR (rssma_SetLevelSweepStopLevel (instrSession, stopLevel));
    CHECKERR (rssma_SetLevelSweepStep (instrSession, stepLin));
    CHECKERR (rssma_SetLevelSweepDwell (instrSession, dwellTime));
    RETURN;
}

/*============================================================================
 * Function: rssma_LevelSweepExecute
 * Purpose:  This function triggers a sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_LevelSweepExecute (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SOUR:SWE:POW:EXEC");
}

/*============================================================================
 * Function: rssma_SetLevelSweepMode
 * Purpose:  This function sets the sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLevelSweepMode (ViSession instrSession,
                                           ViInt32 mode)
{
    INIT;
    
    switch (mode)
    {
    case RSSMA_SWEEP_MODE_AUTO:
        CHECKERR (rssma_SetSweepLevelMode (instrSession, RSSMA_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssma_SetLevelSweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_AUTO));
        CHECKERR (rssma_SetLevelMode (instrSession, RSSMA_POWER_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_SINGLE:
        CHECKERR (rssma_SetSweepLevelMode (instrSession, RSSMA_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssma_SetLevelSweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_SINGLE));
        CHECKERR (rssma_SetLevelMode (instrSession, RSSMA_POWER_MODE_SWEEP));
        CHECKERR (rssma_LevelSweepExecute (instrSession));
        break;
    case RSSMA_SWEEP_MODE_STEP:
        CHECKERR (rssma_SetSweepLevelMode (instrSession, RSSMA_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssma_SetLevelSweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_SINGLE));
        CHECKERR (rssma_SetLevelMode (instrSession, RSSMA_POWER_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_EXT_SINGLE:
        CHECKERR (rssma_SetSweepLevelMode (instrSession, RSSMA_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssma_SetLevelSweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssma_SetLevelMode (instrSession, RSSMA_POWER_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_EXT_STEP:
        CHECKERR (rssma_SetSweepLevelMode (instrSession, RSSMA_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssma_SetLevelSweepStep (instrSession, 0.5));
        CHECKERR (rssma_SetLevelSweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssma_SetLevelMode (instrSession, RSSMA_POWER_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_EXT_START:
        CHECKERR (rssma_SetSweepLevelMode (instrSession, RSSMA_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssma_SetLevelSweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_EAUTO));
        CHECKERR (rssma_SetLevelMode (instrSession, RSSMA_POWER_MODE_SWEEP));
        break;
    default:
        rssma_status = rssma_ParameterError (instrSession, "mode");
    }

    RETURN;
}

/*============================================================================
 * Function: rssma_SetLevelSweepStartLevel
 * Purpose:  This function sets the RF start level in Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLevelSweepStartLevel (ViSession instrSession,
                                                 ViReal64 startLevel)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:STAR",
                              startLevel,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetLevelSweepStartLevel
 * Purpose:  This function returns the RF start level in Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLevelSweepStartLevel (ViSession instrSession,
                                                 ViReal64 *startLevel)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:STAR?",
                              startLevel);
}

/*============================================================================
 * Function: rssma_SetLevelSweepStopLevel
 * Purpose:  This function sets the stop level in Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLevelSweepStopLevel (ViSession instrSession,
                                                ViReal64 stopLevel)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:STOP",
                              stopLevel,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetLevelSweepStopLevel
 * Purpose:  This function returns the stop level in Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLevelSweepStopLevel (ViSession instrSession, 
                                                ViReal64 *stopLevel)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:STOP?",
                              stopLevel);
}

/*============================================================================
 * Function: rssma_SetLevelSweepStep
 * Purpose:  This function sets the step width factor for logarithmic sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLevelSweepStep (ViSession instrSession,
                                           ViReal64 step)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:POW:STEP",
                              step,
                              0.1, 20.0);
}

/*============================================================================
 * Function: rssma_GetLevelSweepStep
 * Purpose:  This function returns the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLevelSweepStep (ViSession instrSession, 
                                           ViReal64 *step)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:POW:STEP?",
                              step);
}

/*============================================================================
 * Function: rssma_SetLevelSweepDwell
 * Purpose:  This function sets the time taken for each level step of the
 *           sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLevelSweepDwell (ViSession instrSession, 
                                            ViReal64 dwellTime)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:POW:DWEL",
                              dwellTime,
                              2E-3, 10.0);
}

/*============================================================================
 * Function: rssma_GetLevelSweepDwell
 * Purpose:  This function returns the time taken for each level step of the
 *           sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLevelSweepDwell (ViSession instrSession,
                                            ViReal64 *dwellTime)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:POW:DWEL?",
                              dwellTime);
}

/*============================================================================
 * Function: rssma_SetLevelSweepManualStep
 * Purpose:  In Sweep mode (:SOUR:POW:MODE SWE) this function sets the level
 *           for the next sweep step in the Step sweep mode (:SOUR:SWE:POW:MODE
 *           MAN). Here only level values between the settings [:SOUR]:POW:STAR
 *           and ...:STOP are permitted. Each sweep step is triggered by a
 *           separate :SOUR:POW:MAN command.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLevelSweepManualStep (ViSession instrSession, 
                                                 ViReal64 manualStep)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:MAN",
                              manualStep,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetLevelSweepManualStep
 * Purpose:  This function returns the level of the next sweep step in the
 *           Step sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLevelSweepManualStep (ViSession instrSession, 
                                                 ViReal64 *manualStep)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:MAN?",
                              manualStep);
}

/*============================================================================
 * Function: rssma_SetLevelSweepPoints
 * Purpose:  This function sets the number of steps in a level sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLevelSweepPoints (ViSession instrSession, 
                                             ViInt32 levelSweepPoints)
{
    return SETPARAMINT32 (instrSession,
                             "SOUR:SWE:POW:POIN",
                             levelSweepPoints,
                             0, 0);
}

/*============================================================================
 * Function: rssma_GetLevelSweepPoints
 * Purpose:  This function returns the number of steps in a level sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLevelSweepPoints (ViSession instrSession, 
                                             ViInt32 *levelSweepPoints)
{
    return GETPARAMINT32 (instrSession,
                             "SOUR:SWE:POW:POIN?",
                             levelSweepPoints);
}

/*============================================================================
 * Function: rssma_confLFFreq
 * Purpose:  This function sets parameters of LF Output.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confLFFreqSweep (ViSession instrSession,
                                         ViInt32 mode, ViReal64 startFreq,
                                         ViReal64 stopFreq, ViInt32 spacing,
                                         ViReal64 step, ViReal64 dwellTime)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetLFSweepMode (instrSession, mode));
    CHECKERR (rssma_SetLFSweepStartFreq (instrSession, startFreq));    CHECKERR (rssma_SetLFSweepStopFreq (instrSession, stopFreq));
    CHECKERR (rssma_SetLFSweepSpacing (instrSession, spacing));
    if (spacing == RSSMA_LFOUTPUT_SWEEP_FREQUENCY_SPACING_LINEAR)
    {
        CHECKERR (rssma_SetLFSweepStepLin (instrSession, step));    }
    else
    {
        CHECKERR (rssma_SetLFSweepStepLog (instrSession, step));    }

    CHECKERR (rssma_SetLFSweepDwellTime (instrSession, dwellTime));
    RETURN;
}

/*============================================================================
 * Function: rssma_LFOutputLFSweepTrigger
 * Purpose:  This function immediately starts an LF sweep. The MODE setting
 *           determines which sweep is executed, e.g.
 *           :SOURce:LFOutput:SWEep:FREQuency:MODE STEP. This function
 *           corresponds to the manual-control command EXECUTE SINGLE SWEEP.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_LFSweepTrigger(ViSession    instrSession)
{
    return rssma_sendCmd (instrSession, "LFO:SWE:EXEC");
}

/*============================================================================
 * Function: rssma_SetLFSweepMode
 * Purpose:  This function sets the sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepMode (ViSession instrSession, ViInt32 mode)
{
    ViStatus    rssma_status;

    switch (mode)
    {
    case RSSMA_SWEEP_MODE_AUTO:
        CHECKERR (rssma_SetSweepLFMode (instrSession, RSSMA_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssma_SetLFSweepTriggerSource (instrSession,
                                                        RSSMA_TRIG_SOURCE_SWEEP_AUTO));
        CHECKERR (rssma_SetLFMode (instrSession, RSSMA_LFOUTPUT_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_SINGLE:
        CHECKERR (rssma_SetSweepLFMode (instrSession, RSSMA_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssma_SetLFSweepTriggerSource (instrSession, RSSMA_TRIG_SOURCE_SWEEP_SINGLE));
        CHECKERR (rssma_SetLFMode (instrSession, RSSMA_LFOUTPUT_FREQUENCY_MODE_SWEEP));
        CHECKERR (rssma_LFSweepTrigger (instrSession));
        break;
    case RSSMA_SWEEP_MODE_STEP:
        CHECKERR (rssma_SetSweepLFMode (instrSession, RSSMA_SWEEP_LOW_MODE_MAN));
        CHECKERR (rssma_SetLFMode (instrSession, RSSMA_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_EXT_SINGLE:
        CHECKERR (rssma_SetLFMode (instrSession, RSSMA_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssma_SetLFSweepTriggerSource (instrSession,RSSMA_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssma_SetLFMode (instrSession, RSSMA_LFOUTPUT_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_EXT_STEP:
        CHECKERR (rssma_SetSweepLFMode (instrSession, RSSMA_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssma_SetLFSweepTriggerSource (instrSession,
                                                        RSSMA_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssma_SetLFMode (instrSession, RSSMA_LFOUTPUT_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMA_SWEEP_MODE_EXT_START:
        CHECKERR (rssma_SetSweepLFMode (instrSession, RSSMA_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssma_SetLFSweepTriggerSource (instrSession,
                                                        RSSMA_TRIG_SOURCE_SWEEP_EAUTO));
        CHECKERR (rssma_SetLFMode (instrSession, RSSMA_LFOUTPUT_FREQUENCY_MODE_SWEEP));
        break;
    default:
        rssma_status = rssma_ParameterError (instrSession, "mode");
    }

    RETURN;
}

/*============================================================================
 * Function: rssma_SetLFSweepSource
 * Purpose:  This function selects the source of the LF sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepSource (ViSession instrSession,
                                          ViInt32 source)
{
    INIT;
    CHECKOPT (K24);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:LFO:SWE:LFS",
                          source,
                          lfoSourceArr);    
}

/*============================================================================
 * Function: rssma_GetLFSweepSource
 * Purpose:  This function returns the source of the LF sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFSweepSource (ViSession instrSession,
                                          ViInt32 *source)
{
    INIT;
    CHECKOPT (K24);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:LFO:SWE:LFS?",
                          source,
                          lfoSourceArr);    
}

/*============================================================================
 * Function: rssma_SetLFSweepStartFreq
 * Purpose:  This function sets the start frequency for the LF Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepStartFreq (ViSession instrSession,
                                             ViReal64  startFrequency)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:FREQ:STAR",
                              startFrequency,
                              0.1, 1.0e6);
}

/*============================================================================
 * Function: rssma_GetLFSweepStartFreq
 * Purpose:  This function Returns the start frequency of the LF Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFSweepStartFreq(ViSession  instrSession,
                                            ViReal64*  startFrequency)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:FREQ:STAR?",
                              startFrequency);
}

/*============================================================================
 * Function: rssma_SetLFSweepStopFreq
 * Purpose:  This function sets the stop frequency for the LF sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepStopFreq(ViSession   instrSession,
                                           ViReal64    stopFrequency)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:FREQ:STOP",
                              stopFrequency,
                              0.1, 1.0e6);
}

/*============================================================================
 * Function: rssma_GetLFSweepStopFreq
 * Purpose:  This function returns the stop frequency of the LF sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFSweepStopFreq(ViSession   instrSession,
                                           ViReal64*   stopFrequency)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:FREQ:STOP?",
                              stopFrequency);
}

/*============================================================================
 * Function: rssma_SetLFSweepSpacing
 * Purpose:  This function selects linear or logarithmic sweep spacing.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepSpacing(ViSession instrSession,
                                          ViInt32 spacing)
{
    return SETPARAMENUM (instrSession,
                          "LFO:SWE:SPAC",
                          spacing,
                          spacingArr);
}

/*============================================================================
 * Function: rssma_GetLFSweepSpacing
 * Purpose:  This function returns linear or logarithmic sweep spacing.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFSweepSpacing(ViSession instrSession,
                                          ViInt32*    spacing)
{
    return GETPARAMENUM (instrSession,
                          "LFO:SWE:SPAC?",
                          spacing,
                          spacingArr);
}

/*============================================================================
 * Function: rssma_SetLFSweepStepLin
 * Purpose:  This function sets the step width for the linear sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepStepLin(ViSession   instrSession,
                                          ViReal64    stepLin)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:SWE:STEP",
                              stepLin,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetLFSweepStepLin
 * Purpose:  This function returns the step width for the linear sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFSweepStepLin(ViSession   instrSession,
                                          ViReal64*   stepLin
)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:SWE:STEP?",
                              stepLin);
}

/*============================================================================
 * Function: rssma_SetLFSweepStepLog
 * Purpose:  This function specifies the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepStepLog(ViSession  instrSession,
                                          ViReal64    stepLog)
{
    return SETPARAMREAL64UNITS(instrSession,
                                   "LFO:SWE:STEP:LOG",
                                   stepLog,
                                   "PCT",
                                   0.01, 50.0);
}

/*============================================================================
 * Function: rssma_GetLFSweepStepLog
 * Purpose:  This function returns the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFSweepStepLog(ViSession  instrSession,
                                          ViReal64*   stepLog)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:SWE:STEP:LOG?",
                              stepLog);
}

/*============================================================================
 * Function: rssma_SetLFSweepDwellTime
 * Purpose:  This function sets the dwell time for each frequency step of the
 *           sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepDwellTime(ViSession    instrSession,
                                            ViReal64     dwellTime)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:SWE:DWEL",
                              dwellTime,
                              2.0e-3, 10.0);
}

/*============================================================================
 * Function: rssma_GetLFSweepDwellTime
 * Purpose:  This function returns the dwell time of frequency step of the
 *           sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFSweepDwellTime(ViSession    instrSession,
                                            ViReal64*    dwellTime)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:SWE:DWEL?",
                              dwellTime);
}

/*============================================================================
 * Function: rssma_SetLFSweepManualStep
 * Purpose:  In Sweep mode (:SOUR:LFO:FREQ:MODE SWE) this function sets the
 *           frequency for the next sweep step in the Step sweep mode
 *           (:SOUR:LFO:SWE:MODE MAN). Here only frequency values between the
 *           settings :SOUR:LFO:FREQ:STAR and ...:STOP are permitted. Each
 *           sweep step is triggered by a separate :SOUR:LFO:FREQ:MAN command.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepManualStep (ViSession instrSession,
                                              ViReal64 manualStep)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:FREQ:MAN",
                              manualStep,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssma_GetLFSweepManualStep
 * Purpose:  In Sweep mode (:SOUR:LFO:FREQ:MODE SWE) this function returns the
 *           frequency of the next sweep step in the Step sweep mode
 *           (:SOUR:LFO:SWE:MODE MAN). Here only frequency values between the
 *           settings :SOUR:LFO:FREQ:STAR and ...:STOP are permitted. Each
 *           sweep step is triggered by a separate :SOUR:LFO:FREQ:MAN command.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFSweepManualStep (ViSession instrSession,
                                              ViReal64 *manualStep)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:FREQ:MAN?",
                              manualStep);
}

/*============================================================================
 * Function: rssma_SetLFSweepPoints
 * Purpose:  This function sets the number of steps in an LF sweep. This
 *           function is linked to the command :LFOutput:SWEep[:FREQuency]:STEP
 *           as follows:
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepPoints(ViSession  instrSession,
                                         ViInt32 lfSweepPoints)
{
    return SETPARAMINT32 (instrSession,
                             "LFO:SWE:POIN",
                             lfSweepPoints,
                             0, 0);
}

/*============================================================================
 * Function: rssma_GetLFSweepPoints
 * Purpose:  This function returns the number of steps in an LF sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFSweepPoints(ViSession  instrSession,
                                         ViInt32*    lfSweepPoints)
{
    return GETPARAMINT32 (instrSession,
                             "LFO:SWE:POIN?",
                             lfSweepPoints);
}


/*============================================================================
 * Function: Set LF Sweep Shape
 * Purpose:  This function sets the cycle mode for a sweep sequence (shape).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFSweepShape (ViSession instrSession,
                                         ViInt32 shape)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:LFO:SWE:FREQ:SHAP",
                          shape,
                          frequencySweepShapeArr);    
}

/*============================================================================
 * Function: Get LF Sweep Shape
 * Purpose:  This function queries the cycle mode for a sweep sequence (shape).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFSweepShape (ViSession instrSession,
                                         ViInt32 *shape)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:LFO:SWE:FREQ:SHAP?",
                          shape,
                          frequencySweepShapeArr);    
}

/*============================================================================
 * Function: rssma_confLFFreq
 * Purpose:  This function sets parameters of LF Output.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confLFFreq (ViSession instrSession,
                                    ViBoolean state,
                                    ViInt32 channel,
                                    ViReal64 LFGenFrequency,
                                    ViReal64 LFGenVoltage)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetLFGenFrequency (instrSession, channel, LFGenFrequency));
    CHECKERR (rssma_SetLFGenVoltage (instrSession, LFGenVoltage));
    
    return rssma_SetLFOutputState (instrSession, state);
}

/*============================================================================
 * Function: rssma_SetLFOutputState
 * Purpose:  This function activates/deactivates the LF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFOutputState(ViSession instrSession,
                                         ViBoolean lfOutputState)
{
    return SETPARAMBOOL (instrSession,
                               "LFO",
                               lfOutputState);
}

/*============================================================================
 * Function: rssma_GetLFOutputState
 * Purpose:  This function returns state the LF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFOutputState(ViSession instrSession,
                                         ViBoolean*    lfOutputState)
{
    return GETPARAMBOOL (instrSession,
                               "LFO?",
                               lfOutputState);
}

/*============================================================================
 * Function: rssma_SetLFGenSource
 * Purpose:  This function selects the internal source to be used for the LF
 *           Output signal. The available selection depends on the options
 *           fitted.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFGenSource(ViSession instrSession,
                                       ViInt32   lfoSource)
{
    INIT;
    
    if (lfoSource != RSSMA_LFOUTPUT_SOURCE_LF1)
    {
        CHECKOPT (K24);
    }
    
    return SETPARAMENUM (instrSession,
                          "LFO:SOUR",
                          lfoSource,
                          lfoSourceArr);
}

/*============================================================================
 * Function: rssma_GetLFGenSource
 * Purpose:  This function returns the internal source to be used for the LF
 *           Output signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFGenSource(ViSession instrSession,
                                       ViInt32*  lfoSource)
{
    return GETPARAMENUM (instrSession,
                          "LFO:SOUR?",
                          lfoSource,
                          lfoSourceArr);
}

/*============================================================================
 * Function: rssma_SetLFGenVoltage
 * Purpose:  This function sets the output voltage of the LF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFGenVoltage (ViSession instrSession,
                                         ViReal64  voltage)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:VOLT",
                              voltage,
                              0.0, 4.0);
}

/*============================================================================
 * Function: rssma_GetLFGenVoltage
 * Purpose:  This function returns the output voltage of the LF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFGenVoltage (ViSession instrSession,
                                         ViReal64 *voltage)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:VOLT?",
                              voltage);
}

/*============================================================================
 * Function: rssma_SetLFGenInternalVoltage
 * Purpose:  This function sets the internal output voltage of the LF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFGenInternalVoltage (ViSession instrSession,
                                                 ViInt32 channel,
                                                 ViReal64 LFGenInternalVoltage)
{
    ViChar  buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPTCONT (K24);
    if (rssma_status != VI_SUCCESS)
    {
        CHECKVIINT32 (instrSession, channel, 1, 1);
    }
    else
    {
        CHECKVIINT32 (instrSession, channel, 1, 2);
    }
    
    sprintf (buffer, "LFO%ld:INT:VOLT", channel);
    
    return SETPARAMREAL64 (instrSession,
                              buffer,
                              LFGenInternalVoltage,
                              0.0, 4.0);
}

/*============================================================================
 * Function: rssma_GetLFGenInternalVoltage
 * Purpose:  This function returns the output internal voltage of the LF
 *           output.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFGenInternalVoltage (ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  *internalVoltage)
{
    ViChar  buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPTCONT (K24);
    if (rssma_status != VI_SUCCESS)
    {
        CHECKVIINT32 (instrSession, channel, 1, 1);
    }
    else
    {
        CHECKVIINT32 (instrSession, channel, 1, 2);
    }

    sprintf (buffer, "LFO%ld:INT:VOLT?", channel);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              internalVoltage);
}

/*============================================================================
 * Function: rssma_SetLFGenFrequency
 * Purpose:  This function sets the frequency of the LF signal for CW mode
 *           (:SOUR::MODE CW mode). In Sweep mode (:SOUR:LFO:FREQ:MODE SWE),
 *           the frequency is linked to the sweep frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFGenFrequency (ViSession instrSession,
                                           ViInt32   channel,
                                           ViReal64  frequency)
{
    ViChar  buffer[BUFFER_SIZE];
    INIT;
    
    CHECKOPTCONT (K24);
    if (rssma_status != VI_SUCCESS)
    {
        CHECKVIINT32 (instrSession, channel, 1, 1);
    }
    else
    {
        CHECKVIINT32 (instrSession, channel, 1, 2);
    }

    sprintf (buffer, "LFO%ld:FREQ", channel);
    
    return SETPARAMREAL64 (instrSession,
                              buffer,
                              frequency,
                              0.1, 10.0e6);
}

/*============================================================================
 * Function: rssma_GetLFGenFrequency
 * Purpose:  This function returns the frequency of the LF signal of CW mode
 *           (:SOUR::MODE CW mode). In Sweep mode (:SOUR:LFO:FREQ:MODE SWE),
 *           the frequency is linked to the sweep frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFGenFrequency (ViSession instrSession,
                                           ViInt32   channel,
                                           ViReal64  *frequency)
{
    ViChar  buffer[BUFFER_SIZE];
    
    INIT;
    
    CHECKOPTCONT (K24);
    if (rssma_status != VI_SUCCESS)
    {
        CHECKVIINT32 (instrSession, channel, 1, 1);
    }
    else
    {
        CHECKVIINT32 (instrSession, channel, 1, 2);
    }

    sprintf (buffer, "LFO%ld:FREQ?", channel);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              frequency);
}

/*============================================================================
 * Function: rssma_SetLFGenShape
 * Purpose:  This function selects the shape of the second LF generator.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFGenShape (ViSession instrSession, ViInt32 shape)
{
    INIT;
    CHECKOPT (K24);
    
    return SETPARAMENUM (instrSession,
                          "LFO2:SHAP",
                          shape,
                          shapeArr);
}

/*============================================================================
 * Function: rssma_GetLFGenShape
 * Purpose:  This function returns the shape of the second LF generator.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFGenShape (ViSession instrSession,
                                       ViInt32 *shape)
{
    INIT;
    CHECKOPT (K24);

    return GETPARAMENUM (instrSession,
                          "LFO2:SHAP?",
                          shape,
                          shapeArr);
}

/*============================================================================
 * Function: rssma_SetLFGenShapeTrapeze
 * Purpose:  This function selects the fall/high/low/rise time for the
 *           trapeze shape of the second LF generator.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFGenShapeTrapeze (ViSession instrSession,
                                              ViInt32 trapeze, ViReal64 time)
{
    ViChar      buffer [BUFFER_SIZE];
    ViInt32     channel = 2;
    INIT;
    CHECKOPT (K24);
    
    switch (trapeze){
        case RSSMA_LFOUTPUT_SHAPE_TRAPEZE_FALL:
        sprintf (buffer, "SOUR:LFO%ld:SHAP:TRAP:FALL", channel);
    
        break;
        
        case RSSMA_LFOUTPUT_SHAPE_TRAPEZE_HIGH:
        sprintf (buffer, "SOUR:LFO%ld:SHAP:TRAP:HIGH", channel);
    
        break;
        
        case RSSMA_LFOUTPUT_SHAPE_TRAPEZE_LOW:
        sprintf (buffer, "SOUR:LFO%ld:SHAP:TRAP:LOW", channel);
    
        break;
        
        case RSSMA_LFOUTPUT_SHAPE_TRAPEZE_RISE:
        sprintf (buffer, "SOUR:LFO%ld:SHAP:TRAP:RISE", channel);
    
        break;
        default:
	        return rssma_ParameterError (instrSession, "trapeze");
    }
        return SETPARAMREAL64 (instrSession,
                                  buffer,
                                  time,
                                  0.2e-6, 2.5);
   
}

/*============================================================================
 * Function: rssma_GetLFGenShapeTrapeze
 * Purpose:  This function returns the fall/high/low/rise time for the
 *           trapeze shape of the second LF generator.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFGenShapeTrapeze (ViSession instrSession,
                                              ViInt32 trapeze, ViReal64 *time)
{
    ViChar      buffer [BUFFER_SIZE];
    ViInt32     channel = 2;
    INIT;
    CHECKOPT (K24);
    
    switch (trapeze){
        case RSSMA_LFOUTPUT_SHAPE_TRAPEZE_FALL:
        sprintf (buffer, "SOUR:LFO%ld:SHAP:TRAP:FALL?", channel);
    
        break;
        
        case RSSMA_LFOUTPUT_SHAPE_TRAPEZE_HIGH:
        sprintf (buffer, "SOUR:LFO%ld:SHAP:TRAP:HIGH?", channel);
    
        break;
        
        case RSSMA_LFOUTPUT_SHAPE_TRAPEZE_LOW:
        sprintf (buffer, "SOUR:LFO%ld:SHAP:TRAP:LOW?", channel);
    
        break;
        
        case RSSMA_LFOUTPUT_SHAPE_TRAPEZE_RISE:
        sprintf (buffer, "SOUR:LFO%ld:SHAP:TRAP:RISE?", channel);
    
        break;
        default:
	        return rssma_ParameterError (instrSession, "trapeze");
    }
        return GETPARAMREAL64 (instrSession,
                                  buffer,
                                  time);
   
}

/*============================================================================
 * Function: Set LIST Attenuator Mode
 * Purpose:  This function selects the ranges of level settings for the list
 *           mode. The level settings are either performed in the low level
 *           or in the high level ranges.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTAttenuatorMode (ViSession instrSession,
                                               ViInt32 attenuatorMode)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:POW:AMOD",
                          attenuatorMode,
                          ListAttenuatorModeArr);
}

/*============================================================================
 * Function: Get LIST Attenuator Mode
 * Purpose:  This function returns the ranges of level settings for the list
 *           mode. The level settings are either performed in the low level
 *           or in the high level ranges.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTAttenuatorMode (ViSession instrSession,
                                               ViInt32 *attenuatorMode)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:LIST:POW:AMOD?",
                          attenuatorMode,
                          ListAttenuatorModeArr);
}

/*============================================================================
 * Function: Set LIST Step Index
 * Purpose:  This function sets the index for step mode. After the trigger
 *           signal the frequency and level settings of the selected index
 *           are processed in List mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTStepIndex (ViSession instrSession,
                                          ViInt32 stepIndex)
{
    return SETPARAMINT32 (instrSession, 
                             "SOUR:LIST:IND",
                             stepIndex,
                             0, 0);
}

/*============================================================================
 * Function: Get LIST Step Index
 * Purpose:  This function returns the index for step mode. After the trigger
 *           signal the frequency and level settings of the selected index
 *           are processed in List mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTStepIndex (ViSession instrSession,
                                          ViInt32 *stepIndex)
{
    return GETPARAMINT32 (instrSession, 
                             "SOUR:LIST:IND?",
                             stepIndex);
}

/*============================================================================
 * Function: Set LIST Start Index
 * Purpose:  This function sets the start index of the index range which
 *           defines a subgroup of frequency/level value pairs in the current
 *           list. Only the values in the set index range (:LIST:INDex:STARt
 *           ... :LIST:INDex:STOP) are processed in List mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTStartIndex (ViSession instrSession,
                                           ViInt32 startIndex)
{
    return SETPARAMINT32 (instrSession, 
                             "SOUR:LIST:IND:STAR",
                             startIndex,
                             0, 0);
}

/*============================================================================
 * Function: Get LIST Start Index
 * Purpose:  This function returns the start index of the index range which
 *           defines a subgroup of frequency/level value pairs in the current
 *           list. Only the values in the set index range (:LIST:INDex:STARt
 *           ... :LIST:INDex:STOP) are processed in List mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTStartIndex (ViSession instrSession,
                                           ViInt32 *startIndex)
{
    return GETPARAMINT32 (instrSession, 
                             "SOUR:LIST:IND:STAR?",
                             startIndex);
}

/*============================================================================
 * Function: Set LIST Stop Index
 * Purpose:  This function sets the stop index of the index range which
 *           defines a subgroup of frequency/level value pairs in the current
 *           list. Only the values in the set index range (:LIST:INDex:STARt
 *           ... :LIST:INDex:STOP) are processed in List mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTStopIndex (ViSession instrSession,
                                          ViInt32 stopIndex)
{
    return SETPARAMINT32 (instrSession, 
                             "SOUR:LIST:IND:STOP",
                             stopIndex,
                             0, 0);
}

/*============================================================================
 * Function: Get LIST Stop Index
 * Purpose:  This function returns the stop index of the index range which
 *           defines a subgroup of frequency/level value pairs in the current
 *           list. Only the values in the set index range (:LIST:INDex:STARt
 *           ... :LIST:INDex:STOP) are processed in List mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTStopIndex (ViSession instrSession,
                                          ViInt32 *stopIndex)
{
    return GETPARAMINT32 (instrSession, 
                             "SOUR:LIST:IND:STOP?",
                             stopIndex);
}

/*============================================================================
 * Function: Learn LIST Mode Data
 * Purpose:  This function learns the selected list, i.e. it determines the
 *           hardware setting for the entire list. The data determined in
 *           this way is stored together with the list. When the list is
 *           activated for the first time, these settings are calculated
 *           automatically.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_LearnLISTModeData (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SOUR:LIST:LEAR");    
}

/*===========================================================================
 * Function: Get List Free
 * Purpose:  This function queries on the free storage space for list mode 
 *			 lists.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTFree (ViSession instrSession,
                                     ViInt32 *freeSpace)
{
    return GETPARAMINT32(instrSession,
                          "SOUR:LIST:FREE?",
                          freeSpace);    
}

/*===========================================================================
 * Function: Set LIST Import Export List Mode
 * Purpose:  This function sets the dexchange list mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTImportExportListMode(
    ViSession   instrSession,
    ViInt32 listMode
)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:MODE",
                          listMode,
                          dexcListModeArr);
}

/*===========================================================================
 * Function: Get LIST Import Export List Mode
 * Purpose:  This function returns the dexchange list mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTImportExportListMode(
    ViSession   instrSession,
    ViInt32*    listMode
)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:MODE?",
                          listMode,
                          dexcListModeArr);
}

/*===========================================================================
 * Function: Set LIST Import Export List File
 * Purpose:  This function calls the File Manager for selecting the list 
 *           mode list to be exported (source) into an ASCII file or the 
 *           destination for the ASCII file to be imported (destination) in. 
 *           
 *           The list mode files are stored with the fixed file extensions 
 *           *.lsw in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR. A path can also be specified in command 
 *           SOUR:LIST:DEXC:SEL, in which case the files are stored or loaded 
 *           in the specified directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTImportExportListFile (ViSession instrSession,
                                                     ViString sourceFile)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:LIST:DEXC:SEL",
                                    "'",
                                    sourceFile);
}

/*===========================================================================
 * Function: Get LIST Import Export List File
 * Purpose:  This function calls the File Manager for querying the list mode 
 *           list to be exported (source) into an ASCII file or the 
 *           destination for the ASCII file to be imported (destination) in.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTImportExportListFile (ViSession instrSession,
                                                     ViChar sourceFile[])
{
    return GETPARAMSTRING (instrSession,
                              "SOUR:LIST:DEXC:SEL?",
                              sourceFile,
                              128);
}

/*===========================================================================
 * Function: Import Export Execute
 * Purpose:  This function starts the export or import of the selected file. 
 *           
 *           When import is selected, the ASCII file is imported as list 
 *           mode list.
 *           
 *           When export is selected, the list mode list is exported into 
 *           the selected ASCII file.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ImportExportExecute(
    ViSession   instrSession
)
{
    return rssma_sendCmd (instrSession, "SOUR:LIST:DEXC:EXEC");
}

/*===========================================================================
 * Function: Set LIST Import Export ASCII File
 * Purpose:  This function calls the File Manager for selecting the ASCII 
 *           file to be imported into a list mode list (source) or the ASCII 
 *           file the list mode list is exported (destination) in.
 *           
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified in command 
 *           SOUR:LIST:DEXC:AFIL:SEL, in which case the files are stored or 
 *           loaded in the specified directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTImportExportASCIIFile(
    ViSession   instrSession,
    ViString    asciiSourceFile
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:LIST:DEXC:AFIL:SEL",
                                    "'",
                                    asciiSourceFile);
}

/*===========================================================================
 * Function: Get LIST Import Export ASCII File
 * Purpose:  This function calls the File Manager for querying the ASCII 
 *           file to be imported into a list mode list (source) or the ASCII 
 *           file the list mode list is exported (destination) in.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTImportExportASCIIFile(
    ViSession   instrSession,
    ViChar _VI_FAR asciiSourceFile[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SOUR:LIST:DEXC:AFIL:SEL?",
                              asciiSourceFile,
                              128);
}

/*===========================================================================
 * Function: Set LIST Import Export ASCII Extension
 * Purpose:  This function selects the file extension of the ASCII file to 
 *           be imported or exported. Selection TXT (text file) or CSV (Excel 
 *           file) is available. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTImportExportASCIIExtension(
    ViSession   instrSession,
    ViInt32 extension
)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:EXT",
                          extension,
                          dexcExtensionArr);
}

/*===========================================================================
 * Function: Get LIST Import Export ASCII Extension
 * Purpose:  This function queries the selected file extension of the ASCII 
 *           file to be imported or exported. Selection TXT (text file) or 
 *           CSV (Excel file) is available. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTImportExportASCIIExtension(
    ViSession   instrSession,
    ViInt32*    extension
)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:EXT?",
                          extension,
                          dexcExtensionArr);
}

/*===========================================================================
 * Function: Set LIST Import Export ASCII Column Separator
 * Purpose:  This function selects the separator between the frequency and 
 *           level column of the ASCII table. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTImportExportASCIIColumnSeparator(
    ViSession   instrSession,
    ViInt32 separator
)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:SEP:COL",
                          separator,
                          dexcSeparatorArr);
}

/*===========================================================================
 * Function: Get LIST Import Export ASCII Column Separator
 * Purpose:  This function queries the selected separator between the 
 *           frequency and level column of the ASCII table. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTImportExportASCIIColumnSeparator(
    ViSession   instrSession,
    ViInt32*    separator
    
)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:SEP:COL?",
                          separator,
                          dexcSeparatorArr);
}

/*===========================================================================
 * Function: Set LIST Import Export ASCII Column Decimal Separator
 * Purpose:  This function selects the decimal separator used in the ASCII 
 *           data between '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTImportExportASCIIColumnDecimalSeparator(
    ViSession   instrSession,
    ViInt32 decimalSeparator
)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:SEP:DEC",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*===========================================================================
 * Function: Get LIST Import Export ASCII Column Decimal Separator
 * Purpose:  This function displays the selected decimal separator used in 
 *           the ASCII data - '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTImportExportASCIIColumnDecimalSeparator(
    ViSession   instrSession,
    ViInt32*    decimalSeparator
)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:SEP:DEC?",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*===========================================================================
 * Function: LIST Import Export Catalog
 * Purpose:  This function requests a list of available ASCII files for 
 *           export/import of list mode data. The individual files are 
 *           separated by commas.
 *            
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified in command 
 *           SOUR:LIST:DEXC:AFIL:CAT?, in which case the files in the 
 *           specified directory are read.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_LISTImportExportCatalog (ViSession instrSession, 
                            ViInt32 arraySize, ViChar _VI_FAR catalogList[])
{
    return rssma_getCatalog (instrSession,
                             ":LIST:DEXC:AFIL:CAT?;*OPC?",
                             arraySize,
                             catalogList,
							"catalogList");

}

/*============================================================================
 * Function: rssma_ConfigureClockSynthesis
 * Purpose:  This function sets parameters of Clock Synthesis.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ConfigureClockSynthesis (ViSession instrSession,
                                                 ViBoolean csynState,
                                                 ViReal64 csynFreq,
                                                 ViBoolean csynVariationActive,
                                                 ViReal64 csynVariationStep)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetCSYNFrequency (instrSession, csynFreq));
    CHECKERR (rssma_SetCSYNVariationActive (instrSession, csynVariationActive));
    CHECKERR (rssma_SetCSYNVariationStep (instrSession, csynVariationStep));    
    return rssma_SetCSYNState (instrSession, csynState);
}                                       
                                       
                                       
/*============================================================================
 * Function: rssma_SetCSYNState
 * Purpose:  This function activates/deactivates generation of a system clock
 *           for differential outputs CLK SYN and CLK SYN N at the rear of
 *           the instrument.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetCSYNState(ViSession instrSession,
                                     ViBoolean csynState)
{
    INIT;
    CHECKOPT (B29);
    
    return SETPARAMBOOL (instrSession,
                               "CSYN:STAT",
                               csynState);
}

/*============================================================================
 * Function: rssma_GetCSYNState
 * Purpose:  This function returns the state of the generation of a system
 *           clock
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetCSYNState (ViSession  instrSession,
                                      ViBoolean* csynState)
{
    INIT;
    CHECKOPT (B29);

    return GETPARAMBOOL (instrSession,
                               "CSYN:STAT?",
                               csynState);
} 

/*============================================================================
 * Function: rssma_SetCSYNFrequency
 * Purpose:  This function sets the frequency of the clock synthesis output
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetCSYNFrequency (ViSession instrSession,
                                          ViReal64  csynFrequency)
{
    INIT;
    CHECKOPT (B29);
    
    return SETPARAMREAL64 (instrSession,
                              "CSYN:FREQ",
                              csynFrequency,
                              100.0e3, 1.5e9);
}

/*============================================================================
 * Function: rssma_GetCSYNFrequency
 * Purpose:  This function returns the frequency of the clock synthesis
 *           output signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetCSYNFrequency (ViSession instrSession,
                                          ViReal64 *csynFrequency)
{
    INIT;
    CHECKOPT (B29);
    
    return GETPARAMREAL64 (instrSession,
                              "CSYN:FREQ?",
                              csynFrequency);
}

/*============================================================================
 * Function: rssma_SetCSYNVariationStep
 * Purpose:  This function sets the step width for the clock synthesis
 *           frequency setting if the frequency values UP/DOWN are used and
 *           variation mode SOUR:FREQ:STEP:MODE USER is selected. The command
 *           is linked to Variation Step for manual control, i.e. the command
 *           also sets the step width of the rotary knob for Variation Active
 *           on.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetCSYNVariationStep (ViSession instrSession,
                                              ViReal64  csynVariationStep)
{
    INIT;
    CHECKOPT (B29);
    
    return SETPARAMREAL64 (instrSession,
                              "CSYN:FREQ:STEP",
                              csynVariationStep,
                              0.0, 9999.99e6);
}

/*============================================================================
 * Function: rssma_GetCSYNVariationStep
 * Purpose:  This function returns the step width for the clock synthesis
 *           frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetCSYNVariationStep (ViSession instrSession,
                                              ViReal64 *csynVariationStep)
{
    INIT;
    CHECKOPT (B29);
    
    return GETPARAMREAL64 (instrSession,
                              "CSYN:FREQ:STEP?",
                              csynVariationStep);
}

/*============================================================================
 * Function: rssma_SetCSYNVariationActive
 * Purpose:  This function activates (USER) or deactivates (DECimal) the
 *           user-defined step width used when varying the clock synthesis
 *           frequency value with the frequency values UP/DOWN. The command
 *           is linked to the command Variation Active for manual control,
 *           i.e. the command also activates/deactivates the user-defined
 *           step width used when varying the frequency value with the
 *           rotary knob
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetCSYNVariationActive(ViSession instrSession,
                                               ViBoolean csynVariationActive
)
{
    INIT;
    CHECKOPT (B29);

    return SETPARAMENUM (instrSession,
                           "CSYN:FREQ:STEP:MODE",
                           (ViInt32) csynVariationActive,
                           variationArr);
}

/*============================================================================
 * Function: rssma_GetCSYNVariationActive
 * Purpose:  This function returns the frequency step mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetCSYNVariationActive (ViSession  instrSession,
                                                ViBoolean* csynVariationActive)
{
    ViInt32     variation = 0;
    INIT;
    CHECKOPT (B29);
    
    CHECKERR (GETPARAMENUM (instrSession,
                             "CSYN:FREQ:STEP:MODE?",
                             &variation,
                             variationArr));
    if (csynVariationActive)
        *csynVariationActive = (ViBoolean)((variation == 1) ? 1 : 0);

    return rssma_status;}

/*============================================================================
 * Function: rssma_SetCSYNPhase
 * Purpose:  This function sets the phase of the clock synthesis output
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetCSYNPhase (ViSession instrSession,
                                      ViReal64  csynPhase)
{
    INIT;
    CHECKOPT (B29);
    
    return SETPARAMREAL64 (instrSession,
                              "CSYN:PHAS",
                              csynPhase,
                              -359.9, 359.9);
}

/*============================================================================
 * Function: rssma_GetCSYNPhase
 * Purpose:  This function returns the phase of the clock synthesis output
 *           signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetCSYNPhase (ViSession instrSession,
                                      ViReal64 *csynPhase)
{
    INIT;
    CHECKOPT (B29);
    
    return GETPARAMREAL64 (instrSession,
                              "CSYN:PHAS?",
                              csynPhase);
}

/*============================================================================
 * Function: rssma_SetCSYNPhaseReference
 * Purpose:  This function adopts the phase set as the current phase.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetCSYNPhaseReference (ViSession instrSession)
{
    INIT;
    CHECKOPT (B29);
    
    return rssma_sendCmd (instrSession, "CSYN:PHAS:REF");
}


/*============================================================================
 * Function: rssma_SetAMState
 * Purpose:  This function activates/deactivates amplitude modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAMState (ViSession instrSession, 
                                    ViBoolean AMState)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:AM:STAT",
                               AMState);
}

/*============================================================================
 * Function: rssma_GetAMState
 * Purpose:  This function returns the state of the amplitude modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAMState (ViSession instrSession, 
                                    ViBoolean *AMState)
{
    return GETPARAMBOOL (instrSession,
                               "SOUR:AM:STAT?",
                               AMState);
}

/*============================================================================
 * Function: Set All Modulations State
 * Purpose:  This function switches the modulations on and off.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAllModulationsState (ViSession instrSession,
                                                ViBoolean modulationStates)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:MOD:ALL:STAT",
                               modulationStates);
}

/*============================================================================
 * Function: rssma_configAM
 * Purpose:  This function sets available parameter of amplitude modulation..
 *===========================================================================*/
ViStatus _VI_FUNC rssma_configAM (ViSession instrSession, ViBoolean AMState,
                                  ViInt32 AMSource, ViReal64 AMDepth,
                                  ViInt32 AMExternalCoupling,
                                  ViInt32 AMInternalSource,
                                  ViReal64 AMInternalDepth1,
                                  ViReal64 LFGen1Frequency, ViInt32 LFGen2Shape,
                                  ViReal64 LFGen2Frequency)
{
    INIT;


    CHECKERR (rssma_SetAMSource (instrSession, AMSource));
    CHECKERR (rssma_SetAMDepth (instrSession, AMDepth));
    if (AMSource == RSSMA_AM_SOURCE_INT)
    {
        CHECKERR (rssma_SetAMInternalSource (instrSession, AMInternalSource));
		CHECKOPTCONT (K24);
        if (rssma_status == VI_SUCCESS)
        {
            CHECKERR (rssma_SetAMInternalDepth (instrSession, 1, AMInternalDepth1));
			CHECKERR (rssma_SetLFGenShape (instrSession, LFGen2Shape));
            CHECKERR (rssma_SetLFGenFrequency (instrSession, 2, LFGen2Frequency));
        }
    
        CHECKERR (rssma_SetLFGenFrequency (instrSession, 1, LFGen1Frequency));
	}
    else
    {
        CHECKERR (rssma_SetAMExternalCoupling (instrSession, AMExternalCoupling));
	}
    
    return rssma_SetAMState (instrSession, AMState);
}


/*============================================================================
 * Function: rssma_SetAMSource
 * Purpose:  This function selects the modulation source for amplitude
 *           modulation. INT is the internal modulation source which is
 *           selected with command AM:SOURce:INTernal and configured in the
 *           SOURce:LFOutput subsystem. The external signal is input at the
 *           AM EXT connector. Internal and external modulation source can be
 *           selected at the same time.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAMSource (ViSession instrSession,
                                     ViInt32   amSource)
{
    return SETPARAMENUM (instrSession,
                          "AM:SOUR",
                          amSource,
                          fmSourceArr);
}

/*============================================================================
 * Function: rssma_GetAMSource
 * Purpose:  This function returns the modulation source of the amplitude
 *           modulation. INT is the internal modulation source. The external
 *           signal is input at the AM EXT connector. Internal and external
 *           modulation source can be selected at the same time.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAMSource (ViSession instrSession,
                                     ViInt32   *amSource)
{
    return GETPARAMENUM (instrSession,
                          "AM:SOUR?",
                          amSource,
                          fmSourceArr);
}

/*============================================================================
 * Function: rssma_SetAMDepth
 * Purpose:  This function sets the modulation depth of the amplitude
 *           modulation in percent.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAMDepth (ViSession instrSession,
                                    ViReal64  amDepth)
{
    return SETPARAMREAL64 (instrSession,
                              "AM:DEPT",
                              amDepth,
                              0.0, 100.0);
}

/*============================================================================
 * Function: rssma_GetAMDepth
 * Purpose:  This function returns the modulation depth of the amplitude
 *           modulation in percent.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAMDepth (ViSession instrSession,
                                    ViReal64  *amDepth)
{
    return GETPARAMREAL64 (instrSession,
                              "AM:DEPT?",
                              amDepth);
}

/*============================================================================
 * Function: rssma_SetAMExternalCoupling
 * Purpose:  This function selects the coupling mode for the external
 *           modulation input (AM EXT) in the case of amplitude modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAMExternalCoupling (ViSession instrSession,
                                               ViInt32   amExtCoupling)
{
    return SETPARAMENUM (instrSession,
                          "AM:EXT:COUP",
                          amExtCoupling,
                          amExtCouplingArr);
}

/*============================================================================
 * Function: rssma_GetAMExternalCoupling
 * Purpose:  This function returns the coupling mode of the external
 *           modulation input (AM EXT) in the case of amplitude modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAMExternalCoupling (ViSession instrSession,
                                     ViInt32   *amExtCoupling)
{
    return GETPARAMENUM (instrSession,
                          "AM:EXT:COUP?",
                          amExtCoupling,
                          amExtCouplingArr);
}

/*============================================================================
 * Function: rssma_GetAMSensitivity
 * Purpose:  This function queries the input sensitivity of the AM EXT input
 *           in %/V. This function is only effective if the external
 *           modulation source is selected (SOUR:AM:SOUR EXT). The returned
 *           value depends on the modulation depth setting (SOUR:AM:DEPTh).
 *           This value is assigned to the voltage value for full modulation
 *           of the input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAMSensitivity (ViSession instrSession,
                                          ViReal64  *amSensitivity)
{
    return GETPARAMREAL64 (instrSession,
                              "AM:SENS?",
                              amSensitivity);
}

/*============================================================================
 * Function: rssma_SetAMInternalSource
 * Purpose:  This function selects which internal modulation source is used.
 *           The available selection depends on the options fitted. The
 *           selected source can be configured in in the SOURce:LFOutput
 *           subsystem.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAMInternalSource (ViSession instrSession,
                                             ViInt32   amIntSource)
{
    INIT;
    
    if (amIntSource != RSSMA_AM_INTERNAL_SOURCE_LF1)
    {
        CHECKOPT (K24);
    }
    
    return SETPARAMENUM (instrSession,
                          "AM:INT:SOUR",
                          amIntSource,
                          lfoSourceArr);
}

/*============================================================================
 * Function: rssma_GetAMInternalSource
 * Purpose:  This function returns the internal modulation source of the
 *           amplitude modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAMInternalSource (ViSession instrSession,
                                             ViInt32   *amIntSource)
{
    return GETPARAMENUM (instrSession,
                          "AM:INT:SOUR?",
                          amIntSource,
                          lfoSourceArr);
}

/*============================================================================
 * Function: rssma_SetAMInternalDepth
 * Purpose:  This function sets the modulation depth for the two LF
 *           generators. The sum of the two values must not exceed the
 *           overall modulation depth set in the upper part of the menu.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAMInternalDepth (ViSession instrSession,
                                            ViInt32   channel,
                                            ViReal64  amIntDepth)
{
    ViChar      buffer [BUFFER_SIZE];
    INIT;
    CHECKOPT (K24);
    
    CHECKVIINT32 (instrSession, channel, 1, 2);

    sprintf (buffer, "AM:INT%ld:DEPT", channel);
    
    return SETPARAMREAL64 (instrSession,
                              buffer,
                              amIntDepth,
                              0.0, 100.0);
}

/*============================================================================
 * Function: rssma_GetAMInternalDepth
 * Purpose:  This function returns the modulation depth for the two LF
 *           generators.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAMInternalDepth (ViSession instrSession,
                                            ViInt32   channel,
                                            ViReal64  *amIntDepth)
{
    ViChar      buffer [BUFFER_SIZE];
    INIT;
    CHECKOPT (K24);
    
    CHECKVIINT32 (instrSession, channel, 1, 2);

    sprintf (buffer, "AM:INT%ld:DEPT?", channel);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              amIntDepth);
}

/*============================================================================
 * Function: Get PM Sensitivity
 * Purpose:  This function queries the input sensitivity of the FM/PM EXT 
 *           input in Hz/V. The function is only effective if the external 
 *           modulation source is selected (rssma_SetFMSource). The returned 
 *           value depends on the modulation deviation setting (SOUR:PM:DEViation). 
 *           This value is assigned to the voltage value for full modulation of the
 *           input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPMSensitivity (ViSession instrSession,
                                          ViReal64 *sensitivity)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:PM:SENS?",
                              sensitivity);
}

/*============================================================================
 * Function: rssma_confPulm
 * Purpose:  This function sets the parameters of pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confPulm (ViSession instrSession, ViBoolean state,
                                  ViInt32 source, ViInt32 polarity,
                                  ViBoolean pulseGeneratorState,
                                  ViReal64 pulsePeriod, ViReal64 pulseWidth,
                                  ViReal64 pulseDelay,
                                  ViBoolean doublePulseState,
                                  ViReal64 doublePulseWidth,
                                  ViReal64 doublePulseDelay, ViInt32 triggerMode)
{
    INIT;
    
    CHECKERR (rssma_SetPULMSource (instrSession, source));
    CHECKERR (rssma_SetPULMPolarity (instrSession, polarity));
    CHECKERR (rssma_SetPGENState (instrSession, pulseGeneratorState));
    CHECKERR (rssma_SetPULMPeriod (instrSession, pulsePeriod));
    CHECKERR (rssma_SetPULMWidth (instrSession, pulseWidth));
    if (instrPtr->instrOptions.K23 != 0)
    {
        CHECKERR (rssma_SetPULMDelay (instrSession, pulseDelay));
        CHECKERR (rssma_SetPULMTriggerMode (instrSession, triggerMode));
        CHECKERR (rssma_SetPULMDoubleState (instrSession, doublePulseState));
        CHECKERR (rssma_SetPULMDoubleDelay (instrSession, doublePulseDelay));
        CHECKERR (rssma_SetPULMDoubleWidth (instrSession, doublePulseWidth));
    }
            
    return rssma_SetPULMState (instrSession, state);
}

/*============================================================================
 * Function: rssma_SetPULMState
 * Purpose:  This function activates/deactivates the pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMState (ViSession instrSession, 
                                      ViBoolean state)
{

    return SETPARAMBOOL (instrSession,
                               "SOUR:PULM:STAT",
                               state);
}

/*============================================================================
 * Function: rssma_GetPULMState
 * Purpose:  This function returns the pulse modulation state.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMState (ViSession instrSession, 
                                      ViBoolean *state)
{

    return GETPARAMBOOL (instrSession,
                               "SOUR:PULM:STAT?",
                               state);
}

/*============================================================================
 * Function: rssma_SetPULMPolarity
 * Purpose:  This function sets the polarity between modulating and modulated
 *           signal. This function is only effective for an external
 *           modulation signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMPolarity (ViSession instrSession,
                                         ViInt32   polarity)
{
    return SETPARAMENUM (instrSession,
                          "PULM:POL",
                          polarity,
                          polarityArr);
}

/*============================================================================
 * Function: rssma_GetPULMPolarity
 * Purpose:  This function returns the polarity between modulating and
 *           modulated signal. This function is only effective for an
 *           external modulation signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMPolarity (ViSession instrSession,
                                         ViInt32   *polarity)
{
    return GETPARAMENUM (instrSession,
                          "PULM:POL?",
                          polarity,
                          polarityArr);
}

/*============================================================================
 * Function: rssma_SetPULMSource
 * Purpose:  This function selects the source for pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMSource (ViSession instrSession,
                                       ViInt32   pulmSource)
{
    return SETPARAMENUM (instrSession,
                          "PULM:SOUR",
                          pulmSource,
                          pulmSourceArr);
}

/*============================================================================
 * Function: rssma_GetPULMSource
 * Purpose:  This function returns the source for pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMSource (ViSession instrSession,
                                       ViInt32   *pulmSource)
{
    return GETPARAMENUM (instrSession,
                          "PULM:SOUR?",
                          pulmSource,
                          pulmSourceArr);
}

/*============================================================================
 * Function: rssma_SetPULMPeriod
 * Purpose:  This function sets the period of the generated pulse. The period
 *           determines the repetition
 *           frequency of the internal signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMPeriod (ViSession instrSession,
                                       ViReal64  period)
{
    return SETPARAMREAL64 (instrSession,
                              "PULM:PER",
                              period,
                              0.0,0.0);
}

/*============================================================================
 * Function: rssma_GetPULMPeriod
 * Purpose:  This function returns the period of the generated pulse.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMPeriod (ViSession instrSession,
                                       ViReal64 *period)
{
    return GETPARAMREAL64 (instrSession,
                              "PULM:PER?",
                              period);
}

/*============================================================================
 * Function: rssma_SetPULMWidth
 * Purpose:  This function sets the width of the generated pulse. The width
 *           determines the pulse length. The pulse width must be at least 20
 *           ns less than the set pulse period.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMWidth (ViSession instrSession,
                                      ViReal64  width)
{
    return SETPARAMREAL64 (instrSession,
                              "PULM:WIDT",
                              width,
                              20.0e-9,1.0);
}

/*============================================================================
 * Function: rssma_GetPULMWidth
 * Purpose:  This function returns the width of the generated pulse.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMWidth (ViSession instrSession,
                                      ViReal64  *period)
{
    INIT;
    CHECKOPT (K23);
    
    return GETPARAMREAL64 (instrSession,
                              "PULM:WIDT?",
                              period);
}

/*============================================================================
 * Function: rssma_SetPULMDelay
 * Purpose:  This function sets the pulse delay. The pulse delay determines
 *           the time that elapses after a trigger event before pulse
 *           modulation starts. The pulse delay is not effective for double
 *           pulse generation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMDelay (ViSession instrSession,
                                      ViReal64  delay)
{
    INIT;
    CHECKOPT (K23);
    
    return SETPARAMREAL64 (instrSession,
                              "PULM:DEL",
                              delay,
                              10.0e-9,100.0);
}

/*============================================================================
 * Function: rssma_GetPULMDelay
 * Purpose:  This function returns the pulse delay.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMDelay (ViSession instrSession,
                                      ViReal64  *delay)
{
    INIT;
    CHECKOPT (K23);
    
    return GETPARAMREAL64 (instrSession,
                              "PULM:DEL?",
                              delay);
}

/*============================================================================
 * Function: rssma_SetPULMDoubleState
 * Purpose:  This function enables/disables double pulse generation. The two
 *           pulses are generated in one pulse period.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMDoubleState (ViSession instrSession, 
                                            ViBoolean state)
{
    INIT;
    CHECKOPT (K23);

    return SETPARAMBOOL (instrSession,
                               "SOUR:PULM:DOUBLE:STAT",
                               state);
}

/*============================================================================
 * Function: rssma_GetPULMDoubleState
 * Purpose:  This function returns the double pulse modulation state.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMDoubleState (ViSession instrSession, 
                                            ViBoolean *state)
{
    INIT;
    CHECKOPT (K23);

    return GETPARAMBOOL (instrSession,
                               "SOUR:PULM:DOUBLE:STAT?",
                               state);
}

/*============================================================================
 * Function: rssma_SetPULMDoubleWidth
 * Purpose:  This function sets the width of the second pulse in case of
 *           double pulse generation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMDoubleWidth (ViSession instrSession,
                                            ViReal64  width)
{
    INIT;
    CHECKOPT (K23);
    
    return SETPARAMREAL64 (instrSession,
                              "PULM:DOUBLE:WIDT",
                              width,
                              20.0e-9,1.0);
}

/*============================================================================
 * Function: rssma_GetPULMDoubleWidth
 * Purpose:  This function returns the width of the second pulse in case of
 *           double pulse generation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMDoubleWidth (ViSession instrSession,
                                            ViReal64  *width)
{
    INIT;
    CHECKOPT (K23);
    
    return GETPARAMREAL64 (instrSession,
                              "PULM:DOUBLE:WIDT?",
                              width);
}

/*============================================================================
 * Function: rssma_SetPULMDoubleDelay
 * Purpose:  This function sets the delay from the start of the first pulse
 *           to the start of the second pulse.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMDoubleDelay (ViSession instrSession,
                                            ViReal64  delay)
{
    INIT;
    CHECKOPT (K23);
    return SETPARAMREAL64 (instrSession,
                              "PULM:DOUBLE:DEL",
                              delay,
                              10.0e-9,100.0);
}

/*============================================================================
 * Function: rssma_GetPULMDoubleDelay
 * Purpose:  This function returns the delay from the start of the first
 *           pulse to the start of the second pulse.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMDoubleDelay (ViSession instrSession,
                                            ViReal64  *delay)
{
    INIT;
    CHECKOPT (K23);
    return GETPARAMREAL64 (instrSession,
                              "PULM:DOUBLE:DEL?",
                              delay);
}

/*============================================================================
 * Function: rssma_SetPULMTriggerMode
 * Purpose:  This function selects the trigger mode for pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMTriggerMode (ViSession instrSession,
                                            ViInt32   pulmTriggerMode)
{
    INIT;
    CHECKOPT (K23);
    return SETPARAMENUM (instrSession,
                          "PULM:TRIG:MODE",
                          pulmTriggerMode,
                          pulmTriggerModeArr);
}

/*============================================================================
 * Function: rssma_GetPULMTriggerMode
 * Purpose:  This function returns the trigger mode for pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTriggerMode (ViSession instrSession,
                                            ViInt32   *pulmTriggerMode)
{
    INIT;
    CHECKOPT (K23);
    return GETPARAMENUM (instrSession,
                          "PULM:TRIG:MODE?",
                          pulmTriggerMode,
                          pulmTriggerModeArr);
}

/*===========================================================================*/
/* Function: Set Frequency Sweep Shape
/* Purpose:  This function sets the cycle mode for a sweep sequence (shape).
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetFrequencySweepShape(
    ViSession   instrSession,
    ViInt32 shape
)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "SOUR:SWE:FREQ:SHAP",
                          shape,
                          frequencySweepShapeArr);    
}

/*===========================================================================*/
/* Function: Get Frequency Sweep Shape
/* Purpose:  This function returns the cycle mode for a sweep sequence 
/*           (shape).
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetFrequencySweepShape(
    ViSession   instrSession,
    ViInt32*    shape
)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "SOUR:SWE:FREQ:SHAP?",
                          shape,
                          frequencySweepShapeArr);    
}

/*===========================================================================*/
/* Function: Set PULM External Trigger Input Impedance
/* Purpose:  This function selects the impedance for external pulse trigger.
/*           The trigger is fed via the PULSE EXT input.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMExternalTriggerInputImpedance (ViSession instrSession,
                                                       ViInt32 externalInputImpedance)
{
    INIT;
    CHECKOPT (K23);
    return SETPARAMENUM (instrSession,
                          "PULM:TRIG:EXT:IMP",
                          externalInputImpedance,
                          dmemExternalInputImpedance);
}

/*===========================================================================*/
/* Function: Get PULM External Trigger Input Impedance
/* Purpose:  This function returns the impedance for external pulse trigger.
/*           The trigger is fed via the PULSE EXT input.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMExternalTriggerInputImpedance (ViSession instrSession,
                                                       ViInt32 *externalInputImpedance)
{
    INIT;
    CHECKOPT (K23);
    return GETPARAMENUM (instrSession,
                          "PULM:TRIG:EXT:IMP?",
                          externalInputImpedance,
                          dmemExternalInputImpedance);
}

/*===========================================================================*/
/* Function: Set PULM External Trigger Slope
/* Purpose:  This function sets the polarity of the active slope of an
/*           applied trigger at the PULSE EXT connector
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMExternalTriggerSlope (ViSession instrSession,
                                                     ViInt32 externalTriggerSlope)
{
    INIT;
    CHECKOPT (K23);
    return SETPARAMENUM (instrSession,
                          "PULM:TRIG:EXT:SLOP",
                          externalTriggerSlope,
                          slopeArr);
}

/*===========================================================================*/
/* Function: Get PULM External Trigger Slope
/* Purpose:  This function returns the polarity of the active slope of an
/*           applied trigger at the PULSE EXT connector
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMExternalTriggerSlope (ViSession instrSession,
                                                     ViInt32 *externalTriggerSlope)
{
    INIT;
    CHECKOPT (K23);
    return GETPARAMENUM (instrSession,
                          "PULM:TRIG:EXT:SLOP?",
                          externalTriggerSlope,
                          slopeArr);
}

/*===========================================================================*/
/* Function: Set PULM External Trigger Slope
/* Purpose:  This function selects the polarity of the external gate signal.
/*           The signal is supplied via the PULSE EXT connector.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMGateInputPolarity (ViSession instrSession,
                                                  ViInt32 gateInputPolarity)
{
    INIT;
    CHECKOPT (K23);
    return SETPARAMENUM (instrSession,
                          "PULM:TRIG:EXT:GATE:POL",
                          gateInputPolarity,
                          polarityArr);
}

/*===========================================================================*/
/* Function: Get PULM External Trigger Slope
/* Purpose:  This function returns the polarity of the external gate signal.
/*           The signal is supplied via the PULSE EXT connector.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMGateInputPolarity (ViSession instrSession,
                                                  ViInt32 *gateInputPolarity)
{
    INIT;
    CHECKOPT (K23);
    return GETPARAMENUM (instrSession,
                          "PULM:TRIG:EXT:GATE:POL?",
                          gateInputPolarity,
                          polarityArr);
}

/*===========================================================================*/
/* Function: Set PULM Mode
/* Purpose:  This function selects the mode for pulse generation.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMF-K23, 
/*           parameter PTRain is only available with option R&S SMF-K27.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMMode(
    ViSession   instrSession,
    ViInt32 mode
)
{
    INIT;
    
    if (mode == RSSMA_PULM_MODE_PTR)
        CHECKOPT (K27);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:MODE",
                          mode,
                          PULMModeArr);    
}

/*===========================================================================*/
/* Function: Get PULM Mode
/* Purpose:  This function returns the mode for pulse generation.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMF-K23, 
/*           parameter PTRain is only available with option R&S SMF-K27.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMMode(
    ViSession   instrSession,
    ViInt32*    mode
)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "SOUR:PULM:MODE?",
                          mode,
                          PULMModeArr);    
}

/*===========================================================================*/
/* Function: PULM Pulse Train Catalog
/* Purpose:  This function requests a list of available pulse train files. 
/*           The individual pulse train files are separated by commas.  
/*           
/*           The files are stored with the fixed file extensions *.pulstrn 
/*           in a directory of the user's choice. The directory applicable to 
/*           the commands is defined with the command MMEMory:CDIR.
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMF-K27.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PULMPulseTrainCatalog(
    ViSession   instrSession,
    ViString    directoryPath,
    ViInt32 arraySize,
    ViChar _VI_FAR  catalogList[]
)
{
    ViChar      buffer[BUFFER_SIZE];
    INIT;
    
    CHECKOPT(K27); 
    
    if (!directoryPath)
        sprintf (buffer, "SOUR:PULM:TRA:CAT?;*OPC?");
    else
        sprintf (buffer, "SOUR:PULM:TRA:CAT? '%s';*OPC?", directoryPath);

    rssma_status = rssma_getCatalog (instrSession,
                                     buffer,
                                     (ViUInt32) arraySize,
                                     catalogList, "catalogList");

    return rssma_status;
}

/*===========================================================================*/
/* Function: PULM Delete Pulse Train File
/* Purpose:  This function deletes the specified pulse train file.
/*           
/*           The files are stored with the fixed file extensions *.pulstrn 
/*           in a directory of the user's choice. The directory applicable to 
/*           the command is defined with the command MMEMory:CDIR. To access 
/*           the files in this directory, only the file name has to be given 
/*           without the path and the file extension. 
/*           
/*           Note(s):
/*           
/*           (1) This function is only available with option R&S SMF-K27.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PULMDeletePulseTrainFile(
    ViSession   instrSession,
    ViString    fileName
)
{
    INIT;
    CHECKOPT(K27);
    
    return SETPARAMSTRINGQUOTED (instrSession,
                              "SOUR:PULM:TRA:DEL",
                              "'",
                              fileName);    
}

/*===========================================================================*/
/* Function: PULM Select Pulse Train File
/* Purpose:  This function selects the specified pulse train file. If a new 
/*           file is to be created, the name can be entered here. The file is 
/*           created if it does not yet exist. The file selected here is 
/*           available for the further processing steps (editing) and is used 
/*           in the instrument when the pulse train mode is activated. 
/*           
/*           The files are stored with the fixed file extensions *.pulstrn 
/*           in a directory of the user's choice. The directory applicable to 
/*           the command is defined with the command MMEMory:CDIR. 
/*           
/*           *RST does not affect data lists.
/*           
/*           Note(s):
/*           (1) This function is only available with option R&S SMA-K27.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PULMSelectPulseTrainFile(
    ViSession   instrSession,
    ViString    fileName
)
{
    INIT;
    CHECKOPT(K27);
    
    return SETPARAMSTRINGQUOTED (instrSession,
                              "SOUR:PULM:TRA:SEL",
                              "'",
                              fileName);    
}

/*===========================================================================*/
/* Function: Set PULM Train Offtime
/* Purpose:  This function fills the Offtime part of the selected file with 
/*           data. The data can be given either as a list of numbers (list 
/*           can be of any length and list entries must be separated by 
/*           commas) or as binary block data. When block data is transferred, 
/*           8 (4) bytes are always interpreted as a floating-point number 
/*           with double accuracy (see the command FORMat:DATA). The maximum 
/*           length is 1023 values.
/*           
/*           *RST does not affect data lists. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMTrainOfftime(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViReal64 _VI_FAR    offtime[]
)
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    INIT;
    
    CHECKOPT(K27);
    CHECKNULL (offtime);

    p2buf = pbuffer = malloc((size_t)  arraySize*20 + 30);
    p2buf += sprintf(pbuffer, ":SOUR:PULM:TRA:OFFT ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%f,", offtime[i]);

    *--p2buf = '\0';

    rssma_status = rssma_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssma_status;
}

/*===========================================================================*/
/* Function: Get PULM Train Offtime
/* Purpose:  This function returns the Offtime part of the selected file 
/*           with data. The data can be given either as a list of numbers 
/*           (list can be of any length and list entries must be separated by 
/*           commas) or as binary block data. When block data is transferred, 
/*           8 (4) bytes are always interpreted as a floating-point number 
/*           with double accuracy (see the command FORMat:DATA). The maximum 
/*           length is 1023 values.
/*           
/*           *RST does not affect data lists. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTrainOfftime(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViReal64 _VI_FAR    offtime[]
)
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMREAL64ARRAY (instrSession,
                                   "SOUR:PULM:TRA:OFFT?",
                                   offtime,
                                   arraySize);
}

/*===========================================================================*/
/* Function: Get Train Offtime Points
/* Purpose:  This function queries queries the length (in points) of the 
/*           offtime component of the selected list. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetTrainOfftimePoints(
    ViSession   instrSession,
    ViInt32*    numberofOfftimePoints
)
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:PULM:TRA:OFFT:POIN?",
                             numberofOfftimePoints);    
}

/*===========================================================================*/
/* Function: Set PULM Train Ontime
/* Purpose:  This function fills the Ontime part of the selected file with 
/*           data. The data can be given either as a list of numbers (list 
/*           can be of any length and list entries must be separated by 
/*           commas) or as binary block data. When block data is transferred, 
/*           8 (4) bytes are always interpreted as a floating-point number 
/*           with double accuracy (see the command FORMat:DATA). The maximum 
/*           length is 1023 values.
/*           
/*           *RST does not affect data lists. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMTrainOntime(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViReal64 _VI_FAR    ontime[]
)
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    INIT;
    
    CHECKOPT(K27);
    CHECKNULL (ontime);

    p2buf = pbuffer = malloc((size_t)  arraySize*20 + 30);
    p2buf += sprintf(pbuffer, ":SOUR:PULM:TRA:ONT ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%f,", ontime[i]);

    *--p2buf = '\0';

    rssma_status = rssma_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssma_status;
}

/*===========================================================================*/
/* Function: Get PULM Train Ontime
/* Purpose:  This function returns the Ontime part of the selected file with 
/*           data. The data can be given either as a list of numbers (list 
/*           can be of any length and list entries must be separated by 
/*           commas) or as binary block data. When block data is transferred, 
/*           8 (4) bytes are always interpreted as a floating-point number 
/*           with double accuracy (see the command FORMat:DATA). The maximum 
/*           length is 1023 values.
/*           
/*           *RST does not affect data lists. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTrainOntime(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViReal64 _VI_FAR    ontime[]
)
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMREAL64ARRAY (instrSession,
                                   "SOUR:PULM:TRA:ONT?",
                                   ontime,
                                   arraySize);
}

/*===========================================================================*/
/* Function: Get Train Ontime Points
/* Purpose:  This function queries queries the length (in points) of the 
/*           ontime component of the selected list. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetTrainOntimePoints(
    ViSession   instrSession,
    ViInt32*    numberofOntimePoints
)
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:PULM:TRA:ONT:POIN?",
                             numberofOntimePoints);    
}

/*===========================================================================*/
/* Function: Set PULM Train Repetition
/* Purpose:  This function sets the number of repetitions for each
/*           ontime/offtime value pair. The maximum length is 1023 values. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMTrainRepetition (ViSession instrSession,
                                                ViInt32 arraySize,
                                                ViInt32 repetition[])
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    INIT;
    
    CHECKOPT(K27);
    CHECKNULL (repetition);

    p2buf = pbuffer = malloc((size_t)  arraySize*20 + 30);
    p2buf += sprintf(pbuffer, ":SOUR:PULM:TRA:REP ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%ld,", repetition[i]);

    *--p2buf = '\0';

    rssma_status = rssma_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssma_status;
}

/*===========================================================================*/
/* Function: Get PULM Train Repetition
/* Purpose:  This function returns the number of repetitions for each
/*           ontime/offtime value pair. The maximum length is 1023 values. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTrainRepetition (ViSession instrSession,
                                                ViInt32 arraySize,
                                                ViInt32 repetition[])
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMINT32ARRAY (instrSession,
                                   "SOUR:PULM:TRA:REP?",
                                   repetition,
                                   arraySize);
}

/*===========================================================================*/
/* Function: Get PULM Train Repetition Points
/* Purpose:  This function queries the length (in points) of the repetition
/*           component of the selected list. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTrainRepetitionPoints (ViSession instrSession,
                                                  ViInt32 *number_ofRepetitionPoints)
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:PULM:TRA:REP:POIN?",
                             number_ofRepetitionPoints);    
}

/*===========================================================================
 * Function: Set PULM Train Import Export List Mode
 * Purpose:  This function selects if pulse train lists should be imported or 
 *           exported.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportListMode (ViSession instrSession,
                                                          ViInt32 listMode)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:MODE",
                          listMode,
                          dexcListModeArr);
}

/*===========================================================================
 * Function: Get PULM Train Import Export List Mode
 * Purpose:  This function returns whether pulse train lists is imported or 
 *           exported.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportListMode (ViSession instrSession,
                                                          ViInt32 *listMode)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:MODE?",
                          listMode,
                          dexcListModeArr);
}

/*===========================================================================
 * Function: Set PULM Train Import Export List File
 * Purpose:  This function calls the File Manager for selecting the pulse train 
 *           list to be exported (source) into an ASCII file or the destination 
 *           for the ASCII file to be imported (destination) in. 
 *           
 *           The list mode files are stored with the fixed file extensions 
 *           *.lsw in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR. A path can also be specified, in which case the 
 *           files are stored or loaded in the specified directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportListFile (ViSession instrSession,
                                                          ViString sourceFile)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:PULM:TRA:DEXC:SEL",
                                    "'",
                                    sourceFile);
}

/*===========================================================================
 * Function: Get PULM Train Import Export List File
 * Purpose:  This function calls the File Manager for querying the pulse
 *           train list to be exported (source) into an ASCII file or the
 *           destination for the ASCII file to be imported (destination) in.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportListFile (ViSession instrSession,
                                                          ViChar sourceFile[])
{
    return GETPARAMSTRING (instrSession,
                              "SOUR:PULM:TRA:DEXC:SEL?",
                              sourceFile,
                              128);
}

/*===========================================================================
 * Function: PULM Train Import Export Execute
 * Purpose:  This function starts the export or import of the selected file. 
 *           
 *           When import is selected, the ASCII file is imported as pulse 
 *           train list.
 *           
 *           When export is selected, the pulse train list is exported into 
 *           the selected ASCII file.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_PULMTrainImportExportExecute (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SOUR:PULM:TRA:DEXC:EXEC");
}

/*===========================================================================
 * Function: Set PULM Train Import Export ASCII File
 * Purpose:  This function calls the File Manager for selecting the ASCII 
 *           file to be imported into a pulse train list (source) or the ASCII 
 *           file the pulse train list is exported (destination) in.
 *           
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified, in which case the 
 *           files are stored or loaded in the specified directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportASCIIFile (ViSession instrSession,
                                                           ViString ASCIISourceFile)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:PULM:TRA:DEXC:AFIL:SEL",
                                    "'",
                                    ASCIISourceFile);
}

/*===========================================================================
 * Function: Get PULM Train Import Export ASCII File
 * Purpose:  This function calls the File Manager for querying the ASCII file
 *           to be imported into a pulse train list (source) or the ASCII
 *           file the pulse train list is exported (destination) in.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportASCIIFile (ViSession instrSession,
                                                           ViChar ASCIISourceFile[])
{
    return GETPARAMSTRING (instrSession,
                              "SOUR:PULM:TRA:DEXC:AFIL:SEL?",
                              ASCIISourceFile,
                              128);
}

/*===========================================================================
 * Function: Set PULM Train Import Export ASCII Extension
 * Purpose:  This function selects the file extension of the ASCII file to 
 *           be imported or exported. Selection TXT (text file) or CSV (Excel 
 *           file) is available. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportASCIIExtension (ViSession instrSession,
                                                                ViInt32 extension)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:EXT",
                          extension,
                          dexcExtensionArr);
}

/*===========================================================================
 * Function: Get PULM Train Import Export ASCII Extension
 * Purpose:  This function queries the selected file extension of the ASCII
 *           file to be imported or exported. Selection TXT (text file) or
 *           CSV (Excel file) is available.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportASCIIExtension (ViSession instrSession,
                                                                ViInt32 *extension)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:EXT?",
                          extension,
                          dexcExtensionArr);
}

/*===========================================================================
 * Function: Set PULM Train Import Export ASCII Column Separator
 * Purpose:  This function selects the separator between the frequency and 
 *           level column of the ASCII table. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportASCIIColumnSeparator (ViSession instrSession,
                                                                      ViInt32 separator)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:SEP:COL",
                          separator,
                          dexcSeparatorArr);
}

/*===========================================================================
 * Function: Get PULM Train Import Export ASCII Column Separator
 * Purpose:  This function returns the separator between the frequency and 
 *           level column of the ASCII table. 
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportASCIIColumnSeparator (ViSession instrSession,
                                                                      ViInt32 *separator)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:SEP:COL?",
                          separator,
                          dexcSeparatorArr);
}

/*===========================================================================
 * Function: Set PULM Train Import Export ASCII Decimal Separator
 * Purpose:  This function selects the decimal separator used in the ASCII 
 *           data between '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMTrainImportExportASCIIDecimalSeparator (ViSession instrSession,
                                                                       ViInt32 decimalSeparator)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:SEP:DEC",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*===========================================================================
 * Function: Get PULM Train Import Export ASCII Decimal Separator
 * Purpose:  This function returns the decimal separator used in the ASCII 
 *           data between '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMTrainImportExportASCIIDecimalSeparator (ViSession instrSession,
                                                                       ViInt32 *decimalSeparator)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:SEP:DEC?",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*===========================================================================
 * Function: PULM Train Import Export Catalog
 * Purpose:  This function requests a list of available ASCII files for 
 *           export/import of pulse train data. The individual files are 
 *           separated by commas.
 *            
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified, in which case the 
 *           files in the specified directory are read.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_PULMTrainImportExportCatalog (ViSession instrSession,
                                                      ViInt32 arraySize,
                                                      ViChar catalogList[])
{
    return rssma_getCatalog (instrSession,
                             ":PULM:TRA:DEXC:AFIL:CAT?;*OPC?",
                             arraySize,
                             catalogList,"catalogList");

}

/*===========================================================================*/
/* Function: Configure DMEM
/* Purpose:  This function configure the DME modulation.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confDMEM (ViSession instrSession, ViBoolean state,
                                  ViInt32 mode, ViBoolean lowEmission,
                                  ViBoolean singlePulse, ViBoolean pulseSquitter,
                                  ViInt32 rangeDistance, ViInt32 rate)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetDMEMState (instrSession, state));
    
    CHECKERR (rssma_SetDMEMMode (instrSession, mode));
    CHECKERR (rssma_SetDMEMLowSpectralEmission (instrSession, lowEmission));    
    CHECKERR (rssma_SetDMEMSinglePulse (instrSession, singlePulse));    
    if (mode == RSSMA_DMEM_MODE_REP)
    {
        CHECKERR (rssma_SetDMEMSquitter (instrSession, pulseSquitter));    
        CHECKERR (rssma_SetDMEMRangeDistance (instrSession, rangeDistance));    }
    else
    {
        CHECKERR (rssma_SetDMEMRate (instrSession, rate));    }
    
    return rssma_status;    
}

/*===========================================================================*/
/* Function: Configure DMEM Echo
/* Purpose:  This function configures the echo pulse. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confDMEMEcho(
    ViSession   instrSession,
    ViBoolean   echoPulse,
    ViReal64    echoAttenuation,
    ViReal64    echoDelay
)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetDMEMEchoPulse (instrSession, echoPulse));
    
    CHECKERR (rssma_SetDMEMEchoAttenuation (instrSession, echoAttenuation));
    CHECKERR (rssma_SetDMEMEchoDelay (instrSession, echoDelay));    
    return rssma_status;    
}

/*===========================================================================*/
/* Function: Configure DMEM Pulse
/* Purpose:  This function configure the pulse.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confDMEMPulse(
    ViSession   instrSession,
    ViInt32 channelSuffix,
    ViInt32 pulseShape,
    ViReal64    pulseRise,
    ViReal64    pulseFall,
    ViReal64    pulseWidth,
    ViReal64    pulseSpacing
)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetDMEMChannelSuffix (instrSession, channelSuffix));
        
    CHECKERR (rssma_SetDMEMPulseShape (instrSession, pulseShape));    
    CHECKERR (rssma_SetDMEMPulseRise (instrSession, pulseRise));    
    CHECKERR (rssma_SetDMEMPulseFall (instrSession, pulseFall));    
    CHECKERR (rssma_SetDMEMPulseWidth (instrSession, pulseWidth));    
    CHECKERR (rssma_SetDMEMPulseSpacing (instrSession, pulseSpacing));    
    return rssma_status;    
}

/*===========================================================================*/
/* Function: Configure DMEM Trigger
/* Purpose:  This function configures the DMEM trigger.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confDMEMTrigger(
    ViSession   instrSession,
    ViInt32 triggerMode,
    ViReal64    triggerDelay,
    ViInt32 externalInputImpedance,
    ViInt32 extTriggerInputSlope
)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetDMEMTriggerMode (instrSession, triggerMode));
    
    CHECKERR (rssma_SetDMEMTriggerDelay (instrSession, triggerDelay));
    CHECKERR (rssma_SetDMEMExternalInputImpedance (instrSession, externalInputImpedance));    
    CHECKERR (rssma_SetDMEMExtTriggerInputSlope (instrSession, extTriggerInputSlope));    
    return rssma_status;    
}

/*===========================================================================*/
/* Function: DMEM Preset
/* Purpose:  This function activate the DME defaultsetting (see *RST value 
/*           column of command descriptions).
/*           
/*           Note(s):
/*           
/*           (1) This command triggers an event and therefore has no *RST 
/*           value.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_DMEMPreset(
    ViSession   instrSession
)
{
    INIT;
    CHECKOPT(K26);
    
    return rssma_sendCmd (instrSession, "SOUR:DME:PRES");    
}



/*===========================================================================*/
/* Function: Get DMEM Reply Efficiency
/* Purpose:  This function queries the measured reply efficiency in percent. 
/*           The measurement is the ratio of the number of measured valid 
/*           reply pulse pairs to transmitted pulse pairs in a measurement 
/*           cycle.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMAnalysisReplyEfficiency(
    ViSession   instrSession,
    ViReal64*   efficiency
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ANAL:EFF?",
                              efficiency);    
}

/*===========================================================================*/
/* Function: Set DMEM Reply Efficiency State
/* Purpose:  This function activate/deactivate the measurement of the 
/*           average reply delay of all valid pulse pairs in a measurement 
/*           cycle. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMReplyEfficiencyState(
    ViSession   instrSession,
    ViBoolean   efficiencyState
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:ANAL:EFF:STAT",
                               efficiencyState);    
}

/*===========================================================================*/
/* Function: Get DMEM Reply Efficiency State
/* Purpose:  This function returns the measurement of the average reply 
/*           delay of all valid pulse pairs in a measurement cycle state. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMReplyEfficiencyState(
    ViSession   instrSession,
    ViBoolean*  efficiencyState
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:ANAL:EFF:STAT?",
                               efficiencyState);    
}

/*===========================================================================*/
/* Function: Get DMEM Reply Efficiency Validity
/* Purpose:  This function queries if there are valid measurement values
/*           available in the set measurement window. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMReplyEfficiencyValidity (ViSession instrSession,
                                                        ViInt32 *efficiencyValidity)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:DME:ANAL:EFF:OK?",
                             efficiencyValidity);    
}

/*===========================================================================*/
/* Function: Set DMEM Measurement Count
/* Purpose:  This function sets the number of pulse pairs which are sent 
/*           from the R&S SMA (= interrogator) to the ground station in one 
/*           measurement cycle.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMMeasurementCount(
    ViSession   instrSession,
    ViInt32 measurementCount
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMINT32 (instrSession,
                             "SOUR:DME:ANAL:GATE:COUN",
                             measurementCount,
                             1,1000);    
}

/*===========================================================================*/
/* Function: Get DMEM Measurement Count
/* Purpose:  This function returns the number of pulse pairs which are sent 
/*           from the R&S SMA (= interrogator) to the ground station in one 
/*           measurement cycle.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMMeasurementCount(
    ViSession   instrSession,
    ViInt32*    measurementCount
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:DME:ANAL:GATE:COUN?",
                             measurementCount);    
}

/*===========================================================================*/
/* Function: Set DMEM Expected Reply Delay
/* Purpose:  This function sets the expected reply delay. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMExpectedReplyDelay(
    ViSession   instrSession,
    ViReal64    replyDelay
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMREAL64 (instrSession,
                             "SOUR:DME:ANAL:GATE:EDEL",
                             replyDelay,
                             0,157e-6);    
}

/*===========================================================================*/
/* Function: Get DMEM Expected Reply Delay
/* Purpose:  This function returns the expected reply delay. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMExpectedReplyDelay(
    ViSession   instrSession,
    ViReal64*   replyDelay
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                             "SOUR:DME:ANAL:GATE:EDEL?",
                             replyDelay);    
}

/*===========================================================================*/
/* Function: Set DMEM Gate Length
/* Purpose:  This function sets the gate length for the measurement window. 
/*           
/*           The measurement gate settings determine the measurement window 
/*           (expected reply delay +/- gate length/2). Only reply pulses 
/*           within the measurement window are used to evaluate the delay 
/*           time and reply efficiency. 
/*           
/*           The delay measurement is averaged within the measurement cycle. 
/*           The reply efficiency is calculated once for each measurement 
/*           cycle.
/*           
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMGateLength(
    ViSession   instrSession,
    ViReal64    gateLength
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMREAL64 (instrSession,
                             "SOUR:DME:ANAL:GATE:LENG",
                             gateLength,
                             0,0);    
}

/*===========================================================================*/
/* Function: Get DMEM Gate Length
/* Purpose:  This function returns the gate length for the measurement 
/*           window. 
/*           
/*           The measurement gate settings determine the measurement window 
/*           (expected reply delay +/- gate length/2). Only reply pulses 
/*           within the measurement window are used to evaluate the delay 
/*           time and reply efficiency. 
/*           
/*           The delay measurement is averaged within the measurement cycle. 
/*           The reply efficiency is calculated once for each measurement 
/*           cycle.
/*           
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMGateLength(
    ViSession   instrSession,
    ViReal64*   gateLength
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                             "SOUR:DME:ANAL:GATE:LENG?",
                             gateLength);    
}

/*===========================================================================*/
/* Function: Get DMEM Normalize
/* Purpose:  This function performs a normalization of the test setup. The 
/*           delay due to the test setup is measured and subsequently 
/*           considered in the reply measurements.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMNormalize(
    ViSession   instrSession,
    ViReal64*   delay
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ANAL:NORM?",
                              delay);    
}

/*===========================================================================*/
/* Function: Get DMEM Peak Level
/* Purpose:  This function queries the measured average peak level of all 
/*           valid pulse pairs in a measurement cycle.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPeakLevel(
    ViSession   instrSession,
    ViReal64*   peakLevel
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ANAL:POW?",
                              peakLevel);    
}

/*===========================================================================*/
/* Function: Set DMEM Peak Level State
/* Purpose:  This function activate/deactivate the measurement of the 
/*           average peak level of all valid pulse pairs in a measurement 
/*           cycle.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMPeakLevelState(
    ViSession   instrSession,
    ViBoolean   peakLevel
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:ANAL:POW:STAT",
                               peakLevel);    
}

/*===========================================================================*/
/* Function: Get DMEM Peak Level State
/* Purpose:  This function returns the measurement of the average peak level 
/*           of all valid pulse pairs in a measurement cycle state.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPeakLevelState(
    ViSession   instrSession,
    ViBoolean*  peakLevel
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:ANAL:POW:STAT?",
                               peakLevel);    
}

/*===========================================================================*/
/* Function: Get DMEM Peak Level Validity
/* Purpose:  This function queries if there are measurement values available
/*           in the set measurement window.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPeakLevelValidity (ViSession instrSession,
                                                  ViInt32 *peakLevelValidity)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMINT32 (instrSession,
                               "SOUR:DME:ANAL:POW:OK?",
                               peakLevelValidity);
}

/*===========================================================================*/
/* Function: Get DMEM Pulse Repetition Frequency
/* Purpose:  This function queries the measured mean pulse repetition rate 
/*           of the DME ground station. For this measurement all received 
/*           pulses of the DME ground station are considered.
/*           
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPulseRepetitionFrequency(
    ViSession   instrSession,
    ViReal64*   pulseRepetitionFrequency
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ANAL:PRR?",
                              pulseRepetitionFrequency);    
}

/*===========================================================================*/
/* Function: Set DMEM Pulse Repetition Frequency State
/* Purpose:  This function activate/deactivate the measurement of the mean 
/*           pulse repetition rate of the DME ground station. For this 
/*           measurement all received pulses of the DME ground station are 
/*           considered.
/*           
/*           This measurement and the measurement of the delay, efficiency 
/*           and average peak power are performed alternately. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMPulseRepetitionFrequencyState(
    ViSession   instrSession,
    ViBoolean   pulseRepFreqStat
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:ANAL:PRR:STAT",
                               pulseRepFreqStat);    
}

/*===========================================================================*/
/* Function: Get DMEM Pulse Repetition Frequency State
/* Purpose:  This function returns the measurement of the mean pulse 
/*           repetition rate of the DME ground station state.. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPulseRepetitionFrequencyState(
    ViSession   instrSession,
    ViBoolean*  pulseRepFreqStat
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:ANAL:PRR:STAT?",
                               pulseRepFreqStat);    
}

/*===========================================================================*/
/* Function: Get DMEM Pulse Repetition Frequency Validity
/* Purpose:  This function queries if there are valid measurement values
/*           available in the set measurement window.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPulseRepetitionFrequencyValidity (ViSession instrSession,
                                                                 ViInt32 *pulseRepetitionFreqValidity)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMINT32 (instrSession,
                               "SOUR:DME:ANAL:PRR:OK?",
                               pulseRepetitionFreqValidity);    
}

/*===========================================================================*/
/* Function: Set DMEM Analysis State
/* Purpose:  This function activate/deactivate the DME analysis. The setting 
/*           is only available after connecting the R&S NRP-Z81 power sensor. 
/*           
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMAnalysisState(
    ViSession   instrSession,
    ViBoolean   analysisState
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:ANAL:STAT",
                               analysisState);    
}

/*===========================================================================*/
/* Function: Get DMEM Analysis State
/* Purpose:  This function returns the DME analysis state. The setting is 
/*           only available after connecting the R&S NRP-Z81 power sensor. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMAnalysisState(
    ViSession   instrSession,
    ViBoolean*  analysisState
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:ANAL:STAT?",
                               analysisState);    
}

/*===========================================================================*/
/* Function: Get DMEM Reply Delay
/* Purpose:  This function queries the measured average reply delay of all 
/*           valid pulse pairs in a measurement cycle. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMReplyDelay(
    ViSession   instrSession,
    ViReal64*   replyDelay
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ANAL:TIME?",
                              replyDelay);    
}

/*===========================================================================*/
/* Function: Set DMEM Reply Delay State
/* Purpose:  This function activate/deactivate the measurement of the 
/*           average reply delay of all valid pulse pairs in a measurement 
/*           cycle. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMReplyDelayState(
    ViSession   instrSession,
    ViBoolean   replyDelayState
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:ANAL:TIME:STAT",
                               replyDelayState);    
}

/*===========================================================================*/
/* Function: Get DMEM Reply Delay State
/* Purpose:  This function returns the measurement of the average reply 
/*           delay of all valid pulse pairs in a measurement cycle state. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMReplyDelayState(
    ViSession   instrSession,
    ViBoolean*  replyDelayState
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:ANAL:TIME:STAT?",
                               replyDelayState);    
}

/*===========================================================================*/
/* Function: Get DMEM Reply Delay Validity
/* Purpose:  This function queries if there are valid measurement values
/*           available in the set measurement window. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMReplyDelayValidity (ViSession instrSession,
                                                   ViInt32 *replyDelayValidity)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMINT32 (instrSession,
                               "SOUR:DME:ANAL:TIME:OK?",
                               replyDelayValidity);    
}

/*===========================================================================*/
/* Function: Get DMEM AnalysisTrigger Level
/* Purpose:  This function determines the trigger level = 50% voltage point 
/*			 of first pulse of the external DME interrogation signal.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMAnalysisTriggerLevel (ViSession instrSession,
                                                     ViReal64 *triggerLevel)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                               "SOUR:DME:ANAL:TRIG:SEAR?",
                               triggerLevel); 
}

/*===========================================================================*/
/* Function: Configure DMEM Id 
/* Purpose:  This function configures the generation of ID.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_confDMEMId(
    ViSession   instrSession,
    ViBoolean   idState,
    ViReal64    idFrequency,
    ViString    idCode,
    ViReal64    idDotLength,
    ViInt32 idPeriod
)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetDMEMIdState (instrSession, idState));
    
    CHECKERR (rssma_SetDMEMIdFrequency (instrSession, idFrequency));
    CHECKERR (rssma_SetDMEMIdCode (instrSession, idCode));    
    CHECKERR (rssma_SetDMEMIdDotLength (instrSession, idDotLength));    
    CHECKERR (rssma_SetDMEMIdPeriod (instrSession, idPeriod));    
    return rssma_status;    
}

/*===========================================================================*/
/* Function: DMEM Id Preset
/* Purpose:  This function sets the default settings for the ID signal. This 
/*           function is available for reply mode only. This command triggers 
/*           an event and therefore has no *RST value.
/*           
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_DMEMIdPreset(
    ViSession   instrSession
)
{
    INIT;
    CHECKOPT(K26);
    
    return rssma_sendCmd (instrSession, "SOUR:DME:ID:PRES");    
}

/*===========================================================================*/
/* Function: Set DMEM Id State
/* Purpose:  This function activate/deactivate the generation of the 
/*           identification sequence (ID signal) of the ground station. The 
/*           Id signal consists of a series of pulse pairs transmitted at a 
/*           rate of 1350 Hz. 
/*           
/*           The transmission of the ground signal is interrupted every 40 
/*           seconds (ID period) and one ID sequence is transmitted instead. 
/*           The "key down time" of the ground signal corresponds to the 
/*           period of transmission for a dot or dash in the morse code ID 
/*           sequence (e.g. 100ms for a dot). During the key down times reply 
/*           pulses are not transmitted, however, they are transmitted 
/*           between the key down times.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMIdState(
    ViSession   instrSession,
    ViBoolean   idState
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:ID:STAT",
                               idState);    
}

/*===========================================================================*/
/* Function: Get DMEM Id State
/* Purpose:  This function returns state of the generation of the 
/*           identification sequence (ID signal) of the ground station. The 
/*           Id signal consists of a series of pulse pairs transmitted at a 
/*           rate of 1350 Hz. 
/*           
/*           The transmission of the ground signal is interrupted every 40 
/*           seconds (ID period) and one ID sequence is transmitted instead. 
/*           The "key down time" of the ground signal corresponds to the 
/*           period of transmission for a dot or dash in the morse code ID 
/*           sequence (e.g. 100ms for a dot). During the key down times reply 
/*           pulses are not transmitted, however, they are transmitted 
/*           between the key down times.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMIdState(
    ViSession   instrSession,
    ViBoolean*  idState
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:ID:STAT?",
                               idState);    
}

/*===========================================================================*/
/* Function: Set DMEM Id Code
/* Purpose:  This function sets the coding of the ID signal by the 
/*           international short name of the airport (e.g. MUC for the Munic 
/*           airport). (ID signal). The ID tone is sent according to the 
/*           selected code. This function is available for reply mode only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMIdCode(
    ViSession   instrSession,
    ViString    idCode
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMSTRINGQUOTED (instrSession,
                              "SOUR:DME:ID:CODE",
                              "\"",
                              idCode);    
}

/*===========================================================================*/
/* Function: Get DMEM Id Code
/* Purpose:  This function returns the coding of the ID signal by the 
/*           international short name of the airport (e.g. MUC for the Munic 
/*           airport). (ID signal). The ID tone is sent according to the 
/*           selected code. This function is available for reply mode only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMIdCode(
    ViSession   instrSession,
    ViChar _VI_FAR  idCode[]
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMSTRING (instrSession,
                              "SOUR:DME:ID:CODE?",
                              idCode,
                              1024);    
}

/*===========================================================================*/
/* Function: Set DMEM Id Time Schema
/* Purpose:  This function selects if the set dot length determines the dash
/*           length (= three times the dot length) (setting STD) or if all
/*           length parameters can be set independently (setting USER. (ID
/*           signal).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMIdTimeSchema (ViSession instrSession,
                                             ViInt32 idTimeSchema)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:DME:ID:TSCH",
                          idTimeSchema,
                          TimeSchemaArr);    
}

/*===========================================================================*/
/* Function: Get DMEM Id Time Schema
/* Purpose:  This function returns whether the set dot length is determined by the dash
/*           length (= three times the dot length) (setting STD) or all
/*           length parameters is set independently (setting USER. (COM/ID
/*           signal).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMIdTimeSchema (ViSession instrSession,
                                             ViInt32 *idTimeSchema)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:DME:ID:TSCH?",
                          idTimeSchema,
                          TimeSchemaArr);    
}

/*===========================================================================*/
/* Function: Set DMEM Id Dot Length
/* Purpose:  This function sets the length of a morse dot by means of 
/*           seconds. The length of the dash (3xdot), space (=dot) and letter 
/*           space (=3xdot) is also determined by this entry. This function 
/*           is available for reply mode only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMIdDotLength(
    ViSession   instrSession,
    ViReal64    idDotLength
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:ID:DOT",
                              idDotLength,
                              50.0e-3,0.1);    
}

/*===========================================================================*/
/* Function: Get DMEM Id Dot Length
/* Purpose:  This function returns the length of a morse dot by means of 
/*           seconds. The length of the dash (3xdot), space (=dot) and letter 
/*           space (=3xdot) is also determined by this entry. This function 
/*           is available for reply mode only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMIdDotLength(
    ViSession   instrSession,
    ViReal64*   idDotLength
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ID:DOT?",
                              idDotLength);    
}

/*===========================================================================*/
/* Function: Set DMEM Id Dash Length
/* Purpose:  This function sets the length of a morse dash by means of
/*           seconds. This function is available for user time schema only
/*           (rssma_SetDMEMIdTimeSchema).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMIdDashLength (ViSession instrSession,
                                             ViReal64 idDashLength)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:ID:DASH",
                              idDashLength,
                              0.0,0.0);    
}

/*===========================================================================*/
/* Function: Get DMEM Id Dash Length
/* Purpose:  This function returns the length of a morse dash by means of
/*           seconds. This function is available for user time schema only
/*           (rssma_SetDMEMIdTimeSchema).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMIdDashLength (ViSession instrSession,
                                             ViReal64 *idDashLength)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ID:DASH?",
                              idDashLength);    
}

/*===========================================================================*/
/* Function: Set DMEM Id Letter Space
/* Purpose:  This function sets the length of a letter space by means of
/*           seconds. This function is available user time schema only
/*           (rssma_SetDMEMIdTimeSchema).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMIdLetterSpace (ViSession instrSession,
                                              ViReal64 idLetterSpace)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:ID:LETT",
                              idLetterSpace,
                              0.0,0.0);    
}

/*===========================================================================*/
/* Function: Get DMEM Id Letter Space
/* Purpose:  This function returns the length of a letter space by means of
/*           seconds. This function is available user time schema only
/*           (rssma_SetDMEMIdTimeSchema).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMIdLetterSpace (ViSession instrSession,
                                              ViReal64 *idLetterSpace)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ID:LETT?",
                              idLetterSpace);    
}

/*===========================================================================*/
/* Function: Set DMEM Id Symbol Space
/* Purpose:  This function sets the length of the symbol space by means of
/*           seconds. This function is available for user time schema only
/*           (rssma_SetDMEMIdTimeSchema).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMIdSymbolSpace (ViSession instrSession,
                                              ViReal64 idSymbolSpace)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:ID:SYMB",
                              idSymbolSpace,
                              0.0,0.0);    
}

/*===========================================================================*/
/* Function: Get DMEM Id Symbol Space
/* Purpose:  This function returns the length of the symbol space by means of
/*           seconds. This function is available for user time schema only
/*           (rssma_SetDMEMIdTimeSchema).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMIdSymbolSpace (ViSession instrSession,
                                              ViReal64 *idSymbolSpace)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ID:SYMB?",
                              idSymbolSpace);    
}

/*===========================================================================*/
/* Function: Set DMEM Id Period
/* Purpose:  This function sets the period of the ID sequence by means of 
/*           seconds. This function is available for reply mode only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMIdPeriod(
    ViSession   instrSession,
    ViInt32 idPeriod
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMINT32 (instrSession,
                             "SOUR:DME:ID:PER",
                             idPeriod,
                             0,120);    
}

/*===========================================================================*/
/* Function: Get DMEM Id Period
/* Purpose:  This function returns the period of the ID sequence by means of 
/*           seconds. This function is available for reply mode only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMIdPeriod(
    ViSession   instrSession,
    ViInt32*    idPeriod
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:DME:ID:PER?",
                             idPeriod);    
}

/*===========================================================================*/
/* Function: Set DMEM Id Frequency
/* Purpose:  This function sets the pulse repetition rate of the ID 
/*           sequence. This function is available for reply mode only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMIdFrequency(
    ViSession   instrSession,
    ViReal64    idFrequency
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:ID:RATE",
                              idFrequency,
                              100.0,10.0e3);    
}

/*===========================================================================*/
/* Function: Get DMEM Id Frequency
/* Purpose:  This function returns the pulse repetition rate of the ID 
/*           sequence. This function is available for reply mode only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMIdFrequency(
    ViSession   instrSession,
    ViReal64*   idFrequency
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ID:RATE?",
                              idFrequency);    
}

/*===========================================================================*/
/* Function: Set DMEM State
/* Purpose:  This function activate/deactivate DME modulation. The DME 
/*           modulation signal can be output at the LF connector.
/*           
/*           Note(s):
/*           
/*           (1) Modulations PULSE, VOR, ILS-GS, ILS-LOC, MKR-BCN, ADF, AM, 
/*           and internal FM/PhiM  are automatically switched off when DME 
/*           modulation is activated. Also, the pulse generator is 
/*           automatically switched off.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMState(
    ViSession   instrSession,
    ViBoolean   state
)
{
    INIT;
    CHECKOPT(K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:STAT",
                               state);    
}

/*===========================================================================*/
/* Function: Get DMEM State
/* Purpose:  This function returns the state of the DME modulation.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMState(
    ViSession   instrSession,
    ViBoolean*  state
)
{
    INIT;
    CHECKOPT(K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:STAT?",
                               state);    
}

/*===========================================================================*/
/* Function: Set DMEM Mode
/* Purpose:  This function selects the mode of the DME modulation. The mode 
/*           determines the signal type that is simulated. The exact timing 
/*           of the signal for each mode is determined by the selected 
/*           channel.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMMode(
    ViSession   instrSession,
    ViInt32 mode
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:DME:MODE",
                          mode,
                          dmemModeArr);
}

/*===========================================================================*/
/* Function: Get DMEM Mode
/* Purpose:  This function returns the mode of the DME modulation.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMMode(
    ViSession   instrSession,
    ViInt32*    mode
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:DME:MODE?",
                          mode,
                          dmemModeArr);
}

/*===========================================================================*/
/* Function: Set DMEM Pulse Shape
/* Purpose:  This function selects the pulse shape.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMPulseShape(
    ViSession   instrSession,
    ViInt32 pulseShape
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:DME:SHAP",
                          pulseShape,
                          dmemPulseShapeArr);
}

/*===========================================================================*/
/* Function: Get DMEM Pulse Shape
/* Purpose:  This function returns the pulse shape.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPulseShape(
    ViSession   instrSession,
    ViInt32*    pulseShape
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:DME:SHAP?",
                          pulseShape,
                          dmemPulseShapeArr);
}

/*===========================================================================*/
/* Function: Set DMEM Pulse Rise
/* Purpose:  This function sets the rise time of the pulse (10% to 90% of 
/*           peak voltage).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMPulseRise(
    ViSession   instrSession,
    ViReal64    pulseRise
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:RISE",
                              pulseRise,
                              0.5e-6,20.0e-6);
}

/*===========================================================================*/
/* Function: Get DMEM Pulse Rise
/* Purpose:  This function returns the rise time of the pulse (10% to 90% of 
/*           peak voltage).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPulseRise(
    ViSession   instrSession,
    ViReal64*   pulseRise
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:RISE?",
                              pulseRise);
}

/*===========================================================================*/
/* Function: Set DMEM Pulse Width
/* Purpose:  This function sets the pulse width (50% to 50% of peak 
/*           voltage).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMPulseWidth(
    ViSession   instrSession,
    ViReal64    pulseWidth
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:WIDT",
                              pulseWidth,
                              1.0e-6,100.0e-6);
}

/*===========================================================================*/
/* Function: Get DMEM Pulse Width
/* Purpose:  This function returns the pulse width (50% to 50% of peak 
/*           voltage).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPulseWidth(
    ViSession   instrSession,
    ViReal64*   pulseWidth
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:WIDT?",
                              pulseWidth);
}

/*===========================================================================*/
/* Function: Set DMEM Pulse Fall
/* Purpose:  This function sets the fall time of the pulse (90% to 10% of 
/*           peak voltage).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMPulseFall(
    ViSession   instrSession,
    ViReal64    pulseFall
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:FALL",
                              pulseFall,
                              0.5e-6,20.0e-6);
}

/*===========================================================================*/
/* Function: Get DMEM Pulse Fall
/* Purpose:  This function returns the fall time of the pulse (90% to 10% of 
/*           peak voltage).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPulseFall(
    ViSession   instrSession,
    ViReal64*   pulseFall
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:FALL?",
                              pulseFall);
}

/*===========================================================================*/
/* Function: Set DMEM Pulse Spacing
/* Purpose:  This function sets the spacing between the first and second 
/*           pulse of a pulse pair (time between half-voltage points on the 
/*           leading edge of each pulse).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMPulseSpacing(
    ViSession   instrSession,
    ViReal64    pulseSpacing
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:PPS",
                              pulseSpacing,
                              1.0e-6,100.0e-6);
}

/*===========================================================================*/
/* Function: Get DMEM Pulse Spacing
/* Purpose:  This function sets the spacing between the first and second 
/*           pulse of a pulse pair (time between half-voltage points on the 
/*           leading edge of each pulse).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMPulseSpacing(
    ViSession   instrSession,
    ViReal64*   pulseSpacing
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:PPS?",
                              pulseSpacing);
}

/*===========================================================================*/
/* Function: Set DMEM Pulse Squitter
/* Purpose:  This function activates the random pulse repetition rate. The 
/*           average repetition rate is 2700 pp/s. The pulse spacing is 
/*           distributed randomly in the range of 60 us to about 1500 us 
/*           according to EUROCAE EN-54 6.2.12.  The squitter pulses are 
/*           constantly sent by the ground station in order to ensure proper 
/*           operation and in order to ease synchronization of the aircraft 
/*           interrogator to the ground station.
/*           
/*           Note(s):
/*           
/*           (1) This function is available for Reply mode on and Echo Pulse 
/*           off only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMSquitter(
    ViSession   instrSession,
    ViBoolean   pulseSquitter
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:SQU",
                               pulseSquitter);
}

/*===========================================================================*/
/* Function: Get DMEM Pulse Squitter
/* Purpose:  This function returns the state of the random pulse repetition 
/*           rate. The average repetition rate is 2700 pp/s. The pulse 
/*           spacing is distributed randomly in the range of 60 us to about 
/*           1500 us according to EUROCAE EN-54 6.2.12.  The squitter pulses 
/*           are constantly sent by the ground station in order to ensure 
/*           proper operation and in order to ease synchronization of the 
/*           aircraft interrogator to the ground station.
/*           
/*           Note(s):
/*           
/*           (1) This function is available for Reply mode on and Echo Pulse 
/*           off only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMSquitter(
    ViSession   instrSession,
    ViBoolean*  pulseSquitter
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:SQU?",
                               pulseSquitter);
}

/*===========================================================================*/
/* Function: Set DMEM Rate
/* Purpose:  This function sets the pulse repetition rate for trigger 
/*           setting Auto or Ext Gated. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMRate(
    ViSession   instrSession,
    ViInt32 rate
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMINT32 (instrSession,
                             "SOUR:DME:RATE",
                             rate,
                             10,60000);
}

/*===========================================================================*/
/* Function: Get DMEM Rate
/* Purpose:  This function reeturns the pulse repetition rate for trigger 
/*           setting Auto or Ext Gated. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMRate(
    ViSession   instrSession,
    ViInt32*    rate
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:DME:RATE?",
                             rate);
}

/*===========================================================================*/
/* Function: Set DMEM Echo Pulse
/* Purpose:  This function activates an echo pulse pair. The echo pulses 
/*           have the same shape and timing as the original pulses.The delay 
/*           and amplitude of the echo pulses can be set. The selection is 
/*           possible for pulse squitter off (SOURce:DME:SQUitter OFF).
/*           
/*           
/*           Note(s):
/*           
/*           (1) This function is available for Reply mode on and Echo Pulse 
/*           off only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMEchoPulse(
    ViSession   instrSession,
    ViBoolean   echoPulse
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:ECHO",
                               echoPulse);
}

/*===========================================================================*/
/* Function: Get DMEM Echo Pulse
/* Purpose:  This function returns an echo pulse pair. The echo pulses have 
/*           the same shape and timing as the original pulses.The delay and 
/*           amplitude of the echo pulses can be set. The selection is 
/*           possible for pulse squitter off (SOURce:DME:SQUitter OFF).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMEchoPulse(
    ViSession   instrSession,
    ViBoolean*  echoPulse
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:ECHO?",
                               echoPulse);
}

/*===========================================================================*/
/* Function: Set DMEM Echo Attenuation
/* Purpose:  This function sets the attenuation of the echo pulses compared 
/*           to the original pulses. A positive attenuation leads to echo 
/*           pulses with reduced amplitude compared to the original pulses, 
/*           and a negative attenuation leads to echo pulses with increased 
/*           amplitude compared to the original pulses. 
/*           
/*           Examples:
/*           (1) DME:ECHO ON - activates the simulation of an echo pulse 
/*           pair.
/*               DME:ECHO:ATT 4 dB - 'an echo is simulated with an amplitude 
/*           that is reduced by 4 dB compared to the original pulse pair.
/*            
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMEchoAttenuation(
    ViSession   instrSession,
    ViReal64    echoAttenuation
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:ECHO:ATT",
                              echoAttenuation,
                              0.0,20.0);
}

/*===========================================================================*/
/* Function: Get DMEM Echo Attenuation
/* Purpose:  This function returns the attenuation of the echo pulses 
/*           compared to the original pulses. A positive attenuation leads to 
/*           echo pulses with reduced amplitude compared to the original 
/*           pulses, and a negative attenuation leads to echo pulses with 
/*           increased amplitude compared to the original pulses. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMEchoAttenuation(
    ViSession   instrSession,
    ViReal64*   echoAttenuation
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:ECHO:ATT?",
                              echoAttenuation);
}

/*===========================================================================*/
/* Function: Set DMEM Echo Delay
/* Purpose:  This function sets the delay from the first original pulse to 
/*           the first echo pulse. 
/*           
/*           Examples:
/*            DME:ECHO ON
/*            DME:ECHO:ATT 4 dB
/*            DME:ECHO:DEL 600 us
/*            
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMEchoDelay(
    ViSession   instrSession,
    ViReal64    echoDelay
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMREAL64 (instrSession,
                             "SOUR:DME:ECHO:DEL",
                             echoDelay,
                             10e-6,1000e-6);
}

/*===========================================================================*/
/* Function: Get DMEM Echo Delay
/* Purpose:  This function returns the delay from the first original pulse 
/*           to the first echo pulse. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMEchoDelay(
    ViSession   instrSession,
    ViReal64*   echoDelay
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMREAL64 (instrSession,
                             "SOUR:DME:ECHO:DEL?",
                             echoDelay);
}

/*===========================================================================*/
/* Function: Set DMEM Trigger Mode
/* Purpose:  This function selects the trigger mode for DME modulation 
/*           signals. For Reply mode only external trigger is possible 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMTriggerMode(
    ViSession   instrSession,
    ViInt32 triggerMode
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:DME:TRIG",
                          triggerMode,
                          chirpTriggerModeArr);
}

/*===========================================================================*/
/* Function: Get DMEM Trigger Mode
/* Purpose:  This function returns the trigger mode for DME modulation 
/*           signals. For Reply mode only external trigger is possible 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMTriggerMode(
    ViSession   instrSession,
    ViInt32*    triggerMode
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:DME:TRIG?",
                          triggerMode,
                          chirpTriggerModeArr);
}

/*===========================================================================*/
/* Function: Set DMEM Trigger Delay
/* Purpose:  This function sets the delay between the external trigger and 
/*           the first DME output pulse (50% voltage point of first pulse). 
/*           This command is effective for external trigger only.
/*           
/*           For DME Reply mode this simulates the defined delay of the DME 
/*           transponder and twice the run time of the signal (from 
/*           interrogator to transponder and back). The delay is a measure of 
/*           the range distance, thus, the two values are interdependent 
/*           according to:
/*           
/*           Delay = X/Y mode delay + range distance * 12.359 nm/s
/*           
/*           (X mode delay = 50 us, Y mode delay is 56 us)
/*           
/*           Changing one value automatically changes the other value. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMTriggerDelay(
    ViSession   instrSession,
    ViReal64    triggerDelay
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMREAL64 (instrSession,
                              "SOUR:DME:TRIG:DEL",
                              triggerDelay,
                              4.0e-6,50.0e-6);
}

/*===========================================================================*/
/* Function: Get DMEM Trigger Delay
/* Purpose:  This function returns the delay between the external trigger 
/*           and the first DME output pulse (50% voltage point of first 
/*           pulse). This command is effective for external trigger only.
/*           
/*           For DME Reply mode this simulates the defined delay of the DME 
/*           transponder and twice the run time of the signal (from 
/*           interrogator to transponder and back). The delay is a measure of 
/*           the range distance, thus, the two values are interdependent 
/*           according to:
/*           
/*           Delay = X/Y mode delay + range distance * 12.359 nm/s
/*           
/*           (X mode delay = 50 us, Y mode delay is 56 us)
/*           
/*           Changing one value automatically changes the other value. 
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMTriggerDelay(
    ViSession   instrSession,
    ViReal64*   triggerDelay
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:TRIG:DEL?",
                              triggerDelay);
}

/*===========================================================================*/
/* Function: Get DMEM Trigger Normalize Delay
/* Purpose:  This function performs a normalization of the test setup. The
/*           delay due to the test setup is measured and subsequently
/*           considered in the trigger delay. This function is effective for
/*           external power sensor trigger only (rssma_SetDMEMTriggerMode).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMTriggerNormalizeDelay (ViSession instrSession,
                                                      ViReal64 *triggerNormalizeDelay)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:DME:TRIG:DEL:NORM?",
                              triggerNormalizeDelay);
}

/*===========================================================================*/
/* Function: Set DMEM Gate Polarity
/* Purpose:  This function sets the polarity of the active level of the
/*           external gate trigger at the PULSE EXT connector. This function
/*           is effective for external gate only (rssma_SetDMEMTriggerMode).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMGatePolarity (ViSession instrSession,
                                             ViInt32 gatePolarity)
{
    INIT;
    CHECKOPT (K26);
    return SETPARAMENUM (instrSession,
                          "SOUR:DME:TRIG:GPOL",
                          gatePolarity,
                          polarityArr);
}

/*===========================================================================*/
/* Function: Get DMEM Gate Polarity
/* Purpose:  This function returns the polarity of the active level of the
/*           external gate trigger at the PULSE EXT connector. This function
/*           is effective for external gate only (rssma_SetDMEMTriggerMode).
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMGatePolarity (ViSession instrSession,
                                             ViInt32 *gatePolarity)
{
    INIT;
    CHECKOPT (K26);
    return GETPARAMENUM (instrSession,
                          "SOUR:DME:TRIG:GPOL?",
                          gatePolarity,
                          polarityArr);
}

/*===========================================================================*/
/* Function: Set DMEM External Input Impedance
/* Purpose:  This function selects the input impedance for the external 
/*           trigger and gate signal input PULSE EXT (10 kOhm or 50 Ohm).This 
/*           command is only available for external or gate trigger mode.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMExternalInputImpedance(
    ViSession   instrSession,
    ViInt32 externalInputImpedance
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:DME:TRIG:IMP",
                          externalInputImpedance,
                          dmemExternalInputImpedance);
}

/*===========================================================================*/
/* Function: Get DMEM External Input Impedance
/* Purpose:  This function returns the input impedance for the external 
/*           trigger and gate signal input PULSE EXT (10 kOhm or 50 Ohm).This 
/*           command is only available for external or gate trigger mode.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMExternalInputImpedance(
    ViSession   instrSession,
    ViInt32*    externalInputImpedance
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:DME:TRIG:IMP?",
                          externalInputImpedance,
                          dmemExternalInputImpedance);
}

/*===========================================================================*/
/* Function: Set DMEM Ext Trigger Input Slope
/* Purpose:  This function sets the polarity of the active slope of an 
/*           applied trigger at the PULSE EXT connector. This command is 
/*           effective for external trigger only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMExtTriggerInputSlope(
    ViSession   instrSession,
    ViInt32 extTriggerInputSlope
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:DME:TRIG:SLOP",
                          extTriggerInputSlope,
                          chirpTriggerSlopeArr);
}

/*===========================================================================*/
/* Function: Get DMEM Ext Trigger Input Slope
/* Purpose:  This function returns the polarity of the active slope of an 
/*           applied trigger at the PULSE EXT connector. This command is 
/*           effective for external trigger only.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMExtTriggerInputSlope(
    ViSession   instrSession,
    ViInt32*    extTriggerInputSlope
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:DME:TRIG:SLOP?",
                          extTriggerInputSlope,
                          chirpTriggerSlopeArr);
}

/*===========================================================================*/
/* Function: Set DMEM Range Distance
/* Purpose:  This function sets the simulated distance between interrogator 
/*           and transponder for reply mode. The distance is given in nautic 
/*           miles (nm). 1 nm is 1852.01 meters and corresponds to a run time 
/*           of 12.359 us.  This function is available for reply mode only.
/*           
/*           The range distance and the external  trigger delay are 
/*           interdependent according to:
/*           
/*           range distance = (trigger delay  X/Y mode delay)/12.359 s/nm
/*           
/*           (X mode delay = 50 us, Y mode delay is 56 us)
/*           
/*           Changing one value automatically changes the other value.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMRangeDistance(
    ViSession   instrSession,
    ViInt32 rangeDistance
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMINT32 (instrSession,
                             "SOUR:DME:RDIS",
                             rangeDistance,
                             0,0);
}

/*===========================================================================*/
/* Function: Get DMEM Range Distance
/* Purpose:  This function returns the simulated distance between 
/*           interrogator and transponder for reply mode. The distance is 
/*           given in nautic miles (nm). 1 nm is 1852.01 meters and 
/*           corresponds to a run time of 12.359 us.  This function is 
/*           available for reply mode only.
/*           
/*           The range distance and the external  trigger delay are 
/*           interdependent according to:
/*           
/*           range distance = (trigger delay  X/Y mode delay)/12.359 s/nm
/*           
/*           (X mode delay = 50 us, Y mode delay is 56 us)
/*           
/*           Changing one value automatically changes the other value.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMRangeDistance(
    ViSession   instrSession,
    ViInt32*    rangeDistance
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:DME:RDIS?",
                             rangeDistance);
}

/*===========================================================================*/
/* Function: Set DMEM Low Spectral Emission
/* Purpose:  This function activates the generation of a DME signal with 
/*           higher spectral purity but less dynamic compared to the default 
/*           mode. The ON/OFF ratio of the pulses is reduced from typ. >100 
/*           dB to typ 35 dB.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMLowSpectralEmission(
    ViSession   instrSession,
    ViBoolean   lowEmission
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:LOW",
                               lowEmission);
}

/*===========================================================================*/
/* Function: Get DMEM Low Spectral Emission
/* Purpose:  This function returns the generation of a DME signal with 
/*           higher spectral purity but less dynamic compared to the default 
/*           mode. The ON/OFF ratio of the pulses is reduced from typ. >100 
/*           dB to typ 35 dB.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMLowSpectralEmission(
    ViSession   instrSession,
    ViBoolean*  lowEmission
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:LOW?",
                               lowEmission);
}

/*===========================================================================*/
/* Function: Set DMEM Channel Suffix
/* Purpose:  This function selects the channel that is simulated.
/*           
/*           Standard compliant X and Y channels differ in the spacing 
/*           between the two pulses of the pulse pair and the delay of the 
/*           groundstation. ICAO indicates the ICAO  channel parameter  
/*			 for selecting the ICAO channel.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMChannelSuffix(
    ViSession   instrSession,
    ViInt32 channelSuffix
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMENUM (instrSession,
                          "SOUR:DME:CSUF",
                          channelSuffix,
                          dmemChannelSuffixArr);
}

/*===========================================================================*/
/* Function: Get DMEM Channel Suffix
/* Purpose:  This function returns the channel that is simulated.
/*           
/*           Standard compliant X and Y channels differ in the spacing 
/*           between the two pulses of the pulse pair and the delay of the 
/*           groundstation.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMChannelSuffix(
    ViSession   instrSession,
    ViInt32*    channelSuffix
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMENUM (instrSession,
                          "SOUR:DME:CSUF?",
                          channelSuffix,
                          dmemChannelSuffixArr);
}

/*===========================================================================*/
/* Function: Set DMEM Reply Efficiency
/* Purpose:  This function sets the relation between reply pulse pairs and 
/*           received trigger signals, e.g. with a set efficiency of 50% only 
/*           every second trigger event leads to the generation of a reply 
/*           pulse pair.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMReplyEfficiency(
    ViSession   instrSession,
    ViInt32 replyEfficiency
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMINT32 (instrSession,
                             "SOUR:DME:EFF",
                             replyEfficiency,
                             0,100);
}

/*===========================================================================*/
/* Function: Get DMEM Reply Efficiency
/* Purpose:  This function returns the relation between reply pulse pairs 
/*           and received trigger signals, e.g. with a set efficiency of 50% 
/*           only every second trigger event leads to the generation of a 
/*           reply pulse pair.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMReplyEfficiency(
    ViSession   instrSession,
    ViInt32*    replyEfficiency
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:DME:EFF?",
                             replyEfficiency);
}

/*===========================================================================*/
/* Function: Set DMEM Single Pulse
/* Purpose:  This function activate/deactivate a single test pulse.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMSinglePulse(
    ViSession   instrSession,
    ViBoolean   singlePulse
)
{
    INIT;
    CHECKOPT (K26);
    
    return SETPARAMBOOL (instrSession,
                               "SOUR:DME:SING",
                               singlePulse);
}

/*===========================================================================*/
/* Function: Get DMEM Single Pulse
/* Purpose:  This function returns a state of the single test pulse.
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMSinglePulse(
    ViSession   instrSession,
    ViBoolean*  singlePulse
)
{
    INIT;
    CHECKOPT (K26);
    
    return GETPARAMBOOL (instrSession,
                               "SOUR:DME:SING?",
                               singlePulse);
}

/*===========================================================================*/
/* Function: Set DMEM ICAO Channel
/* Purpose:  This function sets the ICAO channel number i.e., the DME transmitting
/*           frequency. The RF- frequency is set to the value selected here
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetDMEMICAOChannel (ViSession instrSession,
                                            ViInt32 ICAOChannel,
                                            ViInt32 ICAONumber)
{
 	ViChar buffer[BUFFER_SIZE]="";
	
	
	INIT;
    CHECKOPT (K26); 
	
	CHECKVIINT32 (instrSession, ICAOChannel, 
										   RSSMA_SOURCE_ICAO_CHANNEL_X, RSSMA_SOURCE_ICAO_CHANNEL_Y);
	CHECKVIINT32 (instrSession, ICAONumber, 
										   1, 126);
	sprintf  (buffer, ":SOUR:DME:ICAO:CHAN %s%ld", icaoChanArr[ICAOChannel], ICAONumber);
	
	return rssma_sendCmd(instrSession, buffer);
	
}

/*===========================================================================*/
/* Function: Get DMEM ICAO Channel
/* Purpose:  This function rthe ICAO channel number
/*           
/*           Required Option(s):
/*           R&S SMA-K26
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetDMEMICAOChannel (ViSession instrSession,
                                            ViInt32 *ICAOChannel,
                                            ViInt32 *ICAONumber)
{
 	ViChar buffer[BUFFER_SIZE]="";
	ViChar channel[5]="";
	
	
	INIT;
    CHECKOPT (K26); 
	
	GETPARAMSTRING(instrSession, ":SOUR:DME:ICAO:CHAN?",buffer, BUFFER_SIZE);
	if (sscanf (buffer, "%1s%ld", channel, ICAONumber) != 2)
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE; 
		
	return rssma_getIndexFromString (channel, icaoChanArr, ICAOChannel);
}

		  
/*===========================================================================*/
/* Function: Set DMEM Composite AM State
/* Purpose:  This function activate/deactivate the two-tone signal
/*
/*			 Required Option(s):
/*			 R&S SMA-K26
/*===========================================================================*/		  
ViStatus _VI_FUNC rssma_SetDMEMCompositeAMState (ViSession instrSession,
                                                 ViBoolean state)
{
 	INIT;
    CHECKOPT (K26); 
	
	return SETPARAMBOOL(instrSession,
							  "SOUR:DME:TAC:STAT",
							  state);
}

/*===========================================================================*/
/* Function: Get DMEM Composite AM State
/* Purpose:  This function returns the two-tone signal state
/*
/*			 Required Option(s):
/*			 R&S SMA-K26
/*===========================================================================*/	
ViStatus _VI_FUNC rssma_GetDMEMCompositeAMState (ViSession instrSession,
                                                 ViBoolean *state)
{
 	INIT;
    CHECKOPT (K26); 
	
	return GETPARAMBOOL(instrSession,
							  "SOUR:DME:TAC:STAT?",
							  state);    
}

/*===========================================================================*/
/* Function: Set DMEM Composite AM Modulation Depth
/* Purpose:  This function sets the modulation depth in per cent.
/*
/*			 Required Option(s):
/*			 R&S SMA-K26
/*===========================================================================*/	
ViStatus _VI_FUNC rssma_SetDMEMCompositeAMModulationDepth (ViSession instrSession,
                                                           ViInt32 signalComponent,
                                                           ViReal64 modulationDepth)
{
	INIT;
    CHECKOPT (K26);  
	
	switch (signalComponent){
		case RSSMA_SOURCE_DMEM_AM_SIGNAL_BOTH:
			rssma_status = SETPARAMREAL64 (instrSession,
							  ":SOUR:DME:TAC:DEPT",
							  modulationDepth,
							  0, 100);
		break;
		case RSSMA_SOURCE_DMEM_AM_SIGNAL_CHANN1:
			 rssma_status = SETPARAMREAL64 (instrSession,
							  "SOUR:DME:TAC:INT1:DEPT",
							  modulationDepth,
							  0, 100); 
		break;
		case RSSMA_SOURCE_DMEM_AM_SIGNAL_CHANN2:
			 rssma_status = SETPARAMREAL64 (instrSession,
							  "SOUR:DME:TAC:INT2:DEPT",
							  modulationDepth,
							  0, 100);
		break;
		default:
	        return rssma_ParameterError (instrSession, "signalComponent");
	}
	return rssma_status;
}

/*===========================================================================*/
/* Function: Get DMEM Composite AM Modulation Depth
/* Purpose:  This function returns the modulation depth in per cent.
/*
/*			 Required Option(s):
/*			 R&S SMA-K26
/*===========================================================================*/	
ViStatus _VI_FUNC rssma_GetDMEMCompositeAMModulationDepth (ViSession instrSession,
                                                           ViInt32 signalComponent,
                                                           ViReal64 *modulationDepth)
{
	INIT;
    CHECKOPT (K26);  
	
	switch (signalComponent){
		case RSSMA_SOURCE_DMEM_AM_SIGNAL_BOTH:
			rssma_status = GETPARAMREAL64 (instrSession,
							  ":SOUR:DME:TAC:DEPT?",
							  modulationDepth);
		break;
		case RSSMA_SOURCE_DMEM_AM_SIGNAL_CHANN1:
			 rssma_status = GETPARAMREAL64 (instrSession,
							  "SOUR:DME:TAC:INT1:DEPT?",
							  modulationDepth); 
		break;
		case RSSMA_SOURCE_DMEM_AM_SIGNAL_CHANN2:
			 rssma_status = GETPARAMREAL64 (instrSession,
							  "SOUR:DME:TAC:INT2:DEPT?",
							  modulationDepth);
		break;
		default:
	        return rssma_ParameterError (instrSession, "signalComponent");
	}
	return rssma_status;    
}

/*===========================================================================*/
/* Function: Set DMEM Composite AM Frequency
/* Purpose:  This function sets the individual frequencies of the two-tone signal.
/*
/*			 Required Option(s):
/*			 R&S SMA-K26
/*===========================================================================*/	
ViStatus _VI_FUNC rssma_SetDMEMCompositeAMFrequency (ViSession instrSession,
                                                     ViInt32 signalComponent,
                                                     ViReal64 frequency)
{
	INIT;
    CHECKOPT (K26);  
	
	switch (signalComponent){
		case RSSMA_SOURCE_DMEM_AM_SIGNAL_CHANN1:
			 rssma_status = SETPARAMREAL64 (instrSession,
							  "SOUR:DME:TAC:FREQ1",
							  frequency,
							  0.1, 100000.0); 
		break;
		case RSSMA_SOURCE_DMEM_AM_SIGNAL_CHANN2:
			 rssma_status = SETPARAMREAL64 (instrSession,
							  "SOUR:DME:TAC:FREQ2",
							  frequency,
							  0.1, 100000.0);
		break;
		default:
	        return rssma_ParameterError (instrSession, "signalComponent");
	}
	return rssma_status;    
}

/*===========================================================================*/
/* Function: Get DMEM Composite AM Frequency
/* Purpose:  This function returns the individual frequencies of the two-tone signal.
/*
/*			 Required Option(s):
/*			 R&S SMA-K26
/*===========================================================================*/	
ViStatus _VI_FUNC rssma_GetDMEMCompositeAMFrequency (ViSession instrSession,
                                                     ViInt32 signalComponent,
                                                     ViReal64 *frequency)
{
	INIT;
    CHECKOPT (K26);  
	
	switch (signalComponent){
		case RSSMA_SOURCE_DMEM_AM_SIGNAL_CHANN1:
			 rssma_status = GETPARAMREAL64 (instrSession,
							  "SOUR:DME:TAC:FREQ1?",
							  frequency); 
		break;
		case RSSMA_SOURCE_DMEM_AM_SIGNAL_CHANN2:
			 rssma_status = GETPARAMREAL64 (instrSession,
							  "SOUR:DME:TAC:FREQ2?",
							  frequency);
		break;
		default:
	        return rssma_ParameterError (instrSession, "signalComponent");
	}
	return rssma_status;       
}

/*===========================================================================*/
/* Function: Set DMEM Composite AM Phase
/* Purpose:  This function sets the phase of the two-tone signal.
/*
/*			 Required Option(s):
/*			 R&S SMA-K26
/*===========================================================================*/	
ViStatus _VI_FUNC rssma_SetDMEMCompositeAMPhase (ViSession instrSession,
                                                 ViReal64 phase)
{
	INIT;
    CHECKOPT (K26);  
	
	return SETPARAMREAL64 (instrSession,
							  "SOUR:DME:TAC:PHAS",
							  phase,
							  -180.0, 180.0); ;        
}



/*===========================================================================*/
/* Function: Get DMEM Composite AM Phase
/* Purpose:  This function returns the phase of the two-tone signal.
/*
/*			 Required Option(s):
/*			 R&S SMA-K26
/*===========================================================================*/	
ViStatus _VI_FUNC rssma_GetDMEMCompositeAMPhase (ViSession instrSession,
                                                 ViReal64 *phase)
{
	INIT;
    CHECKOPT (K26);  
	
	return GETPARAMREAL64 (instrSession,
							  "SOUR:DME:TAC:PHAS?",
							  phase); ;      
}


/*===========================================================================*/
/* Function: Set LFGen DC Offset
/* Purpose:  This function sets a DC offset at the LF Output.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetLFGenDCOffset (ViSession instrSession,
                                          ViReal64 DCOffset)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:LFO:OFFS",
                              DCOffset,
                              -4.0, 4.0);
}

/*===========================================================================*/
/* Function: Get LFGen DC Offset
/* Purpose:  This function returns a DC offset at the LF Output.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetLFGenDCOffset (ViSession instrSession,
                                          ViReal64 *DCOffset)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:LFO:OFFS?",
                              DCOffset);
}

/*============================================================================
 * Function: rssma_SetPGENState
 * Purpose:  This function activates/deactivates the pulse generator. The
 *           pulse generator and the pulse modulation are activated
 *           separately.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPGENState (ViSession instrSession, 
                                      ViBoolean state)
{
    return SETPARAMBOOL (instrSession,
                               "PGEN:STAT",
                               state);
}

/*============================================================================
 * Function: rssma_GetPGENState
 * Purpose:  This function returns the pulse generator state.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPGENState (ViSession instrSession, 
                                      ViBoolean *state)
{
    return GETPARAMBOOL (instrSession,
                               "PGEN:STAT?",
                               state);
}

 /*============================================================================
 * Function: rssma_confFM
 * Purpose:  This function returns the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confFM (ViSession instrSession, ViBoolean state,
                                ViInt32 source, ViInt32 mode,
                                ViReal64 FMDeviation, ViReal64 internalDeviation,
                                ViReal64 externalDeviation,
                                ViInt32 externalCoupling,
                                ViInt32 externalInputImpedance,
                                ViInt32 binaryFormat)
{
    ViStatus    rssma_status;     
    externalInputImpedance;

    CHECKERR (rssma_SetFMSource (instrSession, source));
    CHECKERR (rssma_SetFMMode (instrSession, mode));
    switch (source)
    {
        case RSSMA_FM_SOURCE_INTERNAL:
            CHECKERR (rssma_SetFMDeviation (instrSession, FMDeviation));
			break;
        case RSSMA_FM_SOURCE_EXTERNAL:
            CHECKERR (rssma_SetFMDeviation (instrSession, FMDeviation));
            CHECKERR (rssma_SetFMExternalDeviation (instrSession, externalDeviation));
			CHECKERR (rssma_SetFMExtCoupling (instrSession, externalCoupling));        
        break;
        case RSSMA_FM_SOURCE_INTEXT:
            CHECKERR (rssma_SetFMInternalDeviation (instrSession, 1, internalDeviation));
            CHECKERR (rssma_SetFMExternalDeviation (instrSession, externalDeviation));
            CHECKERR (rssma_SetFMExtCoupling (instrSession, externalCoupling));        
        break;
        case RSSMA_FM_SOURCE_EDIG:
        
            CHECKERR (rssma_SetFMExternalDigitalBinaryFormat (instrSession, binaryFormat));
            CHECKERR (rssma_SetFMExternalDeviation (instrSession, externalDeviation));
	}

    return rssma_SetFMState (instrSession, state);
}

 /*============================================================================
 * Function: rssma_confInternalFM
 * Purpose:  This function sets the parameters of internal frequency
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confFMInternal (ViSession instrSession,
                                        ViInt32 internalSource,
                                        ViReal64 internalDeviation1,
                                        ViReal64 LFGen1Frequency,
                                        ViInt32 LFGen2Shape,
                                        ViReal64 LFGen2Frequency)
{
    INIT;     
    
    CHECKERR (rssma_SetFMInternalSource (instrSession, internalSource));

    CHECKERR ( rssma_SetFMInternalDeviation (instrSession, 1, internalDeviation1));
    
    CHECKERR (rssma_SetLFGenFrequency (instrSession, 1, LFGen1Frequency));    
    CHECKOPTCONT (K24);
    if (rssma_status == VI_SUCCESS)
    {
        switch (internalSource )
        {
            case RSSMA_FM_INTERNAL_SOURCE_LF2:
            case RSSMA_FM_INTERNAL_SOURCE_LF12:
            case RSSMA_FM_INTERNAL_SOURCE_LF2NOISE:
                CHECKERR (rssma_SetLFGenShape (instrSession, LFGen2Shape));
                CHECKERR (rssma_SetLFGenFrequency (instrSession, 2, LFGen2Frequency));        }
    }
    else
    {
        rssma_status = VI_SUCCESS;
    }

    RETURN;    
}

/*============================================================================
 * Function: rssma_SetFMState
 * Purpose:  This function activates/deactivates frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFMState (ViSession instrSession, 
                                    ViBoolean state)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);

    return SETPARAMBOOL (instrSession,
                               "FM:STAT",
                               state);
}

/*============================================================================
 * Function: rssma_GetFMState
 * Purpose:  This function returns the frequency modulation state.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFMState (ViSession instrSession, 
                                    ViBoolean *state)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);

    return GETPARAMBOOL (instrSession,
                               "FM:STAT?",
                               state);
}

/*============================================================================
 * Function: rssma_SetFMSource
 * Purpose:  This function sets the source for the FM signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFMSource (ViSession instrSession,
                                     ViInt32   fmSource)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return SETPARAMENUM (instrSession,
                          "FM:SOUR",
                          fmSource,
                          fmSourceArr);
}

/*============================================================================
 * Function: rssma_GetFMSource
 * Purpose:  This function retruns the FM source.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFMSource (ViSession instrSession,
                                     ViInt32   *fmSource)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMENUM (instrSession,
                          "FM:SOUR?",
                          fmSource,
                          fmSourceArr);
}

/*============================================================================
 * Function: rssma_SetFMMode
 * Purpose:  This function selects the mode for the frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFMMode (ViSession instrSession,
                                   ViInt32   fmMode)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return SETPARAMENUM (instrSession,
                          "FM:MODE",
                          fmMode,
                          fmModeArr);
}

/*============================================================================
 * Function: rssma_GetFMMode
 * Purpose:  This function selects the mode for the frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFMMode (ViSession instrSession,
                                   ViInt32   *fmMode)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMENUM (instrSession,
                          "FM:MODE?",
                          fmMode,
                          fmModeArr);
}

/*============================================================================
 * Function: rssma_SetFMDeviation
 * Purpose:  Sets the modulation deviation of the frequency modulation in Hz.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFMDeviation (ViSession instrSession,
                                        ViReal64  deviation)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return SETPARAMREAL64 (instrSession,
                              "FM:DEV",
                              deviation,
                              0.0,20.0e+6);
}

/*============================================================================
 * Function: rssma_GetFMDeviation
 * Purpose:  This function returns the the modulation depth in Hz.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFMDeviation (ViSession instrSession,
                                        ViReal64  *deviation)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMREAL64 (instrSession,
                              "FM:DEV?",
                              deviation);
}

/*============================================================================
 * Function: rssma_SetFMExtCoupling
 * Purpose:  This function selects the coupling modefor the external
 *           modulation input (FM/PM EXT) in the case of frequency
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFMExtCoupling (ViSession instrSession,
                                          ViInt32   fmExtCoupling)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return SETPARAMENUM (instrSession,
                          "FM:EXT:COUP",
                          fmExtCoupling,
                          fmExtCouplingArr);
}

/*============================================================================
 * Function: rssma_GetFMExtCoupling
 * Purpose:  This function returns the coupling modefor the external
 *           modulation input (FM/PM EXT) in the case of frequency
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFMExtCoupling (ViSession instrSession,
                                          ViInt32   *fmExtCoupling)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMENUM (instrSession,
                          "FM:EXT:COUP?",
                          fmExtCoupling,
                          fmExtCouplingArr);
}

/*============================================================================
 * Function: rssma_SetFMInternalSource
 * Purpose:  This function Selects which internal modulation source is used.
 *           The available selection depends on the options fitted.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFMInternalSource (ViSession instrSession,
                                             ViInt32   fmIntSource)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    if (fmIntSource > RSSMA_FM_INTERNAL_SOURCE_LF1)
        CHECKOPT (K24);
    
    return SETPARAMENUM (instrSession,
                          "FM:INT:SOUR",
                          fmIntSource,
                          lfoSourceArr);
}

/*============================================================================
 * Function: rssma_GetFMInternalSource
 * Purpose:  This function retruns the internal FM source.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFMInternalSource (ViSession instrSession,
                                             ViInt32   *fmIntSource)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMENUM (instrSession,
                          "FM:INT:SOUR?",
                          fmIntSource,
                          lfoSourceArr);
}

/*============================================================================
 * Function: rssma_SetFMInternalDeviation
 * Purpose:  This function selects the deviation for the two frequency
 *           modulation signals. The sum of the two values must not exceed
 *           the overall modulation deviation set with command
 *           SOURce:FM:DEViation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFMInternalDeviation (ViSession instrSession,
                                                ViInt32   channel,
                                                ViReal64  fmInternalDeviation)
{
    ViChar  buffer[BUFFER_SIZE];
    INIT;
    
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    if (channel != 1)
    {
        CHECKOPT (K24);
    }
    
    CHECKVIINT32 (instrSession, channel, 1, 2);

    sprintf (buffer, "FM:INT%ld:DEV", channel);
    
    return SETPARAMREAL64 (instrSession,
                              buffer,
                              fmInternalDeviation,
                              0.0, 40.0e6);
}

/*============================================================================
 * Function: rssma_GetFMInternalDeviation
 * Purpose:  This function returns the deviation for the two frequency
 *           modulation signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFMInternalDeviation (ViSession instrSession,
                                                ViInt32   channel,
                                                ViReal64  *fmInternalDeviation)
{
    ViChar  buffer[BUFFER_SIZE];
    
    INIT;
    CHECKOPT (K24);
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    if (channel != 1)
    {
        CHECKOPT (K24);
    }

    CHECKVIINT32 (instrSession, channel, 1, 2);

    sprintf (buffer, "FM:INT%ld:DEV?", channel);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              fmInternalDeviation);
}

/*============================================================================
 * Function: rssma_SetFMExternalDeviation
 * Purpose:  This function sets the deviation for the two frequency
 *           modulation signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFMExternalDeviation (ViSession instrSession,
                                                ViReal64 externalDeviation)
{
    return SETPARAMREAL64 (instrSession,
                              "FM:EXT:DEV",
                              externalDeviation,
                              0.0, 40.0e6);
}

/*============================================================================
 * Function: rssma_GetFMExternalDeviation
 * Purpose:  This function returns the deviation for the two frequency
 *           modulation signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFMExternalDeviation (ViSession instrSession,
                                                ViReal64 *externalDeviation)
{
    return GETPARAMREAL64 (instrSession,
                              "FM:EXT:DEV?",
                              externalDeviation);
}

/*============================================================================
 * Function: rssma_SetFMExternalDigitalBinaryFormat
 * Purpose:  This function selects the binary format for external digital
 *           frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetFMExternalDigitalBinaryFormat (ViSession instrSession,
                                                          ViInt32   fmExtBinaryFormat)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return SETPARAMENUM (instrSession,
                          "FM:EXT:DIG:BFOR",
                          fmExtBinaryFormat,
                          fmExtBinaryFormatArr);
}

/*============================================================================
 * Function: rssma_GetFMExternalDigitalBinaryFormat
 * Purpose:  This function returns the binary format for external digital
 *           frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFMExternalDigitalBinaryFormat (ViSession instrSession,
                                                          ViInt32*  fmExtBinaryFormat)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMENUM (instrSession,
                          "FM:EXT:DIG:BFOR?",
                          fmExtBinaryFormat,
                          fmExtBinaryFormatArr);
}

/*============================================================================
 * Function: Get FM Sensitivity
 * Purpose:  This function queries the input sensitivity of the FM/PM EXT 
 *           input in Hz/V. The function is only effective if the external 
 *           modulation source is selected (rssma_SetFMSource). The returned 
 *           value depends on the modulation deviation setting (SOUR:FM:DEViation). 
 *           This value is assigned to the voltage value for full modulation of the
 *           input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetFMSensitivity (ViSession instrSession,
                                          ViReal64 *sensitivity)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FM:SENS?",
                              sensitivity);
}

 /*============================================================================
 * Function: rssma_confPM
 * Purpose:  This function returns the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confPM (ViSession instrSession, ViBoolean state,
                                ViInt32 source, ViInt32 mode,
                                ViReal64 PMDeviation, ViReal64 internalDeviation,
                                ViReal64 externalDeviation,
                                ViInt32 externalCoupling,
                                ViInt32 externalInputImpedance,
                                ViInt32 binaryFormat)
{
    ViStatus    rssma_status;
    externalInputImpedance;

    CHECKERR (rssma_SetPMSource (instrSession, source));
    CHECKERR (rssma_SetPMMode (instrSession, mode));
    switch (source)
    {
        case RSSMA_FM_SOURCE_INTERNAL:
            CHECKERR (rssma_SetPMInternalDeviation (instrSession, 1, internalDeviation));            break;
        case RSSMA_FM_SOURCE_EXTERNAL:
            CHECKERR (rssma_SetPMDeviation (instrSession, PMDeviation));
            CHECKERR (rssma_SetPMExternalDeviation (instrSession, externalDeviation));            CHECKERR (rssma_SetPMExtCoupling (instrSession, externalCoupling));        
        break;
        case RSSMA_FM_SOURCE_INTEXT:
            CHECKERR (rssma_SetPMExternalDeviation (instrSession, externalDeviation));
            CHECKERR (rssma_SetPMInternalDeviation (instrSession, 1, internalDeviation));
            CHECKERR (rssma_SetPMExtCoupling (instrSession, externalCoupling));        
        break;
        case RSSMA_FM_SOURCE_EDIG:
        
            CHECKERR (rssma_SetPMExternalDigitalBinaryFormat (instrSession, binaryFormat));
            CHECKERR (rssma_SetPMExternalDeviation (instrSession, externalDeviation));    }
    
    return rssma_SetPMState (instrSession, state);
}

 /*============================================================================
 * Function: rssma_confInternalPM
 * Purpose:  This function sets the parameters of internal frequency
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confPMInternal (ViSession instrSession,
                                        ViInt32 internalSource,
                                        ViReal64 internalDeviation1,
                                        ViReal64 LFGen1Frequency,
                                        ViInt32 LFGen2Shape,
                                        ViReal64 LFGen2Frequency)
{
    INIT;
    
    CHECKERR (rssma_SetPMInternalSource (instrSession, internalSource));

    CHECKERR ( rssma_SetPMInternalDeviation (instrSession, 1, internalDeviation1));
    
    CHECKERR (rssma_SetLFGenFrequency (instrSession, 1, LFGen1Frequency));    
    CHECKOPTCONT (K24);
    
    if (rssma_status == VI_SUCCESS)
    {
        switch (internalSource )
        {
            case RSSMA_FM_INTERNAL_SOURCE_LF2:
            case RSSMA_FM_INTERNAL_SOURCE_LF12:
            case RSSMA_FM_INTERNAL_SOURCE_LF2NOISE:
                CHECKERR (rssma_SetLFGenShape (instrSession, LFGen2Shape));
                CHECKERR (rssma_SetLFGenFrequency (instrSession, 2, LFGen2Frequency));        }
    }
    else
    {
        rssma_status = VI_SUCCESS;
    }

    RETURN;    
}

/*============================================================================
 * Function: rssma_SetPMState
 * Purpose:  This function activates/deactivates frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPMState (ViSession instrSession, 
                                    ViBoolean state)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);

    return SETPARAMBOOL (instrSession,
                               "PM:STAT",
                               state);
}

/*============================================================================
 * Function: rssma_GetPMState
 * Purpose:  This function returns the frequency modulation state.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPMState (ViSession instrSession, 
                                    ViBoolean *state)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);

    return GETPARAMBOOL (instrSession,
                               "PM:STAT?",
                               state);
}

/*============================================================================
 * Function: rssma_SetPMSource
 * Purpose:  This function sets the source for the PM signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPMSource (ViSession instrSession,
                                     ViInt32   fmSource)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return SETPARAMENUM (instrSession,
                          "PM:SOUR",
                          fmSource,
                          fmSourceArr);
}

/*============================================================================
 * Function: rssma_GetPMSource
 * Purpose:  This function retruns the PM source.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPMSource (ViSession instrSession,
                                     ViInt32   *fmSource)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMENUM (instrSession,
                          "PM:SOUR?",
                          fmSource,
                          fmSourceArr);
}

/*============================================================================
 * Function: rssma_SetPMMode
 * Purpose:  This function selects the mode for the frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPMMode (ViSession instrSession,
                                   ViInt32   pmMode)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return SETPARAMENUM (instrSession,
                          "PM:MODE",
                          pmMode,
                          pmModeArr);
}

/*============================================================================
 * Function: rssma_GetPMMode
 * Purpose:  This function selects the mode for the frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPMMode (ViSession instrSession,
                                   ViInt32   *pmMode)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMENUM (instrSession,
                          "PM:MODE?",
                          pmMode,
                          pmModeArr);
}

/*============================================================================
 * Function: rssma_SetPMDeviation
 * Purpose:  Sets the modulation deviation of the frequency modulation in Hz.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPMDeviation (ViSession instrSession,
                                        ViReal64  deviation)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return SETPARAMREAL64 (instrSession,
                              "PM:DEV",
                              deviation,
                              0.0,20.0);
}

/*============================================================================
 * Function: rssma_GetPMDeviation
 * Purpose:  This function returns the the modulation depth in Hz.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPMDeviation (ViSession instrSession,
                                        ViReal64  *deviation)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMREAL64 (instrSession,
                              "PM:DEV?",
                              deviation);
}

/*============================================================================
 * Function: rssma_SetPMExtCoupling
 * Purpose:  This function selects the coupling modefor the external
 *           modulation input (PM/PM EXT) in the case of frequency
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPMExtCoupling (ViSession instrSession,
                                          ViInt32   fmExtCoupling)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return SETPARAMENUM (instrSession,
                          "PM:EXT:COUP",
                          fmExtCoupling,
                          fmExtCouplingArr);
}

/*============================================================================
 * Function: rssma_GetPMExtCoupling
 * Purpose:  This function returns the coupling modefor the external
 *           modulation input (PM/PM EXT) in the case of frequency
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPMExtCoupling (ViSession instrSession,
                                          ViInt32   *fmExtCoupling)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMENUM (instrSession,
                          "PM:EXT:COUP?",
                          fmExtCoupling,
                          fmExtCouplingArr);
}

/*============================================================================
 * Function: rssma_SetPMInternalSource
 * Purpose:  This function Selects which internal modulation source is used.
 *           The available selection depends on the options fitted.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPMInternalSource (ViSession instrSession,
                                             ViInt32   fmIntSource)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    if (fmIntSource > RSSMA_FM_INTERNAL_SOURCE_LF1)
        CHECKOPT (K24);
    
    return SETPARAMENUM (instrSession,
                          "PM:INT:SOUR",
                          fmIntSource,
                          lfoSourceArr);
}

/*============================================================================
 * Function: rssma_GetPMInternalSource
 * Purpose:  This function retruns the internal PM source.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPMInternalSource (ViSession instrSession,
                                             ViInt32   *fmIntSource)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMENUM (instrSession,
                          "PM:INT:SOUR?",
                          fmIntSource,
                          lfoSourceArr);
}

/*============================================================================
 * Function: rssma_SetPMInternalDeviation
 * Purpose:  This function selects the deviation for the two frequency
 *           modulation signals. The sum of the two values must not exceed
 *           the overall modulation deviation set with command
 *           SOURce:PM:DEViation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPMInternalDeviation (ViSession instrSession,
                                                ViInt32   channel,
                                                ViReal64  fmInternalDeviation)
{
    ViChar  buffer[BUFFER_SIZE];
    INIT;
    
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);

    CHECKVIINT32 (instrSession, channel, 1, 2);

    if (channel != 1)
    {
        CHECKOPT (K24);
    }

    sprintf (buffer, "PM:INT%ld:DEV", channel);
    
    return SETPARAMREAL64 (instrSession,
                              buffer,
                              fmInternalDeviation,
                              0.0, 20.0);
}

/*============================================================================
 * Function: rssma_GetPMInternalDeviation
 * Purpose:  This function returns the deviation for the two frequency
 *           modulation signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPMInternalDeviation (ViSession instrSession,
                                                ViInt32   channel,
                                                ViReal64  *fmInternalDeviation)
{
    ViChar  buffer[BUFFER_SIZE];
    
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    CHECKVIINT32 (instrSession, channel, 1, 2);

    if (channel != 1)
    {
        CHECKOPT (K24);
    }

    sprintf (buffer, "PM:INT%ld:DEV?", channel);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              fmInternalDeviation);
}

/*============================================================================
 * Function: rssma_SetPMExternalDeviation
 * Purpose:  This function sets the deviation for the two frequency
 *           modulation signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPMExternalDeviation (ViSession instrSession,
                                                ViReal64 externalDeviation)
{
    return SETPARAMREAL64 (instrSession,
                              "PM:EXT:DEV",
                              externalDeviation,
                              0.0, 20.0);
}

/*============================================================================
 * Function: rssma_GetPMExternalDeviation
 * Purpose:  This function returns the deviation for the two frequency
 *           modulation signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPMExternalDeviation (ViSession instrSession,
                                                ViReal64 *externalDeviation)
{
    return GETPARAMREAL64 (instrSession,
                              "PM:EXT:DEV?",
                              externalDeviation);
}

/*============================================================================
 * Function: rssma_SetPMExternalDigitalBinaryFormat
 * Purpose:  This function selects the binary format for external digital
 *           frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPMExternalDigitalBinaryFormat (ViSession instrSession,
                                                          ViInt32   fmExtBinaryFormat)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return SETPARAMENUM (instrSession,
                          "PM:EXT:DIG:BFOR",
                          fmExtBinaryFormat,
                          fmExtBinaryFormatArr);
}

/*============================================================================
 * Function: rssma_GetPMExternalDigitalBinaryFormat
 * Purpose:  This function returns the binary format for external digital
 *           frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPMExternalDigitalBinaryFormat (ViSession instrSession,
                                                          ViInt32*  fmExtBinaryFormat)
{
    INIT;
    CHECKOPTCONT (B20);
    
    if (rssma_status != VI_SUCCESS)
        CHECKOPT (B22);
    
    return GETPARAMENUM (instrSession,
                          "PM:EXT:DIG:BFOR?",
                          fmExtBinaryFormat,
                          fmExtBinaryFormatArr);
}

/*============================================================================
 * Function: Set Level Sweep Attenuator Mode
 * Purpose:  This function selects the ranges of level settings for the list
 *           mode. The level settings are either performed in the low level
 *           or in the high level ranges.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLevelSweepAttenuatorMode (ViSession instrSession,
                                                     ViInt32 attenuatorMode)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:SWE:POW:AMOD",
                          attenuatorMode,
                          ListAttenuatorModeArr);
}

/*============================================================================
 * Function: Get Level Sweep Attenuator Mode
 * Purpose:  This function returns the ranges of level settings for the list
 *           mode. The level settings are either performed in the low level
 *           or in the high level ranges.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLevelSweepAttenuatorMode (ViSession instrSession,
                                                     ViInt32 *attenuatorMode)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:SWE:POW:AMOD?",
                          attenuatorMode,
                          ListAttenuatorModeArr);
}

/*============================================================================
 * Function: Set Level Sweep Shape
 * Purpose:  This function sets the cycle mode for a sweep sequence (shape).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLevelSweepShape (ViSession instrSession,
                                            ViInt32 shape)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:SWE:POW:SHAP",
                          shape,
                          frequencySweepShapeArr);
}

/*============================================================================
 * Function: Get Level Sweep Shape
 * Purpose:  This function returns the cycle mode for a sweep sequence (shape).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLevelSweepShape (ViSession instrSession,
                                            ViInt32 *shape)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:SWE:POW:SHAP?",
                          shape,
                          frequencySweepShapeArr);
}

/*============================================================================
 * Function: Get Level Sweep Spacing
 * Purpose:  This function queries the sweep spacing. The sweep spacing for
 *           level sweeps is always linear.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLevelSweepSpacing (ViSession instrSession,
                                              ViInt32 *spacing)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:SWE:POW:SPAC:MODE?",
                          spacing,
                          spacingArr);
}

/*============================================================================
 * Function: Reset Level Sweep
 * Purpose:  This function resets all active sweeps to the starting point.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ResetLevelSweep (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SOUR:SWE:RES:ALL");   
}

/*============================================================================
 * Function: rssma_confList
 * Purpose:  This function sets all parameters of selected list. If list with
 *           specified name does not exists it will be created.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confList (ViSession instrSession,
                                  ViString listName, ViInt32 listMode,
                                  ViInt32 arraySize, ViReal64 frequencyValues[],
                                  ViReal64 powerValues[], ViReal64 dwellTime)
{
    ViStatus    rssma_status;

    CHECKERR (rssma_SetLISTSelectList (instrSession, listName));

    CHECKERR (rssma_SetLISTFrequencyValues(instrSession, arraySize, frequencyValues));
    CHECKERR (rssma_SetLISTPowerValues(instrSession, arraySize, powerValues));
    CHECKERR (rssma_SetLISTDwellTime(instrSession, dwellTime));
    CHECKERR (rssma_SetLISTMode(instrSession, listMode));
    RETURN;
}

/*============================================================================
 * Function: rssma_LISTCatalog
 * Purpose:  This function requests a list of available lists. The individual
 *           lists are separated by commas.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_LISTCatalog (ViSession instrSession, ViString directoryPath,
                            ViInt32 arraySize, ViChar _VI_FAR catalogList[])
{
    ViStatus    rssma_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!directoryPath)
        sprintf (buffer, ":LIST:CAT?;*OPC?");
    else
        sprintf (buffer, ":LIST:CAT? '%s';*OPC?", directoryPath);

    rssma_status = rssma_getCatalog (instrSession,
                                     buffer,
                                     (ViUInt32) arraySize,
                                     catalogList, "catalogList");

    return rssma_status;
}

/*============================================================================
 * Function: rssma_LISTDeleteList
 * Purpose:  This function deletes the specified list.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_LISTDeleteList (ViSession instrSession, ViString listName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                              ":LIST:DEL",
                              "'",
                              listName);
}

/*============================================================================
 * Function: rssma_LISTDeleteAll
 * Purpose:  This function deletes all lists in the set directory. The List
 *           mode must be deactivated beforehand to ensure that no lists are
 *           selected when this function is called (:SOUR:FREQ:MODE CW or SWE).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_LISTDeleteAll (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, ":LIST:DEL:ALL");
}

/*============================================================================
 * Function: rssma_LISTResetList
 * Purpose:  This function resets the list to the starting point.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_LISTResetList (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, ":SOUR:LIST:RES");
}

/*============================================================================
 * Function: rssma_LISTTriggerExecute
 * Purpose:  This function immediately starts the processing of a list in list
 *           mode. It corresponds to the manual-control command Execute Single.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_LISTTriggerExecute (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "SOUR:LIST:TRIG:EXEC");
}

/*============================================================================
 * Function: rssma_SetLISTMode
 * Purpose:  This function sets the list mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTMode (ViSession instrSession,
                                     ViInt32 listMode)
{
    ViStatus    rssma_status;

    switch (listMode)
    {
    case RSSMA_SWEEP_MODE_AUTO:
        CHECKERR (rssma_SetModeLIST (instrSession, RSSMA_LIST_SWEEP_MODE_AUTO));
        CHECKERR (rssma_SetLISTTriggerSource (instrSession,
                                              RSSMA_LIST_TRIGGER_SOURCE_AUTO));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_LIST));
        break;
    case RSSMA_SWEEP_MODE_SINGLE:
        CHECKERR (rssma_SetModeLIST (instrSession, RSSMA_LIST_SWEEP_MODE_AUTO));
        CHECKERR (rssma_SetLISTTriggerSource (instrSession,
                                                        RSSMA_LIST_TRIGGER_SOURCE_SINGLE));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_LIST));
        CHECKERR (rssma_LISTTriggerExecute (instrSession));
        break;
    case RSSMA_SWEEP_MODE_STEP:
        CHECKERR (rssma_SetModeLIST (instrSession, RSSMA_LIST_SWEEP_MODE_STEP));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_LIST));
        CHECKERR (rssma_SetLISTTriggerSource (instrSession,
                                                        RSSMA_LIST_TRIGGER_SOURCE_SINGLE));
        break;
    case RSSMA_SWEEP_MODE_EXT_SINGLE:
        CHECKERR (rssma_SetModeLIST (instrSession, RSSMA_LIST_SWEEP_MODE_AUTO));
        CHECKERR (rssma_SetLISTTriggerSource (instrSession,
                                              RSSMA_LIST_TRIGGER_SOURCE_EXTERNAL));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_LIST));
        break;
    case RSSMA_SWEEP_MODE_EXT_STEP:
        CHECKERR (rssma_SetModeLIST (instrSession, RSSMA_LIST_SWEEP_MODE_STEP));
        CHECKERR (rssma_SetLISTTriggerSource (instrSession,
                                              RSSMA_LIST_TRIGGER_SOURCE_EXTERNAL));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_LIST));
        break;
    case RSSMA_SWEEP_MODE_EXT_HOP:
        CHECKERR (rssma_SetModeLIST (instrSession, RSSMA_LIST_SWEEP_MODE_STEP));
        CHECKERR (rssma_SetLISTTriggerSource (instrSession,
                                              RSSMA_LIST_TRIGGER_SOURCE_HOP));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_LIST));
        break;
    case RSSMA_SWEEP_MODE_EXT_DHOP:
        CHECKERR (rssma_SetModeLIST (instrSession, RSSMA_LIST_SWEEP_MODE_STEP));
        CHECKERR (rssma_SetLISTTriggerSource (instrSession,
                                              RSSMA_LIST_TRIGGER_SOURCE_DHOP));
        CHECKERR (rssma_SetFrequencyMode (instrSession, RSSMA_FREQUENCY_MODE_LIST));
        break;
    default:
        rssma_status = rssma_ParameterError (instrSession, "listMode");
    }

    RETURN;
}

/*============================================================================
 * Function: rssma_SetLISTDwellTime
 * Purpose:  This function sets the time for which the instrument retains a
 *           setting.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTDwellTime (ViSession instrSession,
                                 ViReal64 dwellTime)
{
    return SETPARAMREAL64 (instrSession,
                              ":SOUR:LIST:DWEL",
                              dwellTime,
                              1.0e-3, 1.0);
}

/*============================================================================
 * Function: rssma_GetLISTDwellTime
 * Purpose:  This function Returns the time for which the instrument retains a
 *           setting.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTDwellTime (ViSession instrSession, ViReal64 *dwellTime)
{
    return GETPARAMREAL64 (instrSession,
                                  ":SOUR:LIST:DWEL?",
                                  dwellTime);
}

/*============================================================================
 * Function: rssma_SetLISTSelectList
 * Purpose:  This function selects the specified list. If a new list is to be
 *           created, the name can be entered here. The list is created if it
 *           does not yet exist. The list selected here is available for the
 *           further processing steps (editing) and is used in the instrument
 *           when the List mode is activated.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTSelectList (ViSession instrSession,
                                            ViString listName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    ":SOUR:LIST:SEL",
                                    "'",
                                    listName);
}

/*============================================================================
 * Function: rssma_GetLISTSelectList
 * Purpose:  This function queries name of the selected list.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTSelectList (ViSession instrSession,
                                           ViChar _VI_FAR listName[])
{
    ViStatus    rssma_status;
    rssma_status = GETPARAMSTRING (instrSession,
                                  "SOUR:LIST:SEL?",
                                  listName,
                                  128);

    return (rssma_status < VI_SUCCESS) ? rssma_status : VI_SUCCESS;
}

/*============================================================================
 * Function: rssma_SetLISTFrequencyValues
 * Purpose:  This function fills the FREQuency part of the selected list with
 *           data. The data can be given either as a list of numbers (list can
 *           be of any length and list entries must be separated by commas) or
 *           as binary block data. When block data is transferred, 8 (4) bytes
 *           are always interpreted as a floating-point number with double
 *           accuracy (see the command FORMat:DATA).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTFrequencyValues (ViSession instrSession,
                                       ViInt32 arraySize,
                                       ViReal64 _VI_FAR frequencyValues[])
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    ViStatus    rssma_status;

    CHECKNULL (frequencyValues);

    p2buf = pbuffer = malloc((size_t)  arraySize*20 + 30);
    p2buf += sprintf(pbuffer, ":FORM ASC;:SOUR:LIST:FREQ ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%f,", frequencyValues[i]);

    *--p2buf = '\0';

    rssma_status = rssma_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssma_status;
}

/*============================================================================
 * Function: rssma_GetLISTFrequencyValues
 * Purpose:  This function queries the FREQuency part of the selected list.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTFrequencyValues (ViSession instrSession,
                                       ViInt32 arraySize,
                                       ViReal64 _VI_FAR frequencyValues[])
{
    return GETPARAMREAL64ARRAY (instrSession,
                                   ":FORM ASC;:LIST:FREQ?",
                                   frequencyValues,
                                   arraySize);
}

/*============================================================================
 * Function: rssma_GetLISTFrequencyPoints
 * Purpose:  This function queries the length (in points) of the FREQuency
 *           component of the selected list.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTFrequencyPoints (ViSession instrSession,
                                       ViInt32 *number_ofFrequencyPoints)
{
    return GETPARAMINT32 (instrSession,
                             ":LIST:FREQ:POIN?",
                             number_ofFrequencyPoints);
}

/*============================================================================
 * Function: rssma_SetLISTPowerValues
 * Purpose:  This function fills the Level part of the selected list with
 *           data. The data can be given either as a list of numbers (list can
 *           be of any length and list entries must be separated by commas) or
 *           as binary block data. When block data is transferred, 8 (4) bytes
 *           are always interpreted as a floating-point number with double
 *           accuracy (see the command FORMat:DATA).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetLISTPowerValues (ViSession instrSession, 
                                            ViInt32 arraySize,
                                            ViReal64 _VI_FAR powerValues[])
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    ViStatus    rssma_status;


    CHECKNULL (powerValues);
    
    p2buf = pbuffer = malloc((size_t)arraySize*20 + 30);
    p2buf += sprintf(pbuffer, ":FORM ASC;:SOUR:LIST:POW ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%fdBm,", powerValues[i]);

    *--p2buf = '\0';

    rssma_status = rssma_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssma_status;
}

/*============================================================================
 * Function: rssma_GetLISTPowerValues
 * Purpose:  This function queries the Level part of the selected list.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTPowerValues (ViSession instrSession, ViInt32 arraySize,
                                   ViReal64 _VI_FAR powerValues[])
{
    return GETPARAMREAL64ARRAY (instrSession,
                                   ":FORM ASC;:LIST:POW?",
                                   powerValues,
                                   arraySize);
}

/*============================================================================
 * Function: rssma_GetLISTPowerPoints
 * Purpose:  This function queries the length (in points) of the LEVel part of
 *           the selected list.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetLISTPowerPoints (ViSession instrSession,
                                   ViInt32 *number_ofPowerPoints)
{
    return GETPARAMINT32 (instrSession,
                             ":LIST:POW:POIN?",
                             number_ofPowerPoints);
}

/*============================================================================
 * Function: rssma_SetModeLIST
 * Purpose:  This function specifies how the list is to be processed (similar
 *           to SOURce:SWEep:MODE).
 *===========================================================================*/
ViStatus rssma_SetModeLIST (ViSession instrSession,
                            ViInt32 listMode)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:MODE",
                          listMode,
                          listModeArr);
}

/*============================================================================
 * Function: rssma_SetLISTTriggerSource
 * Purpose:  This function sets the trigger source for the LIST mode.
 *===========================================================================*/
ViStatus rssma_SetLISTTriggerSource (ViSession instrSession,
                                     ViInt32 triggerSource)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:TRIG:SOUR",
                          triggerSource,
                          listTriggerSourceArr);
}

/*============================================================================
 * Function: Set CSYN DC Offset State
 * Purpose:  This function activates/deactivates the addition of the DC offset
 *           to both clock synthesis output signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetCSYNDCOffsetState (ViSession instrSession,
                                              ViBoolean DCOffsetState)
{
    INIT;
    CHECKOPT (B29);
    
    return SETPARAMBOOL (instrSession,
                               "CSYN:OFFS:STAT",
                               DCOffsetState);    
}

/*============================================================================
 * Function: Get CSYN DC Offset State
 * Purpose:  This function returns returns the state of the addition of the 
 *           DC offset to both clock synthesis output signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetCSYNDCOffsetState (ViSession instrSession,
                                              ViBoolean *DCOffsetState)
{
    INIT;
    CHECKOPT (B29);

    return GETPARAMBOOL (instrSession,
                               "CSYN:OFFS:STAT?",
                               DCOffsetState);    
}

/*============================================================================
 * Function: Set CSYN DC Offset
 * Purpose:  This function sets DC offset added to both clock synthesis output 
 *           signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetCSYNDCOffset (ViSession instrSession,
                                         ViReal64 DCOffset)
{
    INIT;
    CHECKOPT (B29);
    
    return SETPARAMREAL64 (instrSession,
                              "CSYN:OFFS",
                              DCOffset,
                              -5.0, 5.0);   
}

/*============================================================================
 * Function: Set CSYN DC Offset
 * Purpose:  This function returns the DC offset added to both clock synthesis 
 *           output  signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetCSYNDCOffset (ViSession instrSession,
                                         ViReal64 *DCOffset)
{
    INIT;
    CHECKOPT (B29);
    
    return GETPARAMREAL64 (instrSession,
                              "CSYN:OFFS?",
                              DCOffset);    
}

/*============================================================================
 * Function: rssma_confNoise
 * Purpose:  This function sets all parameters of noise modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_confNoise (ViSession instrSession,
                                   ViBoolean bandwidthState, ViReal64 bandwidth,
                                   ViInt32 distribution)
{
    ViStatus    rssma_status;     

    CHECKERR (rssma_SetNOISEBandwidth (instrSession, bandwidth));    
    CHECKERR (rssma_SetNOISEDistribution (instrSession, distribution));    
    return rssma_SetNOISEBandwidthState (instrSession, bandwidthState);
}

/*============================================================================
 * Function: rssma_SetNOISEBandwidthState
 * Purpose:  This function enabled/disables bandwidth limitation of noise.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetNOISEBandwidthState (ViSession instrSession, 
                                                ViBoolean state)
{
    INIT;
    CHECKOPT (K24);

    return SETPARAMBOOL (instrSession,
                               "NOIS:BWID:STAT",
                               state);
}

/*============================================================================
 * Function: rssma_GetNOISEBandwidthState
 * Purpose:  This function returns the bandwidth limitation of noise.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetNOISEBandwidthState (ViSession instrSession, 
                                                ViBoolean *state)
{
    INIT;
    CHECKOPT (K24);

    return GETPARAMBOOL (instrSession,
                               "NOIS:BWID:STAT?",
                               state);
}

/*============================================================================
 * Function: rssma_SetNOISEBandwidth
 * Purpose:  This function sets the noise level in the system bandwidth for
 *           enabled bandwidth limitation. Distinct bandwidth settings
 *           between 10 kHz and 10 MHz in 100 kHz steps (range 100 .. 1 MHz),
 *           1 MHz (range 1 MHz .. 5 MHz) and 5 MHz (5 MHz  10 MHz) are
 *           possible.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetNOISEBandwidth (ViSession instrSession,
                                           ViReal64  bandwidth)
{
    INIT;
    CHECKOPT (K24);
    
    return SETPARAMREAL64 (instrSession,
                              "NOIS:BWID",
                              bandwidth,
                              100.0e3,10.0e+6);
}

/*============================================================================
 * Function: rssma_GetNOISEBandwidth
 * Purpose:  This function returns the noise level in the system bandwidth
 *           for enabled bandwidth limitation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetNOISEBandwidth (ViSession instrSession,
                                           ViReal64  *bandwidth)
{
    INIT;
    CHECKOPT (K24);
    
    return GETPARAMREAL64 (instrSession,
                              "NOIS:BWID?",
                              bandwidth);
}

/*============================================================================
 * Function: rssma_SetNOISEDistribution
 * Purpose:  This function selects the noise power densitiy distribution of
 *           the noise.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetNOISEDistribution (ViSession instrSession,
                                              ViInt32   distribution)
{
    INIT;
    CHECKOPT (K24);
    
    return SETPARAMENUM (instrSession,
                          "NOIS:DIST",
                          distribution,
                          distributionArr);
}

/*============================================================================
 * Function: rssma_GetNOISEDistribution
 * Purpose:  This function returns the noise power densitiy distribution of
 *           the noise.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetNOISEDistribution (ViSession instrSession,
                                              ViInt32*  distribution)
{
    INIT;
    CHECKOPT (K24);
    
    return GETPARAMENUM (instrSession,
                          "NOIS:DIST?",
                          distribution,
                          distributionArr);
}

/*============================================================================
 * Function: rssma_GetNOISERelativeLevel
 * Purpose:  This function returns the level of the noise signal per Hz in
 *           the total bandwidth.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetNOISERelativeLevel (ViSession instrSession,
                                               ViReal64  *relativeLevel)
{
    INIT;
    CHECKOPT (K24);
    
    return GETPARAMREAL64 (instrSession,
                              "NOIS:LEV:REL?",
                              relativeLevel);
}

/*============================================================================
 * Function: rssma_GetNOISEAbsoluteLevel
 * Purpose:  This function returns the level of the noise signal in the
 *           system bandwidth for enabled bandwidth limitation.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetNOISEAbsoluteLevel (ViSession instrSession,
                                               ViReal64  *absoluteLevel)
{
    INIT;
    CHECKOPT (K24);
    
    return GETPARAMREAL64 (instrSession,
                              "NOIS:LEV?",
                              absoluteLevel);
}

/*===========================================================================
 * Function: Get SYSTem BIOS Version NEW
 * Purpose:  This function queries the BIOS version with which the 
 *           instrument complies.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemBIOSVersion(
    ViSession   instrSession,
    ViChar _VI_FAR biosVersion[]
)
{
    return GETPARAMSTRING (instrSession,
                              ":SYST:BIOS:VERS?",
                              biosVersion,
                              256);
}

/*===========================================================================
 * Function: Set SYSTem Identification
 * Purpose:  This function sets the system identification.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemIdentification(
    ViSession   instrSession,
    ViInt32 identification
)
{
    return SETPARAMENUM (instrSession,
                          "SYST:IDEN",
                          identification,
                          identification2Arr);
}

/*===========================================================================
 * Function: Get SYSTem Identification
 * Purpose:  This function returns system identification.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemIdentification(
    ViSession   instrSession,
    ViInt32*    identification
)
{
    ViStatus rssma_status = VI_SUCCESS;
    
    rssma_status = GETPARAMENUM (instrSession,
                                  "SYST:IDEN?",
                                  identification,
                                  identification2Arr);
    
    return (rssma_status < VI_SUCCESS) ? rssma_status : VI_SUCCESS;
    
}

/*===========================================================================*/
/* Function: Get SYSTem Startup Status
/* Purpose:  This function returns if the startup of the instrument is 
/*           completed.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemStartupStatus(
    ViSession   instrSession,
    ViInt32*    startupComplete
)
{
    INIT;
    
    return GETPARAMINT32 (instrSession,
                             "SYST:STAR:COMP?",
                             startupComplete);
}

/*===========================================================================
 * Function: Set SYSTem Network IP Address
 * Purpose:  This function sets the ip address.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemNetworkIPAddress(
    ViSession   instrSession,
    ViString    ipAddress
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:IPAD",
                                    "'",
                                    ipAddress);
}

/*===========================================================================
 * Function: Get SYSTem Network IP Address
 * Purpose:  This function returns the IP address.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkIPAddress(
    ViSession   instrSession,
    ViInt32     arraySize,
    ViChar _VI_FAR  ipAddress[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:IPAD?",
                              ipAddress,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network IP Address Subnet Mask
 * Purpose:  This function sets the subnet mask.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemNetworkIPAddressSubnetMask(
    ViSession   instrSession,
    ViString    ipAddressSubnetMask
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:IPAD:SUBN:MASK",
                                    "'",
                                    ipAddressSubnetMask);
}

/*===========================================================================
 * Function: Get SYSTem Network IP Address Subnet Mask
 * Purpose:  This function returns the IP address subnet mask.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkIPAddressSubnetMask(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar _VI_FAR  ipAddressSubnetMask[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:IPAD:SUBN:MASK?",
                              ipAddressSubnetMask,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network IP Address Gateway
 * Purpose:  This function sets the ip adress gateway.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemNetworkIPAddressGateway(
    ViSession   instrSession,
    ViString    ipAddressGateway
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:IPAD:GAT",
                                    "'",
                                    ipAddressGateway);
}

/*===========================================================================
 * Function: Get SYSTem Network IP Address Gateway
 * Purpose:  This function returns the gateway.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkIPAddressGateway(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar _VI_FAR  ipAddressGateway[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:IPAD:GAT?",
                              ipAddressGateway,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network IP Address Mode
 * Purpose:  This function sets the IP address mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemNetworkIPAddressMode(
    ViSession   instrSession,
    ViInt32 ipAddressMode
)
{
    return SETPARAMENUM (instrSession,
                          "SYST:COMM:NETW:IPAD:MODE",
                          ipAddressMode,
                          ipAddressModeArr);
}

/*===========================================================================
 * Function: Get SYSTem Network IP Address Mode
 * Purpose:  This function returns the IP address mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkIPAddressMode(
    ViSession   instrSession,
    ViInt32*    ipAddressMode
)
{
    return GETPARAMENUM (instrSession,
                          "SYST:COMM:NETW:IPAD:MODE?",
                          ipAddressMode,
                          ipAddressModeArr);
}

/*===========================================================================
 * Function: Set SYSTem Network Common Domain
 * Purpose:  This function sets the common domain.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemNetworkCommonDomain(
    ViSession   instrSession,
    ViString    commonDomain
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:COMM:DOM",
                                    "'",
                                    commonDomain);
}

/*===========================================================================
 * Function: Get SYSTem Network Common Domain
 * Purpose:  This function returns the common domain.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkCommonDomain(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar _VI_FAR  commonDomain[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:COMM:DOM?",
                              commonDomain,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network Common Hostname
 * Purpose:  This function sets the individual computer name of the R&S SMA.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemNetworkCommonHostname (ViSession instrSession,
                                                        ViString commonHostname)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:COMM:HOST",
                                    "'",
                                    commonHostname);    
}

/*===========================================================================
 * Function: Get SYSTem Network Common Hostname
 * Purpose:  This function returns the individual computer name of the R&S SMA.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkCommonHostname (ViSession instrSession,
                                                        ViInt32 arraySize,
                                                        ViChar commonHostname[])
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:COMM:HOST?",
                              commonHostname,
                              arraySize);    
}

/*===========================================================================
 * Function: Set SYSTem Network Domain
 * Purpose:  This function sets the domain.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemNetworkDomain(
    ViSession   instrSession,
    ViString    domain
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:DOM",
                                    "'",
                                    domain);
}

/*===========================================================================
 * Function: Get SYSTem Network Domain
 * Purpose:  This function returns the domain.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkDomain(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar _VI_FAR  domain[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:DOM?",
                              domain,
                              arraySize);
}

/*===========================================================================
 * Function: Get SYSTem Network Workgroup
 * Purpose:  This function returns the individual workgroup name of 
 *			 the instrument.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkWorkgroup (ViSession instrSession,
                                                   ViInt32 arraySize,
                                                   ViChar _VI_FAR workgroupName[])
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:WORK?",
                              workgroupName,
                              arraySize);    
}

/*===========================================================================
 * Function: Set SYSTem Network Gateway
 * Purpose:  This function sets the gateway.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemNetworkGateway(
    ViSession   instrSession,
    ViString    gateway
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:GAT",
                                    "'",
                                    gateway);
}

/*===========================================================================
 * Function: Get SYSTem Network Gateway
 * Purpose:  This function returns the gateway.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkGateway(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar _VI_FAR  gateway[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:GAT?",
                              gateway,
                              arraySize);
}

/*===========================================================================
 * Function: Get SYSTem Network Resource
 * Purpose:  This function returns the resource string.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkResource(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar _VI_FAR  resource[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:RES?",
                              resource,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network Subnet Mask
 * Purpose:  This function sets the subnet mask.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemNetworkSubnetMask(
    ViSession   instrSession,
    ViString    subnetMask
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:SUBN:MASK",
                                    "'",
                                    subnetMask);
}

/*===========================================================================
 * Function: Get SYSTem Network Subnet Mask
 * Purpose:  This function returns the subnet mask.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkSubnetMask(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar _VI_FAR  subnetMask[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:SUBN:MASK?",
                              subnetMask,
                              arraySize);
}

/*===========================================================================
 * Function: Get SYSTem Network Mac Address
 * Purpose:  This function returns the MAC Address.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetworkMACAddress(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar _VI_FAR  macAddress[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:MACaddress?",
                              macAddress,
                              arraySize);
}

/*===========================================================================
 * Function: Get SYSTem Network
 * Purpose:  This function retrieves the current network settings. They can
 *           be queried with the corresponding functions.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemNetwork(
    ViSession   instrSession
)
{
    return rssma_sendCmd (instrSession,"SYST:COMM:NETW:GET");
}

/*============================================================================
 * Function: rssma_SetInputTriggerSlope
 * Purpose:  This function sets the active slope of an externally applied
 *           trigger signal at the INST TRIG inputs (BNC connector at the rear
 *           of the instrument for Path A, pin at the AUX I/O intf for
 *           Path B). The setting is effective for both inputs at the same
 *           time.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetInstrumentTriggerInputSlope (ViSession instrSession,
                                                        ViInt32 inputTriggerSlope)
{
    return SETPARAMENUM (instrSession,
                          "INP:TRIG:SLOP",
                          inputTriggerSlope,
                          slopeArr);
}

/*============================================================================
 * Function: rssma_GetInputTriggerSlope
 * Purpose:  This function returns the active slope of an externally applied
 *           trigger signal at the INST TRIG inputs (BNC connector at the rear
 *           of the instrument for Path A, pin at the AUX I/O intf for
 *           Path B). The setting is effective for both inputs at the same
 *           time.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetInstrumentTriggerInputSlope (ViSession instrSession,
                                                        ViInt32 *inputTriggerSlope)
{
    return GETPARAMENUM (instrSession,
                          "INP:TRIG:SLOP?",
                          inputTriggerSlope,
                          slopeArr);
}

/*============================================================================
 * Function: rssma_TRIGgerAllSweeps
 * Purpose:  This function starts all sweeps which are activated for the
 *           respective path. The sweep to be executed depends on the
 *           respective MODE setting (:SOURce<1|2>:SWEep:POWer|FREQuency:MODE
 *           STEP). This function corresponds to the manual-control command
 *           Execute Trigger.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_TriggerAllSweeps (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "TRIG");
}

/*============================================================================
 * Function: rssma_TRIGgerRFFrequencySweep
 * Purpose:  This function immediately starts an RF frequency sweep. The sweep
 *           to be executed depends on the respective MODE setting
 *           (:SOURce:SWEep:FREQuency:MODE SING). This function corresponds to
 *           the manual-control command Execute Trigger.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_TriggerRFFrequencySweep (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "TRIG:FSW");
}

/*============================================================================
 * Function: rssma_TRIGgerRFLevelSweep
 * Purpose:  This function immediately starts an RF level sweep. The sweep to
 *           be executed depends on the set sweep mode
 *           (:SOURce<1|2>:SWEep:POWer:MODE). This function corresponds to the
 *           manual-control command Execute Trigger.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_TriggerRFLevelSweep (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "TRIG:PSW");
}

/*============================================================================
 * Function: rssma_TRIGgerListMode
 * Purpose:  This function immediately starts the processing of a list in LIST
 *           mode. It corresponds to the manual-control command Execute
 *           Trigger.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_TriggerListMode (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "TRIG:LIST");
}

/*===========================================================================*/
/* Function: Set Angle Units
/* Purpose:  This function sets the default unit for angles. It is valid for 
/*           all commands which determine angle values. It does not influence 
/*           the manual control parameter unit and the display.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetAngleUnits(
    ViSession   instrSession,
    ViInt32 angleUnits
)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "UNIT:ANGL",
                          angleUnits,
                          angleUnitsArr);
}

/*===========================================================================*/
/* Function: Get Angle Units
/* Purpose:  This function returns the unit for angles. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetAngleUnits(
    ViSession   instrSession,
    ViInt32*    angleUnits
)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "UNIT:ANGL?",
                          angleUnits,
                          angleUnitsArr);
}

/*===========================================================================*/
/* Function: Set Power Units
/* Purpose:  This function sets the default unit for power. It is valid for 
/*           all commands which determine power values. It does not influence 
/*           the manual control parameter unit and the display.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetPowerUnits(
    ViSession   instrSession,
    ViInt32 powerUnits
)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "UNIT:POW",
                          powerUnits,
                          powerUnitsArr);
}

/*===========================================================================*/
/* Function: Get Power Units
/* Purpose:  This function returns the unit for power.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetPowerUnits(
    ViSession   instrSession,
    ViInt32*    powerUnits
)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "UNIT:POW?",
                          powerUnits,
                          powerUnitsArr);
}

/*===========================================================================*/
/* Function: Set Velocity Units
/* Purpose:  This function sets the default unit for velocity. It is valid 
/*           for all commands which determine velocity values. It does not 
/*           influence the manual control parameter unit and the display.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetVelocityUnits(
    ViSession   instrSession,
    ViInt32 velocityUnits
)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "UNIT:VEL",
                          velocityUnits,
                          velocityUnitsArr);
}

/*===========================================================================*/
/* Function: Get Velocity Units
/* Purpose:  This function returns the default unit for velocity. It is 
/*           valid for all commands which determine velocity values. It does 
/*           not influence the manual control parameter unit and the display.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetVelocityUnits(
    ViSession   instrSession,
    ViInt32*    velocityUnits
)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "UNIT:VEL?",
                          velocityUnits,
                          velocityUnitsArr);
}

/*============================================================================
 * Function: rssma_SetPULMExtInpImpedance
 * Purpose:  This function sets the impedance for external feed via the PM
 *           EXT input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPULMExtInpImpedance (ViSession instrSession,
                                              ViInt32   pulmInputImpedance)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:INP:MOD:IMP",
                          pulmInputImpedance,
                          impedanceArr);    
}

/*============================================================================
 * Function: rssma_GetPULMExtInpImpedance
 * Purpose:  This function sets the impedance of the EXT MOD input.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPULMExtInpImpedance (ViSession instrSession,
                                                ViInt32   *pulmInputImpedance)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:INP:MOD:IMP?",
                          pulmInputImpedance,
                          impedanceArr);    
}

/*============================================================================
 * Function: Set Pulse External Input Impedance
 * Purpose:  This function sets the impedance for external feed via the PULSE
 *           EXT input
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetPulseExtInpImpedance (ViSession instrSession,
                                                 ViInt32 pulseExternalInputImpedance)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:INP:PULM:IMP",
                          pulseExternalInputImpedance,
                          pulseExternalInputImpedanceArr);    
}

/*============================================================================
 * Function: Get Pulse External Input Impedance
 * Purpose:  This function returns the impedance for external feed via the PULSE
 *           EXT input
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetPulseExtInpImpedance (ViSession instrSession,
                                                 ViInt32 *pulseExternalInputImpedance)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:INP:PULM:IMP?",
                          pulseExternalInputImpedance,
                          pulseExternalInputImpedanceArr);    
}

/*===========================================================================*/
/* Function: Set Image File Name
/* Purpose:  This function creates/selects a file into which the hardcopy 
/*           will be stored after the HCOP:EXEC command is sent. The 
/*           directory is either defined with the command MMEMory:CDIR or the 
/*           path is specified together with the file name. Access via remote 
/*           control is possible using the commands of the MMEM-Subsystem.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetImageFileName(
    ViSession   instrSession,
    ViString    fileName
)
{
    INIT;
    
    return SETPARAMSTRINGQUOTED (instrSession,
                              "HCOP:FILE:NAME",
                              "'",
                              fileName);    
}

/*===========================================================================*/
/* Function: Get Image File Name
/* Purpose:  This function returns a file name where the hardcopy will be 
/*           stored after the HCOP:EXEC command is sent. The directory is 
/*           either defined with the command MMEMory:CDIR or the path is 
/*           specified together with the file name. Access via remote control 
/*           is possible using the commands of the MMEM-Subsystem.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetImageFileName(
    ViSession   instrSession,
    ViChar _VI_FAR  fileName[]
)
{
    INIT;
    
    return GETPARAMSTRING (instrSession,
                              "HCOP:FILE:NAME?",
                              fileName,
                              1024);    
}

/*===========================================================================*/
/* Function: Set Image Format
/* Purpose:  This function sets the image format in which the hard copy 
/*           should be stored.
/*           
/*           Example(s):
/*           HCOP:DEV:LANG BMP
/*             - 'selects bmp as imge format.
/*           HCOP:FILE 'C:/TEMP/snapshot001'
/*             - 'defines directory, path and file for storing the hard 
/*           copy.
/*           HCOP
/*             - 'executees the hard copy procedure.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_SetImageFormat(
    ViSession   instrSession,
    ViInt32 imageFormat
)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "HCOP:DEV:LANG",
                          imageFormat,
                          imageFormatArr);    
}

/*===========================================================================*/
/* Function: Get Image Format
/* Purpose:  This function returns the image format in which the hard copy 
/*           should be stored.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetImageFormat(
    ViSession   instrSession,
    ViInt32*    imageFormat
)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "HCOP:DEV:LANG?",
                          imageFormat,
                          imageFormatArr);    
}

/*===========================================================================*/
/* Function: Get Image Snapshot
/* Purpose:  This function transfers the hardcopy (snapshot) as a NByte 
/*           stream to the remote client. 
/*===========================================================================*/
ViStatus _VI_FUNC rssma_GetImageSnapshot(
    ViSession   instrSession, ViString destination
)
{
    return rssma_readDataToFile (instrSession, "HCOP:DATA?", destination);
}

/*============================================================================
 * Function: Set Image Size
 * Purpose:  This function sets the size of the hardcopy in number of pixels. 
 *           The first value of the size setting defines the width, the second 
 *           value the height of the image.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetImageSize (ViSession instrSession,
                                      ViInt32 imageSize)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "HCOP:IMAG:SIZE",
                          imageSize,
                          hcopImageSizeArr);     
}

/*============================================================================
 * Function: Get Image Size
 * Purpose:  This function returns the size of the hardcopy in number of pixels. 
 *           The first value of the size setting defines the width, the second 
 *           value the height of the image.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetImageSize (ViSession instrSession,
                                      ViInt32 *imageSize)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "HCOP:IMAG:SIZE?",
                          imageSize,
                          hcopImageSizeArr);         
}

/*============================================================================
 * Function: Set Output Device
 * Purpose:  This function sets the output device.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetOutputDevice (ViSession instrSession,
                                         ViInt32 outputDevice)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "HCOP:DEV",
                          outputDevice,
                          hcopDeviceArr);        
}

/*============================================================================
 * Function: Get Output Device
 * Purpose:  This function returns the output device.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetOutputDevice (ViSession instrSession,
                                         ViInt32 *outputDevice)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "HCOP:DEV?",
                          outputDevice,
                          hcopDeviceArr);               
}

/*============================================================================
 * Function: Save Hard Copy
 * Purpose:  This function triggers the generation of a hardcopy of 
 *           the display.. The data is written into the file selected/created 
 *           with the HCOP:FILE commands.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SaveHardCopy (ViSession instrSession)
{
    INIT;
    
    return rssma_sendCmd (instrSession,
                          "HCOP:EXEC");
}

/*============================================================================
 * Function: Set Automatic Naming State
 * Purpose:  This function activates/deactivates automatic naming of 
 *           the hardcopy files.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAutomaticNamingState (ViSession instrSession,
                                                 ViBoolean state)
{
    INIT;
    
    return SETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:STAT",
                               state);
}

/*============================================================================
 * Function: Get Automatic Naming State
 * Purpose:  This function returns state of automatic naming of the hardcopy 
 *           files.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAutomaticNamingState (ViSession instrSession,
                                                 ViBoolean *state)
{
    INIT;
    
    return GETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:STAT?",
                               state);    
}

/*============================================================================
 * Function: Get Automatic File Path
 * Purpose:  This function returns the file name generated using 
 *           the automatic naming settings. As default the automatically 
 *           generated file name is composed of:
 *           "<Path>/<Prefix><YYYY><MM><DD><Number>.<Format>". Each component 
 *           can be deactivated/activated separately to individually design 
 *           the file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAutomaticFilePath (ViSession instrSession,
                                              ViInt32 arraySize,
                                              ViChar filePath[])
{
    return GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO?",
                              filePath,
                              arraySize);   
}

/*============================================================================
 * Function: Get Automatic File Name
 * Purpose:  This function returns the file name generated using 
 *           the automatic naming settings. As default the automatically 
 *           generated file name is composed of:
 *           "<Path>/<Prefix><YYYY><MM><DD><Number>.<Format>". Each component 
 *           can be deactivated/activated separately to individually design 
 *           the file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAutomaticFileName (ViSession instrSession,
                                              ViChar _VI_FAR fileName[])
{
    INIT;
    
    return GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:FILE?",
                              fileName,
                              1024);   
}

/*============================================================================
 * Function: Set Automatic Directory
 * Purpose:  This function defines the directory into which the hardcopy files 
 *           will be stored if auto naming is activated 
 *           (HCOP:FILE:AUTO:STAT ON).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAutomaticDirectory (ViSession instrSession,
                                               ViString directoryName)
{
    INIT;
    
    return SETPARAMSTRINGQUOTED (instrSession,
                              "HCOP:FILE:AUTO:DIR",
                              "'",
                              directoryName);     
}

/*============================================================================
 * Function: Get Automatic Directory
 * Purpose:  This function returns the directory into which the hardcopy files 
 *           will be stored if auto naming is activated 
 *           (HCOP:FILE:AUTO:STAT ON).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAutomaticDirectory (ViSession instrSession,
                                               ViChar _VI_FAR directory[])
{
    INIT;
    
    return GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:DIR?",
                              directory,
                              1024);      
}

/*============================================================================
 * Function: Delete Image Files
 * Purpose:  This function deletes all files with extensions "bmp", "img", 
 *           "png" and "xpm" in the directory set for automatic naming.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_DeleteImageFiles (ViSession instrSession)
{
    INIT;
    
    return rssma_sendCmd (instrSession,
                          "HCOP:FILE:AUTO:DIR:CLE");    
}

/*============================================================================
 * Function: Set Automatic File Date State
 * Purpose:  This function activates/deactivates the usage of the day/month/year 
 *           in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAutomaticFileDateState (ViSession instrSession,
                                                   ViInt32 fileNamePart,
                                                   ViBoolean state)
{
    INIT;
    
    CHECKVIINT32 (instrSession, fileNamePart, 0, 2);
    
    switch (fileNamePart){
        case RSSMA_HCOPY_AUTO_FILE_DAY:
            rssma_status = SETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:DAY:STAT",
                               state);        
        break;
        case RSSMA_HCOPY_AUTO_FILE_MONTH:
            rssma_status = SETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:MONT:STAT",
                               state);
        break;                         
        case RSSMA_HCOPY_AUTO_FILE_YEAR:
            rssma_status = SETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:YEAR:STAT",
                               state);
        break;
    }
    return rssma_status;
}

/*============================================================================
 * Function: Get Automatic File Date State
 * Purpose:  This function returns the state of usage of the day/month/year in 
 *           the automatic file name
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAutomaticFileDateState (ViSession instrSession,
                                                   ViInt32 fileNamePart,
                                                   ViBoolean *state)
{
    INIT;
    
    CHECKVIINT32 (instrSession, fileNamePart, 0, 2);
    
    switch (fileNamePart){
        case RSSMA_HCOPY_AUTO_FILE_DAY:
            rssma_status = GETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:DAY:STAT?",
                               state);        
        break;
        case RSSMA_HCOPY_AUTO_FILE_MONTH:
            rssma_status = GETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:MONT:STAT?",
                               state);
        break;                         
        case RSSMA_HCOPY_AUTO_FILE_YEAR:
            rssma_status = GETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:YEAR:STAT?",
                               state);
        break;
    }
    return rssma_status;     
}

/*============================================================================
 * Function: Set Automatic File Date State
 * Purpose:  This function queries the day/month/year part in the automatic 
 *           file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAutomaticFileDate (ViSession instrSession,
                                              ViInt32 fileNamePart,
                                              ViChar _VI_FAR part[])
{
    INIT;
    
    CHECKVIINT32 (instrSession, fileNamePart, 0, 2);
    
    switch (fileNamePart){
        case RSSMA_HCOPY_AUTO_FILE_DAY:
            rssma_status = GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:DAY?",
                              part,
                              1024);         
        break;
        case RSSMA_HCOPY_AUTO_FILE_MONTH:
            rssma_status = GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:MONT?",
                              part,
                              1024);  
        break;                         
        case RSSMA_HCOPY_AUTO_FILE_YEAR:
            rssma_status = GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:YEAR?",
                              part,
                              1024);  
        break;
    } 
    return rssma_status; 
}

/*============================================================================
 * Function: Set Automatic File Prefix State
 * Purpose:  This function activates/deactivates the usage of the prefix 
 *           in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAutomaticFilePrefixState (ViSession instrSession,
                                                     ViBoolean state)
{
    INIT; 
    
    return SETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:PREF:STAT",
                               state);
}

/*============================================================================
 * Function: Get Automatic File Prefix State
 * Purpose:  This function returns the state of the usage of the prefix 
 *           in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAutomaticFilePrefixState (ViSession instrSession,
                                                     ViBoolean *state)
{
    INIT; 
    
    return GETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:PREF:STAT?",
                               state);    
}

/*============================================================================
 * Function: Set Automatic File Prefix
 * Purpose:  This function defines the prefix part in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetAutomaticFilePrefix (ViSession instrSession,
                                                ViString prefix)
{
    INIT;
    
    return SETPARAMSTRINGQUOTED (instrSession,
                              "HCOP:FILE:AUTO:PREF",
                              "'",
                              prefix);     
}

/*============================================================================
 * Function: Get Automatic File Prefix
 * Purpose:  This function returns the prefix part in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAutomaticFilePrefix (ViSession instrSession,
                                                ViChar _VI_FAR fileName[])
{
    INIT;
    
    return GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:PREF?",
                              fileName,
                              1024);     
}

/*============================================================================
 * Function: Get Automatic File Number
 * Purpose:  This function returns queries the generated number in the automatic
 *           file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetAutomaticFileNumber (ViSession instrSession,
                                                ViInt32 *number)
{
    INIT;
    
    return GETPARAMINT32 (instrSession,
                              "HCOP:FILE:AUTO:NUMB?",
                              number);        
}

/*============================================================================
 * Function: rssma_ClearStatus
 * Purpose:  CLEAR STATUS sets the status byte (STB), the standard event
 *           register (ESR), and the EVENt part of the QUEStionable and
 *           OPERation register to zero. The function does not change the
 *           enable and transition parts of the registers. It deletes the
 *           output buffer.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ClearStatus (ViSession instrSession)
{
    return rssma_sendCmd ( instrSession, "*CLS" );
}

/*============================================================================
 * Function: rssma_EventStatusEnable
 * Purpose:  EVENT STATUS ENABLE sets the event status enable register to the
 *           specified value. The *ESE? query command returns the content of
 *           the event status enable register in decimal form.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_EventStatusEnable (ViSession instrSession,
                                           ViInt32 registerValue)
{
    return SETPARAMINT32 (instrSession,
                             "*ESE",
                             registerValue,
                             0, 255);
}

/*============================================================================
 * Function: rssma_EventStatusQuery
 * Purpose:  STANDARD EVENT STATUS QUERY returns the content of the event
 *           status register in decimal form (0...255) and then sets the
 *           register to zero.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_EventStatusQuery (ViSession instrSession,
                                          ViInt32 *registerValue)
{
    return GETPARAMINT32 (instrSession,
                             "*ESR?",
                             registerValue);
}

/*============================================================================
 * Function: rssma_IdentificationQuery
 * Purpose:  IDENTIFICATION QUERY queries the instrument identification. The
 *           instrument type, serial number and firmware version are returned.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_IdentificationQuery (ViSession instrSession,
                                             ViChar _VI_FAR instrumentIdentification[])
{
    ViStatus rssma_status = VI_SUCCESS;

    rssma_status = GETPARAMSTRING (instrSession,
                              "*IDN?",
                              instrumentIdentification,
                              256);

    return (rssma_status < VI_SUCCESS) ? rssma_status : VI_SUCCESS;
}

/*============================================================================
 * Function: rssma_IndividualStatusQuery
 * Purpose:  INDIVIDUAL STATUS QUERY returns the content of the IST flag in
 *           decimal form (0 | 1). The IST flag is the status bit which is sent
 *           during a parallel poll.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_IndividualStatusQuery (ViSession instrSession,
                                      ViInt32 *ISTFlag)
{
    return GETPARAMINT32 (instrSession,
                             "*IST?",
                             ISTFlag);
}

/*============================================================================
 * Function: rssma_OperationComplete
 * Purpose:  OPERATION COMPLETE sets bit 0 in the event status register when
 *           all previous commands have been processed. This bit can be used to
 *           trigger a service request.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_OperationComplete (ViSession instrSession)
{
    ViStatus            rssma_status;
    rssma_instrRange    instrPtr = NULL;
    ViBoolean           err_check;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    if (instrPtr == NULL)
        return VI_ERROR_INV_SESSION;

    err_check = instrPtr -> errorChecking;
    
    CHECKERR (rssma_errorCheckState (instrSession, VI_FALSE));
    rssma_status = rssma_sendCmd (instrSession, "*OPC");
    (void) rssma_errorCheckState (instrSession, err_check);
    
    return rssma_status;
}

/*============================================================================
 * Function: rssma_OperationCompleteQuery
 * Purpose:  OPERATION COMPLETE QUERY returns a 1 when all previous commands
 *           have been processed. It is important to ensure that the timeout
 *           set at the IEC/IEEE bus is long enough.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_OperationCompleteQuery (ViSession instrSession,
                                       ViInt32 *OPCState)
{
    return GETPARAMINT32 (instrSession,
                             "*OPC?",
                             OPCState);
}

/*============================================================================
 * Function: rssma_OptionIdentificationQuery
 * Purpose:  OPTION IDENTIFICATION QUERY queries the options configured in the
 *           instrument and returns a list of the installed options. The
 *           options are separated by commas.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_OptionIdentificationQuery (ViSession instrSession,
                                          ViChar _VI_FAR optionIdentification[])
{
    return GETPARAMSTRING (instrSession,
                              "*OPT?",
                              optionIdentification,
                              1024);
}

/*============================================================================
 * Function: rssma_ParallelPollRegisterEnable
 * Purpose:  PARALLEL POLL REGISTER ENABLE sets the parallel poll enable
 *           register to the specified value. The *PRE? query command returns
 *           the content of the parallel poll enable register in decimal form.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ParallelPollRegisterEnable (ViSession instrSession,
                                           ViInt32 registerValue)
{
    return SETPARAMINT32 (instrSession,
                             "*PRE",
                             registerValue,
                             0, 655);
}

/*============================================================================
 * Function: rssma_ParallelPollRegisterQuery
 * Purpose:  PARALLEL POLL REGISTER QUERY returns the content of the parallel
 *           poll enable register in decimal form.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ParallelPollRegisterQuery (ViSession instrSession,
                                          ViInt32 *registerValue)
{
    return GETPARAMINT32 (instrSession,
                             "*PRE?",
                             registerValue);
}

/*============================================================================
 * Function: rssma_PowerOnStatusClear
 * Purpose:  POWER ON STATUS CLEAR determines whether the content of the
 *           ENABle registers are retained or reset at power on.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_PowerOnStatusClear (ViSession instrSession,
                                   ViInt32 registerValue)
{
    return SETPARAMINT32 (instrSession,
                             "*PSC",
                             registerValue,
                             0, 1);
}

/*============================================================================
 * Function: rssma_PowerOnStatusClearQuery
 * Purpose:  POWER ON STATUS CLEAR QUERY retrieves the contents of the
 *           Power-on-Status-Clear flag. The answer is either 0 or 1.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_PowerOnStatusClearQuery (ViSession instrSession,
                                        ViInt32 *registerValue)
{
    return GETPARAMINT32 (instrSession,
                             "*PSC?",
                             registerValue);
}

/*============================================================================
 * Function: rssma_Recall
 * Purpose:  RECALL calls up the instrument status which was stored under the
 *           specified number using the *SAV command or using the
 *           MMEMory:STORe:STATe command. "0" corresponds to the power-on
 *           state, unless a new instrument setting was stored under this
 *           number in the meantime using the MMEMory:STORe:STATe command. This
 *           function can also be used to call up the 3 intermediate instrument
 *           states which are stored with *SAV.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_Recall (ViSession instrSession, ViInt32 instrumentStatus)
{
    return SETPARAMINT32 (instrSession,
                             "*RCL",
                             instrumentStatus,
                             0, 10);
}

/*============================================================================
 * Function: rssma_Save
 * Purpose:  SAVE stores the current device state under the specified number
 *           (see also *RCL). This function is used to store the 3 intermediate
 *           instrument states and to store the current instrument state
 *           internally when the instrument is switched off. This state is then
 *           restored at power on.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_Save (ViSession instrSession, ViInt32 instrumentStatus)
{
    return SETPARAMINT32 (instrSession,
                             "*SAV",
                             instrumentStatus,
                             0, 10);
}

/*============================================================================
 * Function: rssma_ServiceRequestEnable
 * Purpose:  SERVICE REQUEST ENABLE sets the service request enable register
 *           to the specified value. Bit 6 (MSS enable bit) remains 0. This
 *           function determines the conditions under which a service request
 *           is triggered. The *SRE? query command reads out the content of the
 *           service request enable register in decimal form. Bit 6 is always
 *           0.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ServiceRequestEnable (ViSession instrSession,
                                     ViInt32 registerValue)
{
    return SETPARAMINT32 (instrSession,
                             "*SRE",
                             registerValue,
                             0, 255);
}

/*============================================================================
 * Function: rssma_ServiceRequestEnableQuery
 * Purpose:  SERVICE REQUEST ENABLE reads out the content of the service
 *           request enable register in decimal form. Bit 6 is always 0.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_ServiceRequestEnableQuery (ViSession instrSession,
                                          ViInt32 *registerValue)
{
    return GETPARAMINT32 (instrSession,
                             "*SRE?",
                             registerValue);
}

/*============================================================================
 * Function: rssma_StatusByteQuery
 * Purpose:  READ STATUS BYTE QUERY reads out the content of the status byte
 *           in decimal form.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_StatusByteQuery (ViSession instrSession, ViInt32 *statusByte)
{
    return GETPARAMINT32 (instrSession,
                             "*STB?",
                             statusByte);
}

/*============================================================================
 * Function: rssma_Trigger
 * Purpose:  TRIGGER triggers all actions which are waiting for a trigger
 *           event. Specific trigger events can be triggered by means of the
 *           "TRIGger" command system (sweep and lists) or the SOURce
 *           subsystems (baseband).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_Trigger (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "*TRG");
}

/*===========================================================================*/
/* Function: Send Trigger and Wait for OPC                                   */
/* Purpose:  This function triggers all actions waiting for a trigger event  */
/*           in the currently active window and waits for operation completed*/
/*           (OPC) before returning the status code.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_TriggerWaitOPC (ViSession instrSession, ViInt32 timeout)
{
    ViStatus rssma_status = VI_SUCCESS;
    ViChar buffer[BUFFER_SIZE] = "";

    CHECKVIINT32 (instrSession, timeout, 0, 600000);
									 
    CHECKERR (rssma_waitOPC (instrSession, "*TRG;*OPC?", buffer, timeout));

    return rssma_status;
}

/*============================================================================
 * Function: rssma_SelfTestQuery
 * Purpose:  SELF TEST QUERY triggers all self tests of the instrument and
 *           outputs an rssma_status code in decimal form (see Service Manual supplied
 *           with the instrument).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SelfTestQuery (ViSession instrSession, ViInt32 *errorCode)
{
    return GETPARAMINT32 (instrSession,
                             "*TST?",
                             errorCode);
}

/*============================================================================
 * Function: rssma_WaittoContinue
 * Purpose:  WAIT-to-CONTINUE does not allow subsequent commands to be
 *           processed until all previous commands have been executed and all
 *           signals are in their transient condition.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_WaittoContinue (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "*WAI");
}

/*===========================================================================*/
/* Function: Preset Signal Path
/* Purpose:  This function presets all parameters which are related to the 
/*           signal path.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_PresetSignalPath(
    ViSession   instrSession
)
{
    INIT;
    
    return rssma_sendCmd (instrSession, "SOUR:PRES");
}

/*============================================================================
 * Function: rssma_CALibrationAll
 * Purpose:  This function starts all internal adjustments for which no
 *           external measuring equipment is needed. With two-path instruments,
 *           adjustment is performed for both paths.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_CALibrationAll (ViSession instrSession, ViInt32 timeout,
                                        ViInt32 *response)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL (response);

    sprintf (buffer, "CAL:ALL?");

    CHECKERR (rssma_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssma_status = RSSMA_ERROR_CALIBRATION_FAILED;

    return rssma_status;
}

/*============================================================================
 * Function: rssma_CALibrationClockSynthesis
 * Purpose:  This function starts all internal adjustments for which no
 *           external measuring equipment is needed. With two-path instruments,
 *           adjustment is performed for both paths.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_CALibrationClockSynthesis (ViSession instrSession, ViInt32 timeout,
                                                   ViInt32   *response)
{
    ViChar      buffer[BUFFER_SIZE];

    INIT;
    
    CHECKOPT (B29);
    
    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL (response);

    sprintf (buffer, "CAL:CSYN?");

    CHECKERR (rssma_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssma_status = RSSMA_ERROR_CALIBRATION_FAILED;

    return rssma_status;
}

/*============================================================================
 * Function: rssma_CALibrationFrequency
 * Purpose:  This function starts all adjustments which affect the frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_CALibrationFrequency (ViSession instrSession,
                                              ViInt32 timeout,
                                              ViInt32 *response)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL (response);

    sprintf (buffer, "CAL:FREQ:MEAS?");

    CHECKERR (rssma_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssma_status = RSSMA_ERROR_CALIBRATION_FAILED;

    return rssma_status;
}

/*============================================================================
 * Function: rssma_CALibrationModulationGenerators
 * Purpose:  This function performs all adjustments which affect the internal
 *           modulation generators.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_CALibrationModulationGenerators (ViSession instrSession,
                                                         ViInt32 timeout,
                                                         ViInt32 *response)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL (response);

    sprintf (buffer, "CAL:LFO?");

    CHECKERR (rssma_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssma_status = RSSMA_ERROR_CALIBRATION_FAILED;

    return rssma_status;
}

/*============================================================================
 * Function: rssma_CALibrationLevel
 * Purpose:  This function starts all adjustments which affect the level.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_CALibrationLevel (ViSession instrSession,
                                          ViInt32 timeout,
                                          ViInt32 *response)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL (response);

    sprintf (buffer, "CAL:LEV:MEAS?");

    CHECKERR (rssma_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssma_status = RSSMA_ERROR_CALIBRATION_FAILED;

    return rssma_status;
}

/*============================================================================
 * Function: rssma_SetCALibrationLevelState
 * Purpose:  This function switches on or off use of external level
 *           correction data. This switch can be used in case of corrupt
 *           external level correction data.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetCALibrationLevelState(ViSession instrSession,
                                                 ViBoolean state)
{
    return SETPARAMBOOL (instrSession,
                               "CAL:LEV:STAT",
                               state);
}

/*============================================================================
 * Function: rssma_GetCALibrationLevelState
 * Purpose:  This function returns state of external level correction data.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetCALibrationLevelState(ViSession   instrSession,
                                                 ViBoolean*  state)
{
    return GETPARAMBOOL (instrSession,
                               "CAL:LEV:STAT?",
                               state);
}

/*============================================================================
 * Function: Set CALibration Level Adjustment Data
 * Purpose:  This function selects the data used for external level correction. 
 *           By default the instrument uses correction data obtained in 
 *           the factory before delivery. In addition, customer data can be 
 *           used for external level correction. The customer data is obtained 
 *           using an R&S NRP power sensor.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetCALibrationLevelAdjustmentData (ViSession instrSession,
                                                           ViInt32 adjustmentData)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "CAL:LEV:EXT:DATA",
                          adjustmentData,
                          calLevelAdjDataArr);    
}

/*============================================================================
 * Function: Get CALibration Level Adjustment Data (*N)
 * Purpose:  This function returns the data used for external level correction.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetCALibrationLevelAdjustmentData (ViSession instrSession,
                                                           ViInt32 *adjustmentData)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "CAL:LEV:EXT:DATA?",
                          adjustmentData,
                          calLevelAdjDataArr);    
}

/*===========================================================================*/
/* Function: CALibration FM Offset
/* Purpose:  This function starts all adjustment for the FM/PhiM modulator.
/*           
/*           Note:
/*           
/*           If the calibration fails, this function returns an error code.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_CALibrationFMOffset(
    ViSession   instrSession,
    ViInt32 timeout,
    ViInt32*    response
)
{
   ViStatus    rssma_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL (response);

    sprintf (buffer, "CAL:FMOF:MEAS?");

    CHECKERR (rssma_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssma_status = RSSMA_ERROR_CALIBRATION_FAILED;

    return rssma_status;
}

/*===========================================================================*/
/* Function: CALibration DMEM Pulse Slope
/* Purpose:  This function starts all adjustment of the DME pulse slope for 
/*           best linearity.
/*           
/*           Note:
/*           
/*           If the calibration fails, this function returns an error code.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_CALibrationDMEMPulseSlope(
    ViSession   instrSession,
    ViInt32 timeout,
    ViInt32*    response
)
{
    ViChar      buffer[BUFFER_SIZE];
    INIT;
    
    CHECKOPT (K26);
    
    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL (response);

    sprintf (buffer, "CAL:DME:MEAS?");

    CHECKERR (rssma_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssma_status = RSSMA_ERROR_CALIBRATION_FAILED;

    return rssma_status;
}

/*============================================================================
 * Function: rssma_DIAGnosticModules
 * Purpose:  This function checks the assemblies available in the instrument
 *           using the variant and revision state.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_DIAGnosticModules (ViSession instrSession, 
                                           ViString moduleName, 
                                           ViInt32 arraySize,
                                           ViChar _VI_FAR moduleConfiguration[])
{
    ViChar      buffer [BUFFER_SIZE];

    CHECKNULL (moduleConfiguration);

    if (!moduleName)
        sprintf (buffer, "DIAG:BGIN?;*OPC?");
    else
        sprintf (buffer, "DIAG:BGIN? '%s';*OPC?", moduleName);

    return rssma_getCatalog (instrSession,
                             buffer,
                             (ViUInt32) arraySize,
                             moduleConfiguration, "moduleConfiguration");
}

/*============================================================================
 * Function: rssma_DIAGnosticModuleNames
 * Purpose:  This function queries the names of the assemblies available in
 *           the instrument.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_DIAGnosticModuleNames (ViSession instrSession, 
                                               ViInt32 arraySize, 
                                               ViChar _VI_FAR moduleNames[])
{
    ViChar  buffer [BUFFER_SIZE];

    sprintf (buffer, "DIAG:BGIN:CAT?;*OPC?");

    return rssma_getCatalog (instrSession,
                             buffer,
                             (ViUInt32) arraySize,
                             moduleNames, "moduleNames");
}

/*============================================================================
 * Function: rssma_DIAGnosticTestPoints
 * Purpose:  This function queries the test points available in the
 *           instrument. A detailed description of the test points can be found
 *           in Chapter 3 of the Service Manual (supplied with the instrument).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_DIAGnosticTestPoints (ViSession instrSession, 
                                              ViInt32 arraySize, 
                                              ViChar _VI_FAR testPoints[])
{
    ViChar  buffer [BUFFER_SIZE];

    sprintf (buffer, "DIAG:POIN:CAT?;*OPC?");

    return rssma_getCatalog (instrSession,
                             buffer,
                             (ViUInt32) arraySize,
                             testPoints,"testPoints");
}

/*============================================================================
 * Function: rssma_DIAGnosticTestPointVoltages
 * Purpose:  This function triggers voltage measurement at the specified test
 *           point and returns the measured voltage. A detailed description of
 *           the test points can be found in Chapter 3 of the Service Manual
 *           (supplied with the instrument).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_DIAGnosticTestPointVoltages (ViSession instrSession,
                                                     ViString testPointName,
                                                     ViReal64 *measuredVoltage)
{
    ViChar      buffer [BUFFER_SIZE];

    CHECKNULL (testPointName);

    sprintf (buffer, "DIAG:POIN? '%s'", testPointName);

    return GETPARAMREAL64 (instrSession,
                              buffer,
                              measuredVoltage);
}

/*****************************************************************************
 * Function: DIAGnostic Operation Time
 * Purpose:  This function queries the number of operation hours.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_DIAGnosticOperationTime(
    ViSession   instrSession,
    ViUInt32*   operationTime
)
{
    return GETPARAMUINT32 (instrSession,
                              "DIAG:INFO:OTIM?",
                              operationTime);
}

/*****************************************************************************
 * Function: DIAGnostic Power On Events
 * Purpose:  This function queries the number of power-on events.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_DIAGnosticPowerOnEvents(
    ViSession   instrSession,
    ViUInt32*   powerOnEvents
)
{
    return GETPARAMUINT32 (instrSession,
                              "DIAG:INFO:POC?",
                              powerOnEvents);
}

/*============================================================================
 * Function: rssma_TESTDirect
 * Purpose:  The respective hardware assembly responds directly to this
 *           function; any safety mechanisms are bypassed.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_TESTDirect (ViSession instrSession, 
                                    ViInt32 command, 
                                    ViChar _VI_FAR assemblyResponse[])
{
    ViChar  buffer [BUFFER_SIZE];

    CHECKVIINT32 (instrSession, command, 0, 7);

    sprintf (buffer, "TEST:DIR? '%s',0", testCommandsArr[command]);

    return GETPARAMSTRING (instrSession,
                              buffer,
                              assemblyResponse,
                              256);
}

/*****************************************************************************
 * Function: TEST All Start
 * Purpose:  This function performs a selftest on all installed hardware 
 *           options.
 *           
 *           Remote-control command(s):
 *           TEST<hw>:ALL:STARt
 *****************************************************************************/
ViStatus _VI_FUNC rssma_TESTAllStart(
    ViSession   instrSession
)
{
    INIT;
    
    return rssma_sendCmd (instrSession, "TEST:ALL:STAR");
}

/*****************************************************************************
 * Function: TEST All Result
 * Purpose:  This function queries the result of the performed selftest.
 *           
 *           Remote-control command(s):
 *           TEST<hw>:ALL:RESult?
 *****************************************************************************/
ViStatus _VI_FUNC rssma_TESTAllResult(
    ViSession   instrSession,
    ViChar _VI_FAR  result[]
)
{
    INIT;
    
    return GETPARAMSTRING (instrSession, 
                              "TEST:ALL:RES?",
                              result,
                              512);
}

/*===========================================================================
 * Function: Set Display Power Save State
 * Purpose:  This function sets the display power save state. With the 
 *           screen-save mode activated the display including backlight is 
 *           completely switched off after the elapse of the wait time (see 
 *           command DISPlay:PSAVe:HOLDoff) when no entries via front panel, 
 *           external mouse or external keyboard are made.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetDisplayPowerSaveState(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus rssma_status = VI_SUCCESS;
    
    rssma_status = SETPARAMBOOL (instrSession,
                                       "DISP:PSAV:STAT",
                                       state);
    
    return (rssma_status < VI_SUCCESS) ? rssma_status : VI_SUCCESS;
}

/*===========================================================================
 * Function: Get Display Power Save State
 * Purpose:  This function returns the state of display power save.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetDisplayPowerSaveState(
    ViSession   instrSession,
    ViBoolean*  state
)
{
    ViStatus rssma_status = VI_SUCCESS;
    
    rssma_status = GETPARAMBOOL (instrSession,
                                       "DISP:PSAV:STAT?",
                                       state);
    
    return (rssma_status < VI_SUCCESS) ? rssma_status : VI_SUCCESS;
}

/*===========================================================================
 * Function: Set Display Power Save Holdoff
 * Purpose:  This function sets the display power save holdoff.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetDisplayPowerSaveHoldoff(
    ViSession   instrSession,
    ViInt32 holdoff
)
{
    ViStatus rssma_status = VI_SUCCESS;
    
    rssma_status = SETPARAMINT32 (instrSession,
                                     "DISP:PSAV:HOLD",
                                     holdoff,
                                     1, 60);
    
    return (rssma_status < VI_SUCCESS) ? rssma_status : VI_SUCCESS;
}

/*===========================================================================
 * Function: Get Display Power Save Holdoff
 * Purpose:  This function returns the state of display power save holdoff.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetDisplayPowerSaveHoldoff(
    ViSession   instrSession,
    ViInt32*    holdoff
)
{
    ViStatus rssma_status = VI_SUCCESS;
    
    rssma_status = GETPARAMINT32 (instrSession,
                                     "DISP:PSAV:HOLD?",
                                     holdoff);
    
    return (rssma_status < VI_SUCCESS) ? rssma_status : VI_SUCCESS;
}

/*===========================================================================
 * Function: Set Display Update State
 * Purpose:  This function switches the update of the display on/off. A
 *           switchover from remote control to manual control always sets the
*            status of the update of the display to ON.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetDisplayUpdateState (ViSession instrSession,
                                               ViBoolean state)
{
    return SETPARAMBOOL (instrSession,
                               "SYST:DISP:UPD",
                               state);    
}

/*===========================================================================
 * Function: Get Display Update State
 * Purpose:  This function queries the state of update of the display on/off.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetDisplayUpdateState (ViSession instrSession,
                                               ViBoolean *state)
{
   return GETPARAMBOOL (instrSession,
                               "SYST:DISP:UPD?",
                               state);    
}

/*============================================================================
 * Function: Display Dialog Open
 * Purpose:  This function opens the specified dialog. To determine the dialog 
 *			 identifier, use function rssma_getDisplayDialogID.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_diplayDialogOpen (ViSession instrSession,
                                          ViString dialog)
{
	return SETPARAMSTRINGQUOTED(instrSession,
                               ":DISP:DIAL:OPEN",
                               "\"",
							   dialog);       
}

/*============================================================================
 * Function: Display Dialog Close
 * Purpose:  This function closes the specified dialog. To determine the dialog 
 *			 identifier, use function rssma_getDisplayDialogID.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_diplayDialogClose (ViSession instrSession,
                                           ViString dialog)
{
	return SETPARAMSTRINGQUOTED(instrSession,
                               ":DISP:DIAL:CLOS",
							   "\"",
                               dialog);         
}

/*============================================================================
 * Function: Display Dialog ID
 * Purpose:  This function returns the array of the open dialogs identifiers.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_getDisplayDialogID (ViSession instrSession,
                                            ViInt32 arraySize,
                                            ViChar _VI_FAR dialogIDs[])
{
	return rssma_getCatalogWithSeparator(instrSession,
                               ":DISP:DIAL:ID?",
							   ' ',
							   (ViUInt32) arraySize,
							   dialogIDs, 
                               "dialogIDs");     
}

/*============================================================================
 * Function: rssma_MMEMoryCatalog
 * Purpose:  This function reads out the subdirectories and files in the
 *           specified directory. If no directory is specified, the default
 *           directory selected with the MMEM:CDIR command is read out on the
 *           default drive selected with the MMEM:MSIS command.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryCatalog (ViSession instrSession,
                               ViString directoryPath, ViInt32 arraySize,
                               ViChar _VI_FAR catalogList[])
{
    ViChar  buffer [BUFFER_SIZE];

    if (!directoryPath)
        sprintf (buffer, "MMEM:CAT?;*OPC?");
    else
        sprintf (buffer, "MMEM:CAT? '%s';*OPC?", directoryPath);

    return rssma_getCatalog (instrSession,
                             buffer,
                             (ViUInt32) arraySize,
                             catalogList, "catalogList");
}

/*============================================================================
 * Function: rssma_MMEMoryCatalogLength
 * Purpose:  This function reads out the number of files in the specified
 *           directory. If no directory is specified, the default directory
 *           selected with the MMEM:CDIR command is read out on the default
 *           drive selected with the MMEM:MSIS command.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryCatalogLength (ViSession instrSession,
                                     ViString directoryPath,
                                     ViInt32 *catalogLength)
{
    ViChar  buffer [BUFFER_SIZE];

    if (!directoryPath)
        sprintf (buffer, "MMEM:CAT:LENG?");
    else
        sprintf (buffer, "MMEM:CAT:LENG? '%s'", directoryPath);

    return GETPARAMINT32 (instrSession,
                             buffer,
                             catalogLength);
}

/*============================================================================
 * Function: rssma_MMEMoryChangeDirectory
 * Purpose:  This function changes the default directory. This directory is
 *           used for all subsequent MMEM commands if no path is specified with
 *           them. It is also possible to change to a higher directory using
 *           two dots '..'.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryChangeDirectory (ViSession instrSession,
                                       ViString directoryName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                            "MMEM:CDIR",
                            "'",
                            directoryName);
}

/*============================================================================
 * Function: rssma_MMEMoryCopy
 * Purpose:  This function copies the first specified file to the second
 *           specified file. Instead of just a file, this function can also be
 *           used to copy a complete directory together with all its files.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryCopy (ViSession instrSession, ViString sourcePath,
                            ViString destinationPath)
{
    ViStatus    rssma_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKNULL (sourcePath);

    if (destinationPath)
        sprintf (buffer, "MMEM:COPY '%s','%s'", sourcePath, destinationPath);
    else
        sprintf (buffer, "MMEM:COPY '%s'", sourcePath);

    rssma_status = rssma_sendCmd (instrSession, buffer);

    return rssma_status;
}

/*============================================================================
 * Function: rssma_MMEMoryBlockDataWrite
 * Purpose:  This function writes the block data <binary block data> to the
 *           file identified by <file_name>.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryBlockDataWrite (ViSession instrSession,
                                      ViString fileName,
                                      ViInt32 binaryBlockLength,
                                      ViChar _VI_FAR binaryBlockData[])
{
    ViStatus    rssma_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViChar*     pbuffer;
    ViChar*     p2buf;

    CHECKNULL (fileName);
    CHECKNULL (binaryBlockData);

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    p2buf = pbuffer = malloc (strlen (fileName) + binaryBlockLength + 1000);

    p2buf += sprintf (pbuffer, "MMEM:DATA '%s',#%ld%ld",
                               fileName,
                               (ViInt32)(floor(log10((double)binaryBlockLength)) + 1),
                               binaryBlockLength);

    memcpy (p2buf, binaryBlockData, (size_t) binaryBlockLength);
//    p2buf[binaryBlockLength] = '\n';

    rssma_status = viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) ((p2buf - pbuffer) + binaryBlockLength), VI_NULL);

    if (pbuffer)
        free (pbuffer);

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL));

    return rssma_status;
}

/*============================================================================
 * Function: rssma_MMEMoryBlockDataRead
 * Purpose:  This function transfers the specified file from the R&S SMA to
 *           the IEC/IEEE bus and then on to the control computer. It is
 *           important to ensure that the intermediate memory on the control
 *           computer is large enough to take the file. In this case, the
 *           setting for the IEC/IEEE-bus terminator is irrelevant.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryBlockDataRead (ViSession instrSession,
                                              ViString fileName,
                                              ViInt32 binaryBlockLength,
                                              ViChar _VI_FAR binaryBlockData[])
{
    ViStatus    rssma_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViUInt32    count           = 0,
                lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
                format[32],
                *pbuffer = NULL, *p2buffer = NULL;

    CHECKNULL (fileName);

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    sprintf (buffer, "MMEM:DATA? '%s'\n", fileName);

    CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL));
    CHECKERR (rssma_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1lu", &lengthOfLength) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    sprintf (format, "%%%lud", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    p2buffer += lengthOfLength;

    if (intfType == VI_INTF_GPIB)
        if ((rssma_status = viWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL)) < VI_SUCCESS)
            {
            free (pbuffer);
            return rssma_status;
            }

    if ((rssma_status = rssma_checkStatus (instrSession)) < VI_SUCCESS)
        {
        free (pbuffer);
        return rssma_status;
        }

    if (count > (ViUInt32) binaryBlockLength)
        {
        rssma_status = RSSMA_WARNING_SMALL_BUFFER;
        count = (ViUInt32) binaryBlockLength;
        memcpy (binaryBlockData, p2buffer, (size_t) count);
        }
    else
        {
        rssma_status = VI_SUCCESS;
        memcpy (binaryBlockData, p2buffer, (size_t) count);
        }

    free (pbuffer);

    return rssma_status;
}

/*============================================================================
 * Function: rssma_MMEMoryDirectoryCatalog
 * Purpose:  This function reads out the subdirectories of the specified
 *           directory. If no directory is specified, the default directory
 *           selected with the MMEM:CDIR command is read out. The directories
 *           are output in a list (the list entries are separated by commas).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryDirectoryCatalog (ViSession instrSession,
                                        ViString directoryPath,
                                        ViInt32 arraySize, ViChar _VI_FAR catalogList[])
{
    ViStatus    rssma_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!directoryPath)
        sprintf (buffer, "MMEM:DCAT?;*OPC?");
    else
        sprintf (buffer, "MMEM:DCAT? '%s';*OPC?", directoryPath);

    rssma_status = rssma_getCatalog (instrSession,
                                     buffer,
                                     (ViUInt32) arraySize,
                                     catalogList, "catalogList");

    return rssma_status;
}

/*============================================================================
 * Function: rssma_MMEMoryDirectoryCatalogLength
 * Purpose:  This function reads out the number of subdirectories in the
 *           specified directory. If no directory is specified, the directory
 *           selected with the MMEM:CDIR command is read out.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryDirectoryCatalogLength (ViSession instrSession,
                                              ViString directoryPath,
                                              ViInt32 *catalogLength)
{
    ViStatus    rssma_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!directoryPath)
        sprintf (buffer, "MMEM:DCAT:LENG?");
    else
        sprintf (buffer, "MMEM:DCAT:LENG? '%s'", directoryPath);

    rssma_status = GETPARAMINT32 (instrSession,
                                     buffer,
                                     catalogLength);

    return rssma_status;
}

/*============================================================================
 * Function: rssma_MMEMoryDelete
 * Purpose:  This function deletes the specified file.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryDelete (ViSession instrSession, ViString fileName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                              "MMEM:DEL",
                              "'",
                              fileName);
}

/*============================================================================
 * Function: rssma_MMEMoryLoad
 * Purpose:  This function loads the specified file stored under the specified
 *           name in an internal memory. If 0 is used, the instrument setting
 *           of the selected file is set directly in the R&S SMA. The *RCL
 *           command is used to load the intermediate instrument setting (see
 *           the section Common Functions (Commands)).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryLoad (ViSession instrSession,
                            ViInt32 instrumentSetting, ViString fileName)
{
    ViChar buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, instrumentSetting, 0, 10);
    CHECKNULL (fileName);

    sprintf (buffer, "MMEM:LOAD:STAT %ld,'%s'", instrumentSetting, fileName);

    return rssma_sendCmd (instrSession, buffer);
}

/*============================================================================
 * Function: rssma_MMEMoryMakeDirectory
 * Purpose:  This function creates a new subdirectory in the specified
 *           directory. If no directory is specified, a subdirectory is created
 *           in the default directory. This function can also be used to create
 *           a directory tree.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryMakeDirectory (ViSession instrSession,
                                     ViString directoryName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                              "MMEM:MDIR",
                              "'",
                              directoryName);
}

/*============================================================================
 * Function: rssma_MMEMoryMove
 * Purpose:  This function renames an existing file ('File Source') if no path
 *           is specified for 'File Destination'. Otherwise the file is moved
 *           to the specified path and stored under the original file name or,
 *           if specified, a new file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryMove (ViSession instrSession, ViString fileSource,
                            ViString fileDestination)
{
    ViStatus    rssma_status;
    ViChar*     pbuffer;

    CHECKNULL (fileSource);
    CHECKNULL (fileDestination);

    pbuffer = malloc (strlen (fileSource) + strlen (fileDestination) + 20);

    sprintf (pbuffer, "MMEM:MOVE '%s','%s'", fileSource, fileDestination);

    rssma_status = rssma_sendCmd (instrSession, pbuffer);

    free (pbuffer);

    RETURN;
}

/*============================================================================
 * Function: rssma_MMEMoryRemoveDirectory
 * Purpose:  This function deletes the specified subdirectory in the specified
 *           directory. If no directory is specified, the subdirectory with the
 *           specified name is deleted in the default directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryRemoveDirectory (ViSession instrSession,
                                       ViString directoryName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                              "MMEM:RDIR",
                              "'",
                              directoryName);
}

/*============================================================================
 * Function: rssma_MMEMoryStore
 * Purpose:  This function stores the current instrument setting in the
 *           specified file. If 0 is specified, the current instrument setting
 *           is stored directly in the specified file. The intermediate
 *           instrument settings can be stored using the *SAV command (see the
 *           section Common Functions (Commands)).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_MMEMoryStore (ViSession instrSession,
                             ViInt32 instrumentSetting, ViString fileName)
{
    ViChar buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, instrumentSetting, 0, 10);
    CHECKNULL (fileName);

    sprintf (buffer, "MMEM:STOR:STAT %ld,'%s'", instrumentSetting, fileName);

    return rssma_sendCmd (instrSession, buffer);
}

/*============================================================================
 * Function: rssma_setRegister
 * Purpose:  This function sets the value of selected status register
 *===========================================================================*/
ViStatus _VI_FUNC rssma_setRegister (ViSession instrSession, ViInt32 registerGroup,
                            ViInt32 registerType, ViInt32 value)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, registerGroup, 0, 1);
    CHECKVIINT32 (instrSession, registerType, 2, 4);

    sprintf (buffer, "STAT:%s:%s", regGroupArr[registerGroup], regTypeArr[registerType]);

    return SETPARAMINT32 (instrSession,
                             buffer,
                             value,
                             0, 0);
}

/*============================================================================
 * Function: rssma_getRegister
 * Purpose:  This function returns the value of selected status register
 *===========================================================================*/
ViStatus _VI_FUNC rssma_getRegister (ViSession instrSession, ViInt32 registerGroup,
                            ViInt32 registerType, ViInt32* value)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, registerGroup, 0, 1);
    CHECKVIINT32 (instrSession, registerType, 0, 4);

    sprintf (buffer, "STAT:%s:%s?", regGroupArr[registerGroup], regTypeArr[registerType]);

    return GETPARAMINT32 (instrSession,
                             buffer,
                             value);
}

/*============================================================================
 * Function: rssma_STATusPRESet
 * Purpose:  This function resets the status registers. All PTRansition parts
 *           are set to FFFFh (32767), i.e. all transitions from 0 to 1 are
 *           detected. All NTRansition parts are set to 0, i.e. a transition
 *           from 1 to 0 in a CONDition bit is not detected. The ENABle parts
 *           of STATus:OPERation and STATus:QUEStionable are set to 0, i.e. all
 *           events in these registers are not passed on.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_STATusPRESet (ViSession instrSession)
{
    return rssma_sendCmd (instrSession, "STAT:PRES");
}

/*============================================================================
 * Function: rssma_SetSYSTemBeeper
 * Purpose:  This function switches the key beep on or off.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemBeeper (ViSession instrSession,
                                ViBoolean systemBeeper)
{
    return SETPARAMBOOL (instrSession,
                               "SYST:BEEP:STAT",
                               systemBeeper);
}

/*============================================================================
 * Function: rssma_GetSYSTemBeeper
 * Purpose:  This function returns status of the key beep on or off.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemBeeper (ViSession instrSession,
                                ViBoolean *systemBeeper)
{
    return GETPARAMBOOL (instrSession,
                               "SYST:BEEP:STAT?",
                               systemBeeper);
}

/*============================================================================
 * Function: rssma_SYSTemErrorQueueAll
 * Purpose:  This function queries all entries in the rssma_status queue and then
 *           deletes them. Positive error numbers denote instrument-specific
 *           errors, and negative rssma_status numbers denote rssma_status messages defined
 *           by SCPI (see Chapter 9). If the error queue is empty, 0 ("No
 *           error") is returned.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SYSTemErrorQueueAll (ViSession instrSession,
                                    ViInt32 arraySize, ViChar _VI_FAR errorQueue[])
{
    return GETPARAMSTRING (instrSession,
                              "SYST:ERR:ALL?",
                              errorQueue,
                              (ViUInt32) arraySize);
}

/*============================================================================
 * Function: rssma_SYSTemErrorQueueCodeAll
 * Purpose:  This function queries all entries in the error queue and then
 *           deletes them. Only the error numbers are returned and not the
 *           entire error text. Positive error numbers denote
 *           instrument-specific errors, and negative error numbers denote
 *           error messages defined by SCPI (see Chapter 9). If the error queue
 *           is empty, 0 ("No Error") is returned.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SYSTemErrorQueueCodeAll (ViSession instrSession,
                                        ViInt32 arraySize,
                                        ViChar _VI_FAR errorQueueCode[])
{
    return GETPARAMSTRING (instrSession,
                              "SYST:ERR:CODE:ALL?",
                              errorQueueCode,
                              (ViUInt32) arraySize);
}

/*============================================================================
 * Function: rssma_SYSTemErrorQueueCodeNext
 * Purpose:  This function queries the oldest entry in the error queue and
 *           then deletes it. Only the error number is returned and not the
 *           entire error text. Positive error numbers denote
 *           instrument-specific errors, and negative error numbers denote
 *           error messages defined by SCPI (see Chapter 9). If the error queue
 *           is empty, 0 is returned.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SYSTemErrorQueueCodeNext (ViSession instrSession,
                                         ViInt32 *errorQueueCodeNext)
{
    return GETPARAMINT32 (instrSession,
                             "SYST:ERR:CODE?",
                             errorQueueCodeNext);
}

/*============================================================================
 * Function: rssma_SYSTemErrorQueueCount
 * Purpose:  This function queries the number of entries in the error queue.
 *           If the error queue is empty, '0' is returned.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SYSTemErrorQueueCount (ViSession instrSession,
                                      ViInt32 *errorQueueCount)
{
    return GETPARAMINT32 (instrSession,
                             "SYST:ERR:COUN?",
                             errorQueueCount);
}

/*============================================================================
 * Function: System Error All
 * Purpose:  This function returns a list of all errors existing at the time 
 *			 when the query is started.
 *			 This list corresponds to the display on the info page under
 *			 manual control.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SYSTemErrorAll (ViSession instrSession,
                                        ViInt32 arraySize, ViChar _VI_FAR errorQueue[])
{
    return GETPARAMSTRING (instrSession,
                              "SYST:SERR?",
                              errorQueue,
                              (ViUInt32) arraySize);    
}

/*===========================================================================
 * Function: SYSTem USB Resource
 * Purpose:  This function queries the USB resource.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SYSTemUSBResource (ViSession instrSession,
                                           ViInt32 arraySize,
                                           ViChar USBResource[])
{
    return GETPARAMSTRING (instrSession,
                              ":SYST:COMM:USB:RES?",
                              USBResource,
                              arraySize);
}

/*****************************************************************************
 * Function: SYSTem GPIB Resource
 * Purpose:  This function queries the visa resource string for the GPIB
 *           remote control interface. This string is used for remote control
 *           of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SYSTemGPIBResource(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar    gpibResource[]
)
{
    INIT;

    return GETPARAMSTRING (instrSession,
                              ":SYST:COMM:GPIB:RES?",
                              gpibResource,
                              (ViUInt32) arraySize);
}

/*****************************************************************************
 * Function: SYSTem Serial Resource
 * Purpose:  This function queries the visa resource string for the serial
 *           remote control interface. This string is used for remote control
 *           of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SYSTemSerialResource(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar    serialResource[]
)
{
    INIT;

    return GETPARAMSTRING (instrSession,
                              ":SYST:COMM:SER:RES?",
                              serialResource,
                              (ViUInt32) arraySize);
}

/*****************************************************************************
 * Function: SYSTem HiSLIP Resource
 * Purpose:  This function queries the VISA resource string. This string is 
 *			 used for remote control of the instrument with HiSLIP protocol.
 * 
 *           Remote-control command(s):
 *           :SYSTem:COMMunicate:HISLip:RESource?
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SYSTemHISLIPResource (ViSession instrSession,
                                              ViInt32 arraySize,
                                              ViChar hiSLIPResource[])
{
    INIT;
	
	return GETPARAMSTRING (instrSession,
                              ":SYST:COMM:HISL:RES?",
                              hiSLIPResource,
                              (ViUInt32) arraySize);    
}

/*****************************************************************************
 * Function: SYSTem Socket Resource
 * Purpose:  This function queries the VISA resource string. This string is 
 *			 used for remote control of the instrument with Socket interface.
 * 
 *           Remote-control command(s):
 *           :SYSTem:COMMunicate:SOCKet:RESource?
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SYSTemSocketResource (ViSession instrSession,
                                              ViInt32 arraySize,
                                              ViChar socketResource[])
{
    INIT;
	
	return GETPARAMSTRING (instrSession,
                              ":SYST:COMM:SOCK:RES?",
                              socketResource,
                              (ViUInt32) arraySize);        
}

/*****************************************************************************
 * Function: SYSTem Option User Defined
 * Purpose:  This function defines the OPT string for selection user defined
 *           (SYST:IDEN USER).
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SYSTemOptionUserDefined(
    ViSession   instrSession,
    ViString    userDefinedIdentification
)
{
    INIT;

    return SETPARAMSTRING (instrSession,
                              ":SYST:ORES",
                              userDefinedIdentification);
}

/*****************************************************************************
 * Function: SYSTem Identification User Defined
 * Purpose:  This function defines the identification string for selection 
 *			 user defined (SYST:IDEN USER).
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SYSTemIdentificationUserDefined (ViSession instrSession,
                                                         ViString userDefinedIdentification)
{
    INIT;

    return SETPARAMSTRING (instrSession,
                              ":SYST:IRES",
                              userDefinedIdentification);
}

/*****************************************************************************
 * Function: Set SYSTem Annotation Amplitude 
 * Purpose:  This function sets the displaing of currently selected level 
 *			 in the header of the instrument. 
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetSYSTemAnnotationAmplitude (ViSession instrSession,
                                                      ViBoolean amplitude)
{
    INIT;

    return SETPARAMBOOL (instrSession,
                              ":DISP:ANN:AMPL",
                              amplitude);        
}

/*****************************************************************************
 * Function: Get SYSTem Annotation Amplitude 
 * Purpose:  This function returns the state of displaying currently selected 
 *			 level in the header of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetSYSTemAnnotationAmplitude (ViSession instrSession,
                                                      ViBoolean *amplitude)
{
    INIT;

    return GETPARAMBOOL (instrSession,
                              ":DISP:ANN:AMPL?",
                              amplitude);     
}

/*****************************************************************************
 * Function: Set SYSTem Annotation Frequency 
 * Purpose:  This function sets the displaing of currently selected frequency 
 *			 in the header of the instrument. 
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetSystemAnnotationFrequency (ViSession instrSession,
                                                      ViBoolean frequency)
{
    INIT;

    return SETPARAMBOOL (instrSession,
                              ":DISP:ANN:FREQ",
                              frequency);    
}

/*****************************************************************************
 * Function: Get SYSTem Annotation Frequency 
 * Purpose:  This function returns the state of displaying currently selected 
 *			 frequency in the header of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetSystemAnnotationFrequency (ViSession instrSession,
                                                      ViBoolean *frequency)
{
    INIT;

    return GETPARAMBOOL (instrSession,
                              ":DISP:ANN:FREQ?",
                              frequency);         
}

/*****************************************************************************
 * Function: Set SYSTem Annotation All 
 * Purpose:  This function sets the displaing of currently selected level and 
 * 			 frequency in the header of the instrument. 
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetSYSTemAnnotationAll (ViSession instrSession,
                                                ViBoolean state)
{
    INIT;

    return SETPARAMBOOL (instrSession,
                              ":DISP:ANN",
                              state);      
}

/*****************************************************************************
 * Function: Get SYSTem Annotation All 
 * Purpose:  This function returns the state of displaying currently selected 
 *			 level and frequency in the header of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetSYSTemAnnotationAll (ViSession instrSession,
                                                ViBoolean *state)
{
    INIT;

    return GETPARAMBOOL (instrSession,
                              ":DISP:ANN?",
                              state);         
}

/*============================================================================
 * Function: rssma_SetSYSTemKeyboardLock
 * Purpose:  This function (Keyboard LOCk) disables the front panel keyboard
 *           of the R&S SMA including the  LOCAL  key, or enables it again
 *           (OFF).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemKeyboardLock (ViSession instrSession,
                                      ViBoolean keyboardLock)
{
    return SETPARAMBOOL (instrSession,
                               "SYST:KLOC",
                               keyboardLock);
}

/*============================================================================
 * Function: rssma_GetSYSTemKeyboardLock
 * Purpose:  This function returns if the front panel keyboard of the R&S SMA
 *           including the LOCAL key is disabled or enabled.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemKeyboardLock (ViSession instrSession,
                                      ViBoolean *keyboardLock)
{
    return GETPARAMBOOL (instrSession,
                               "SYST:KLOC?",
                               keyboardLock);
}

/*****************************************************************************
 * Function: Set SYSTem Display Lock
 * Purpose:  This function (Display LOCk) disables the manual operation via 
 *           the display, or enables it again (OFF).
 *           
 *           
 *           Remote-control command(s):
 *           SYSTem:DLOCk ON | OFF
 *****************************************************************************/
ViStatus _VI_FUNC rssma_SetSYSTemDisplayLock(
    ViSession   instrSession,
    ViBoolean   displayLock
)
{
    INIT;
    
    return SETPARAMBOOL (instrSession,
                               "SYST:DLOC",
                               displayLock);
}

/*****************************************************************************
 * Function: Get SYSTem Display Lock
 * Purpose:  This function returns if the front panel keyboard of the R&S 
 *           Signal Generator including the LOCAL key is disabled or enabled.
 *           
 *           
 *           Remote-control command(s):
 *           SYSTem:KLOCk?
 *           
 *****************************************************************************/
ViStatus _VI_FUNC rssma_GetSYSTemDisplayLock(
    ViSession   instrSession,
    ViBoolean*  displayLock
)
{
    INIT;
    
    return GETPARAMBOOL (instrSession,
                               "SYST:DLOC?",
                               displayLock);
}

/*============================================================================
 * Function: rssma_SetSYSTemProtect
 * Purpose:  This function activates and deactivates the specified protection
 *           level. There are several protection levels which disable specific
 *           service functions (authorized personnel of R&S Service Deparments
 *           only).
 *===========================================================================*/
ViStatus _VI_FUNC rssma_SetSYSTemProtect (ViSession instrSession,
                                 ViInt32 protectionLevel, ViBoolean protection,
                                 ViString password)
{
    ViChar  buffer [BUFFER_SIZE];

    CHECKVIINT32 (instrSession, protectionLevel, 1, 5);
    CHECKNULL (password);

    if (protection == VI_FALSE)
        sprintf (buffer, "SYST:PROT%ld OFF,%s", protectionLevel, password);
    else
        sprintf (buffer, "SYST:PROT%ld ON", protectionLevel);

    return rssma_sendCmd (instrSession, buffer);
}

/*============================================================================
 * Function: rssma_GetSYSTemVersion
 * Purpose:  This function queries the SCPI version with which the instrument
 *           complies.
 *===========================================================================*/
ViStatus _VI_FUNC rssma_GetSYSTemVersion (ViSession instrSession,
                                 ViChar _VI_FAR SCPIVersion[])
{
    ViStatus rssma_status = VI_SUCCESS;

    rssma_status = GETPARAMSTRING (instrSession,
                              "SYST:VERS?",
                              SCPIVersion,
                              32);

    return (rssma_status < VI_SUCCESS) ? rssma_status : VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Read To File From Instrument                                    */
/* Purpose:  This function is used to read data from the instrument and      */
/*           write it to a user specified file.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_readToFile (ViSession instrSession, ViString source,
                           ViString destination)
{
#if 0
    ViStatus    rssma_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViUInt32    count           = 0,
                lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
                format[32],
                *pbuffer = NULL, *p2buffer = NULL;
    FILE        *targetFile;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    sprintf (buffer, ":MMEM:DATA? '%s'\n", source);

    CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL));
    CHECKERR (rssma_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1lu", &lengthOfLength) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    sprintf (format, "%%%lud", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    p2buffer += lengthOfLength;

    if (intfType == VI_INTF_GPIB)
        if ((rssma_status = viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL)) < VI_SUCCESS)
            {
            free (pbuffer);
            return rssma_status;
            }

    if ((rssma_status = rssma_checkStatus (instrSession)) < VI_SUCCESS)
        {
        free (pbuffer);
        return rssma_status;
        }

    /* Store data from instrument to file on host computer */
    if ((targetFile = fopen (destination, "wb")) == NULL)
        {
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    if (count != (ViUInt32) fwrite (p2buffer, sizeof (ViByte), (size_t)(count), targetFile))
        {
        (void) fclose (targetFile);
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_WRITE;
        }

    (void) fclose (targetFile);

    if (pbuffer) /* If not NULL pointer, then deallocate memory. */
        free (pbuffer);

    if ((rssma_status = rssma_checkStatus (instrSession)) < 0)
        return rssma_status;

    return rssma_status;
#endif
    ViChar      buffer[BUFFER_SIZE];
    
    sprintf (buffer, ":MMEM:DATA? '%s'\n", source);
    
    return rssma_readDataToFile (instrSession, buffer, destination);
}

/*===========================================================================*/
/* Function: Write From File To Instrument                                   */
/* Purpose:  This function is used to read data from a user specified file   */
/*           and write it to the instrument.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_writeFromFile (ViSession instrSession, ViString source,
                              ViString destination)
{
    ViStatus    rssma_status;
    FILE        *source_file;
    ViInt32     file_size;
    ViByte      *data;
    
    
    if ((source_file = fopen (source, "rb")) == NULL)
        return VI_ERROR_INSTR_FILE_OPEN;

    /* Get size of file */
    (void) fseek (source_file, 0, SEEK_END);
    file_size = ftell (source_file);
    (void) fseek (source_file, 0, SEEK_SET);

    data=malloc(sizeof(char)*file_size);
    (void) fread (data, sizeof (ViByte), (size_t) file_size, source_file);
    (void) fclose (source_file);

    rssma_status = rssma_MMEMoryBlockDataWrite (instrSession, destination, file_size,
                                           (ViChar*) data);

    free (data);
    return rssma_status;
}

/*===========================================================================*/
/* Function: State Checking                                                  */
/* Purpose:  This function switches ON/OFF state checking of the instrument  */
/*           (reading of the Standard Event Register and checking it for     */
/*           error). Nearly all driver function are using state checking.    */
/*           Switch this function to ON when debug your application. For     */
/*           better bus throughput and instruments performance switch it OFF.*/
/*===========================================================================*/
ViStatus _VI_FUNC rssma_errorCheckState (ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus    rssma_status = VI_SUCCESS;
    rssma_instrRange instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    CHECKNULL (instrPtr);
    
    instrPtr -> errorChecking = state;

    return rssma_status;
}

/*===========================================================================*/
/* Function: Option Checking                                                 */
/* Purpose:  This function switches ON/OFF option checking of the instrument */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_optionCheckState (ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus    rssma_status = VI_SUCCESS;
    rssma_instrRange instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> optionChecking = state;

    return rssma_status;
}

/*===========================================================================*/
/* Function: Range Checking                                                  */
/* Purpose:  This function switches ON/OFF range checking of the instrument  */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_rangeCheckState (ViSession instrSession,
                                         ViBoolean rangeChecking)
{
    ViStatus    rssma_status = VI_SUCCESS;
    rssma_instrRange instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> rangeChecking = rangeChecking;

    return rssma_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus rssma_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "%s\n", writeBuffer));

    return rssma_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_readInstrData (ViSession instrSession,
                                        ViInt32 numberBytesToRead,
                                        ViChar _VI_FAR readBuffer[],
                                        ViPInt32 numBytesRead)
{
    ViStatus    rssma_status = VI_SUCCESS;
    ViUInt32    tmpCnt = 0;

    if (numBytesRead != NULL)
        *numBytesRead = 0L;

    CHECKERR (viRead (instrSession, (ViBuf)readBuffer, (ViUInt32) numberBytesToRead, &tmpCnt));

    if (numBytesRead)
        *numBytesRead = (ViInt32)tmpCnt;

    return rssma_status;
}

/*===========================================================================*/
/* Function: Device Preset
/* Purpose:  This function presets all parameters which are not related to 
/*           the signal path. This includes presetting the LF generator.
/*===========================================================================*/
ViStatus _VI_FUNC rssma_DevicePreset(
    ViSession   instrSession
)
{
    INIT;
    
    return rssma_sendCmd (instrSession, "DEV:PRES");
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_reset (ViSession instrSession)
{
    ViStatus rssma_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    CHECKERR (viPrintf (instrSession, "*RST;*CLS\n"));
    CHECKERR (rssma_defaultInstrSetup (instrSession));

    return rssma_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_self_test (ViSession instrSession,
                                    ViPInt16 testResult,
                                    ViChar _VI_FAR testMessage[])
{
    ViStatus rssma_status = VI_SUCCESS;

    CHECKERR( viQueryf (instrSession, "*TST?\n", "%hd", testResult));

    sprintf (testMessage, "%s", (*testResult == 0) ? "Self-Test Passed" : "Self-Test Failed");

    return rssma_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result. If the error query function is not supported by the */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_ERROR_QUERY.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_error_query (ViSession instrSession,
                                      ViPInt32 errorCode,
                                      ViChar _VI_FAR errorMessage[])
{
    ViStatus    rssma_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE] = "";
    ViInt32     oldCode = 0;
    ViChar      oldMessage[256] = "";
    ViChar      *p2buf = NULL;
    ViInt32     i = 0;
	ViUInt32    retCnt = 0;

	p2buf = (char *)malloc (1);

    *p2buf='\0';
        
    do
	{
		CHECKERR (viWrite (instrSession, (ViBuf) ":SYST:ERR?\n", 11, VI_NULL));
		CHECKERR (viRead (instrSession, (ViBuf)buffer, BUFFER_SIZE, &retCnt));

	    if  (sscanf (buffer, "%ld,%*[^0-9a-zA-Z]%256[^\"]", &oldCode, oldMessage) != 2)
        {
            return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }
        else
        {
            if (oldCode != 0)
            {
                *errorCode = oldCode;
                p2buf = (char *)realloc (p2buf, strlen(p2buf) + strlen (oldMessage) + 2);
                strcat (p2buf, oldMessage);
                strcat (p2buf, "\n");
                i++;
            }
            else if (i == 0)
            {
                *errorCode = oldCode;
                strncpy (errorMessage, oldMessage, 1024);
            }
        }

    } while (oldCode != 0);
    
    if (i > 0)
    {
        p2buf[strlen (p2buf) - 1] = '\0'; 
        strncpy (errorMessage, p2buf, 1024);
    }
   
	if (p2buf) free (p2buf);

    return rssma_status;    
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_error_message (ViSession instrSession,
                                        ViStatus statusCode,
                                        ViChar _VI_FAR errMessage[])
{
    ViStatus rssma_status = VI_SUCCESS;
    ViInt16 i;

    static rssma_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,                 "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,                    "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,                "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,              "WARNING: Error Query not supported"},
        {VI_WARN_NSUP_REV_QUERY,                "WARNING: Revision Query not supported"},
        {VI_ERROR_FAIL_ID_QUERY,                "ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE,                 "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,              "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,             "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE,  "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER,              "ERROR: Parameter out of range"},

        /* --- Instrument specific error codes --- */

        {RSSMA_ERROR_INSTRUMENT_ERROR,          "ERROR: Instrument status error"},
        {RSSMA_ERROR_INVALID_CONFIGURATION,     "ERROR: Instrument configuration error"},
        {RSSMA_ERROR_INSTRUMENT_OPTION,         "ERROR: Option not installed"},
        {RSSMA_ERROR_INSTRUMENT_NSUP_MODEL,     "ERROR: Required Instrument model is not available."},
        {RSSMA_ERROR_SETTINGS_CONFLICT,         "ERROR: Instrument settings conflict."},
        {RSSMA_ERROR_CALIBRATION_FAILED,        "ERROR: Calibration failed."},

        /* --- Instrument specific warnings --- */

        {RSSMA_WARNING_SMALL_BUFFER,            "WARNING: Buffer too small"},

        {VI_NULL, VI_NULL}
    };

    if (statusCode == VI_ERROR_INSTR_PARAMETER)
	{
	    rssma_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		sprintf (errMessage, "ERROR: Invalid parameter '%s'", instrPtr->parameterErrorMessage);
	}
	else
	{
		rssma_status = viStatusDesc (instrSession, statusCode, errMessage);
	    if ((rssma_status != VI_SUCCESS) || (strstr (errMessage, "Unknown status value") != NULL))
	        {
	        for (i = 0; statusDescArray[i].stringName; i++)
	            {
	            if (statusDescArray[i].stringVal == statusCode)
	                {
	                sprintf (errMessage, "%s", statusDescArray[i].stringName);
	                return (VI_SUCCESS);
	                }
	            }
	        sprintf (errMessage, "Unknown Error 0x%X", statusCode);
	        return (VI_WARN_UNKNOWN_STATUS);
	        }
	}

    rssma_status = VI_SUCCESS;
    return rssma_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_revision_query (ViSession instrSession,
                    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViStatus rssma_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "*IDN?\n"));
    //CHECKERR (viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[\r\n]", instrRev));
    CHECKERR (viScanf (instrSession, "%*[^-]-%[^ (\r\n]", instrRev));

    sprintf (driverRev, "%s", rssma_REVISION);

    return rssma_status;
}

/*===========================================================================*/
/* Function: Delay                                                           */
/* Purpose : With a call to rssma_delay, the current program is suspended    */
/*           from execution for the number of seconds specified by the       */
/*           argument seconds. rssma_delay is accurate to a millisecond.     */
/*           This function is platform independent.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_delay (/*@unused@*/ViSession instrSession, ViReal64 seconds)
{
  clock_t StartTime = clock();
  instrSession;
  
  while ((((double)(clock() - StartTime))/CLOCKS_PER_SEC) < (seconds)){
  #ifdef _WIN32
    #ifdef _CVI_
          ProcessSystemEvents ();
    #else
          Sleep(0);
    #endif
  #endif
  }
  return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssma_close (ViSession instrSession)
{
    rssma_instrRange instrPtr = NULL;
    ViSession rmSession = 0;
    ViStatus rssma_status = VI_SUCCESS;

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));
    CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr != NULL)
        free (instrPtr);

    rssma_status = viClose (instrSession);
    (void) viClose (rmSession);

    return rssma_status;
}

/*===========================================================================*/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rssma_reset operation and by the     */
/*           rssma_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/
/*===========================================================================*/
ViStatus rssma_defaultInstrSetup (ViSession instrSession)
{
    ViChar              buffer[BUFFER_SIZE] = "";

    ViStatus            rssma_status = VI_SUCCESS;
    rssma_instrRange    instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    CHECKNULL (instrPtr);
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> errorChecking = VI_TRUE;
    instrPtr -> optionChecking = VI_TRUE;
    instrPtr -> rangeChecking = VI_TRUE;
    strcpy (instrPtr -> instrDriverRevision, rssma_REVISION);

    /* Check for instrument version */
    CHECKERR (viQueryf (instrSession, "*IDN?\n", "%*[^,],%[^,],%*[\r\n]", buffer));
    
    /* Check for instruments options */
    if ((rssma_status = rssma_OptionIdentificationQuery (instrSession, buffer)) < VI_SUCCESS)
        return rssma_status;
    else
        rssma_status = VI_SUCCESS;

    instrPtr->instrOptions.B20 = (strstr (buffer, "SMA-B20") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.B22 = (strstr (buffer, "SMA-B22") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.B29 = (strstr (buffer, "SMA-B29") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.B80 = (strstr (buffer, "SMA-B80") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.B81 = (strstr (buffer, "SMA-B81") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.B103 = (strstr (buffer, "SMA-B103") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.B106 = (strstr (buffer, "SMA-B106") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K23 = (strstr (buffer, "SMA-K23") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K24 = (strstr (buffer, "SMA-K24") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K25 = (strstr (buffer, "SMA-K25") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K83 = (strstr (buffer, "SMA-K83") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K27 = (strstr (buffer, "SMA-K27") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K26 = (strstr (buffer, "SMA-K26") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K20 = (strstr (buffer, "SMA-K20") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K22 = (strstr (buffer, "SMA-K22") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K28 = (strstr (buffer, "SMA-K28") != VI_NULL) ? 1 : 0;
    
    return rssma_status;
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rssma_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rssma_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rssma_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession,
                             ViStatus currentStatus)
{
    rssma_instrRange instrPtr = NULL;

    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL)
            free (instrPtr);

    (void) viClose (*openInstrSession);
    (void) viClose (openRMSession);
    *openInstrSession = VI_NULL;

    return currentStatus;
}

/*============================================================================
 * Function: rssma_SetSweepMode
 * Purpose:  This function sets the sweep mode.
 *===========================================================================*/
ViStatus rssma_SetSweepMode (ViSession instrSession, 
                             ViInt32   mode)
{
    
    return SETPARAMENUM (instrSession,
                          "SOUR:SWE:MODE",
                          mode,
                          lfSweepModeArr);
}

/*============================================================================
 * Function: rssma_SetFrequencySweepTriggerSource
 * Purpose:  This function sets the trigger source for the RF frequency sweep.
 *===========================================================================*/
ViStatus rssma_SetFrequencySweepTriggerSource (ViSession instrSession,
                                               ViInt32 triggerSourceRFFreqSweep)
{
    return SETPARAMENUM (instrSession,
                          "TRIG:FSW:SOUR",
                          triggerSourceRFFreqSweep,
                          triggerRFSourceArr);
}

/*============================================================================
 * Function: rssma_SetFrequencyMode
 * Purpose:  This function sets the instrument operating mode and therefore
 *           also the functions used to set the output frequency.
 *===========================================================================*/
ViStatus rssma_SetFrequencyMode (ViSession instrSession,
                                 ViInt32 frequencyMode)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:FREQ:MODE",
                          frequencyMode,
                          frequencyModeArr);
}

/*============================================================================
 * Function: rssma_SetLFMode
 * Purpose:  This function sets the instrument operating mode and therefore
 *           also the functions used to set the output frequency.
 *===========================================================================*/
ViStatus rssma_SetLFMode (ViSession instrSession, ViInt32 LFOutputMode)
{
    return SETPARAMENUM (instrSession,
                          "LFO:FREQ:MODE",
                          LFOutputMode,
                          lfOutputModeArr);
}

/*============================================================================
 * Function: rssma_SetLFOutputLFSweepMode
 * Purpose:  This function sets the cycle mode of the LF sweep.
 *===========================================================================*/
ViStatus rssma_SetSweepLFMode(ViSession    instrSession,
                              ViInt32  lfSweepMode)
{
    return SETPARAMENUM (instrSession,
                          "LFO:SWE:MODE",
                          lfSweepMode,
                          lfSweepModeArr);
}

/*============================================================================
 * Function: rssma_SetLFSweepTriggerSource
 * Purpose:  This function sets the trigger source for the LF sweep. The
 *           trigger is triggered by the command
 *           :SOURce:LFOutput:SWEep[:FREQuency]EXECute.
 *===========================================================================*/
ViStatus rssma_SetLFSweepTriggerSource (ViSession instrSession,
                                        ViInt32 triggerSourceLFSweep)
{
    return SETPARAMENUM (instrSession,
                          "TRIG0:SWE:SOUR",
                          triggerSourceLFSweep,
                          triggerRFSourceArr);
}

/*============================================================================
 * Function: rssma_SetLevelSweepTriggerSource
 * Purpose:  This function sets the trigger source for the RF level sweep.
 *===========================================================================*/
ViStatus rssma_SetLevelSweepTriggerSource (ViSession instrSession,
                                             ViInt32 triggerSourceRFLevelSweep)
{
    return SETPARAMENUM (instrSession,
                              "TRIG:PSW:SOUR",
                              triggerSourceRFLevelSweep,
                              triggerRFSourceArr);
}

/*============================================================================
 * Function: rssma_SetSweepLevelMode
 * Purpose:  This function sets the cycle mode of the level sweep.
 *===========================================================================*/
ViStatus rssma_SetSweepLevelMode (ViSession instrSession,
                                  ViInt32 sweepLevelMode)
{
    return SETPARAMENUM (instrSession,
                              "SOUR:SWE:POW:MODE",
                              sweepLevelMode,
                              lfSweepModeArr);
}

/*============================================================================
 * Function: rssma_SetLevelMode
 * Purpose:  This function sets the instrument operating mode and therefore
 *           also the functions used to set the output level.
 *===========================================================================*/
ViStatus rssma_SetLevelMode (ViSession instrSession,
                                    ViInt32 RFLevelMode)
{
    return SETPARAMENUM (instrSession,
                              "SOUR:POW:MODE",
                              RFLevelMode,
                              frequencyModeArr);
}
